@@ -26,12 +26,12 @@
 #include <pthread.h> /* PTHREAD_* pthread_*() */
 
 #include <fcntl.h> /* open() */
-#include <unistd.h>
+#include <unistd.h> /* select() */
 
 #include <assert.h> /* assert() */
 #include <errno.h> /* errno */
 #include <stddef.h> /* wchar_t NULL */
-#include <stdlib.h> /* free() malloc() */
+#include <stdlib.h> /* EXIT_FAILURE _Exit() free() malloc() */
 #include <string.h>
 #include <sys/stat.h> /* O_RDONLY */
 #include <sys/types.h> /* pid_t ssize_t */
@@ -46,11 +46,12 @@
 #include "ui/cancellation.h"
 #include "ui/statusline.h"
 #include "utils/env.h"
+#include "utils/fs.h"
 #include "utils/log.h"
 #include "utils/path.h"
 #include "utils/str.h"
 #include "utils/utils.h"
-#include "commands_completion.h"
+#include "cmd_completion.h"
 #include "status.h"
 
 /**
@@ -321,8 +322,9 @@ background_and_wait_for_status(char cmd[], int cancellable, int *cancelled)
 
 		(void)set_sigchld(0);
 
-		(void)execve(cfg.shell, make_execv_array(cfg.shell, cmd), environ);
-		exit(127);
+		(void)execve(get_execv_path(cfg.shell), make_execv_array(cfg.shell, cmd),
+				environ);
+		_Exit(127);
 	}
 
 	if(cancellable)
@@ -334,7 +336,8 @@ background_and_wait_for_status(char cmd[], int cancellable, int *cancelled)
 	{
 		if(errno != EINTR)
 		{
-			LOG_SERROR_MSG(errno, "Failed waiting for process: "PRINTF_PID_T, pid);
+			LOG_SERROR_MSG(errno, "Failed waiting for process: %" PRINTF_ULL,
+					(unsigned long long)pid);
 			status = -1;
 			break;
 		}
@@ -448,7 +451,7 @@ background_and_wait_for_errors(char cmd[], int cancellable)
 			 * compilation errors in case __USE_BSD is defined.  Anonymous type with
 			 * "const int" is composed via compound literal expression. */
 			int status = get_proc_exit_status(pid);
-			result = WEXITSTATUS(status);
+			result = (status != -1 && WIFEXITED(status)) ? WEXITSTATUS(status) : -1;
 		}
 	}
 
@@ -499,16 +502,16 @@ background_and_capture(char *cmd, int user_sh, FILE **out, FILE **err)
 		close(error_pipe[0]);
 		if(dup2(out_pipe[1], STDOUT_FILENO) == -1)
 		{
-			exit(-1);
+			_Exit(EXIT_FAILURE);
 		}
 		if(dup2(error_pipe[1], STDERR_FILENO) == -1)
 		{
-			exit(-1);
+			_Exit(EXIT_FAILURE);
 		}
 
-		sh = user_sh ? cfg.shell : "/bin/sh";
+		sh = user_sh ? get_execv_path(cfg.shell) : "/bin/sh";
 		execvp(sh, make_execv_array(sh, cmd));
-		exit(-1);
+		_Exit(127);
 	}
 
 	close(out_pipe[1]);
@@ -525,7 +528,7 @@ static pid_t
 background_and_capture_internal(char cmd[], int user_sh, FILE **out, FILE **err,
 		int out_pipe[2], int err_pipe[2])
 {
-	wchar_t *args[5];
+	wchar_t *args[4];
 	char cwd[PATH_MAX];
 	int code;
 	wchar_t *final_wide_cmd;
@@ -537,9 +540,8 @@ background_and_capture_internal(char cmd[], int user_sh, FILE **out, FILE **err,
 		return (pid_t)-1;
 
 	cwd[0] = '\0';
-	if(getcwd(cwd, sizeof(cwd)) != NULL)
+	if(get_cwd(cwd, sizeof(cwd)) != NULL)
 	{
-		to_forward_slash(cwd);
 		if(is_unc_path(cwd))
 		{
 			(void)chdir(get_tmpdir());
@@ -548,14 +550,13 @@ background_and_capture_internal(char cmd[], int user_sh, FILE **out, FILE **err,
 
 	final_wide_cmd = to_wide(cmd);
 
-	wide_sh = to_wide(user_sh ? "cmd" : cfg.shell);
-	if(user_sh || curr_stats.shell_type == ST_CMD)
+	wide_sh = to_wide(user_sh ? cfg.shell : "cmd");
+	if(!user_sh || curr_stats.shell_type == ST_CMD)
 	{
 		args[0] = wide_sh;
-		args[1] = L"/U";
-		args[2] = L"/C";
-		args[3] = final_wide_cmd;
-		args[4] = NULL;
+		args[1] = L"/C";
+		args[2] = final_wide_cmd;
+		args[3] = NULL;
 	}
 	else
 	{
@@ -671,7 +672,7 @@ start_background_job(const char *cmd, int skip_errors)
 		if(dup2(error_pipe[1], STDERR_FILENO) == -1)
 		{
 			perror("dup2");
-			exit(-1);
+			_Exit(EXIT_FAILURE);
 		}
 		close(STDIN_FILENO);
 		close(STDOUT_FILENO);
@@ -685,19 +686,19 @@ start_background_job(const char *cmd, int skip_errors)
 			if(dup2(nullfd, STDIN_FILENO) == -1)
 			{
 				perror("dup2 for stdin");
-				exit(-1);
+				_Exit(EXIT_FAILURE);
 			}
 			if(dup2(nullfd, STDOUT_FILENO) == -1)
 			{
 				perror("dup2 for stdout");
-				exit(-1);
+				_Exit(EXIT_FAILURE);
 			}
 		}
 
 		setpgid(0, 0);
 
-		execve(cfg.shell, make_execv_array(cfg.shell, command), environ);
-		exit(-1);
+		execve(get_execv_path(cfg.shell), make_execv_array(cfg.shell, command), environ);
+		_Exit(127);
 	}
 	else
 	{
@@ -714,9 +715,11 @@ start_background_job(const char *cmd, int skip_errors)
 	free(command);
 #else
 	BOOL ret;
-	STARTUPINFO startup = {};
+	STARTUPINFOW startup = {};
 	PROCESS_INFORMATION pinfo;
 	char *command;
+	char *sh_cmd;
+	wchar_t *wide_cmd;
 
 	command = cfg.fast_run ? fast_run_complete(cmd) : strdup(cmd);
 	if(command == NULL)
@@ -724,21 +727,27 @@ start_background_job(const char *cmd, int skip_errors)
 		return -1;
 	}
 
-	ret = CreateProcess(NULL, command, NULL, NULL, 0, 0, NULL, NULL, &startup,
+	sh_cmd = win_make_sh_cmd(command);
+	free(command);
+
+	wide_cmd = to_wide(sh_cmd);
+	ret = CreateProcessW(NULL, wide_cmd, NULL, NULL, 0, 0, NULL, NULL, &startup,
 			&pinfo);
+	free(wide_cmd);
+
 	if(ret != 0)
 	{
 		CloseHandle(pinfo.hThread);
 
-		job = add_background_job(pinfo.dwProcessId, command, pinfo.hProcess,
+		job = add_background_job(pinfo.dwProcessId, sh_cmd, pinfo.hProcess,
 				BJT_COMMAND);
 		if(job == NULL)
 		{
-			free(command);
+			free(sh_cmd);
 			return -1;
 		}
 	}
-	free(command);
+	free(sh_cmd);
 	if(ret == 0)
 	{
 		return 1;
@@ -757,6 +766,8 @@ bg_execute(const char descr[], const char op_descr[], int total, int important,
 		bg_task_func task_func, void *args)
 {
 	pthread_t id;
+	pthread_attr_t attr;
+	int ret;
 
 	background_task_args *const task_args = malloc(sizeof(*task_args));
 	if(task_args == NULL)
@@ -772,7 +783,20 @@ bg_execute(const char descr[], const char op_descr[], int total, int important,
 	if(task_args->job == NULL)
 	{
 		free(task_args);
-		return 2;
+		return 1;
+	}
+
+	if(pthread_attr_init(&attr) != 0)
+	{
+		free(task_args);
+		return 1;
+	}
+
+	if(pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED) != 0)
+	{
+		free(task_args);
+		(void)pthread_attr_destroy(&attr);
+		return 1;
 	}
 
 	replace_string(&task_args->job->bg_op.descr, op_descr);
@@ -783,17 +807,19 @@ bg_execute(const char descr[], const char op_descr[], int total, int important,
 		ui_stat_job_bar_add(&task_args->job->bg_op);
 	}
 
-	if(pthread_create(&id, NULL, &background_task_bootstrap, task_args) != 0)
+	ret = 0;
+	if(pthread_create(&id, &attr, &background_task_bootstrap, task_args) != 0)
 	{
 		/* Mark job as finished with error. */
 		task_args->job->running = 0;
 		task_args->job->exit_code = 1;
 
 		free(task_args);
-		return 3;
+		ret = 1;
 	}
 
-	return 0;
+	(void)pthread_attr_destroy(&attr);
+	return ret;
 }
 
 /* Creates structure that describes background job and registers it in the list
@@ -806,9 +832,8 @@ static job_t *
 add_background_job(pid_t pid, const char cmd[], HANDLE hprocess, BgJobType type)
 #endif
 {
-	job_t *new;
-
-	if((new = malloc(sizeof(job_t))) == 0)
+	job_t *new = malloc(sizeof(*new));
+	if(new == NULL)
 	{
 		show_error_msg("Memory error", "Unable to allocate enough memory");
 		return NULL;
@@ -839,7 +864,7 @@ add_background_job(pid_t pid, const char cmd[], HANDLE hprocess, BgJobType type)
 	return new;
 }
 
-/* Pthreads entry point for a new background task.  Performs correct
+/* pthreads entry point for a new background task.  Performs correct
  * startup/exit with related updates of internal data structures.  Returns
  * result for this thread. */
 static void *
@@ -950,4 +975,4 @@ bg_op_set_descr(bg_op_t *bg_op, const char descr[])
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
