@@ -23,10 +23,13 @@
 #include <assert.h> /* assert() */
 #include <ctype.h> /* tolower() */
 #include <stddef.h> /* NULL size_t wchar_t */
-#include <stdlib.h> /* free() malloc() qsort() */
+#include <stdlib.h> /* free() qsort() */
 #include <string.h> /* strcpy() strlen() */
 #include <wchar.h> /* wcscpy() wcslen() */
+#include <wctype.h> /* iswcntrl() */
 
+#include "compat/reallocarray.h"
+#include "engine/text_buffer.h"
 #include "utils/macros.h"
 #include "utils/str.h"
 
@@ -41,6 +44,7 @@ key_pair_t;
 
 static int notation_sorter(const void *first, const void *second);
 static const key_pair_t * find_notation(const wchar_t str[]);
+static const char * wchar_to_spec(const wchar_t c[], size_t *len, int bs);
 
 /* All notation fields must be written in lower case. */
 static key_pair_t key_pairs[] = {
@@ -533,7 +537,7 @@ substitute_specsw(const wchar_t cmd[])
 	const size_t len = wcslen(cmd) + 1;
 	const wchar_t *s = cmd;
 
-	buf = malloc(len*sizeof(wchar_t));
+	buf = reallocarray(NULL, len, sizeof(wchar_t));
 	if(buf == NULL)
 	{
 		free(buf);
@@ -593,18 +597,45 @@ find_notation(const wchar_t str[])
 	return NULL;
 }
 
-const char *
-wchar_to_spec(const wchar_t c[], size_t *len)
+char *
+wstr_to_spec(const wchar_t str[])
+{
+	vle_textbuf *const descr = vle_tb_create();
+
+	const size_t str_len = wcslen(str);
+	size_t seq_len;
+	size_t i;
+
+	for(i = 0U; i < str_len; i += seq_len)
+	{
+		if(str[i] == L' ')
+		{
+			vle_tb_append(descr, " ");
+			seq_len = 1U;
+		}
+		else
+		{
+			vle_tb_append(descr, wchar_to_spec(&str[i], &seq_len, (i == 0U)));
+		}
+	}
+
+	return vle_tb_release(descr);
+}
+
+/* Converts unicode character(s) starting at c into string form representing
+ * corresponding key.  Upon exit *len is set to number of used characters from
+ * the string.  Returns pointer to internal buffer. */
+static const char *
+wchar_to_spec(const wchar_t c[], size_t *len, int bs)
 {
 	/* TODO: refactor this function wchar_to_spec() */
 
-	static char buf[32];
+	static char buf[256];
 
 	*len = 1;
 	switch(*c)
 	{
 		case L' ':          strcpy(buf, "<space>");    break;
-		case L'\b':         strcpy(buf, "<bs>");       break;
 		case L'\r':         strcpy(buf, "<cr>");       break;
 		case L'\n':         strcpy(buf, "<c-j>");      break;
 		case L'\177':       strcpy(buf, "<del>");      break;
@@ -618,7 +649,16 @@ wchar_to_spec(const wchar_t c[], size_t *len)
 		case KEY_BTAB:      strcpy(buf, "<s-tab>");    break;
 		case KEY_PPAGE:     strcpy(buf, "<pageup>");   break;
 		case KEY_NPAGE:     strcpy(buf, "<pagedown>"); break;
-		case KEY_BACKSPACE: strcpy(buf, "<bs>");       break;
+
+		case L'\b':
+			if(!bs)
+			{
+				goto def;
+			}
+			/* Fall through. */
+		case KEY_BACKSPACE:
+			strcpy(buf, "<bs>");
+			break;
 
 		case L'\033':
 			if(c[1] == L'[' && c[2] == 'Z')
@@ -638,6 +678,7 @@ wchar_to_spec(const wchar_t c[], size_t *len)
 			break;
 
 		default:
+		def:
 			if(*c == '\n' || (*c > L' ' && *c < 256))
 			{
 				buf[0] = *c;
@@ -684,15 +725,22 @@ wchar_to_spec(const wchar_t c[], size_t *len)
 				buf[2] += (*c - KEY_F0)/10;
 				buf[3] += (*c - KEY_F0)%10;
 			}
-			else
+			else if(iswcntrl(*c))
 			{
 				strcpy(buf, "<c-A>");
 				buf[3] = tolower(buf[3] + *c - 1);
 			}
+			else
+			{
+				const wchar_t wchar[] = { *c, L'\0' };
+				char *const mb = to_multibyte(wchar);
+				copy_str(buf, sizeof(buf), mb);
+				free(mb);
+			}
 			break;
 	}
 	return buf;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
