@@ -33,22 +33,32 @@
 #include "utils/path.h"
 #include "utils/string_array.h"
 #include "utils/utils.h"
+#include "filelist.h"
 #include "macros.h"
 #include "types.h"
 
+static var_t chooseopt_builtin(const call_info_t *call_info);
 static var_t executable_builtin(const call_info_t *call_info);
 static var_t expand_builtin(const call_info_t *call_info);
 static var_t filetype_builtin(const call_info_t *call_info);
 static int get_fnum(const char position[]);
+static var_t getpanetype_builtin(const call_info_t *call_info);
 static var_t has_builtin(const call_info_t *call_info);
+static var_t layoutis_builtin(const call_info_t *call_info);
+static var_t paneisat_builtin(const call_info_t *call_info);
 static var_t system_builtin(const call_info_t *call_info);
 
 static const function_t functions[] = {
-	{ "executable", 1, &executable_builtin },
-	{ "expand",     1, &expand_builtin },
-	{ "filetype",   1, &filetype_builtin },
-	{ "has",        1, &has_builtin },
-	{ "system",     1, &system_builtin },
+	/* Name          Description                 Argc  Handler  */
+	{ "chooseopt",   "query choose options",       1, &chooseopt_builtin },
+	{ "executable",  "check for executable file",  1, &executable_builtin },
+	{ "expand",      "expand macros in a string",  1, &expand_builtin },
+	{ "filetype",    "retrieve type of a file",    1, &filetype_builtin },
+	{ "getpanetype", "retrieve type of file list", 0, &getpanetype_builtin},
+	{ "has",         "check for specific ability", 1, &has_builtin },
+	{ "layoutis",    "query current layout",       1, &layoutis_builtin },
+	{ "paneisat",    "query pane location",        1, &paneisat_builtin },
+	{ "system",      "execute external command",   1, &system_builtin },
 };
 
 void
@@ -59,10 +69,46 @@ init_builtin_functions(void)
 	{
 		int result = function_register(&functions[i]);
 		assert(result == 0 && "Builtin function registration error");
+		(void)result;
 	}
 }
 
-/* Checks whether executable exists at absolute path orin directories listed in
+/* Retrieves values of options related to file choosing as a string.  On unknown
+ * arguments empty string is returned. */
+static var_t
+chooseopt_builtin(const call_info_t *call_info)
+{
+	var_val_t var_val = { .string = NULL };
+	char *type;
+
+	type = var_to_string(call_info->argv[0]);
+	if(strcmp(type, "files") == 0)
+	{
+		var_val.string = curr_stats.chosen_files_out;
+	}
+	else if(strcmp(type, "dir") == 0)
+	{
+		var_val.string = curr_stats.chosen_dir_out;
+	}
+	else if(strcmp(type, "cmd") == 0)
+	{
+		var_val.string = curr_stats.on_choose;
+	}
+	else if(strcmp(type, "delimiter") == 0)
+	{
+		var_val.string = curr_stats.output_delimiter;
+	}
+	free(type);
+
+	if(var_val.string == NULL)
+	{
+		var_val.string = "";
+	}
+
+	return var_new(VTYPE_STRING, var_val);
+}
+
+/* Checks whether executable exists at absolute path or in directories listed in
  * $PATH when path isn't absolute.  Checks for various executable extensions on
  * Windows.  Returns boolean value describing result of the check. */
 static var_t
@@ -133,10 +179,60 @@ get_fnum(const char position[])
 	{
 		return curr_view->list_pos;
 	}
+	return -1;
+}
+
+/* Retrieves type of current pane as a string. */
+static var_t
+getpanetype_builtin(const call_info_t *call_info)
+{
+	FileView *const view = curr_view;
+	var_val_t var_val;
+
+	if(flist_custom_active(view))
+	{
+		var_val.string = (view->custom.unsorted ? "very-custom" : "custom");
+	}
 	else
 	{
-		return -1;
+		var_val.string = "regular";
 	}
+
+	return var_new(VTYPE_STRING, var_val);
+}
+
+/* Checks current layout configuration.  Returns boolean value that reflects
+ * state of specified layout type. */
+static var_t
+layoutis_builtin(const call_info_t *call_info)
+{
+	char *type;
+	int result;
+
+	type = var_to_string(call_info->argv[0]);
+	if(strcmp(type, "only") == 0)
+	{
+		result = (curr_stats.number_of_windows == 1);
+	}
+	else if(strcmp(type, "split") == 0)
+	{
+		result = (curr_stats.number_of_windows == 2);
+	}
+	else if(strcmp(type, "vsplit") == 0)
+	{
+		result = (curr_stats.number_of_windows == 2 && curr_stats.split == VSPLIT);
+	}
+	else if(strcmp(type, "hsplit") == 0)
+	{
+		result = (curr_stats.number_of_windows == 2 && curr_stats.split == HSPLIT);
+	}
+	else
+	{
+		result = 0;
+	}
+	free(type);
+
+	return var_from_bool(result);
 }
 
 /* Allows examining internal parameters from scripts to e.g. figure out
@@ -166,6 +262,44 @@ has_builtin(const call_info_t *call_info)
 	return result;
 }
 
+/* Checks for relative position of current pane.  Returns boolean value that
+ * reflects state of specified location. */
+static var_t
+paneisat_builtin(const call_info_t *call_info)
+{
+	char *loc;
+	int result;
+
+	const int only = (curr_stats.number_of_windows == 1);
+	const int vsplit = (curr_stats.split == VSPLIT);
+	const int first = (curr_view == &lwin);
+
+	loc = var_to_string(call_info->argv[0]);
+	if(strcmp(loc, "top") == 0)
+	{
+		result = (only || vsplit || first);
+	}
+	else if(strcmp(loc, "bottom") == 0)
+	{
+		result = (only || vsplit || !first);
+	}
+	else if(strcmp(loc, "left") == 0)
+	{
+		result = (only || !vsplit || first);
+	}
+	else if(strcmp(loc, "right") == 0)
+	{
+		result = (only || !vsplit || !first);
+	}
+	else
+	{
+		result = 0;
+	}
+	free(loc);
+
+	return var_from_bool(result);
+}
+
 /* Runs the command in shell and returns its output (joined standard output and
  * standard error streams).  All trailing newline characters are stripped to
  * allow easy appending to command output.  Returns the output. */
@@ -206,4 +340,4 @@ system_builtin(const call_info_t *call_info)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
