@@ -34,27 +34,29 @@
 
 #include <assert.h> /* assert() */
 #include <limits.h> /* INT_MIN */
-#include <stddef.h> /* size_t */
+#include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* FILE snprintf() */
 #include <stdlib.h> /* free() */
 #include <string.h> /* memmove() memset() strdup() */
 
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
+#include "../compat/reallocarray.h"
+#include "../int/term_title.h"
 #include "../io/iop.h"
 #include "../modes/dialogs/msg_dialog.h"
 #include "../ui/ui.h"
 #include "../utils/env.h"
 #include "../utils/file_streams.h"
 #include "../utils/fs.h"
-#include "../utils/fs_limits.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
 #include "../utils/str.h"
 #include "../utils/path.h"
 #include "../utils/utils.h"
-#include "../bookmarks.h"
-#include "../commands.h"
+#include "../cmd_core.h"
 #include "../filelist.h"
+#include "../marks.h"
 #include "../opt_handlers.h"
 #include "../status.h"
 #include "../types.h"
@@ -95,11 +97,11 @@ static int try_vifm_vifmrc_for_vifmrc(void);
 static void store_config_paths(void);
 static void setup_dirs(void);
 static void copy_help_file(void);
+static void create_scripts_dir(void);
 static void copy_rc_file(void);
-static void add_default_bookmarks(void);
+static void add_default_marks(void);
 static int source_file_internal(FILE *fp, const char filename[]);
 static void show_sourcing_error(const char filename[], int line_num);
-static int is_conf_file(const char file[]);
 static void disable_history(void);
 static void free_view_history(FileView *view);
 static void decrease_history(size_t new_len, size_t removed_count);
@@ -126,28 +128,25 @@ cfg_init(void)
 	cfg.sort_numbers = 0;
 	cfg.follow_links = 1;
 	cfg.fast_run = 0;
-	cfg.confirm = 1;
+	cfg.confirm = CONFIRM_DELETE | CONFIRM_PERM_DELETE;
 	cfg.vi_command = strdup("vim");
 	cfg.vi_cmd_bg = 0;
 	cfg.vi_x_command = strdup("");
 	cfg.vi_x_cmd_bg = 0;
 	cfg.use_trash = 1;
-
-	{
-		char fuse_home[PATH_MAX];
-		int update_stat;
-		snprintf(fuse_home, sizeof(fuse_home), "%s/vifm_FUSE", get_tmpdir());
-		update_stat = cfg_set_fuse_home(fuse_home);
-		assert(update_stat == 0);
-	}
-
 	cfg.use_term_multiplexer = 0;
 	cfg.use_vim_help = 0;
 	cfg.wild_menu = 0;
+	cfg.wild_popup = 0;
+
+	cfg.sug.flags = 0;
+	cfg.sug.maxregfiles = 5;
+	cfg.sug.delay = 500;
+
 	cfg.ignore_case = 0;
 	cfg.smart_case = 0;
 	cfg.hl_search = 1;
-	cfg.vifm_info = VIFMINFO_BOOKMARKS;
+	cfg.vifm_info = VIFMINFO_MARKS | VIFMINFO_BOOKMARKS;
 	cfg.auto_ch_pos = 1;
 	cfg.scroll_off = 0;
 	cfg.gdefault = 0;
@@ -173,6 +172,7 @@ cfg_init(void)
 			"-type d \\( ! -readable -o ! -executable \\) -prune");
 	cfg.grep_prg = strdup("grep -n -H -I -r %i %a %s");
 	cfg.locate_prg = strdup("locate %a");
+	cfg.delete_prg = strdup("");
 
 	cfg.trunc_normal_sb_msgs = 0;
 	cfg.shorten_title_paths = 1;
@@ -182,38 +182,40 @@ cfg_init(void)
 	cfg.cd_path = strdup(env_get_def("CDPATH", DEFAULT_CD_PATH));
 	replace_char(cfg.cd_path, ':', ',');
 
-	cfg.filelist_col_padding = 1;
+	cfg.extra_padding = 1;
 	cfg.side_borders_visible = 1;
 	cfg.display_statusline = 1;
 
 	cfg.border_filler = strdup(" ");
 
+	cfg.set_title = term_title_restorable();
+
 	cfg.chase_links = 0;
 
 	cfg.timeout_len = 1000;
 	cfg.min_timeout_len = 150;
 
-	/* Fill cfg.word_chars as if it was initialized from isspace() fuction. */
+	/* Fill cfg.word_chars as if it was initialized from isspace() function. */
 	memset(&cfg.word_chars, 1, sizeof(cfg.word_chars));
 	cfg.word_chars['\x00'] = 0; cfg.word_chars['\x09'] = 0;
 	cfg.word_chars['\x0a'] = 0; cfg.word_chars['\x0b'] = 0;
 	cfg.word_chars['\x0c'] = 0; cfg.word_chars['\x0d'] = 0;
 	cfg.word_chars['\x20'] = 0;
 
-#ifndef _WIN32
-	copy_str(cfg.log_file, sizeof(cfg.log_file), "/var/log/vifm-startup-log");
-#else
-	{
-		char exe_dir[PATH_MAX];
-		(void)get_exe_dir(exe_dir, sizeof(exe_dir));
-		snprintf(cfg.log_file, sizeof(cfg.log_file), "%s/startup-log", exe_dir);
-	}
-#endif
+	cfg.view_dir_size = VDS_SIZE;
+
+	cfg.log_file[0] = '\0';
 
 	cfg_set_shell(env_get_def("SHELL", DEFAULT_SHELL_CMD));
 
-	memset(&cfg.decorations, '\0', sizeof(cfg.decorations));
-	cfg.decorations[FT_DIR][DECORATION_SUFFIX] = '/';
+	memset(&cfg.type_decs, '\0', sizeof(cfg.type_decs));
+	cfg.type_decs[FT_DIR][DECORATION_SUFFIX][0] = '/';
+
+	cfg.name_decs = NULL;
+	cfg.name_dec_count = 0;
+
+	cfg.fast_file_cloning = 0;
+	cfg.cvoptions = 0;
 }
 
 void
@@ -517,20 +519,31 @@ store_config_paths(void)
 {
 	LOG_FUNC_ENTER;
 
+	const char *const trash_dir_fmt =
+#ifndef _WIN32
+			"%%r/.vifm-Trash-%%u,%s/" TRASH ",%%r/.vifm-Trash";
+#else
+			"%%r/.vifm-Trash,%s/" TRASH;
+#endif
+
+	char *fuse_home;
 	const char *trash_base = path_exists_at(env_get(VIFM_EV), TRASH, DEREF)
 	                       ? cfg.config_dir
 	                       : cfg.data_dir;
-	const char *log_base = path_exists(cfg.data_dir, DEREF)
-	                     ? cfg.data_dir
-	                     : cfg.config_dir;
+	const char *base = path_exists(cfg.data_dir, DEREF)
+	                 ? cfg.data_dir
+	                 : cfg.config_dir;
 
 	snprintf(cfg.home_dir, sizeof(cfg.home_dir), "%s/", env_get(HOME_EV));
 	copy_str(cfg.config_dir, sizeof(cfg.config_dir), env_get(VIFM_EV));
 	snprintf(cfg.colors_dir, sizeof(cfg.colors_dir), "%s/colors/",
 			cfg.config_dir);
-	snprintf(cfg.trash_dir, sizeof(cfg.trash_dir), "%%r/.vifm-Trash,%s/" TRASH,
-			trash_base);
-	snprintf(cfg.log_file, sizeof(cfg.log_file), "%s/" LOG, log_base);
+	snprintf(cfg.trash_dir, sizeof(cfg.trash_dir), trash_dir_fmt, trash_base);
+	snprintf(cfg.log_file, sizeof(cfg.log_file), "%s/" LOG, base);
+
+	fuse_home = format_str("%s/fuse/", base);
+	(void)cfg_set_fuse_home(fuse_home);
+	free(fuse_home);
 }
 
 /* Ensures existence of configuration and data directories.  Performs first run
@@ -544,12 +557,14 @@ setup_dirs(void)
 
 	if(is_dir(cfg.config_dir))
 	{
-		/* We rely on this file for :help, so make sure it's there. */
+		/* We rely on this file for :help, so make sure it's there on every run. */
 		copy_help_file();
+
+		create_scripts_dir();
 		return;
 	}
 
-	if(!is_dir(cfg.config_dir) && make_path(cfg.config_dir, S_IRWXU) != 0)
+	if(make_path(cfg.config_dir, S_IRWXU) != 0)
 	{
 		return;
 	}
@@ -563,7 +578,7 @@ setup_dirs(void)
 	snprintf(rc_file, sizeof(rc_file), "%s/" VIFMRC, cfg.config_dir);
 	env_set(MYVIFMRC_EV, rc_file);
 
-	add_default_bookmarks();
+	add_default_marks();
 }
 
 /* Copies help file from shared files to the ~/.vifm directory if it's not
@@ -589,6 +604,38 @@ copy_help_file(void)
 	(void)iop_cp(&args);
 }
 
+/* Responsible for creation of scripts/ directory if it doesn't exist (in which
+ * case a README file is created as well). */
+static void
+create_scripts_dir(void)
+{
+	char scripts[PATH_MAX];
+	char readme[PATH_MAX];
+	FILE *fp;
+
+	snprintf(scripts, sizeof(scripts), "%s/" SCRIPTS_DIR, cfg.config_dir);
+	if(create_path(scripts, S_IRWXU) != 0)
+	{
+		return;
+	}
+
+	snprintf(readme, sizeof(readme), "%s/README", scripts);
+	fp = os_fopen(readme, "w");
+	if(fp == NULL)
+	{
+		return;
+	}
+
+	fputs("This directory is dedicated for user-supplied scripts/executables.\n"
+				"vifm modifies its PATH environment variable to let user run those\n"
+				"scripts without specifying full path.  All subdirectories are added\n"
+				"as well.  File in a subdirectory overrules file with the same name\n"
+				"in parent directories.  Restart might be needed to recognize files\n"
+				"in newly created or renamed subdirectories.", fp);
+
+	fclose(fp);
+}
+
 /* Copies example vifmrc file from shared files to the ~/.vifm directory. */
 static void
 copy_rc_file(void)
@@ -609,20 +656,25 @@ copy_rc_file(void)
 	(void)iop_cp(&args);
 }
 
-/* Adds 'H' and 'z' default bookmarks. */
+/* Adds 'H' and 'z' default marks. */
 static void
-add_default_bookmarks(void)
+add_default_marks(void)
 {
 	LOG_FUNC_ENTER;
 
-	set_user_bookmark('H', cfg.home_dir, NO_BOOKMARK_FILE);
-	set_user_bookmark('z', cfg.config_dir, NO_BOOKMARK_FILE);
+	set_user_mark('H', cfg.home_dir, NO_MARK_FILE);
+	set_user_mark('z', cfg.config_dir, NO_MARK_FILE);
 }
 
 void
 cfg_load(void)
 {
 	const char *rc;
+	const int prev_global_local_settings = curr_stats.global_local_settings;
+
+	/* Make changes of usually local settings during configuration affect all
+	 * views. */
+	curr_stats.global_local_settings = 1;
 
 	/* Try to load global configuration. */
 	(void)cfg_source_file(PACKAGE_SYSCONF_DIR "/" VIFMRC);
@@ -633,6 +685,8 @@ cfg_load(void)
 	{
 		(void)cfg_source_file(rc);
 	}
+
+	curr_stats.global_local_settings = prev_global_local_settings;
 }
 
 int
@@ -730,46 +784,13 @@ source_file_internal(FILE *fp, const char filename[])
 static void
 show_sourcing_error(const char filename[], int line_num)
 {
+	curr_stats.save_msg = 1;
+
 	/* User choice is saved by prompt_error_msgf internally. */
-	(void)prompt_error_msgf("File Sourcing Error", "Error in %s at %d line",
+	(void)prompt_error_msgf("File Sourcing Error", "Error in %s at line %d",
 			filename, line_num);
-}
 
-int
-cfg_has_old_format(void)
-{
-	const char *const myvifmrc = env_get(MYVIFMRC_EV);
-	return (myvifmrc != NULL) && is_conf_file(myvifmrc);
-}
-
-/* Checks whether file is configuration file (has at least one line which starts
- * with a hash symbol).  Returns non-zero if yes, otherwise zero is returned. */
-static int
-is_conf_file(const char file[])
-{
-	FILE *const fp = os_fopen(file, "r");
-	char *line = NULL;
-
-	if(fp != NULL)
-	{
-		while((line = read_line(fp, line)) != NULL)
-		{
-			if(skip_whitespace(line)[0] == '#')
-			{
-				break;
-			}
-		}
-		fclose(fp);
-		free(line);
-	}
-	return line != NULL;
-}
-
-int
-cfg_has_old_color_schemes(void)
-{
-	return !is_dir(cfg.colors_dir)
-	    && path_exists_at(cfg.config_dir, "colorschemes", DEREF);
+	curr_stats.save_msg = 0;
 }
 
 const char *
@@ -893,16 +914,17 @@ reduce_view_history(FileView *view, int size)
 static void
 reallocate_history(size_t new_len)
 {
-	const size_t hist_item_len = sizeof(history_t)*new_len;
-	const size_t str_item_len = sizeof(char *)*new_len;
+	lwin.history = reallocarray(lwin.history, new_len, sizeof(history_t));
+	rwin.history = reallocarray(rwin.history, new_len, sizeof(history_t));
 
-	lwin.history = realloc(lwin.history, hist_item_len);
-	rwin.history = realloc(rwin.history, hist_item_len);
-
-	cfg.cmd_hist.items = realloc(cfg.cmd_hist.items, str_item_len);
-	cfg.search_hist.items = realloc(cfg.search_hist.items, str_item_len);
-	cfg.prompt_hist.items = realloc(cfg.prompt_hist.items, str_item_len);
-	cfg.filter_hist.items = realloc(cfg.filter_hist.items, str_item_len);
+	cfg.cmd_hist.items = reallocarray(cfg.cmd_hist.items, new_len,
+			sizeof(char *));
+	cfg.search_hist.items = reallocarray(cfg.search_hist.items, new_len,
+			sizeof(char *));
+	cfg.prompt_hist.items = reallocarray(cfg.prompt_hist.items, new_len,
+			sizeof(char *));
+	cfg.filter_hist.items = reallocarray(cfg.filter_hist.items, new_len,
+			sizeof(char *));
 }
 
 /* Zeroes new elements of the history.  The old_len specifies old history size,
@@ -1028,5 +1050,13 @@ cfg_parent_dir_is_visible(int in_root)
 			(!in_root && (cfg.dot_dirs & DD_NONROOT_PARENT)));
 }
 
+int
+cfg_confirm_delete(int to_trash)
+{
+	return to_trash
+	     ? (cfg.confirm & CONFIRM_DELETE)
+	     : (cfg.confirm & CONFIRM_PERM_DELETE);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
