@@ -22,9 +22,9 @@
 
 #include <stddef.h> /* size_t wchar_t */
 
+#include "../compat/fs_limits.h"
+#include "../ui/color_scheme.h"
 #include "../ui/ui.h"
-#include "../utils/fs_limits.h"
-#include "../color_scheme.h"
 #include "../types.h"
 #include "hist.h"
 
@@ -49,13 +49,60 @@ typedef enum
 }
 DotDirs;
 
-/* Indexes for cfg.decorations. */
+/* Possible flags that regulate key suggestions. */
+typedef enum
+{
+	SF_NORMAL            = 1 << 0, /* Display in normal mode. */
+	SF_VISUAL            = 1 << 1, /* Display in visual mode. */
+	SF_VIEW              = 1 << 2, /* Display in view mode. */
+	SF_OTHERPANE         = 1 << 3, /* Display list in other pane, if available. */
+	SF_DELAY             = 1 << 4, /* Postpone suggestions by small delay. */
+	SF_KEYS              = 1 << 5, /* Include keys suggestions in results. */
+	SF_MARKS             = 1 << 6, /* Include marks suggestions in results. */
+	SF_REGISTERS         = 1 << 7, /* Include registers suggestions in results. */
+	NUM_SUGGESTION_FLAGS =      8  /* Number of flags. */
+}
+SuggestionFlags;
+
+/* What should be displayed as size of a directory in a view by default. */
+typedef enum
+{
+	VDS_SIZE,   /* Size of a directory (not its content). */
+	VDS_NITEMS, /* Number of items in a directory. */
+}
+ViewDirSize;
+
+/* Indexes for cfg.type_decs. */
 enum
 {
 	DECORATION_PREFIX, /* The symbol, which is prepended to file name. */
 	DECORATION_SUFFIX, /* The symbol, which is appended to file name. */
 };
 
+/* Operations that require user confirmation. */
+enum
+{
+	CONFIRM_DELETE      = 1, /* Deletion to trash. */
+	CONFIRM_PERM_DELETE = 2, /* Permanent deletion. */
+};
+
+/* Custom view configuration. */
+enum
+{
+	CVO_AUTOCMDS    = 1, /* Trigger autocommands on entering/leaving [v]cv. */
+	CVO_LOCALOPTS   = 2, /* Reset local options on entering/leaving [v]cv. */
+	CVO_LOCALFILTER = 4, /* Reset local filter on entering/leaving [v]cv. */
+};
+
+/* File decoration description. */
+typedef struct
+{
+	struct matchers_t *matchers; /* Name matcher object. */
+	char prefix[9];              /* File name prefix. */
+	char suffix[9];              /* File name suffix. */
+}
+file_dec_t;
+
 typedef struct config_t
 {
 	char home_dir[PATH_MAX];   /* Ends with a slash. */
@@ -82,7 +129,8 @@ typedef struct config_t
 	int use_iec_prefixes;
 	int wrap_quick_view;
 	char *time_format;
-	char *fuse_home; /* This one should be set using set_fuse_home() function. */
+	/* This one should be set using cfg_set_fuse_home() function. */
+	char *fuse_home;
 
 	/* History of command-line commands. */
 	hist_t cmd_hist;
@@ -95,12 +143,31 @@ typedef struct config_t
 
 	col_scheme_t cs;
 
-	int undo_levels; /* Maximum number of changes that can be undone. */
+	int undo_levels;  /* Maximum number of changes that can be undone. */
 	int sort_numbers; /* Natural sort of (version) numbers within text. */
 	int follow_links; /* Follow links on l or Enter. */
-	int confirm; /* Ask user about permanent deletion of files. */
 	int fast_run;
+
+	int confirm; /* About which operations user should be asked (CONFIRM_*). */
+
+	/* Whether wild menu should be used. */
 	int wild_menu;
+	/* Whether wild menu should be a popup instead of a bar. */
+	int wild_popup;
+
+	/* Settings related to suggestions. */
+	struct
+	{
+		/* Combination of SuggestionFlags, configures when and how to show
+		 * suggestions. */
+		int flags;
+		/* Maximum number of register files to display in suggestions. */
+		int maxregfiles;
+		/* Delay before displaying suggestions (in milliseconds). */
+		int delay;
+	}
+	sug;
+
 	int ignore_case;
 	int smart_case;
 	int hl_search;
@@ -122,12 +189,20 @@ typedef struct config_t
 	int columns; /* Terminal width in characters. */
 	/* Controls displaying of dot directories.  Combination of DotDirs flags. */
 	int dot_dirs;
-	char decorations[FT_COUNT][2]; /* File type specific refixes and suffixes. */
+	/* File type specific prefixes and suffixes ('classify'). */
+	char type_decs[FT_COUNT][2][9];
+	/* File name specific prefixes and suffixes ('classify'). */
+	file_dec_t *name_decs;
+	/* Size of name_decs array. */
+	int name_dec_count;
 	int filter_inverted_by_default; /* Default inversion value for :filter. */
+
+	/* Invocation formats for external applications. */
 	char *apropos_prg; /* apropos tool calling pattern. */
-	char *find_prg; /* find tool calling pattern. */
-	char *grep_prg; /* grep tool calling pattern. */
-	char *locate_prg; /* locate tool calling pattern. */
+	char *find_prg;    /* find tool calling pattern. */
+	char *grep_prg;    /* grep tool calling pattern. */
+	char *locate_prg;  /* locate tool calling pattern. */
+	char *delete_prg;  /* File removal application. */
 
 	/* Message shortening controlled by 'shortmess'. */
 	int trunc_normal_sb_msgs; /* Truncate normal status bar messages if needed. */
@@ -140,8 +215,9 @@ typedef struct config_t
 	char *cd_path;
 
 	/* Whether there should be reserved single character width space before and
-	 * after file list column inside a view. */
-	int filelist_col_padding;
+	 * after file list column inside a view and first and last columns and lines
+	 * for a quick view. */
+	int extra_padding;
 
 	/* Whether side borders are visible (separator in the middle isn't
 	 * affected). */
@@ -153,6 +229,9 @@ typedef struct config_t
 	/* Per line pattern for borders. */
 	char *border_filler;
 
+	/* Whether terminal title should be updated or not. */
+	int set_title;
+
 	/* Whether directory path should always be resolved to real path (all symbolic
 	 * link expanded). */
 	int chase_links;
@@ -161,6 +240,14 @@ typedef struct config_t
 	int min_timeout_len; /* Minimum period on waiting for the input. */
 
 	char word_chars[256]; /* Whether corresponding character is a word char. */
+
+	ViewDirSize view_dir_size; /* Type of size display for directories in view. */
+
+	/* Controls use of fast file cloning for file systems that support it. */
+	int fast_file_cloning;
+
+	/* Whether various things should be reset on entering/leaving custom views. */
+	int cvoptions;
 }
 config_t;
 
@@ -181,14 +268,6 @@ void cfg_load(void);
 /* Returns non-zero on error. */
 int cfg_source_file(const char filename[]);
 
-/* Checks whether vifmrc file (pointed to by the $MYVIFMRC) has old format.
- * Returns non-zero if so, otherwise zero is returned. */
-int cfg_has_old_format(void);
-
-/* Checks whether color scheme are stored in obsolete single-file format.
- * Returns non-zero if so, otherwise zero is returned. */
-int cfg_has_old_color_schemes(void);
-
 /* Gets editor invocation command.  Sets *bg to indicate whether the command
  * should be executed in background.  Returns pointer to a string from
  * configuration variables. */
@@ -235,7 +314,12 @@ int cfg_is_word_wchar(wchar_t c);
  * and zero otherwise. */
 int cfg_parent_dir_is_visible(int in_root);
 
+/* Checks whether file deletion (possibly into trash) requires user confirmation
+ * according to current configuration.  Returns non-zero if so, otherwise zero
+ * is returned. */
+int cfg_confirm_delete(int to_trash);
+
 #endif /* VIFM__CFG__CONFIG_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
