@@ -22,31 +22,33 @@
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
 #include <stddef.h> /* NULL size_t */
-#include <stdio.h> /* fscanf() fgets() fputc() snprintf() */
-#include <stdlib.h> /* abs() free() realloc() */
-#include <string.h> /* memset() strtol() strcmp() strchr() strlen() */
+#include <stdio.h> /* fgets() fprintf() fputc() fscanf() snprintf() */
+#include <stdlib.h> /* abs() free() */
+#include <string.h> /* memcpy() memset() strtol() strcmp() strchr() strlen() */
 
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
+#include "../compat/reallocarray.h"
 #include "../engine/cmds.h"
 #include "../engine/options.h"
+#include "../ui/fileview.h"
 #include "../ui/ui.h"
 #include "../utils/file_streams.h"
 #include "../utils/filter.h"
 #include "../utils/fs.h"
-#include "../utils/fs_limits.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
-#include "../utils/matcher.h"
+#include "../utils/matchers.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
 #include "../utils/utils.h"
-#include "../bookmarks.h"
-#include "../commands.h"
+#include "../bmarks.h"
+#include "../cmd_core.h"
 #include "../dir_stack.h"
 #include "../filelist.h"
 #include "../filetype.h"
-#include "../fileview.h"
+#include "../marks.h"
 #include "../opt_handlers.h"
 #include "../registers.h"
 #include "../status.h"
@@ -68,15 +70,18 @@ static void update_info_file(const char filename[]);
 static void process_hist_entry(FileView *view, const char dir[],
 		const char file[], int pos, char ***lh, int *nlh, int **lhp, size_t *nlhp);
 static char * convert_old_trash_path(const char trash_path[]);
-static int assoc_exists(assoc_list_t *assocs, const char pattern[],
-		const char cmd[]);
 static void write_options(FILE *const fp);
 static void write_assocs(FILE *fp, const char str[], char mark,
 		assoc_list_t *assocs, int prev_count, char *prev[]);
+static void write_doubling_commas(FILE *fp, const char str[]);
 static void write_commands(FILE *const fp, char *cmds_list[], char *cmds[],
 		int ncmds);
-static void write_bookmarks(FILE *const fp, const char non_conflicting_bmarks[],
+static void write_marks(FILE *const fp, const char non_conflicting_marks[],
 		char *marks[], const int timestamps[], int nmarks);
+static void write_bmarks(FILE *const fp, char *bmarks[], const int timestamps[],
+		int nbmarks);
+static void write_bmark(const char path[], const char tags[], time_t timestamp,
+		void *arg);
 static void write_tui_state(FILE *const fp);
 static void write_view_history(FILE *fp, FileView *view, const char str[],
 		char mark, int prev_count, char *prev[], int pos[]);
@@ -91,6 +96,7 @@ static void remove_leading_whitespace(char line[]);
 static const char * escape_spaces(const char *str);
 static void put_sort_info(FILE *fp, char leading_char, const FileView *view);
 static int read_optional_number(FILE *f);
+static int read_number(const char line[], long *value);
 static size_t add_to_int_array(int **array, size_t len, int what);
 
 void
@@ -121,12 +127,12 @@ read_info_file(int reread)
 			{
 				FileView *v = curr_view;
 				curr_view = (line_val[0] == '[') ? &lwin : &rwin;
-				process_set_args(line_val + 1);
+				process_set_args(line_val + 1, 1, 1);
 				curr_view = v;
 			}
 			else
 			{
-				process_set_args(line_val);
+				process_set_args(line_val, 1, 1);
 			}
 		}
 		else if(type == LINE_TYPE_FILETYPE || type == LINE_TYPE_XFILETYPE)
@@ -134,7 +140,7 @@ read_info_file(int reread)
 			if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 			{
 				char *error;
-				matcher_t *m;
+				matchers_t *ms;
 				const int x = (type == LINE_TYPE_XFILETYPE);
 
 				/* Prevent loading of old builtin fake associations. */
@@ -143,15 +149,15 @@ read_info_file(int reread)
 					continue;
 				}
 
-				m = matcher_alloc(line_val, 0, 1, &error);
-				if(m == NULL)
+				ms = matchers_alloc(line_val, 0, 1, "", &error);
+				if(ms == NULL)
 				{
 					/* Ignore error description. */
 					free(error);
 				}
 				else
 				{
-					ft_set_programs(m, line2, x,
+					ft_set_programs(ms, line2, x,
 							curr_stats.exec_env_type == EET_EMULATOR_WITH_X);
 				}
 			}
@@ -161,15 +167,15 @@ read_info_file(int reread)
 			if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 			{
 				char *error;
-				matcher_t *const m = matcher_alloc(line_val, 0, 1, &error);
-				if(m == NULL)
+				matchers_t *const ms = matchers_alloc(line_val, 0, 1, "", &error);
+				if(ms == NULL)
 				{
 					/* Ignore error description. */
 					free(error);
 				}
 				else
 				{
-					ft_set_viewers(m, line2);
+					ft_set_viewers(ms, line2);
 				}
 			}
 		}
@@ -185,20 +191,32 @@ read_info_file(int reread)
 				}
 			}
 		}
-		else if(type == LINE_TYPE_BOOKMARK)
+		else if(type == LINE_TYPE_MARK)
 		{
 			if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 			{
 				if((line3 = read_vifminfo_line(fp, line3)) != NULL)
 				{
 					const int timestamp = read_optional_number(fp);
-					setup_user_bookmark(line_val[0], line2, line3, timestamp);
+					setup_user_mark(line_val[0], line2, line3, timestamp);
+				}
+			}
+		}
+		else if(type == LINE_TYPE_BOOKMARK)
+		{
+			if((line2 = read_vifminfo_line(fp, line2)) != NULL)
+			{
+				long timestamp;
+				if((line3 = read_vifminfo_line(fp, line3)) != NULL &&
+						read_number(line3, &timestamp))
+				{
+					(void)bmarks_setup(line_val, line2, (size_t)timestamp);
 				}
 			}
 		}
 		else if(type == LINE_TYPE_ACTIVE_VIEW)
 		{
-			/* don't change active view on :restart command */
+			/* Don't change active view on :restart command. */
 			if(line_val[0] == 'r' && !reread)
 			{
 				ui_views_update_titles();
@@ -293,7 +311,7 @@ read_info_file(int reread)
 		}
 		else if(type == LINE_TYPE_REG)
 		{
-			append_to_register(line_val[0], line_val + 1);
+			regs_append(line_val[0], line_val + 1);
 		}
 		else if(type == LINE_TYPE_LWIN_FILT)
 		{
@@ -345,6 +363,10 @@ read_info_file(int reread)
 static void
 get_sort_info(FileView *view, const char line[])
 {
+	char *const sort = curr_stats.restart_in_progress
+	                 ? ui_view_sort_list_get(view)
+	                 : view->sort;
+
 	int j = 0;
 	while(*line != '\0' && j < SK_COUNT)
 	{
@@ -353,7 +375,7 @@ get_sort_info(FileView *view, const char line[])
 		if(endptr != line)
 		{
 			line = endptr;
-			view->sort[j++] = MIN(SK_LAST, MAX(-SK_LAST, sort_opt));
+			view->sort_g[j++] = MIN(SK_LAST, MAX(-SK_LAST, sort_opt));
 		}
 		else
 		{
@@ -361,7 +383,12 @@ get_sort_info(FileView *view, const char line[])
 		}
 		line = skip_char(line, ',');
 	}
-	memset(&view->sort[j], SK_NONE, sizeof(view->sort) - j);
+	memset(&view->sort_g[j], SK_NONE, sizeof(view->sort_g) - j);
+	if(j == 0)
+	{
+		view->sort_g[0] = SK_DEFAULT;
+	}
+	memcpy(sort, view->sort_g, sizeof(view->sort));
 
 	fview_sorting_updated(view);
 }
@@ -387,20 +414,27 @@ ensure_history_not_full(hist_t *hist)
 	}
 }
 
+/* Loads single history entry from vifminfo into the view. */
 static void
 get_history(FileView *view, int reread, const char *dir, const char *file,
 		int pos)
 {
+	const int list_rows = view->list_rows;
+
 	if(view->history_num == cfg.history_len)
 	{
 		cfg_resize_histories(cfg.history_len + 1);
 	}
 
 	if(!reread)
+	{
 		view->list_rows = 1;
+	}
 	save_view_history(view, dir, file, pos);
 	if(!reread)
-		view->list_rows = 0;
+	{
+		view->list_rows = list_rows;
+	}
 }
 
 /* Sets view property specified by the type to the value. */
@@ -510,13 +544,15 @@ update_info_file(const char filename[])
 	int ncmds_list = -1;
 	char **ft = NULL, **fx = NULL, **fv = NULL, **cmds = NULL, **marks = NULL;
 	char **lh = NULL, **rh = NULL, **cmdh = NULL, **srch = NULL, **regs = NULL;
-	int *lhp = NULL, *rhp = NULL, *bt = NULL;
+	int *lhp = NULL, *rhp = NULL, *bt = NULL, *bmt = NULL;
 	char **prompt = NULL, **filter = NULL, **trash = NULL;
+	char **bmarks = NULL;
 	int nft = 0, nfx = 0, nfv = 0, ncmds = 0, nmarks = 0, nlh = 0, nrh = 0;
 	int ncmdh = 0, nsrch = 0, nregs = 0, nprompt = 0, nfilter = 0, ntrash = 0;
+	int nbmarks = 0;
 	char **dir_stack = NULL;
 	int ndir_stack = 0;
-	char *non_conflicting_bmarks;
+	char *non_conflicting_marks;
 
 	if(cfg.vifm_info == 0)
 		return;
@@ -524,11 +560,11 @@ update_info_file(const char filename[])
 	cmds_list = list_udf();
 	while(cmds_list[++ncmds_list] != NULL);
 
-	non_conflicting_bmarks = strdup(valid_bookmarks);
+	non_conflicting_marks = strdup(valid_marks);
 
 	if((fp = os_fopen(filename, "r")) != NULL)
 	{
-		size_t nlhp = 0UL, nrhp = 0UL, nbt = 0UL;
+		size_t nlhp = 0UL, nrhp = 0UL, nbt = 0UL, nbmt = 0UL;
 		char *line = NULL, *line2 = NULL, *line3 = NULL, *line4 = NULL;
 		while((line = read_vifminfo_line(fp, line)) != NULL)
 		{
@@ -542,7 +578,7 @@ update_info_file(const char filename[])
 			{
 				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 				{
-					if(!assoc_exists(&filetypes, line_val, line2))
+					if(!ft_assoc_exists(&filetypes, line_val, line2))
 					{
 						nft = add_to_string_array(&ft, nft, 2, line_val, line2);
 					}
@@ -552,7 +588,7 @@ update_info_file(const char filename[])
 			{
 				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 				{
-					if(!assoc_exists(&xfiletypes, line_val, line2))
+					if(!ft_assoc_exists(&xfiletypes, line_val, line2))
 					{
 						nfx = add_to_string_array(&fx, nfx, 2, line_val, line2);
 					}
@@ -562,7 +598,7 @@ update_info_file(const char filename[])
 			{
 				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
 				{
-					if(!assoc_exists(&fileviewers, line_val, line2))
+					if(!ft_assoc_exists(&fileviewers, line_val, line2))
 					{
 						nfv = add_to_string_array(&fv, nfv, 2, line_val, line2);
 					}
@@ -610,7 +646,7 @@ update_info_file(const char filename[])
 					}
 				}
 			}
-			else if(type == LINE_TYPE_BOOKMARK)
+			else if(type == LINE_TYPE_MARK)
 			{
 				const char mark = line_val[0];
 				if(line_val[1] != '\0')
@@ -624,14 +660,14 @@ update_info_file(const char filename[])
 						const int timestamp = read_optional_number(fp);
 						const char mark_str[] = { mark, '\0' };
 
-						if(!char_is_one_of(valid_bookmarks, mark))
+						if(!char_is_one_of(valid_marks, mark))
 						{
 							continue;
 						}
 
-						if(is_bookmark_older(mark, timestamp))
+						if(is_mark_older(mark, timestamp))
 						{
-							char *const pos = strchr(non_conflicting_bmarks, mark);
+							char *const pos = strchr(non_conflicting_marks, mark);
 							if(pos != NULL)
 							{
 								nmarks = add_to_string_array(&marks, nmarks, 3, mark_str, line2,
@@ -644,6 +680,23 @@ update_info_file(const char filename[])
 					}
 				}
 			}
+			else if(type == LINE_TYPE_BOOKMARK)
+			{
+				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
+				{
+					if((line3 = read_vifminfo_line(fp, line3)) != NULL)
+					{
+						long timestamp;
+						if(read_number(line3, &timestamp) &&
+								bmark_is_older(line_val, timestamp))
+						{
+							nbmarks = add_to_string_array(&bmarks, nbmarks, 2, line_val,
+									line2);
+							nbmt = add_to_int_array(&bmt, nbmt, timestamp);
+						}
+					}
+				}
+			}
 			else if(type == LINE_TYPE_TRASH)
 			{
 				if((line2 = read_vifminfo_line(fp, line2)) != NULL)
@@ -700,8 +753,10 @@ update_info_file(const char filename[])
 			}
 			else if(type == LINE_TYPE_REG)
 			{
-				if(register_exists(line_val[0]))
+				if(regs_exists(line_val[0]))
+				{
 					continue;
+				}
 				nregs = add_to_string_array(&regs, nregs, 1, line);
 			}
 		}
@@ -736,9 +791,14 @@ update_info_file(const char filename[])
 			write_commands(fp, cmds_list, cmds, ncmds);
 		}
 
+		if(cfg.vifm_info & VIFMINFO_MARKS)
+		{
+			write_marks(fp, non_conflicting_marks, marks, bt, nmarks);
+		}
+
 		if(cfg.vifm_info & VIFMINFO_BOOKMARKS)
 		{
-			write_bookmarks(fp, non_conflicting_bmarks, marks, bt, nmarks);
+			write_bmarks(fp, bmarks, bmt, nbmarks);
 		}
 
 		if(cfg.vifm_info & VIFMINFO_TUI)
@@ -813,13 +873,16 @@ update_info_file(const char filename[])
 	free(lhp);
 	free(rhp);
 	free(bt);
+	free(bmt);
 	free_string_array(cmdh, ncmdh);
 	free_string_array(srch, nsrch);
 	free_string_array(regs, nregs);
 	free_string_array(prompt, nprompt);
+	free_string_array(filter, nfilter);
 	free_string_array(trash, ntrash);
+	free_string_array(bmarks, nbmarks);
 	free_string_array(dir_stack, ndir_stack);
-	free(non_conflicting_bmarks);
+	free(non_conflicting_marks);
 }
 
 /* Handles single directory history entry, possibly skipping merging it in. */
@@ -859,49 +922,23 @@ convert_old_trash_path(const char trash_path[])
 	return strdup(trash_path);
 }
 
-/* Checks that given pair of pattern and command exists in specified list of
- * associations.  Returns non-zero if so, otherwise zero is returned. */
-static int
-assoc_exists(assoc_list_t *assocs, const char pattern[], const char cmd[])
-{
-	int i;
-	for(i = 0; i < assocs->count; ++i)
-	{
-		int j;
-
-		const assoc_t assoc = assocs->list[i];
-		if(strcmp(matcher_get_expr(assoc.matcher), pattern) == 0)
-		{
-			continue;
-		}
-
-		for(j = 0; j < assoc.records.count; ++j)
-		{
-			const assoc_record_t ft_record = assoc.records.list[j];
-			if(strcmp(ft_record.command, cmd) == 0)
-			{
-				return 1;
-			}
-		}
-	}
-	return 0;
-}
-
 /* Writes current values of all options into vifminfo file. */
 static void
 write_options(FILE *const fp)
 {
+	const char *str;
+
 	fputs("\n# Options:\n", fp);
 	fprintf(fp, "=aproposprg=%s\n", escape_spaces(cfg.apropos_prg));
 	fprintf(fp, "=%sautochpos\n", cfg.auto_ch_pos ? "" : "no");
 	fprintf(fp, "=cdpath=%s\n", cfg.cd_path);
 	fprintf(fp, "=%schaselinks\n", cfg.chase_links ? "" : "no");
 	fprintf(fp, "=columns=%d\n", cfg.columns);
-	fprintf(fp, "=%sconfirm\n", cfg.confirm ? "" : "no");
 	fprintf(fp, "=cpoptions=%s%s%s\n",
 			cfg.filter_inverted_by_default ? "f" : "",
 			cfg.selection_is_primary ? "s" : "",
 			cfg.tab_switches_pane ? "t" : "");
+	fprintf(fp, "=deleteprg=%s\n", escape_spaces(cfg.delete_prg));
 	fprintf(fp, "=%sfastrun\n", cfg.fast_run ? "" : "no");
 	if(strcmp(cfg.border_filler, " ") != 0)
 	{
@@ -918,6 +955,7 @@ write_options(FILE *const fp)
 	fprintf(fp, "=%signorecase\n", cfg.ignore_case ? "" : "no");
 	fprintf(fp, "=%sincsearch\n", cfg.inc_search ? "" : "no");
 	fprintf(fp, "=%slaststatus\n", cfg.display_statusline ? "" : "no");
+	fprintf(fp, "=%stitle\n", cfg.set_title ? "" : "no");
 	fprintf(fp, "=lines=%d\n", cfg.lines);
 	fprintf(fp, "=locateprg=%s\n", escape_spaces(cfg.locate_prg));
 	fprintf(fp, "=mintimeoutlen=%d\n", cfg.min_timeout_len);
@@ -926,7 +964,8 @@ write_options(FILE *const fp)
 	fprintf(fp, "=%sscrollbind\n", cfg.scroll_bind ? "" : "no");
 	fprintf(fp, "=scrolloff=%d\n", cfg.scroll_off);
 	fprintf(fp, "=shell=%s\n", escape_spaces(cfg.shell));
-	fprintf(fp, "=shortmess=%s\n", escape_spaces(get_option_value("shortmess")));
+	fprintf(fp, "=shortmess=%s\n",
+			escape_spaces(get_option_value("shortmess", OPT_GLOBAL)));
 #ifndef _WIN32
 	fprintf(fp, "=slowfs=%s\n", escape_spaces(cfg.slow_fs_list));
 #endif
@@ -938,7 +977,7 @@ write_options(FILE *const fp)
 	fprintf(fp, "=timeoutlen=%d\n", cfg.timeout_len);
 	fprintf(fp, "=%strash\n", cfg.use_trash ? "" : "no");
 	fprintf(fp, "=tuioptions=%s%s\n",
-			cfg.filelist_col_padding ? "p" : "",
+			cfg.extra_padding ? "p" : "",
 			cfg.side_borders_visible ? "s" : "");
 	fprintf(fp, "=undolevels=%d\n", cfg.undo_levels);
 	fprintf(fp, "=vicmd=%s%s\n", escape_spaces(cfg.vi_command),
@@ -946,16 +985,25 @@ write_options(FILE *const fp)
 	fprintf(fp, "=vixcmd=%s%s\n", escape_spaces(cfg.vi_x_command),
 			cfg.vi_cmd_bg ? " &" : "");
 	fprintf(fp, "=%swrapscan\n", cfg.wrap_scan ? "" : "no");
-	fprintf(fp, "=[viewcolumns=%s\n", escape_spaces(lwin.view_columns));
-	fprintf(fp, "=]viewcolumns=%s\n", escape_spaces(rwin.view_columns));
-	fprintf(fp, "=[%slsview\n", lwin.ls_view ? "" : "no");
-	fprintf(fp, "=]%slsview\n", rwin.ls_view ? "" : "no");
-	fprintf(fp, "=[%snumber\n", (lwin.num_type & NT_SEQ) ? "" : "no");
-	fprintf(fp, "=]%snumber\n", (rwin.num_type & NT_SEQ) ? "" : "no");
-	fprintf(fp, "=[numberwidth=%d\n", lwin.num_width);
-	fprintf(fp, "=]numberwidth=%d\n", rwin.num_width);
-	fprintf(fp, "=[%srelativenumber\n", (lwin.num_type & NT_REL) ? "" : "no");
-	fprintf(fp, "=]%srelativenumber\n", (rwin.num_type & NT_REL) ? "" : "no");
+	fprintf(fp, "=[viewcolumns=%s\n", escape_spaces(lwin.view_columns_g));
+	fprintf(fp, "=]viewcolumns=%s\n", escape_spaces(rwin.view_columns_g));
+	fprintf(fp, "=[sortgroups=%s\n", escape_spaces(lwin.sort_groups_g));
+	fprintf(fp, "=]sortgroups=%s\n", escape_spaces(rwin.sort_groups_g));
+	fprintf(fp, "=[%slsview\n", lwin.ls_view_g ? "" : "no");
+	fprintf(fp, "=]%slsview\n", rwin.ls_view_g ? "" : "no");
+	fprintf(fp, "=[%snumber\n", (lwin.num_type_g & NT_SEQ) ? "" : "no");
+	fprintf(fp, "=]%snumber\n", (rwin.num_type_g & NT_SEQ) ? "" : "no");
+	fprintf(fp, "=[numberwidth=%d\n", lwin.num_width_g);
+	fprintf(fp, "=]numberwidth=%d\n", rwin.num_width_g);
+	fprintf(fp, "=[%srelativenumber\n", (lwin.num_type_g & NT_REL) ? "" : "no");
+	fprintf(fp, "=]%srelativenumber\n", (rwin.num_type_g & NT_REL) ? "" : "no");
+
+	fprintf(fp, "%s", "=confirm=");
+	if(cfg.confirm & CONFIRM_DELETE)
+		fprintf(fp, "%s", "delete,");
+	if(cfg.confirm & CONFIRM_PERM_DELETE)
+		fprintf(fp, "%s", "permdelete,");
+	fprintf(fp, "\n");
 
 	fprintf(fp, "%s", "=dotdirs=");
 	if(cfg.dot_dirs & DD_ROOT_PARENT)
@@ -964,14 +1012,59 @@ write_options(FILE *const fp)
 		fprintf(fp, "%s", "nonrootparent,");
 	fprintf(fp, "\n");
 
-	fprintf(fp, "=classify=%s\n", escape_spaces(classify_to_str()));
+	fprintf(fp, "%s", "=suggestoptions=");
+	if(cfg.sug.flags & SF_NORMAL)
+		fprintf(fp, "%s", "normal,");
+	if(cfg.sug.flags & SF_VISUAL)
+		fprintf(fp, "%s", "visual,");
+	if(cfg.sug.flags & SF_VIEW)
+		fprintf(fp, "%s", "view,");
+	if(cfg.sug.flags & SF_OTHERPANE)
+		fprintf(fp, "%s", "otherpane,");
+	if(cfg.sug.flags & SF_KEYS)
+		fprintf(fp, "%s", "keys,");
+	if(cfg.sug.flags & SF_MARKS)
+		fprintf(fp, "%s", "marks,");
+	if(cfg.sug.flags & SF_DELAY)
+	{
+		if(cfg.sug.delay == 500)
+		{
+			fprintf(fp, "%s", "delay,");
+		}
+		else
+		{
+			fprintf(fp, "delay:%d,", cfg.sug.delay);
+		}
+	}
+	if(cfg.sug.flags & SF_REGISTERS)
+	{
+		if(cfg.sug.maxregfiles == 5)
+		{
+			fprintf(fp, "%s", "registers,");
+		}
+		else
+		{
+			fprintf(fp, "registers:%d,", cfg.sug.maxregfiles);
+		}
+	}
+	fprintf(fp, "\n");
+
+	fprintf(fp, "%s", "=iooptions=");
+	if(cfg.fast_file_cloning)
+		fprintf(fp, "%s", "fastfilecloning,");
+	fprintf(fp, "\n");
+
+	fprintf(fp, "=dirsize=%s", cfg.view_dir_size == VDS_SIZE ? "size" : "nitems");
+
+	str = classify_to_str();
+	fprintf(fp, "=classify=%s\n", escape_spaces(str == NULL ? "" : str));
 
 	fprintf(fp, "=vifminfo=options");
 	if(cfg.vifm_info & VIFMINFO_FILETYPES)
 		fprintf(fp, ",filetypes");
 	if(cfg.vifm_info & VIFMINFO_COMMANDS)
 		fprintf(fp, ",commands");
-	if(cfg.vifm_info & VIFMINFO_BOOKMARKS)
+	if(cfg.vifm_info & VIFMINFO_MARKS)
 		fprintf(fp, ",bookmarks");
 	if(cfg.vifm_info & VIFMINFO_TUI)
 		fprintf(fp, ",tui");
@@ -999,7 +1092,9 @@ write_options(FILE *const fp)
 
 	fprintf(fp, "=%svimhelp\n", cfg.use_vim_help ? "" : "no");
 	fprintf(fp, "=%swildmenu\n", cfg.wild_menu ? "" : "no");
-	fprintf(fp, "=wordchars=%s\n", escape_spaces(get_option_value("wordchars")));
+	fprintf(fp, "=wildstyle=%s\n", cfg.wild_popup ? "popup" : "bar");
+	fprintf(fp, "=wordchars=%s\n",
+			escape_spaces(get_option_value("wordchars", OPT_GLOBAL)));
 	fprintf(fp, "=%swrap\n", cfg.wrap_quick_view ? "" : "no");
 }
 
@@ -1031,14 +1126,15 @@ write_assocs(FILE *fp, const char str[], char mark, assoc_list_t *assocs,
 
 			if(ft_record.description[0] == '\0')
 			{
-				fprintf(fp, "%c%s\n\t%s\n", mark, matcher_get_expr(assoc.matcher),
-						ft_record.command);
+				fprintf(fp, "%c%s\n\t", mark, matchers_get_expr(assoc.matchers));
 			}
 			else
 			{
-				fprintf(fp, "%c%s\n\t{%s}%s\n", mark, matcher_get_expr(assoc.matcher),
-						ft_record.description, ft_record.command);
+				fprintf(fp, "%c%s\n\t{%s}", mark, matchers_get_expr(assoc.matchers),
+						ft_record.description);
 			}
+			write_doubling_commas(fp, ft_record.command);
+			fputc('\n', fp);
 		}
 	}
 
@@ -1048,6 +1144,20 @@ write_assocs(FILE *fp, const char str[], char mark, assoc_list_t *assocs,
 	}
 }
 
+/* Prints the string into the file doubling commas in process. */
+static void
+write_doubling_commas(FILE *fp, const char str[])
+{
+	while(*str != '\0')
+	{
+		if(*str == ',')
+		{
+			fputc(',', fp);
+		}
+		fputc(*str++, fp);
+	}
+}
+
 /* Writes user-defined commands to vifminfo file.  cmds_list is a NULL
  * terminated list of commands existing in current session, cmds is a list of
  * length ncmds with unseen commands read from vifminfo. */
@@ -1067,40 +1177,70 @@ write_commands(FILE *const fp, char *cmds_list[], char *cmds[], int ncmds)
 	}
 }
 
-/* Writes bookmarks to vifminfo file.  marks is a list of length nmarks
- * bookmarks read from vifminfo. */
+/* Writes marks to vifminfo file.  marks is a list of length nmarks marks read
+ * from vifminfo. */
 static void
-write_bookmarks(FILE *const fp, const char non_conflicting_bmarks[],
+write_marks(FILE *const fp, const char non_conflicting_marks[],
 		char *marks[], const int timestamps[], int nmarks)
 {
-	int active_bookmarks[NUM_BOOKMARKS];
-	const int len = init_active_bookmarks(valid_bookmarks, active_bookmarks);
+	int active_marks[NUM_MARKS];
+	const int len = init_active_marks(valid_marks, active_marks);
 	int i;
 
-	fputs("\n# Bookmarks:\n", fp);
-	for(i = 0; i < len; i++)
+	fputs("\n# Marks:\n", fp);
+	for(i = 0; i < len; ++i)
 	{
-		const int index = active_bookmarks[i];
-		const char mark = index2mark(index);
-		if(!is_spec_bookmark(index) && char_is_one_of(non_conflicting_bmarks, mark))
+		const int index = active_marks[i];
+		const char m = index2mark(index);
+		if(!is_spec_mark(index) && char_is_one_of(non_conflicting_marks, m))
 		{
-			const bookmark_t *const bookmark = get_bookmark(index);
+			const mark_t *const mark = get_mark(index);
 
-			fprintf(fp, "'%c\n", mark);
-			fprintf(fp, "\t%s\n", bookmark->directory);
-			fprintf(fp, "\t%s\n", bookmark->file);
-			fprintf(fp, TIME_T "\n", bookmark->timestamp);
+			fprintf(fp, "%c%c\n", LINE_TYPE_MARK, m);
+			fprintf(fp, "\t%s\n", mark->directory);
+			fprintf(fp, "\t%s\n", mark->file);
+			fprintf(fp, "%lld\n", (long long)mark->timestamp);
 		}
 	}
 	for(i = 0; i < nmarks; i += 3)
 	{
-		fprintf(fp, "'%c\n", marks[i][0]);
+		fprintf(fp, "%c%c\n", LINE_TYPE_MARK, marks[i][0]);
 		fprintf(fp, "\t%s\n", marks[i + 1]);
 		fprintf(fp, "\t%s\n", marks[i + 2]);
 		fprintf(fp, "%d\n", timestamps[i/3]);
 	}
 }
 
+/* Writes bookmarks to vifminfo file.  bmarks is a list of length nbmarks marks
+ * read from vifminfo. */
+static void
+write_bmarks(FILE *const fp, char *bmarks[], const int timestamps[],
+		int nbmarks)
+{
+	int i;
+
+	fputs("\n# Bookmarks:\n", fp);
+
+	bmarks_list(&write_bmark, fp);
+
+	for(i = 0; i < nbmarks; i += 2)
+	{
+		fprintf(fp, "%c%s\n", LINE_TYPE_BOOKMARK, bmarks[i]);
+		fprintf(fp, "\t%s\n", bmarks[i + 1]);
+		fprintf(fp, "\t%d\n", timestamps[i/2]);
+	}
+}
+
+/* bmarks_list() callback that writes a bookmark into vifminfo. */
+static void
+write_bmark(const char path[], const char tags[], time_t timestamp, void *arg)
+{
+	FILE *const fp = arg;
+	fprintf(fp, "%c%s\n", LINE_TYPE_BOOKMARK, path);
+	fprintf(fp, "\t%s\n", tags);
+	fprintf(fp, "\t%d\n", (int)timestamp);
+}
+
 /* Writes state of the TUI to vifminfo file. */
 static void
 write_tui_state(FILE *const fp)
@@ -1156,7 +1296,7 @@ write_history(FILE *fp, const char str[], char mark, int prev_count,
 	}
 }
 
-/* Writes bookmarks to vifminfo file.  regs is a list of length nregs registers
+/* Writes registers to vifminfo file.  regs is a list of length nregs registers
  * read from vifminfo. */
 static void
 write_registers(FILE *const fp, char *regs[], int nregs)
@@ -1170,11 +1310,11 @@ write_registers(FILE *const fp, char *regs[], int nregs)
 	}
 	for(i = 0; valid_registers[i] != '\0'; i++)
 	{
-		const registers_t *const reg = find_register(valid_registers[i]);
+		const reg_t *const reg = regs_find(valid_registers[i]);
 		if(reg != NULL)
 		{
 			int j;
-			for(j = 0; j < reg->num_files; j++)
+			for(j = 0; j < reg->nfiles; ++j)
 			{
 				if(reg->files[j] != NULL)
 				{
@@ -1269,9 +1409,10 @@ remove_leading_whitespace(char line[])
 	}
 }
 
-/* Returns pointer to a statically allocated buffer */
+/* Escapes spaces in the string.  Returns pointer to a statically allocated
+ * buffer. */
 static const char *
-escape_spaces(const char *str)
+escape_spaces(const char str[])
 {
 	static char buf[4096];
 	char *p;
@@ -1295,9 +1436,7 @@ static void
 put_sort_info(FILE *fp, char leading_char, const FileView *view)
 {
 	int i = -1;
-	const char *const sort = (flist_custom_active(view) && view->custom.unsorted)
-	                       ? view->custom.sort
-	                       : view->sort;
+	const char *const sort = ui_view_sort_list_get(view);
 
 	fputc(leading_char, fp);
 	while(++i < SK_COUNT && abs(sort[i]) <= SK_LAST)
@@ -1330,12 +1469,21 @@ read_optional_number(FILE *f)
 	return num;
 }
 
+/* Converts line to number.  Returns non-zero on success and zero otherwise. */
+static int
+read_number(const char line[], long *value)
+{
+	char *endptr;
+	*value = strtol(line, &endptr, 10);
+	return *line != '\0' && *endptr == '\0';
+}
+
 static size_t
 add_to_int_array(int **array, size_t len, int what)
 {
 	int *p;
 
-	p = realloc(*array, sizeof(int)*(len + 1));
+	p = reallocarray(*array, len + 1, sizeof(*p));
 	if(p != NULL)
 	{
 		*array = p;
@@ -1346,4 +1494,4 @@ add_to_int_array(int **array, size_t len, int what)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
