@@ -23,8 +23,10 @@
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h>
 #include <stdlib.h> /* calloc() malloc() free() realloc() */
-#include <string.h>
+#include <string.h> /* strdup() */
 
+#include "../compat/reallocarray.h"
+#include "../utils/darray.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
 #include "../utils/str.h"
@@ -42,31 +44,40 @@ typedef enum
 	BUILTIN_ABBR,
 	BUILTIN_CMD,
 	USER_CMD,
-}CMD_TYPE;
+}
+CMD_TYPE;
 
 typedef struct cmd_t
 {
 	char *name;
+	const char *descr; /* Brief description of the command. */
 	int id;
 	CMD_TYPE type;
 	int passed;
 
-	cmd_handler handler;
-	char *cmd;
+	cmd_handler handler; /* Handler for builtin commands. */
+	char *cmd;           /* Command-line for user-defined commands. */
 
-	int range;
-	int cust_sep;
-	int emark;
-	int qmark;
-	int expand;
 	int min_args, max_args;
-	int regexp;
-	int select;
-	int bg;
-	int quote;
+
+	unsigned int range : 1;            /* Handles ranges. */
+	unsigned int cust_sep : 1;         /* Custom separator of arguments. */
+	unsigned int emark : 1;            /* Supports emark flag. */
+	unsigned int envvars : 1;          /* Expand environment variables. */
+	unsigned int select : 1;           /* Select files in a range. */
+	unsigned int bg : 1;               /* Bg (can have " &" at the end). */
+	unsigned int regexp : 1;           /* Process /.../-arguments. */
+	unsigned int quote : 1;            /* Process '- and "-quoted args. */
+	unsigned int comment : 1;          /* Trailing comment is allowed. */
+	unsigned int qmark : 1;            /* No args after qmark. */
+	unsigned int args_after_qmark : 1; /* Args after qmark are allowed. */
+	unsigned int macros_for_cmd : 1;   /* Expand macros w/o special escaping. */
+	unsigned int macros_for_shell : 1; /* Expand macros with shell escaping. */
+	unsigned int : 0;                  /* Padding. */
 
 	struct cmd_t *next;
-}cmd_t;
+}
+cmd_t;
 
 typedef struct
 {
@@ -74,7 +85,8 @@ typedef struct
 	cmd_add_t user_cmd_handler;
 	cmd_handler command_handler;
 	int udf_count;
-}inner_t;
+}
+inner_t;
 
 /* List of characters, which are treated as range separators. */
 static const char *RANGE_SEPARATORS = ",;";
@@ -99,14 +111,14 @@ static void complete_cmd_name(const char cmd_name[], int user_only);
 TSTATIC int add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf);
 static int comclear_cmd(const cmd_info_t *cmd_info);
 static int command_cmd(const cmd_info_t *cmd_info);
+static void init_command_flags(cmd_t *cmd, int flags);
 static const char * get_user_cmd_name(const char cmd[], char buf[],
 		size_t buf_len);
 static int is_correct_name(const char name[]);
 static cmd_t * insert_cmd(cmd_t *after);
 static int delcommand_cmd(const cmd_info_t *cmd_info);
 TSTATIC char ** dispatch_line(const char args[], int *count, char sep,
-		int regexp, int quotes, int *last_arg, int *last_begin, int *last_end);
-TSTATIC void unescape(char s[], int regexp);
+		int regexp, int quotes, int comments, int *last_arg, int (**positions)[2]);
 static int is_separator(char c, char sep);
 
 void
@@ -114,17 +126,20 @@ init_cmds(int udf, cmds_conf_t *conf)
 {
 	static cmd_add_t commands[] = {
 		{
-			.name = "comclear",   .abbr = "comc", .handler = comclear_cmd,   .id = COMCLEAR_CMD_ID,   .quote = 0,
-			.range = 0,           .emark = 0,     .qmark = 0,                .regexp = 0,             .select = 0,
-			.expand = 0,          .bg = 0,        .min_args = 0,             .max_args = 0,
+			.name = "comclear",   .abbr = "comc", .handler = comclear_cmd,   .id = COMCLEAR_CMD_ID,
+			.descr = "remove all user-defined :commands",
+			.flags = 0,
+			.min_args = 0,             .max_args = 0,
 		}, {
-			.name = "command",    .abbr = "com",  .handler = command_cmd,    .id = COMMAND_CMD_ID,    .quote = 0,
-			.range = 0,           .emark = 1,     .qmark = 0,                .regexp = 0,             .select = 0,
-			.expand = 0,          .bg = 0,        .min_args = 0,             .max_args = NOT_DEF,
+			.name = "command",    .abbr = "com",  .handler = command_cmd,    .id = COMMAND_CMD_ID,
+			.descr = "display/define user-defined :command",
+			.flags = HAS_EMARK,
+			.min_args = 0,             .max_args = NOT_DEF,
 		}, {
-			.name = "delcommand", .abbr = "delc", .handler = delcommand_cmd, .id = DELCOMMAND_CMD_ID, .quote = 0,
-			.range = 0,           .emark = 1,     .qmark = 0,                .regexp = 0,             .select = 0,
-			.expand = 0,          .bg = 0,        .min_args = 1,             .max_args = 1,
+			.name = "delcommand", .abbr = "delc", .handler = delcommand_cmd, .id = DELCOMMAND_CMD_ID,
+			.descr = "undefine user-defined :command",
+			.flags = HAS_EMARK,
+			.min_args = 1,             .max_args = 1,
 		}
 	};
 
@@ -181,7 +196,7 @@ execute_cmd(const char cmd[])
 	int execution_code;
 	size_t last_arg_len;
 	char *last_arg;
-	int last_end;
+	int last_end = 0;
 	cmds_conf_t *cc = cmds_conf;
 
 	init_cmd_info(&cmd_info);
@@ -210,11 +225,8 @@ execute_cmd(const char cmd[])
 	if(udf_is_ambiguous(cmd_name))
 		return CMDS_ERR_UDF_IS_AMBIGUOUS;
 
-	cur = inner->head.next;
-	while(cur != NULL && strcmp(cur->name, cmd_name) < 0)
-		cur = cur->next;
-
-	if(cur == NULL || strncmp(cmd_name, cur->name, strlen(cmd_name)) != 0)
+	cur = find_cmd(cmd_name);
+	if(cur == NULL)
 		return CMDS_ERR_INVALID_CMD;
 
 	args = parse_tail(cur, cmd, &cmd_info);
@@ -223,7 +235,7 @@ execute_cmd(const char cmd[])
 
 	/* Set background flag and remove background mark from raw arguments, when
 	 * command supports backgrounding. */
-	last_arg = get_last_argument(cmd_info.raw_args, &last_arg_len);
+	last_arg = get_last_argument(cmd_info.raw_args, cur->quote, &last_arg_len);
 	if(cur->bg && *last_arg == '&' && *vle_cmds_at_arg(last_arg + 1) == '\0')
 	{
 		cmd_info.bg = 1;
@@ -231,29 +243,35 @@ execute_cmd(const char cmd[])
 	}
 
 	if(cur->select)
-		cc->select_range(cur->id, &cmd_info);
-
-	if(cur->expand)
 	{
-		char *p = NULL;
-		if(cur->expand & 1)
-			cmd_info.args = cc->expand_macros(cmd_info.raw_args, cur->expand & 4,
-					&cmd_info.usr1, &cmd_info.usr2);
-		if(cur->expand & 2)
-		{
-			p = cmd_info.args;
-			cmd_info.args = cc->expand_envvars(p ? p : cmd_info.raw_args);
-		}
+		cc->select_range(cur->id, &cmd_info);
+	}
+
+	if(cur->macros_for_cmd || cur->macros_for_shell)
+	{
+		cmd_info.args = cc->expand_macros(cmd_info.raw_args, cur->macros_for_shell,
+				&cmd_info.usr1, &cmd_info.usr2);
+	}
+	if(cur->envvars)
+	{
+		char *const p = cmd_info.args;
+		cmd_info.args = cc->expand_envvars(p ? p : cmd_info.raw_args);
 		free(p);
 	}
-	else
+	if(cmd_info.args == NULL)
 	{
 		cmd_info.args = strdup(cmd_info.raw_args);
 	}
+
 	cmd_info.argv = dispatch_line(cmd_info.args, &cmd_info.argc, cmd_info.sep,
-			cur->regexp, cur->quote, NULL, NULL, &last_end);
+			cur->regexp, cur->quote, cur->comment, NULL, &cmd_info.argvp);
+	if(cmd_info.argc > 0)
+	{
+		last_end = cmd_info.argvp[cmd_info.argc - 1][1];
+	}
 	cmd_info.args[last_end] = '\0';
 
+	/* TODO: extract a method that would check all these error conditions. */
 	if((cmd_info.begin != NOT_DEF || cmd_info.end != NOT_DEF) && !cur->range)
 	{
 		execution_code = CMDS_ERR_NO_RANGE_ALLOWED;
@@ -270,7 +288,7 @@ execute_cmd(const char cmd[])
 	{
 		execution_code = CMDS_ERR_NO_QMARK_ALLOWED;
 	}
-	else if(cmd_info.qmark && cur->qmark == 1 && *cmd_info.args != '\0')
+	else if(cmd_info.qmark && !cur->args_after_qmark && *cmd_info.args != '\0')
 	{
 		execution_code = CMDS_ERR_TRAILING_CHARS;
 	}
@@ -307,13 +325,18 @@ execute_cmd(const char cmd[])
 	free(cmd_info.raw_args);
 	free(cmd_info.args);
 	free_string_array(cmd_info.argv, cmd_info.argc);
+	free(cmd_info.argvp);
 
 	return execution_code;
 }
 
+/* Applies limit shifts and ensures that its value is not out of bounds.
+ * Returns pointer to part of string after parsed piece. */
 static const char *
 correct_limit(const char cmd[], cmd_info_t *cmd_info)
 {
+	cmd_info->count = (cmd_info->end == NOT_DEF) ? 1 : (cmd_info->end + 1);
+
 	while(*cmd == '+' || *cmd == '-')
 	{
 		int n = 1;
@@ -325,11 +348,19 @@ correct_limit(const char cmd[], cmd_info_t *cmd_info)
 			n = strtol(cmd, &p, 10);
 			cmd = p;
 		}
+
 		if(plus)
+		{
 			cmd_info->end += n;
+			cmd_info->count += n;
+		}
 		else
+		{
 			cmd_info->end -= n;
+			cmd_info->count -= n;
+		}
 	}
+
 	if(cmd_info->end < 0)
 		cmd_info->end = 0;
 	if(cmd_info->end > cmds_conf->end)
@@ -412,11 +443,21 @@ get_cmd_id(const char cmd[])
 	return get_cmd_info(cmd, &info);
 }
 
+const char *
+get_cmd_args(const char cmd[])
+{
+	cmd_info_t info = {};
+	(void)get_cmd_info(cmd, &info);
+	return info.raw_args;
+}
+
+/* Initializes command info structure with reasonable defaults. */
 static void
 init_cmd_info(cmd_info_t *cmd_info)
 {
 	cmd_info->begin = NOT_DEF;
 	cmd_info->end = NOT_DEF;
+	cmd_info->count = NOT_DEF;
 	cmd_info->emark = 0;
 	cmd_info->qmark = 0;
 	cmd_info->raw_args = NULL;
@@ -436,8 +477,7 @@ get_cmd_info(const char cmd[], cmd_info_t *info)
 {
 	cmd_info_t cmd_info;
 	char cmd_name[MAX_CMD_NAME_LEN];
-	cmd_t *cur;
-	size_t len;
+	cmd_t *c;
 
 	init_cmd_info(&cmd_info);
 
@@ -446,19 +486,14 @@ get_cmd_info(const char cmd[], cmd_info_t *info)
 		return CMDS_ERR_INVALID_CMD;
 
 	cmd = get_cmd_name(cmd, cmd_name, sizeof(cmd_name));
-	cur = inner->head.next;
-	while(cur != NULL && strcmp(cur->name, cmd_name) < 0)
-		cur = cur->next;
-
-	len = strlen(cmd_name);
-	if(cur == NULL || strncmp(cmd_name, cur->name, len) != 0)
+	c = find_cmd(cmd_name);
+	if(c == NULL)
 		return -1;
 
-	if(parse_tail(cur, cmd, &cmd_info) == NULL)
-		return -1;
+	cmd_info.raw_args = (char *)parse_tail(c, cmd, &cmd_info);
 
 	*info = cmd_info;
-	return cur->id;
+	return c->id;
 }
 
 int
@@ -765,9 +800,10 @@ complete_cmd_args(cmd_t *cur, const char args[], cmd_info_t *cmd_info,
 	{
 		int argc;
 		char **argv;
+		int (*argvp)[2];
 		int last_arg = 0;
 
-		argv = dispatch_line(args, &argc, ' ', 0, 1, &last_arg, NULL, NULL);
+		argv = dispatch_line(args, &argc, ' ', 0, 1, 0, &last_arg, &argvp);
 
 		cmd_info->args = (char *)args;
 		cmd_info->argc = argc;
@@ -775,6 +811,7 @@ complete_cmd_args(cmd_t *cur, const char args[], cmd_info_t *cmd_info,
 		result += cmds_conf->complete_args(cur->id, cmd_info, last_arg, arg);
 
 		free_string_array(argv, argc);
+		free(argvp);
 	}
 	return result;
 }
@@ -798,8 +835,10 @@ complete_cmd_name(const char cmd_name[], int user_only)
 			;
 		else if(cur->name[0] == '\0')
 			;
+		else if(cur->type == USER_CMD)
+			vle_compl_add_match(cur->name, cur->cmd);
 		else
-			vle_compl_add_match(cur->name);
+			vle_compl_add_match(cur->name, cur->descr);
 		cur = cur->next;
 	}
 
@@ -807,10 +846,10 @@ complete_cmd_name(const char cmd_name[], int user_only)
 }
 
 void
-add_builtin_commands(const cmd_add_t *cmds, int count)
+add_builtin_commands(const cmd_add_t cmds[], int count)
 {
 	int i;
-	for(i = 0; i < count; i++)
+	for(i = 0; i < count; ++i)
 	{
 		int ret_code;
 		assert(cmds[i].min_args >= 0);
@@ -834,6 +873,7 @@ add_builtin_commands(const cmd_add_t *cmds, int count)
 				assert(ret_code == 0);
 			}
 		}
+		(void)ret_code;
 	}
 }
 
@@ -882,25 +922,22 @@ add_builtin_cmd(const char name[], int abbr, const cmd_add_t *conf)
 		return -1;
 	}
 
-	if((new = insert_cmd(cur)) == NULL)
+	new = insert_cmd(cur);
+	if(new == NULL)
+	{
 		return -1;
+	}
+
 	new->name = strdup(name);
+	new->descr = conf->descr;
 	new->id = conf->id;
 	new->handler = conf->handler;
 	new->type = abbr ? BUILTIN_ABBR : BUILTIN_CMD;
 	new->passed = 0;
-	new->range = conf->range;
-	new->cust_sep = conf->cust_sep;
-	new->emark = conf->emark;
-	new->qmark = conf->qmark;
-	new->expand = conf->expand;
+	new->cmd = NULL;
 	new->min_args = conf->min_args;
 	new->max_args = conf->max_args;
-	new->regexp = conf->regexp;
-	new->select = conf->select;
-	new->bg = conf->bg;
-	new->quote = conf->quote;
-	new->cmd = NULL;
+	init_command_flags(new, conf->flags);
 
 	return 0;
 }
@@ -998,26 +1035,44 @@ command_cmd(const cmd_info_t *cmd_info)
 	}
 
 	new->name = strdup(cmd_name);
+	new->descr = NULL;
 	new->id = USER_CMD_ID;
 	new->type = USER_CMD;
 	new->passed = 0;
 	new->cmd = strdup(args);
-	new->range = inner->user_cmd_handler.range;
-	new->cust_sep = inner->user_cmd_handler.cust_sep;
-	new->emark = inner->user_cmd_handler.emark;
-	new->qmark = inner->user_cmd_handler.qmark;
-	new->expand = inner->user_cmd_handler.expand;
 	new->min_args = inner->user_cmd_handler.min_args;
 	new->max_args = inner->user_cmd_handler.max_args;
-	new->regexp = inner->user_cmd_handler.regexp;
-	new->select = inner->user_cmd_handler.select;
-	new->bg = inner->user_cmd_handler.bg;
-	new->quote = inner->user_cmd_handler.quote;
+	init_command_flags(new, inner->user_cmd_handler.flags);
 
-	inner->udf_count++;
+	++inner->udf_count;
 	return 0;
 }
 
+/* Initializes flag fields of *cmd from set of HAS_* flags. */
+static void
+init_command_flags(cmd_t *cmd, int flags)
+{
+	assert((flags & (HAS_QMARK_NO_ARGS | HAS_QMARK_WITH_ARGS)) !=
+			(HAS_QMARK_NO_ARGS | HAS_QMARK_WITH_ARGS) && "Wrong flags combination.");
+	assert((flags & (HAS_MACROS_FOR_CMD | HAS_MACROS_FOR_SHELL)) !=
+			(HAS_MACROS_FOR_CMD | HAS_MACROS_FOR_SHELL) &&
+			"Wrong flags combination.");
+
+	cmd->range = ((flags & HAS_RANGE) != 0);
+	cmd->cust_sep = ((flags & HAS_CUST_SEP) != 0);
+	cmd->emark = ((flags & HAS_EMARK) != 0);
+	cmd->envvars = ((flags & HAS_ENVVARS) != 0);
+	cmd->select = ((flags & HAS_SELECTION_SCOPE) != 0);
+	cmd->bg = ((flags & HAS_BG_FLAG) != 0);
+	cmd->regexp = ((flags & HAS_REGEXP_ARGS) != 0);
+	cmd->quote = ((flags & HAS_QUOTED_ARGS) != 0);
+	cmd->comment = ((flags & HAS_COMMENT) != 0);
+	cmd->qmark = ((flags & (HAS_QMARK_NO_ARGS | HAS_QMARK_WITH_ARGS)) != 0);
+	cmd->args_after_qmark = ((flags & HAS_QMARK_WITH_ARGS) != 0);
+	cmd->macros_for_cmd = ((flags & HAS_MACROS_FOR_CMD) != 0);
+	cmd->macros_for_shell = ((flags & HAS_MACROS_FOR_SHELL) != 0);
+}
+
 static const char *
 get_user_cmd_name(const char cmd[], char buf[], size_t buf_len)
 {
@@ -1055,13 +1110,17 @@ is_correct_name(const char name[])
 	return 1;
 }
 
+/* Allocates new command node and inserts it after the specified one.  Returns
+ * new uninitialized node or NULL on error. */
 static cmd_t *
 insert_cmd(cmd_t *after)
 {
-	cmd_t *new;
-  new = malloc(sizeof(*new));
+	cmd_t *const new = malloc(sizeof(*new));
 	if(new == NULL)
+	{
 		return NULL;
+	}
+
 	new->next = after->next;
 	after->next = new;
 	return new;
@@ -1094,16 +1153,25 @@ delcommand_cmd(const cmd_info_t *cmd_info)
 }
 
 char *
-get_last_argument(const char cmd[], size_t *len)
+get_last_argument(const char cmd[], int quotes, size_t *len)
 {
 	int argc;
 	char **argv;
+	int (*argvp)[2];
 	int last_start = 0;
 	int last_end = 0;
 
-	argv = dispatch_line(cmd, &argc, ' ', 0, 1, NULL, &last_start, &last_end);
+	argv = dispatch_line(cmd, &argc, ' ', 0, quotes, 0, NULL, &argvp);
+
+	if(argc > 0)
+	{
+		last_start = argvp[argc - 1][0];
+		last_end = argvp[argc - 1][1];
+	}
+
 	*len = last_end - last_start;
 	free_string_array(argv, argc);
+	free(argvp);
 	return (char *)cmd + last_start;
 }
 
@@ -1112,23 +1180,22 @@ get_last_argument(const char cmd[], size_t *len)
  * unmatched quotes and to zero on all other errors). */
 TSTATIC char **
 dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
-		int *last_pos, int *last_begin, int *last_end)
+		int comments, int *last_pos, int (**positions)[2])
 {
 	char *cmdstr;
 	int len;
 	int i;
 	int st;
 	const char *args_beg;
-	char** params;
+	char **params;
+	int (*argvp)[2] = NULL;
+	DA_INSTANCE(argvp);
 
 	enum { BEGIN, NO_QUOTING, S_QUOTING, D_QUOTING, R_QUOTING, ARG, QARG } state;
 
 	if(last_pos != NULL)
 		*last_pos = 0;
-	if(last_begin != NULL)
-		*last_begin = 0;
-	if(last_end != NULL)
-		*last_end = 0;
+	*positions = NULL;
 
 	*count = 0;
 	params = NULL;
@@ -1151,7 +1218,8 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 					st = i + 1;
 					state = S_QUOTING;
 				}
-				else if(sep == ' ' && cmdstr[i] == '"' && quotes)
+				else if(cmdstr[i] == '"' && ((sep == ' ' && quotes) ||
+							(comments && strchr(&cmdstr[i + 1], '"') == NULL)))
 				{
 					st = i + 1;
 					state = D_QUOTING;
@@ -1178,9 +1246,13 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 					st = i--;
 					state = NO_QUOTING;
 				}
-				if(state != BEGIN && cmdstr[i] != '\0' && last_begin != NULL)
+				if(state != BEGIN && cmdstr[i] != '\0')
 				{
-					*last_begin = i;
+					if(DA_EXTEND(argvp) != NULL)
+					{
+						DA_COMMIT(argvp);
+						argvp[DA_SIZE(argvp) - 1U][0] = i;
+					}
 				}
 				break;
 			case NO_QUOTING:
@@ -1228,15 +1300,21 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 			case ARG:
 			case QARG:
 				assert(0 && "Dispatch line state machine is broken");
+				break;
 		}
 		if(state == ARG || state == QARG)
 		{
+			/* Found another argument. */
+
+			const int end = (args - args_beg) + ((state == ARG) ? i : (i + 1));
 			char *last_arg;
 			const char c = cmdstr[i];
-			/* found another argument */
 			cmdstr[i] = '\0';
-			if(last_end != NULL)
-				*last_end = (args - args_beg) + ((state == ARG) ? i : (i + 1));
+
+			if(DA_SIZE(argvp) != 0U)
+			{
+				argvp[DA_SIZE(argvp) - 1U][1] = end;
+			}
 
 			*count = add_to_string_array(&params, *count, 1, &cmdstr[st]);
 			if(*count == 0)
@@ -1257,13 +1335,26 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 		}
 	}
 
+	if(comments && state == D_QUOTING && strchr(&cmdstr[st], '"') == NULL)
+	{
+		state = BEGIN;
+		--st;
+		if(DA_SIZE(argvp) != 0U)
+		{
+			DA_REMOVE(argvp, &argvp[DA_SIZE(argvp) - 1U]);
+		}
+	}
+
 	free(cmdstr);
 
-	if(*count == 0 || (state != BEGIN && state != NO_QUOTING) ||
+	if(*count == 0 || (size_t)*count != DA_SIZE(argvp) ||
+			(state != BEGIN && state != NO_QUOTING) ||
 			put_into_string_array(&params, *count, NULL) != *count + 1)
 	{
+		free(argvp);
 		free_string_array(params, *count);
-		*count = (state == S_QUOTING || state == D_QUOTING) ? -1 : 0;
+		*count = (state == S_QUOTING || state == D_QUOTING || state == R_QUOTING)
+		       ? -1 : 0;
 		return NULL;
 	}
 
@@ -1272,37 +1363,21 @@ dispatch_line(const char args[], int *count, char sep, int regexp, int quotes,
 		*last_pos = (args - args_beg) + st;
 	}
 
+	*positions = argvp;
 	return params;
 }
 
-TSTATIC void
-unescape(char s[], int regexp)
-{
-	char *p;
-
-	p = s;
-	while(s[0] != '\0')
-	{
-		if(s[0] == '\\' && (!regexp || s[1] == '/'))
-			s++;
-		*p++ = s[0];
-		if(s[0] != '\0')
-		{
-			s++;
-		}
-	}
-	*p = '\0';
-}
-
 char **
 list_udf(void)
 {
-	char **list;
 	char **p;
 	cmd_t *cur;
 
-	if((list = malloc(sizeof(*list)*(inner->udf_count*2 + 1))) == NULL)
+	char **const list = reallocarray(NULL, inner->udf_count*2 + 1, sizeof(*list));
+	if(list == NULL)
+	{
 		return NULL;
+	}
 
 	p = list;
 
@@ -1401,4 +1476,4 @@ is_separator(char c, char sep)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
