@@ -16,6 +16,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+/* WARNING: code implementing execution of keys deserves a prize in nomination
+ *          "insanely perplexed control flow".  You have been warned.
+ * TODO: seriously, indirect recursion and jumps between routines is a horrible
+ *       mess, even not touching tree representation this should be possible to
+ *       rewrite with master loop that simply goes through the string in
+ *       left-to-right order. */
+
 #include "keys.h"
 
 #include <assert.h> /* assert() */
@@ -27,12 +34,22 @@
 #include <wctype.h> /* iswdigit() */
 #include <wchar.h> /* wcscat() wcslen() */
 
+#include "../compat/reallocarray.h"
 #include "../utils/macros.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
-#include "../utils/test_helpers.h"
 #include "mode.h"
 
+/* Type of key chunk. */
+typedef enum
+{
+	BUILTIN_WAIT_POINT, /* Infinite wait of next key press. */
+	BUILTIN_KEYS,       /* Normal builtin key. */
+	BUILTIN_NIM_KEYS,   /* NIM - number in the middle. */
+	USER_CMD,           /* User mapping. */
+}
+KeyType;
+
 typedef struct key_chunk_t
 {
 	wchar_t key;
@@ -40,11 +57,17 @@ typedef struct key_chunk_t
 	size_t children_count;
 	int enters;            /* To prevent stack overflow and manager lifetime. */
 	int deleted;           /* Postpone free() call for proper lazy deletion. */
+	KeyType type;          /* General key type. */
 	key_conf_t conf;
 	struct key_chunk_t *child;
 	struct key_chunk_t *parent;
 	struct key_chunk_t *prev, *next;
-}key_chunk_t;
+}
+key_chunk_t;
+
+/* Callback of traverse_children(). */
+typedef void (*traverse_func)(const key_chunk_t *chunk, const wchar_t lhs[],
+		void *arg);
 
 static key_chunk_t *builtin_cmds_root;
 static key_chunk_t *selectors_root;
@@ -69,6 +92,10 @@ static int dispatch_keys(const wchar_t keys[], keys_info_t *keys_info,
 		int no_remap, int prev_count);
 static int dispatch_keys_at_root(const wchar_t keys[], keys_info_t *keys_info,
 		key_chunk_t *root, key_info_t key_info, int no_remap);
+static int dispatch_selector(const wchar_t keys[], keys_info_t *keys_info,
+		key_info_t master_key_info, key_chunk_t *master_curr, int no_remap);
+static int fill_key_info(const wchar_t **keys, key_info_t *key_info,
+		int prev_count, int *result);
 static int contains_chain(key_chunk_t *root, const wchar_t *begin,
 		const wchar_t *end);
 static int execute_next_keys(key_chunk_t *curr, const wchar_t keys[],
@@ -89,17 +116,26 @@ static const wchar_t * get_count(const wchar_t keys[], int *count);
 static int is_at_count(const wchar_t keys[]);
 static int combine_counts(int count_a, int count_b);
 static key_chunk_t * find_user_keys(const wchar_t *keys, int mode);
+static int add_list_of_keys(key_chunk_t *root, keys_add_info_t cmds[],
+		size_t len);
 static key_chunk_t * add_keys_inner(key_chunk_t *root, const wchar_t *keys);
-static int fill_list(const key_chunk_t *curr, size_t len, wchar_t **list);
+static void list_chunk(const key_chunk_t *chunk, const wchar_t lhs[],
+		void *arg);
 static void inc_counter(const keys_info_t *const keys_info, const size_t by);
 static int is_recursive(void);
 static int execute_mapping_handler(const key_conf_t *const info,
 		key_info_t key_info, keys_info_t *const keys_info);
 static void pre_execute_mapping_handler(const keys_info_t *const keys_info);
 static void post_execute_mapping_handler(const keys_info_t *const keys_info);
+static void keys_suggest(const key_chunk_t *root, const wchar_t keys[],
+		const wchar_t prefix[], vle_keys_list_cb cb, int custom_only);
+static void traverse_children(const key_chunk_t *chunk, const wchar_t prefix[],
+		traverse_func cb, void *param);
+static void suggest_chunk(const key_chunk_t *chunk, const wchar_t lhs[],
+		void *arg);
 
 void
-init_keys(int modes_count, int *key_mode_flags)
+vle_keys_init(int modes_count, int *key_mode_flags)
 {
 	assert(key_mode_flags != NULL);
 	assert(modes_count > 0);
@@ -121,13 +157,20 @@ init_keys(int modes_count, int *key_mode_flags)
 }
 
 void
-clear_keys(void)
+vle_keys_reset(void)
 {
 	free_forest(builtin_cmds_root, max_modes);
 	free_forest(user_cmds_root, max_modes);
 	free_forest(selectors_root, max_modes);
 
 	free(def_handlers);
+
+	builtin_cmds_root = NULL;
+	selectors_root = NULL;
+	user_cmds_root = NULL;
+	max_modes = 0;
+	mode_flags = NULL;
+	def_handlers = NULL;
 }
 
 /* Releases array of trees of length size including trees. */
@@ -143,7 +186,7 @@ free_forest(key_chunk_t *forest, size_t size)
 }
 
 void
-clear_user_keys(void)
+vle_keys_user_clear(void)
 {
 	int i;
 
@@ -170,8 +213,10 @@ free_tree(key_chunk_t *root)
 		free_chunk(root->next);
 	}
 
-	if(root->conf.type == USER_CMD || root->conf.type == BUILTIN_CMD)
+	if(root->type == USER_CMD)
+	{
 		free(root->conf.data.cmd);
+	}
 }
 
 static void
@@ -188,35 +233,35 @@ free_chunk(key_chunk_t *chunk)
 }
 
 void
-set_def_handler(int mode, default_handler handler)
+vle_keys_set_def_handler(int mode, default_handler handler)
 {
 	def_handlers[mode] = handler;
 }
 
 /* This function should never be called from this module, only externally. */
 int
-execute_keys(const wchar_t keys[])
+vle_keys_exec(const wchar_t keys[])
 {
 	return execute_keys_general_wrapper(keys, 0, 0, 0);
 }
 
 /* This function should never be called from this module, only externally. */
 int
-execute_keys_no_remap(const wchar_t keys[])
+vle_keys_exec_no_remap(const wchar_t keys[])
 {
 	return execute_keys_general_wrapper(keys, 0, 0, 1);
 }
 
 /* This function should never be called from this module, only externally. */
 int
-execute_keys_timed_out(const wchar_t keys[])
+vle_keys_exec_timed_out(const wchar_t keys[])
 {
 	return execute_keys_general_wrapper(keys, 1, 0, 0);
 }
 
 /* This function should never be called from this module, only externally. */
 int
-execute_keys_timed_out_no_remap(const wchar_t keys[])
+vle_keys_exec_timed_out_no_remap(const wchar_t keys[])
 {
 	return execute_keys_general_wrapper(keys, 1, 0, 1);
 }
@@ -266,33 +311,19 @@ dispatch_keys(const wchar_t keys[], keys_info_t *keys_info, int no_remap,
 	key_info_t key_info;
 	int result;
 
-	keys = get_reg(keys, &key_info.reg);
-	if(keys == NULL)
+	if(fill_key_info(&keys, &key_info, prev_count, &result) != 0)
 	{
-		return KEYS_WAIT;
-	}
-	if(key_info.reg == L'\x1b' || key_info.reg == L'\x03')
-	{
-		return 0;
+		return result;
 	}
 
-	keys = get_count(keys, &key_info.count);
-	key_info.count = combine_counts(key_info.count, prev_count);
-	key_info.multi = L'\0';
-
+	result = KEYS_UNKNOWN;
 	if(!no_remap)
 	{
-		key_chunk_t *const root = keys_info->selector
-		                        ? &selectors_root[vle_mode_get()]
-		                        : &user_cmds_root[vle_mode_get()];
-		result = dispatch_keys_at_root(keys, keys_info, root, key_info, no_remap);
-	}
-	else
-	{
-		result = KEYS_UNKNOWN;
+		result = dispatch_keys_at_root(keys, keys_info,
+				&user_cmds_root[vle_mode_get()], key_info, no_remap);
 	}
 
-	if(result == KEYS_UNKNOWN && !keys_info->selector)
+	if(result == KEYS_UNKNOWN)
 	{
 		result = dispatch_keys_at_root(keys, keys_info,
 				&builtin_cmds_root[vle_mode_get()], key_info, no_remap);
@@ -312,7 +343,7 @@ dispatch_keys_at_root(const wchar_t keys[], keys_info_t *keys_info,
 	int has_duplicate;
 	int result;
 
-	/* The loop finds longest match of the input (keys) amoung registered
+	/* The loop finds longest match of the input (keys) among registered
 	 * shortcuts. */
 	curr = root;
 	while(*keys != L'\0')
@@ -320,14 +351,12 @@ dispatch_keys_at_root(const wchar_t keys[], keys_info_t *keys_info,
 		key_chunk_t *p;
 		int number_in_the_middle = 0;
 
-		p = curr->child;
-		while(p != NULL && p->key < *keys)
+		for(p = curr->child; p != NULL && p->key < *keys; p = p->next)
 		{
-			if(p->conf.type == BUILTIN_NIM_KEYS)
+			if(p->type == BUILTIN_NIM_KEYS)
 			{
 				number_in_the_middle = 1;
 			}
-			p = p->next;
 		}
 
 		if(p == NULL || p->key != *keys)
@@ -335,13 +364,12 @@ dispatch_keys_at_root(const wchar_t keys[], keys_info_t *keys_info,
 			if(curr == root)
 				return KEYS_UNKNOWN;
 
-			while(p != NULL)
+			for(; p != NULL; p = p->next)
 			{
-				if(p->conf.type == BUILTIN_NIM_KEYS)
+				if(p->type == BUILTIN_NIM_KEYS)
 				{
 					number_in_the_middle = 1;
 				}
-				p = p->next;
 			}
 
 			if(curr->conf.followed != FOLLOWED_BY_NONE &&
@@ -362,32 +390,29 @@ dispatch_keys_at_root(const wchar_t keys[], keys_info_t *keys_info,
 				}
 			}
 
-			if(curr->conf.type == BUILTIN_WAIT_POINT)
+			if(curr->type == BUILTIN_WAIT_POINT)
 			{
 				return KEYS_UNKNOWN;
 			}
 
 			has_duplicate = root == &user_cmds_root[vle_mode_get()] &&
 					contains_chain(&builtin_cmds_root[vle_mode_get()], keys_start, keys);
-			result = execute_next_keys(curr, curr->conf.type == USER_CMD ? keys : L"",
+			result = execute_next_keys(curr, curr->type == USER_CMD ? keys : L"",
 					&key_info, keys_info, has_duplicate, no_remap);
-			if(curr->conf.type == USER_CMD)
+			if(curr->type == USER_CMD)
 				return result;
 			if(IS_KEYS_RET_CODE(result))
 			{
-				if(result == KEYS_WAIT_SHORT)
-					return KEYS_UNKNOWN;
-
-				return result;
+				return (result == KEYS_WAIT_SHORT) ? KEYS_UNKNOWN : result;
 			}
 			inc_counter(keys_info, keys - keys_start);
 			return execute_keys_general(keys, 0, keys_info->mapped, no_remap);
 		}
-		keys++;
+		++keys;
 		curr = p;
 	}
 
-	if(*keys == '\0' && curr->conf.type != BUILTIN_WAIT_POINT &&
+	if(*keys == '\0' && curr->type != BUILTIN_WAIT_POINT &&
 			curr->children_count > 0 && curr->conf.data.handler != NULL &&
 			!keys_info->after_wait)
 	{
@@ -409,6 +434,112 @@ dispatch_keys_at_root(const wchar_t keys[], keys_info_t *keys_info,
 	return result;
 }
 
+/* Dispatches keys passed in as a selector followed by arbitrary other keys.
+ * Returns error code. */
+static int
+dispatch_selector(const wchar_t keys[], keys_info_t *keys_info,
+		key_info_t master_key_info, key_chunk_t *master_curr, int no_remap)
+{
+	const wchar_t *keys_start = keys;
+	key_chunk_t *curr = &selectors_root[vle_mode_get()];
+	key_info_t key_info;
+	int result;
+
+	if(fill_key_info(&keys, &key_info, master_key_info.count, &result) != 0)
+	{
+		return result;
+	}
+
+	/* The loop finds longest match of the input (keys) among registered
+	 * shortcuts. */
+	while(*keys != L'\0')
+	{
+		key_chunk_t *p;
+
+		for(p = curr->child; p != NULL && p->key < *keys; p = p->next)
+		{
+			/* Advance. */
+		}
+		if(p == NULL || p->key != *keys)
+		{
+			break;
+		}
+		++keys;
+		curr = p;
+	}
+
+	/* Handle ambiguous selector. */
+	if(*keys == '\0' && curr->type != BUILTIN_WAIT_POINT &&
+			curr->children_count > 0 && curr->conf.data.handler != NULL &&
+			!keys_info->after_wait)
+	{
+		return KEYS_WAIT_SHORT;
+	}
+
+	/* Execute the selector. */
+	if(curr->conf.followed == FOLLOWED_BY_MULTIKEY && keys[0] != L'\0')
+	{
+		const wchar_t mk[] = { keys[0], L'\0' };
+		result = execute_next_keys(curr, mk, &key_info, keys_info, 0, no_remap);
+		++keys;
+	}
+	else
+	{
+		result = keys[0] == L'\0'
+		       ? execute_next_keys(curr, L"", &key_info, keys_info, 0, no_remap)
+		       : dispatch_key(key_info, keys_info, curr, L"");
+	}
+	if(IS_KEYS_RET_CODE(result))
+	{
+		return result;
+	}
+
+	/* We used this count in selector, so don't pass it to command. */
+	master_key_info.count = NO_COUNT_GIVEN;
+
+	/* Execute command that requested the selector. */
+	result = execute_mapping_handler(&master_curr->conf, master_key_info,
+			keys_info);
+	if(IS_KEYS_RET_CODE(result))
+	{
+		return result;
+	}
+
+	inc_counter(keys_info, keys - keys_start);
+
+	/* execute_keys_general_wrapper() treats empty input as an error. */
+	if(keys[0] == L'\0')
+	{
+		return 0;
+	}
+	/* Process the rest of the line. */
+	return execute_keys_general(keys, keys_info->after_wait, 0, no_remap);
+}
+
+/* Fills *key_info advancing input if necessary.  Returns zero on success,
+ * otherwise non-zero is returned and *result is set. */
+static int
+fill_key_info(const wchar_t **keys, key_info_t *key_info, int prev_count,
+		int *result)
+{
+	*keys = get_reg(*keys, &key_info->reg);
+	if(*keys == NULL)
+	{
+		*result = KEYS_WAIT;
+		return 1;
+	}
+	if(key_info->reg == L'\x1b' || key_info->reg == L'\x03')
+	{
+		*result = 0;
+		return 1;
+	}
+
+	*keys = get_count(*keys, &key_info->count);
+	key_info->count = combine_counts(key_info->count, prev_count);
+	key_info->multi = L'\0';
+	return 0;
+}
+
 static int
 contains_chain(key_chunk_t *root, const wchar_t *begin, const wchar_t *end)
 {
@@ -433,7 +564,7 @@ contains_chain(key_chunk_t *root, const wchar_t *begin, const wchar_t *end)
 		curr = p;
 	}
 	return (curr->conf.followed == FOLLOWED_BY_NONE &&
-			curr->conf.type != BUILTIN_WAIT_POINT);
+			curr->type != BUILTIN_WAIT_POINT);
 }
 
 /* Handles the rest of the keys after first one has been determined (in curr).
@@ -447,8 +578,8 @@ execute_next_keys(key_chunk_t *curr, const wchar_t keys[], key_info_t *key_info,
 
 	if(*keys == L'\0')
 	{
-		int wait_point = (conf->type == BUILTIN_WAIT_POINT);
-		wait_point = wait_point || (conf->type == USER_CMD &&
+		int wait_point = (curr->type == BUILTIN_WAIT_POINT);
+		wait_point = wait_point || (curr->type == USER_CMD &&
 				conf->followed != FOLLOWED_BY_NONE);
 
 		if(wait_point)
@@ -464,10 +595,8 @@ execute_next_keys(key_chunk_t *curr, const wchar_t keys[], key_info_t *key_info,
 			return KEYS_UNKNOWN;
 		}
 	}
-	else if(conf->type != USER_CMD)
+	else if(curr->type != USER_CMD)
 	{
-		int result;
-
 		if(conf->followed == FOLLOWED_BY_MULTIKEY)
 		{
 			key_info->multi = keys[0];
@@ -475,16 +604,7 @@ execute_next_keys(key_chunk_t *curr, const wchar_t keys[], key_info_t *key_info,
 		}
 
 		keys_info->selector = 1;
-		result = dispatch_keys(keys, keys_info, no_remap, key_info->count);
-		keys_info->selector = 0;
-
-		if(IS_KEYS_RET_CODE(result))
-		{
-			return result;
-		}
-
-		/* We used this count in selector, so don't pass it to command. */
-		key_info->count = NO_COUNT_GIVEN;
+		return dispatch_selector(keys, keys_info, *key_info, curr, no_remap);
 	}
 
 	return dispatch_key(*key_info, keys_info, curr, keys);
@@ -498,7 +618,7 @@ dispatch_key(key_info_t key_info, keys_info_t *keys_info, key_chunk_t *curr,
 {
 	const key_conf_t *const conf = &curr->conf;
 
-	if(conf->type != USER_CMD && conf->type != BUILTIN_CMD)
+	if(curr->type != USER_CMD)
 	{
 		const int result = execute_mapping_handler(conf, key_info, keys_info);
 		const int finish_dispatching = result != 0
@@ -586,7 +706,7 @@ execute_after_remapping(const wchar_t rhs[], const wchar_t left_keys[],
 	int result;
 	if(rhs[0] == L'\0' && left_keys[0] == L'\0')
 	{
-		/* Nop command executed correctly. */
+		/* No-operation command "executed" correctly. */
 		result = 0;
 	}
 	else if(rhs[0] == L'\0')
@@ -739,38 +859,34 @@ combine_counts(int count_a, int count_b)
 	}
 }
 
-TSTATIC key_conf_t *
-add_cmd(const wchar_t keys[], int mode)
-{
-	key_chunk_t *curr = add_keys_inner(&builtin_cmds_root[mode], keys);
-	return &curr->conf;
-}
-
 int
-add_user_keys(const wchar_t *keys, const wchar_t *cmd, int mode, int no_r)
+vle_keys_user_add(const wchar_t lhs[], const wchar_t rhs[], int mode, int no_r)
 {
-	key_chunk_t *curr;
+	key_chunk_t *curr = add_keys_inner(&user_cmds_root[mode], lhs);
+	if(curr == NULL)
+	{
+		return -1;
+	}
 
-	curr = add_keys_inner(&user_cmds_root[mode], keys);
-	if(curr->conf.type == USER_CMD)
+	if(curr->type == USER_CMD)
 	{
 		free((void*)curr->conf.data.cmd);
 	}
 
-	curr->conf.type = USER_CMD;
-	curr->conf.data.cmd = vifm_wcsdup(cmd);
+	curr->type = USER_CMD;
+	curr->conf.data.cmd = vifm_wcsdup(rhs);
 	curr->no_remap = no_r;
 	return 0;
 }
 
 int
-has_user_keys(const wchar_t *keys, int mode)
+vle_keys_user_exists(const wchar_t keys[], int mode)
 {
 	return find_user_keys(keys, mode) != NULL;
 }
 
 int
-remove_user_keys(const wchar_t *keys, int mode)
+vle_keys_user_remove(const wchar_t keys[], int mode)
 {
 	key_chunk_t *curr, *p;
 
@@ -778,7 +894,7 @@ remove_user_keys(const wchar_t *keys, int mode)
 		return -1;
 
 	free(curr->conf.data.cmd);
-	curr->conf.type = BUILTIN_WAIT_POINT;
+	curr->type = BUILTIN_WAIT_POINT;
 	curr->conf.data.handler = NULL;
 
 	p = curr;
@@ -804,7 +920,7 @@ remove_user_keys(const wchar_t *keys, int mode)
 		curr = parent;
 	}
 	while(curr->parent != NULL && curr->parent->conf.data.handler == NULL &&
-			curr->parent->conf.type == BUILTIN_WAIT_POINT &&
+			curr->parent->type == BUILTIN_WAIT_POINT &&
 			curr->parent->children_count == 0);
 
 	return 0;
@@ -824,51 +940,49 @@ find_user_keys(const wchar_t *keys, int mode)
 		curr = p;
 		keys++;
 	}
-	return (curr->conf.type == USER_CMD) ? curr : NULL;
-}
-
-TSTATIC key_conf_t *
-add_selector(const wchar_t keys[], int mode)
-{
-	key_chunk_t *curr = add_keys_inner(&selectors_root[mode], keys);
-	return &curr->conf;
+	return (curr->type == USER_CMD) ? curr : NULL;
 }
 
 int
-add_cmds(keys_add_info_t *cmds, size_t len, int mode)
+vle_keys_add(keys_add_info_t cmds[], size_t len, int mode)
+{
+	return add_list_of_keys(&builtin_cmds_root[mode], cmds, len);
+}
+
+int
+vle_keys_add_selectors(keys_add_info_t cmds[], size_t len, int mode)
+{
+	return add_list_of_keys(&selectors_root[mode], cmds, len);
+}
+
+/* Registers cmds[0 .. len-1] keys specified tree.  Returns non-zero on error,
+ * otherwise zero is returned. */
+static int
+add_list_of_keys(key_chunk_t *root, keys_add_info_t cmds[], size_t len)
 {
 	int result = 0;
 	size_t i;
 
-	for(i = 0; i < len; i++)
+	for(i = 0U; i < len; ++i)
 	{
-		key_conf_t *curr;
-
-		curr = add_cmd(cmds[i].keys, mode);
+		key_chunk_t *const curr = add_keys_inner(root, cmds[i].keys);
 		if(curr == NULL)
+		{
 			result = -1;
+			continue;
+		}
+
+		curr->conf = cmds[i].info;
+		if(curr->conf.nim)
+		{
+			curr->type = BUILTIN_NIM_KEYS;
+		}
 		else
-			*curr = cmds[i].info;
-	}
-
-	return result;
-}
-
-int
-add_selectors(keys_add_info_t *cmds, size_t len, int mode)
-{
-	int result = 0;
-	size_t i;
-
-	for(i = 0; i < len; i++)
-	{
-		key_conf_t *curr;
-
-		curr = add_selector(cmds[i].keys, mode);
-		if(curr == NULL)
-			result = -1;
-		else
-			*curr = cmds[i].info;
+		{
+			curr->type = (curr->conf.followed == FOLLOWED_BY_NONE)
+			           ? BUILTIN_KEYS
+			           : BUILTIN_WAIT_POINT;
+		}
 	}
 
 	return result;
@@ -894,10 +1008,14 @@ add_keys_inner(key_chunk_t *root, const wchar_t *keys)
 			if(c == NULL)
 				return NULL;
 			c->key = *keys;
-			c->conf.type = (keys[1] == L'\0') ? BUILTIN_KEYS : BUILTIN_WAIT_POINT;
+			c->type = (keys[1] == L'\0') ? BUILTIN_KEYS : BUILTIN_WAIT_POINT;
 			c->conf.data.handler = NULL;
 			c->conf.data.cmd = NULL;
 			c->conf.followed = FOLLOWED_BY_NONE;
+			c->conf.suggest = NULL;
+			c->conf.descr = NULL;
+			c->conf.nim = 0;
+			c->conf.skip_suggestion = 0;
 			c->prev = prev;
 			c->next = p;
 			c->child = NULL;
@@ -930,132 +1048,45 @@ add_keys_inner(key_chunk_t *root, const wchar_t *keys)
 	return curr;
 }
 
-wchar_t **
-list_cmds(int mode)
+void
+vle_keys_list(int mode, vle_keys_list_cb cb)
 {
-	int i, j;
-	int count;
-	wchar_t **result;
+	const key_chunk_t *user = &user_cmds_root[mode];
+	const key_chunk_t *builtin = &builtin_cmds_root[mode];
 
-	count = user_cmds_root[mode].children_count;
-	count += 1;
-	count += builtin_cmds_root[mode].children_count;
-	result = calloc(count + 1, sizeof(*result));
-	if(result == NULL)
-		return NULL;
-
-	if(fill_list(&user_cmds_root[mode], 0, result) < 0)
+	/* Don't traverse empty tries. */
+	if(user->children_count != 0U)
 	{
-		free_wstring_array(result, count);
-		return NULL;
-	}
+		traverse_children(user, L"", &list_chunk, cb);
 
-	j = -1;
-	for(i = 0; i < count && result[i] != NULL; i++)
-	{
-		if(result[i][0] == L'\0')
+		/* Put separator only when it's needed. */
+		if(builtin->children_count != 0U)
 		{
-			free(result[i]);
-			result[i] = NULL;
-			if(j == -1)
-				j = i;
-		}
-	}
-	if(j == -1)
-		j = i;
-
-	result[j++] = vifm_wcsdup(L"");
-
-	if(fill_list(&builtin_cmds_root[mode], 0, result + j) < 0)
-	{
-		free_wstring_array(result, count);
-		return NULL;
-	}
-
-	for(i = j; i < count && result[i] != NULL; i++)
-	{
-		if(result[i][0] == L'\0')
-		{
-			free(result[i]);
-			result[i] = NULL;
+			cb(L"", L"", "");
 		}
 	}
 
-	return result;
+	if(builtin->children_count != 0U)
+	{
+		traverse_children(builtin, L"", &list_chunk, cb);
+	}
 }
 
-static int
-fill_list(const key_chunk_t *curr, size_t len, wchar_t **list)
+/* Invokes list callback (in arg) for leaf chunks. */
+static void
+list_chunk(const key_chunk_t *chunk, const wchar_t lhs[], void *arg)
 {
-	size_t i;
-	const key_chunk_t *child;
-	size_t count;
-
-	count = curr->children_count;
-	if(curr->conf.type == USER_CMD)
-		count++;
-	if(count == 0)
-		count = 1;
-
-	for(i = 0; i < count; i++)
+	if(chunk->children_count == 0 || chunk->type == USER_CMD)
 	{
-		wchar_t *t;
-
-		t = realloc(list[i], sizeof(wchar_t)*(len + 1));
-		if(t == NULL)
-			return -1;
-
-		list[i] = t;
-		if(len > 0)
-			t[len - 1] = curr->key;
-		t[len] = L'\0';
+		const wchar_t *rhs = (chunk->type == USER_CMD) ? chunk->conf.data.cmd : L"";
+		const char *descr = (chunk->conf.descr == NULL) ? "" : chunk->conf.descr;
+		vle_keys_list_cb cb = arg;
+		cb(lhs, rhs, descr);
 	}
-
-	i = 0;
-	if(curr->children_count == 0 || curr->conf.type == USER_CMD)
-	{
-		const wchar_t *s;
-		wchar_t *t;
-
-		s = (curr->conf.type == USER_CMD) ? curr->conf.data.cmd : L"<built in>";
-		t = realloc(list[0], sizeof(wchar_t)*(len + 1 + wcslen(s) + 1));
-		if(t == NULL)
-			return -1;
-
-		list[0] = t;
-		t[wcslen(t) + 1] = L'\0';
-		wcscpy(t + wcslen(t) + 1, s);
-		if(curr->children_count == 0)
-			return 1;
-
-		list++;
-		i = 1;
-	}
-
-	child = curr->child;
-	while(child != NULL)
-	{
-		int filled = fill_list(child, len + 1, list);
-		if(filled < 0)
-			return -1;
-
-		list += filled;
-		i += filled;
-
-		child = child->next;
-	}
-
-	while(i < count--)
-	{
-		free(*list);
-		*list++ = NULL;
-	}
-
-	return i;
 }
 
 size_t
-get_key_counter(void)
+vle_keys_counter(void)
 {
 	return counter;
 }
@@ -1077,13 +1108,13 @@ inc_counter(const keys_info_t *const keys_info, const size_t by)
 static int
 is_recursive(void)
 {
-	return enters_counter > 1;
+	return (enters_counter > 1);
 }
 
 int
-is_inside_mapping(void)
+vle_keys_inside_mapping(void)
 {
-	return inside_mapping != 0;
+	return (inside_mapping != 0);
 }
 
 /* Executes handler for a mapping, if any.  Error or success code is
@@ -1118,5 +1149,155 @@ post_execute_mapping_handler(const keys_info_t *const keys_info)
 	assert(inside_mapping >= 0 && "Calls to pre/post funcs should be balanced");
 }
 
+void
+vle_keys_suggest(const wchar_t keys[], vle_keys_list_cb cb, int custom_only)
+{
+	keys_suggest(&user_cmds_root[vle_mode_get()], keys, L"key: ", cb,
+			custom_only);
+	keys_suggest(&builtin_cmds_root[vle_mode_get()], keys, L"key: ", cb,
+			custom_only);
+}
+
+/* Looks up possible continuations of keys for the given root and calls cb on
+ * them. */
+static void
+keys_suggest(const key_chunk_t *root, const wchar_t keys[],
+		const wchar_t prefix[], vle_keys_list_cb cb, int custom_only)
+{
+	const key_chunk_t *curr = root;
+
+	while(*keys != L'\0')
+	{
+		const key_chunk_t *p;
+		int number_in_the_middle = 0;
+
+		/* Look up current key among children of current node (might be root), while
+		 * inspecting NIM as well. */
+		for(p = curr->child; p != NULL && p->key < *keys; p = p->next)
+		{
+			if(p->type == BUILTIN_NIM_KEYS)
+			{
+				number_in_the_middle = 1;
+			}
+		}
+
+		/* Go to the next character if a match found. */
+		if(p != NULL && p->key == *keys)
+		{
+			++keys;
+			curr = p;
+			continue;
+		}
+
+		/* No match for the first character is fatal for the lookup. */
+		if(curr == root)
+		{
+			return;
+		}
+
+		/* Need to inspect all children for NIM. */
+		for(; p != NULL && !number_in_the_middle; p = p->next)
+		{
+			if(p->type == BUILTIN_NIM_KEYS)
+			{
+				number_in_the_middle = 1;
+			}
+		}
+
+		/* Give up if this isn't one of cases where next character is not presented
+		 * in the tree by design. */
+		if(curr->conf.followed != FOLLOWED_BY_NONE &&
+				(!number_in_the_middle || !is_at_count(keys)))
+		{
+			break;
+		}
+
+		/* Skip over number in the middle, if any. */
+		if(number_in_the_middle)
+		{
+			int count;
+			const wchar_t *new_keys = get_count(keys, &count);
+			if(new_keys != keys)
+			{
+				keys = new_keys;
+				continue;
+			}
+		}
+
+		break;
+	}
+
+	if(!custom_only && *keys == L'\0')
+	{
+		/* Suggest all children. */
+		const key_chunk_t *child;
+		for(child = curr->child; child != NULL; child = child->next)
+		{
+			traverse_children(child, prefix, &suggest_chunk, cb);
+		}
+	}
+
+	if(curr->type == BUILTIN_WAIT_POINT)
+	{
+		if(curr->conf.followed == FOLLOWED_BY_SELECTOR)
+		{
+			/* Suggest selectors. */
+			keys_suggest(&selectors_root[vle_mode_get()], keys, L"sel: ", cb,
+					custom_only);
+		}
+		else if(curr->conf.followed == FOLLOWED_BY_MULTIKEY)
+		{
+			/* Invoke optional external function to provide suggestions. */
+			if(curr->conf.suggest != NULL)
+			{
+				curr->conf.suggest(cb);
+			}
+		}
+	}
+}
+
+/* Visit every child of the tree and calls cb with param on it. */
+static void
+traverse_children(const key_chunk_t *chunk, const wchar_t prefix[],
+		traverse_func cb, void *param)
+{
+	const key_chunk_t *child;
+
+	const size_t prefix_len = wcslen(prefix);
+	wchar_t item[prefix_len + 1U + 1U];
+	wcscpy(item, prefix);
+	item[prefix_len] = chunk->key;
+	item[prefix_len + 1U] = L'\0';
+
+	cb(chunk, item, param);
+
+	for(child = chunk->child; child != NULL; child = child->next)
+	{
+		traverse_children(child, item, cb, param);
+	}
+}
+
+/* Invokes suggestion callback (in arg) for leaf chunks. */
+static void
+suggest_chunk(const key_chunk_t *chunk, const wchar_t lhs[], void *arg)
+{
+	vle_keys_list_cb cb = arg;
+
+	if(chunk->conf.skip_suggestion)
+	{
+		return;
+	}
+
+	if(chunk->type == USER_CMD)
+	{
+		cb(lhs, chunk->conf.data.cmd, "");
+	}
+	else if(chunk->children_count == 0 ||
+			chunk->conf.followed != FOLLOWED_BY_NONE)
+	{
+		cb(lhs, L"", (chunk->conf.descr == NULL) ? "" : chunk->conf.descr);
+	}
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0: */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
