@@ -23,11 +23,11 @@
 #include <ctype.h>
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h>
-#include <stdlib.h>
+#include <stdlib.h> /* realloc() */
 #include <string.h> /* memset() strchr() strcmp() strcpy() strlen() */
 
+#include "../compat/reallocarray.h"
 #include "../utils/str.h"
-#include "../utils/string_array.h"
 #include "completion.h"
 #include "text_buffer.h"
 
@@ -54,21 +54,28 @@ SetOp;
 /* Type of function accepted by the for_each_char_of() function. */
 typedef void (*mod_t)(char buffer[], char value);
 
-static opt_t * add_option_inner(const char name[], OPT_TYPE type, int val_count,
-		const char *vals[], opt_handler handler);
-static int allocates_str_value(OPT_TYPE type);
-static void print_changed_options(void);
-static int process_option(const char arg[]);
-static int handle_all_pseudo(const char arg[], const char suffix[]);
-static void print_options(void);
-static opt_t * get_option(const char option[]);
+/* Type of callback for the enum_options() function. */
+typedef void (*opt_traverse)(opt_t *opt);
+
+static void reset_options(OPT_SCOPE scope);
+static opt_t * add_option_inner(const char name[], const char descr[],
+		OPT_TYPE type, OPT_SCOPE scope, int val_count, const char *vals[][2],
+		opt_handler handler);
+static void print_if_changed(opt_t *opt);
+static int process_option(const char arg[], OPT_SCOPE real_scope,
+		OPT_SCOPE scope, int *print);
+static int handle_all_pseudo(const char arg[], const char suffix[],
+		OPT_SCOPE scope, int *print);
+static void set_print_void(opt_t *opt);
+static void enum_options(OPT_SCOPE scope, opt_traverse traverser);
+static void enum_unique_options(opt_traverse traverser);
+static opt_t * get_option(const char option[], OPT_SCOPE scope);
+static opt_t * pick_option(opt_t *opts[2], OPT_SCOPE scope);
 static int set_on(opt_t *opt);
 static int set_off(opt_t *opt);
 static int set_inv(opt_t *opt);
-static int set_set(opt_t *opt, const char value[]);
 static int set_reset(opt_t *opt);
-static int set_add(opt_t *opt, const char value[]);
-static int set_remove(opt_t *opt, const char value[]);
+static int uses_str_value(OPT_TYPE type);
 static int set_hat(opt_t *opt, const char value[]);
 static void notify_option_update(opt_t *opt, OPT_OP op, optval_t val);
 static int set_op(opt_t *opt, const char value[], SetOp op);
@@ -86,9 +93,11 @@ static char * str_add(char old[], const char value[]);
 static int str_remove(char old[], const char value[]);
 static int find_val(const opt_t *opt, const char value[]);
 static int set_print(const opt_t *opt);
-static const char * extract_option(const char args[], char buf[], int replace);
+static char * extract_option(const char **argsp, int replace);
 static char * skip_alphas(const char str[]);
-static void complete_option_name(const char buf[], int bool_only, int pseudo);
+static void complete_option_name(const char buf[], int bool_only, int pseudo,
+		OPT_SCOPE scope);
+static int option_matches(const opt_t *opt, OPT_SCOPE scope);
 static const char * get_opt_full_name(opt_t *const opt);
 static int complete_option_value(const opt_t *opt, const char beginning[]);
 static int complete_list_value(const opt_t *opt, const char beginning[]);
@@ -102,7 +111,7 @@ static const char MIDDLE_CHARS[] = "^+-=:";
 static int *opts_changed;
 
 static opt_t *options;
-static size_t options_count;
+static size_t option_count;
 
 void
 init_options(int *opts_changed_flag)
@@ -113,9 +122,9 @@ init_options(int *opts_changed_flag)
 }
 
 void
-reset_option_to_default(const char name[])
+reset_option_to_default(const char name[], OPT_SCOPE scope)
 {
-	opt_t *opt = find_option(name);
+	opt_t *opt = find_option(name, scope);
 	if(opt != NULL)
 	{
 		set_reset(opt);
@@ -124,11 +133,17 @@ reset_option_to_default(const char name[])
 
 void
 reset_options_to_default(void)
+{
+	reset_options(OPT_ANY);
+}
+
+static void
+reset_options(OPT_SCOPE scope)
 {
 	size_t i;
-	for(i = 0U; i < options_count; ++i)
+	for(i = 0U; i < option_count; ++i)
 	{
-		if(options[i].full == NULL)
+		if(options[i].full == NULL && option_matches(&options[i], scope))
 		{
 			set_reset(&options[i]);
 		}
@@ -139,44 +154,59 @@ void
 clear_options(void)
 {
 	size_t i;
-	for(i = 0U; i < options_count; ++i)
+	for(i = 0U; i < option_count; ++i)
 	{
 		free(options[i].name);
-		if(allocates_str_value(options[i].type))
+		if(uses_str_value(options[i].type))
 		{
+			/* Abbreviations share default value with full option. */
 			if(options[i].full == NULL)
+			{
 				free(options[i].def.str_val);
+			}
 			free(options[i].val.str_val);
 		}
 	}
 	free(options);
 	options = NULL;
-	options_count = 0;
+	option_count = 0U;
 }
 
 void
-add_option(const char name[], const char abbr[], OPT_TYPE type, int val_count,
-		const char *vals[], opt_handler handler, optval_t def)
+add_option(const char name[], const char abbr[], const char descr[],
+		OPT_TYPE type, OPT_SCOPE scope, int val_count, const char *vals[][2],
+		opt_handler handler, optval_t def)
 {
 	opt_t *full;
 
 	assert(name != NULL);
 	assert(abbr != NULL);
 
-	full = add_option_inner(name, type, val_count, vals, handler);
+	full = add_option_inner(name, descr, type, scope, val_count, vals, handler);
 	if(full == NULL)
+	{
 		return;
+	}
+
 	if(abbr[0] != '\0')
 	{
-		char *full_name = full->name;
-		opt_t *abbreviated;
+		/* Save pointer to name used in this module for use below. */
+		const char *const full_name = full->name;
 
-		abbreviated = add_option_inner(abbr, type, val_count, vals, handler);
+		opt_t *abbreviated;
+		abbreviated = add_option_inner(abbr, descr, type, scope, val_count, vals,
+				handler);
 		if(abbreviated != NULL)
+		{
 			abbreviated->full = full_name;
-		full = find_option(full_name);
+		}
+
+		/* Inserting an option invalidates pointers to other options, so reload
+		 * it. */
+		full = find_option(full_name, scope);
 	}
-	if(allocates_str_value(type))
+
+	if(uses_str_value(type))
 	{
 		full->def.str_val = strdup(def.str_val);
 		full->val.str_val = strdup(def.str_val);
@@ -190,9 +220,16 @@ add_option(const char name[], const char abbr[], OPT_TYPE type, int val_count,
 #ifndef NDEBUG
 	switch(type)
 	{
+		int i;
+
 		case OPT_CHARSET:
-			assert((size_t)full->val_count == strlen(*full->vals) &&
+			assert((size_t)full->val_count == 1U + strlen(full->vals[0][0]) &&
 					"Number of character set items is incorrect.");
+			for(i = 1; i < full->val_count; ++i)
+			{
+				assert(full->vals[i][0][0] != '\0' && full->vals[i][0][1] == '\0' &&
+						"Incorrect character set item.");
+			}
 			break;
 
 		default:
@@ -205,18 +242,23 @@ add_option(const char name[], const char abbr[], OPT_TYPE type, int val_count,
 /* Adds option to internal array of option descriptors.  Returns a pointer to
  * the descriptor or NULL on out of memory error. */
 static opt_t *
-add_option_inner(const char name[], OPT_TYPE type, int val_count,
-		const char *vals[], opt_handler handler)
+add_option_inner(const char name[], const char descr[], OPT_TYPE type,
+		OPT_SCOPE scope, int val_count, const char *vals[][2], opt_handler handler)
 {
 	opt_t *p;
 
-	p = realloc(options, sizeof(*options)*(options_count + 1));
-	if(p == NULL)
-		return NULL;
-	options = p;
-	options_count++;
+	assert(scope != OPT_ANY && "OPT_ANY can't be an option type.");
+	assert(find_option(name, scope) == NULL && "Duplicated option.");
 
-	p = options + options_count - 1;
+	p = reallocarray(options, option_count + 1, sizeof(*options));
+	if(p == NULL)
+	{
+		return NULL;
+	}
+	options = p;
+	option_count++;
+
+	p = options + option_count - 1;
 
 	while((p - 1) >= options && strcmp((p - 1)->name, name) > 0)
 	{
@@ -225,7 +267,9 @@ add_option_inner(const char name[], OPT_TYPE type, int val_count,
 	}
 
 	p->name = strdup(name);
+	p->descr = descr;
 	p->type = type;
+	p->scope = scope;
 	p->handler = handler;
 	p->val_count = val_count;
 	p->vals = vals;
@@ -234,21 +278,13 @@ add_option_inner(const char name[], OPT_TYPE type, int val_count,
 	return p;
 }
 
-/* Checks whether given option type allocates memory for strings on heap.
- * Returns non-zero if so, otherwise zero is returned. */
-static int
-allocates_str_value(OPT_TYPE type)
-{
-	return (type == OPT_STR || type == OPT_STRLIST || type == OPT_CHARSET);
-}
-
 void
-set_option(const char name[], optval_t val)
+set_option(const char name[], optval_t val, OPT_SCOPE scope)
 {
-	opt_t *opt = find_option(name);
+	opt_t *opt = find_option(name, scope);
 	assert(opt != NULL && "Wrong option name.");
 
-	if(opt->type == OPT_STR || opt->type == OPT_STRLIST)
+	if(uses_str_value(opt->type))
 	{
 		(void)replace_string(&opt->val.str_val, val.str_val);
 	}
@@ -259,90 +295,127 @@ set_option(const char name[], optval_t val)
 }
 
 int
-set_options(const char args[])
+set_options(const char args[], OPT_SCOPE scope)
 {
-	int err = 0;
+	int errors = 0;
 
 	if(*args == '\0')
 	{
-		print_changed_options();
+		enum_options(scope, &print_if_changed);
 		return 0;
 	}
 
 	while(*args != '\0')
 	{
-		char buf[1024];
+		int print;
 
-		args = extract_option(args, buf, 1);
-		if(args == NULL)
+		char *const opt = extract_option(&args, 1);
+		if(args == NULL || opt == NULL)
+		{
+			free(opt);
 			return -1;
-		err += process_option(buf);
+		}
+
+		if(*args == '\0' && *opt == '\0')
+		{
+			/* Stop on reaching comment. */
+			free(opt);
+			break;
+		}
+
+		if(scope == OPT_ANY)
+		{
+			const int error = process_option(opt, scope, OPT_LOCAL, &print);
+			errors += error;
+			if(!print && !error)
+			{
+				errors += process_option(opt, scope, OPT_GLOBAL, &print);
+			}
+		}
+		else
+		{
+			errors += process_option(opt, scope, scope, &print);
+		}
+
+		free(opt);
 	}
-	return err;
+	return errors;
 }
 
-/* Prints options, which value differs from the default one. */
+/* Prints the option if its current value differs from its default value. */
 static void
-print_changed_options(void)
+print_if_changed(opt_t *opt)
 {
-	size_t i;
-	for(i = 0; i < options_count; ++i)
+	if(uses_str_value(opt->type))
 	{
-		opt_t *opt = &options[i];
-
-		if(opt->full != NULL)
-			continue;
-
-		if(opt->type == OPT_STR || opt->type == OPT_STRLIST)
+		if(strcmp(opt->val.str_val, opt->def.str_val) == 0)
 		{
-			if(strcmp(opt->val.str_val, opt->def.str_val) == 0)
-				continue;
+			return;
 		}
-		else if(opt->val.int_val == opt->def.int_val)
-		{
-			continue;
-		}
-
-		set_print(&options[i]);
 	}
+	else if(opt->val.int_val == opt->def.int_val)
+	{
+		return;
+	}
+
+	(void)set_print(opt);
 }
 
-/* Processes one :set statement.  Returns zero on success, otherwize non-zero is
+/* Processes one :set statement.  Returns zero on success, otherwise non-zero is
  * returned. */
 static int
-process_option(const char arg[])
+process_option(const char arg[], OPT_SCOPE real_scope, OPT_SCOPE scope,
+		int *print)
 {
 	char optname[OPTION_NAME_MAX + 1];
 	int err;
 	const char *suffix;
 	opt_t *opt;
 
+	*print = 0;
+
 	suffix = skip_alphas(arg);
 
 	copy_str(optname, suffix - arg + 1, arg);
 
 	if(strcmp(optname, "all") == 0)
 	{
-		return handle_all_pseudo(arg, suffix);
+		return handle_all_pseudo(arg, suffix, real_scope, print);
 	}
 
-	opt = get_option(optname);
+	opt = get_option(optname, scope);
 	if(opt == NULL)
 	{
-		vle_tb_append_linef(vle_err, "%s: %s", "Unknown option", arg);
+		/* Silently return on attempts to set global-only option with :setlocal. */
+		if(scope == OPT_LOCAL && get_option(optname, OPT_GLOBAL) != NULL)
+		{
+			return 0;
+		}
+
+		if(optname[0] == '\0')
+		{
+			vle_tb_append_linef(vle_err, "%s: %s", "No valid option name in", arg);
+		}
+		else
+		{
+			vle_tb_append_linef(vle_err, "%s: %s", "Unknown option", optname);
+		}
 		return 1;
 	}
 
 	err = 0;
 	if(*suffix == '\0')
 	{
-		opt_t *o = find_option(optname);
+		opt_t *o = find_option(optname, scope);
 		if(o != NULL)
 		{
 			if(o->type == OPT_BOOL)
 				err = set_on(opt);
 			else
+			{
+				*print = 1;
 				err = set_print(o);
+			}
 		}
 		else if(strncmp(optname, "no", 2) == 0)
 		{
@@ -363,7 +436,10 @@ process_option(const char arg[])
 		if(*suffix == '!')
 			err = set_inv(opt);
 		else if(*suffix == '?')
+		{
+			*print = 1;
 			err = set_print(opt);
+		}
 		else
 			err = set_reset(opt);
 	}
@@ -396,17 +472,21 @@ process_option(const char arg[])
 }
 
 /* Handles "all" pseudo-option.  Actual action is determined by the suffix.
- * Returns zero on success, otherwize non-zero is returned. */
+ * Returns zero on success, otherwise non-zero is returned. */
 static int
-handle_all_pseudo(const char arg[], const char suffix[])
+handle_all_pseudo(const char arg[], const char suffix[], OPT_SCOPE scope,
+		int *print)
 {
+	*print = 0;
+
 	switch(*suffix)
 	{
 		case '\0':
-			print_options();
+			*print = 1;
+			enum_options(scope, &set_print_void);
 			return 0;
 		case '&':
-			reset_options_to_default();
+			reset_options(scope);
 			return 0;
 
 		default:
@@ -415,53 +495,134 @@ handle_all_pseudo(const char arg[], const char suffix[])
 	}
 }
 
-/* Prints values of all options. */
+/* Wrapper for set_print() to make its prototype compatible with
+ * enum_options(). */
 static void
-print_options(void)
+set_print_void(opt_t *opt)
+{
+	(void)set_print(opt);
+}
+
+/* Enumerates options of specified scope calling the traverser for each one. */
+static void
+enum_options(OPT_SCOPE scope, opt_traverse traverser)
 {
 	size_t i;
-	for(i = 0U; i < options_count; ++i)
+
+	if(scope == OPT_ANY)
 	{
-		if(options[i].full == NULL)
+		enum_unique_options(traverser);
+		return;
+	}
+
+	for(i = 0U; i < option_count; ++i)
+	{
+		if(options[i].full == NULL && options[i].scope == scope)
 		{
-			set_print(&options[i]);
+			traverser(&options[i]);
 		}
 	}
 }
 
+/* Enumerates options preferring local options to global ones on calling the
+ * traverser. */
+static void
+enum_unique_options(opt_traverse traverser)
+{
+	size_t g = 0U, l = 0U;
+
+	do
+	{
+		while(g < option_count &&
+				(options[g].scope != OPT_GLOBAL || options[g].full != NULL))
+		{
+			++g;
+		}
+		while(l < option_count &&
+				(options[l].scope != OPT_LOCAL || options[l].full != NULL))
+		{
+			++l;
+		}
+
+		if(l < option_count && g < option_count)
+		{
+			const int cmp = strcmp(options[l].name, options[g].name);
+			if(cmp == 0)
+			{
+				traverser(&options[l]);
+				++l;
+				++g;
+			}
+			else
+			{
+				traverser(&options[(cmp < 0) ? l++ : g++]);
+			}
+		}
+		else if(l < option_count)
+		{
+			traverser(&options[l++]);
+		}
+		else if(g < option_count)
+		{
+			traverser(&options[g++]);
+		}
+	}
+	while(g < option_count || l < option_count);
+}
+
 /* Finds option by its name.  Understands "no" and "inv" boolean option
  * prefixes.  Returns NULL for unknown option names. */
 static opt_t *
-get_option(const char option[])
+get_option(const char option[], OPT_SCOPE scope)
 {
 	opt_t *opt;
 
-	opt = find_option(option);
+	opt = find_option(option, scope);
 	if(opt != NULL)
 		return opt;
 
 	if(strncmp(option, "no", 2) == 0)
-		return find_option(option + 2);
+		return find_option(option + 2, scope);
 	else if(strncmp(option, "inv", 3) == 0)
-		return find_option(option + 3);
+		return find_option(option + 3, scope);
 	else
 		return NULL;
 }
 
 opt_t *
-find_option(const char option[])
+find_option(const char option[], OPT_SCOPE scope)
 {
-	int l = 0, u = options_count - 1;
+	int l = 0, u = option_count - 1;
 	while(l <= u)
 	{
-		int i = (l + u)/2;
-		int comp = strcmp(option, options[i].name);
+		const size_t i = l + (u - l)/2;
+		const int comp = strcmp(option, options[i].name);
 		if(comp == 0)
 		{
-			if(options[i].full == NULL)
-				return &options[i];
+			opt_t *opt;
+			opt_t *opts[2];
+
+			opts[0] = &options[i];
+
+			if(i > 0 && strcmp(option, options[i - 1].name) == 0)
+			{
+				opts[1] = &options[i - 1];
+			}
+			else if(i < option_count - 1 && strcmp(option, options[i + 1].name) == 0)
+			{
+				opts[1] = &options[i + 1];
+			}
 			else
-				return find_option(options[i].full);
+			{
+				opts[1] = NULL;
+			}
+
+			opt = pick_option(opts, scope);
+			if(opt != NULL && opt->full != NULL)
+			{
+				return find_option(opt->full, scope);
+			}
+			return opt;
 		}
 		else if(comp < 0)
 		{
@@ -475,6 +636,42 @@ find_option(const char option[])
 	return NULL;
 }
 
+/* Of two options (global and local, when second exists) picks the one that
+ * matches the scope in a best way (exact match is preferred as well as local
+ * option over global one).  Returns picked option or NULL on no match. */
+static opt_t *
+pick_option(opt_t *opts[2], OPT_SCOPE scope)
+{
+	if(opts[1] == NULL)
+	{
+		if(option_matches(opts[0], scope))
+		{
+			return opts[0];
+		}
+		return NULL;
+	}
+
+	if(opts[0]->scope == OPT_LOCAL)
+	{
+		opt_t *const opt = opts[0];
+		opts[0] = opts[1];
+		opts[1] = opt;
+	}
+
+	switch(scope)
+	{
+		case OPT_GLOBAL:
+			return opts[0];
+		case OPT_LOCAL:
+		case OPT_ANY:
+			return opts[1];
+
+		default:
+			assert(0 && "Unhandled scope?");
+			return opts[1];
+	}
+}
+
 /* Turns boolean option on.  Returns non-zero in case of error. */
 static int
 set_on(opt_t *opt)
@@ -520,9 +717,7 @@ set_inv(opt_t *opt)
 	return 0;
 }
 
-/* Assigns value to an option of all kinds except boolean.  Returns non-zero in
- * case of error. */
-static int
+int
 set_set(opt_t *opt, const char value[])
 {
 	if(opt->type == OPT_BOOL)
@@ -559,6 +754,11 @@ set_set(opt_t *opt, const char value[])
 	{
 		char *end;
 		int int_val = strtoll(value, &end, 10);
+		if(*end != '\0')
+		{
+			vle_tb_append_linef(vle_err, "Not a number: %s", value);
+			return -1;
+		}
 		if(opt->val.int_val != int_val)
 		{
 			opt->val.int_val = int_val;
@@ -567,7 +767,7 @@ set_set(opt_t *opt, const char value[])
 	}
 	else if(opt->type == OPT_CHARSET)
 	{
-		const size_t valid_len = strspn(value, *opt->vals);
+		const size_t valid_len = strspn(value, *opt->vals[0]);
 		if(valid_len != strlen(value))
 		{
 			vle_tb_append_linef(vle_err, "Illegal character: <%c>", value[valid_len]);
@@ -590,8 +790,7 @@ set_set(opt_t *opt, const char value[])
 static int
 set_reset(opt_t *opt)
 {
-	if(opt->type == OPT_STR || opt->type == OPT_STRLIST ||
-			opt->type == OPT_CHARSET)
+	if(uses_str_value(opt->type))
 	{
 		if(replace_if_changed(&opt->val.str_val, opt->def.str_val))
 		{
@@ -606,12 +805,19 @@ set_reset(opt_t *opt)
 	return 0;
 }
 
-/* Adds value(s) to the option (+= operator).  Returns zero on success. */
+/* Checks whether given option type allocates memory for strings on heap.
+ * Returns non-zero if so, otherwise zero is returned. */
 static int
+uses_str_value(OPT_TYPE type)
+{
+	return (type == OPT_STR || type == OPT_STRLIST || type == OPT_CHARSET);
+}
+
+int
 set_add(opt_t *opt, const char value[])
 {
 	if(opt->type != OPT_INT && opt->type != OPT_SET && opt->type != OPT_STRLIST &&
-			opt->type != OPT_CHARSET)
+			opt->type != OPT_CHARSET && opt->type != OPT_STR)
 		return -1;
 
 	if(opt->type == OPT_INT)
@@ -637,7 +843,7 @@ set_add(opt_t *opt, const char value[])
 	}
 	else if(opt->type == OPT_CHARSET)
 	{
-		const size_t valid_len = strspn(value, *opt->vals);
+		const size_t valid_len = strspn(value, *opt->vals[0]);
 		if(valid_len != strlen(value))
 		{
 			vle_tb_append_linef(vle_err, "Illegal character: <%c>", value[valid_len]);
@@ -650,16 +856,26 @@ set_add(opt_t *opt, const char value[])
 	}
 	else if(*value != '\0')
 	{
-		opt->val.str_val = str_add(opt->val.str_val, value);
+		if(opt->type == OPT_STR)
+		{
+			size_t len = strlen(opt->val.str_val);
+			if(strappend(&opt->val.str_val, &len, value) != 0)
+			{
+				vle_tb_append_line(vle_err, "Memory allocation error");
+				return -1;
+			}
+		}
+		else
+		{
+			opt->val.str_val = str_add(opt->val.str_val, value);
+		}
 		notify_option_update(opt, OP_MODIFIED, opt->val);
 	}
 
 	return 0;
 }
 
-/* Removes value(s) from the option (-= operator).  Returns non-zero on
- * success. */
-static int
+int
 set_remove(opt_t *opt, const char value[])
 {
 	if(opt->type != OPT_INT && opt->type != OPT_SET && opt->type != OPT_STRLIST &&
@@ -737,21 +953,21 @@ notify_option_update(opt_t *opt, OPT_OP op, optval_t val)
 static int
 set_op(opt_t *opt, const char value[], SetOp op)
 {
-	char val[1024];
 	const char *p;
 	const int old_val = opt->val.set_items;
 	int new_val = (op == SO_SET) ? 0 : old_val;
 
 	while(*value != '\0')
 	{
+		char *val;
 		int i;
-		p = strchr(value, ',');
-		if(p == NULL)
-			p = value + strlen(value);
 
-		copy_str(val, p - value + 1, value);
+		p = until_first(value, ',');
 
+		val = format_str("%.*s", (int)(p - value), value);
 		i = find_val(opt, val);
+		free(val);
+
 		if(i != -1)
 		{
 			if(op == SO_SET || op == SO_ADD)
@@ -959,36 +1175,42 @@ str_remove(char old[], const char value[])
 	return changed;
 }
 
-/* Returns index of the value in the list of options value, or -1 if value name
+/* Returns index of the value in the list of option values, or -1 if value name
  * is wrong. */
 static int
 find_val(const opt_t *opt, const char value[])
 {
-	return string_array_pos((char **)opt->vals, opt->val_count, value);
+	int i;
+	for(i = 0; i < opt->val_count; ++i)
+	{
+		if(strcmp(opt->vals[i][0], value) == 0)
+		{
+			return i;
+		}
+	}
+	return -1;
 }
 
 /* Prints value of the given option to text buffer. */
 static int
 set_print(const opt_t *opt)
 {
-	char buf[1024];
 	if(opt->type == OPT_BOOL)
 	{
-		snprintf(buf, sizeof(buf), "%s%s",
-				opt->val.bool_val ? "  " : "no", opt->name);
+		vle_tb_append_linef(vle_err, "%s%s", opt->val.bool_val ? "  " : "no",
+				opt->name);
 	}
 	else
 	{
-		snprintf(buf, sizeof(buf), "  %s=%s", opt->name, get_value(opt));
+		vle_tb_append_linef(vle_err, "  %s=%s", opt->name, get_value(opt));
 	}
-	vle_tb_append_line(vle_err, buf);
 	return 0;
 }
 
 const char *
-get_option_value(const char name[])
+get_option_value(const char name[], OPT_SCOPE scope)
 {
-	opt_t *opt = find_option(name);
+	opt_t *opt = find_option(name, scope);
 	assert(opt != NULL && "Wrong option name.");
 	if(opt == NULL)
 	{
@@ -1017,20 +1239,22 @@ get_value(const opt_t *opt)
 	}
 	else if(opt->type == OPT_ENUM)
 	{
-		copy_str(buf, sizeof(buf), opt->vals[opt->val.enum_item]);
+		copy_str(buf, sizeof(buf), opt->vals[opt->val.enum_item][0]);
 	}
 	else if(opt->type == OPT_SET)
 	{
 		int i, first = 1;
 		buf[0] = '\0';
-		for(i = 0; i < opt->val_count; i++)
+		for(i = 0; i < opt->val_count; ++i)
+		{
 			if(opt->val.set_items & (1 << i))
 			{
 				char *p = buf + strlen(buf);
 				snprintf(p, sizeof(buf) - (p - buf), "%s%s",
-						first ? "" : ",", opt->vals[i]);
+						first ? "" : ",", opt->vals[i][0]);
 				first = 0;
 			}
+		}
 	}
 	else
 	{
@@ -1040,36 +1264,44 @@ get_value(const opt_t *opt)
 }
 
 void
-complete_options(const char args[], const char **start)
+complete_options(const char args[], const char **start, OPT_SCOPE scope)
 {
-	char buf[1024];
 	int bool_only;
 	int is_value_completion;
 	opt_t *opt = NULL;
-	char * p;
+	char *last_opt = strdup("");
+	char *p;
 
 	/* Skip all options except the last one. */
 	*start = args;
-	buf[0] = '\0';
 	while(*args != '\0')
 	{
-		*start = args;
-		args = extract_option(args, buf, 0);
-		if(args == NULL)
+		free(last_opt);
+		last_opt = extract_option(&args, 0);
+
+		if(args == NULL || (*args == '\0' && is_null_or_empty(last_opt)))
 		{
-			vle_compl_add_match(buf);
+			/* Just exit on error or reaching a comment. */
+			if(last_opt != NULL)
+			{
+				vle_compl_put_match(last_opt, "");
+			}
+			else
+			{
+				vle_compl_add_match("", "");
+			}
 			return;
 		}
 	}
-	if(strlen(buf) != (size_t)(args - *start))
+	if(strlen(last_opt) != (size_t)(args - *start))
 	{
 		*start = args;
-		buf[0] = '\0';
+		last_opt[0] = '\0';
 	}
 
 	bool_only = 0;
 
-	p = skip_alphas(buf);
+	p = skip_alphas(last_opt);
 	is_value_completion =  (p[0] == '=' || p[0] == ':');
 	is_value_completion |= (p[0] == '-' && p[1] == '=');
 	is_value_completion |= (p[0] == '+' && p[1] == '=');
@@ -1078,34 +1310,34 @@ complete_options(const char args[], const char **start)
 	{
 		char t = *p;
 		*p = '\0';
-		opt = get_option(buf);
+		opt = get_option(last_opt, scope);
 		*p++ = t;
 		if(t != '=' && t != ':')
 			p++;
 		if(opt != NULL && (opt->type == OPT_SET || opt->type == OPT_STRLIST))
 		{
-			char *t = strrchr(buf, ',');
+			char *t = strrchr(last_opt, ',');
 			if(t != NULL)
 				p = t + 1;
 		}
-		*start += p - buf;
+		*start += p - last_opt;
 	}
-	else if(strncmp(buf, "no", 2) == 0)
+	else if(strncmp(last_opt, "no", 2) == 0)
 	{
 		*start += 2;
-		memmove(buf, buf + 2, strlen(buf) - 2 + 1);
+		memmove(last_opt, last_opt + 2, strlen(last_opt) - 2 + 1);
 		bool_only = 1;
 	}
-	else if(strncmp(buf, "inv", 3) == 0)
+	else if(strncmp(last_opt, "inv", 3) == 0)
 	{
 		*start += 3;
-		memmove(buf, buf + 3, strlen(buf) - 3 + 1);
+		memmove(last_opt, last_opt + 3, strlen(last_opt) - 3 + 1);
 		bool_only = 1;
 	}
 
 	if(!is_value_completion)
 	{
-		complete_option_name(buf, bool_only, !bool_only);
+		complete_option_name(last_opt, bool_only, !bool_only, scope);
 	}
 	else if(opt != NULL)
 	{
@@ -1115,7 +1347,7 @@ complete_options(const char args[], const char **start)
 		}
 		else if(*p == '\0' && opt->type != OPT_BOOL)
 		{
-			vle_compl_put_match(escape_chars(get_value(opt), " |"));
+			vle_compl_put_match(escape_chars(get_value(opt), " |"), "");
 		}
 	}
 
@@ -1126,32 +1358,54 @@ complete_options(const char args[], const char **start)
 	}
 	else
 	{
-		vle_compl_add_last_match(is_value_completion ? p : buf);
+		vle_compl_add_last_match(is_value_completion ? p : last_opt);
 	}
+
+	free(last_opt);
 }
 
-/* Extracts next option from option list.  Returns NULL on error and next call
- * start position on success. */
-static const char *
-extract_option(const char args[], char buf[], int replace)
+/* Extracts next option from option list.  Returns NULL on error and option
+ * string on success.  On reaching trailing comment, empty string is returned.
+ * *argsp is advanced according to parsing. */
+static char *
+extract_option(const char **argsp, int replace)
 {
 	int quote = 0;
 	int slash = 0;
 
+	const char *args = *argsp;
+	char *opt = NULL;
+	size_t opt_len = 0U;
+
+	if(*args == '\"' && strchr(args + 1, '"') == NULL)
+	{
+		/* This is a comment. */
+		*argsp = args + strlen(args);
+		return strdup("");
+	}
+
 	while(*args != '\0')
 	{
 		if(slash == 1)
 		{
-			*buf++ = *args++;
+			if(strappendch(&opt, &opt_len, *args++) != 0)
+			{
+				goto error;
+			}
+
 			slash = 0;
 		}
 		else if(*args == '\\')
 		{
 			slash = 1;
 			if(replace && (quote == 0 || quote == 2))
-				args++;
-			else
-				*buf++ = *args++;
+			{
+				++args;
+			}
+			else if(strappendch(&opt, &opt_len, *args++) != 0)
+			{
+				goto error;
+			}
 		}
 		else if(*args == ' ' && quote == 0)
 		{
@@ -1159,20 +1413,31 @@ extract_option(const char args[], char buf[], int replace)
 			args = skip_whitespace(args);
 			if(char_is_one_of(ENDING_CHARS, *args))
 			{
-				*buf++ = *args++;
+				if(strappendch(&opt, &opt_len, *args++) != 0)
+				{
+					goto error;
+				}
 				if(*args != '\0' && !isspace(*args))
 				{
-					buf -= args - p - 1;
-					strcpy(buf, p);
-					return NULL;
+					opt_len -= args - p - 1U;
+					if(strappend(&opt, &opt_len, p) != 0)
+					{
+						goto error;
+					}
+					*argsp = NULL;
+					return opt;
 				}
 				args = skip_whitespace(args);
 			}
 			else if(char_is_one_of(MIDDLE_CHARS, *args))
 			{
-					while(*args != '\0' && !isspace(*args))
-						*buf++ = *args++;
-					*buf = '\0';
+				while(*args != '\0' && !isspace(*args))
+				{
+					if(strappendch(&opt, &opt_len, *args++) != 0)
+					{
+						goto error;
+					}
+				}
 			}
 			break;
 		}
@@ -1196,13 +1461,25 @@ extract_option(const char args[], char buf[], int replace)
 			quote = 0;
 			args++;
 		}
-		else
+		else if(strappendch(&opt, &opt_len, *args++) != 0)
 		{
-			*buf++ = *args++;
+			goto error;
 		}
 	}
-	*buf = '\0';
-	return args;
+
+	if(quote != 0)
+	{
+		/* Probably an unmatched quote. */
+		goto error;
+	}
+
+	*argsp = args;
+	return opt;
+
+error:
+	free(opt);
+	*argsp = NULL;
+	return NULL;
 }
 
 /* Returns pointer to the first non-alpha character in the string. */
@@ -1215,40 +1492,57 @@ skip_alphas(const char str[])
 }
 
 void
-complete_real_option_names(const char beginning[])
+complete_real_option_names(const char beginning[], OPT_SCOPE scope)
 {
-	complete_option_name(beginning, 0, 0);
+	complete_option_name(beginning, 0, 0, scope);
+	vle_compl_finish_group();
+	vle_compl_add_last_match(beginning);
 }
 
 /* Completes name of an option.  The pseudo parameter controls whether pseudo
  * options should be enumerated (e.g. "all"). */
 static void
-complete_option_name(const char buf[], int bool_only, int pseudo)
+complete_option_name(const char buf[], int bool_only, int pseudo,
+		OPT_SCOPE scope)
 {
 	const size_t len = strlen(buf);
 	size_t i;
 
-	if(pseudo && strncmp(buf, "all", len) == 0)
+	assert(scope != OPT_ANY && "Won't complete for this scope.");
+	if(scope == OPT_ANY)
 	{
-		vle_compl_add_match("all");
+		return;
 	}
 
-	for(i = 0U; i < options_count; ++i)
+	if(pseudo && strncmp(buf, "all", len) == 0)
+	{
+		vle_compl_add_match("all", "pseudo-option that groups others");
+	}
+
+	for(i = 0U; i < option_count; ++i)
 	{
 		opt_t *const opt = &options[i];
 
-		if(bool_only && opt->type != OPT_BOOL)
+		if((bool_only && opt->type != OPT_BOOL) || !option_matches(opt, scope))
 		{
 			continue;
 		}
 
 		if(strncmp(buf, opt->name, len) == 0)
 		{
-			vle_compl_add_match(get_opt_full_name(opt));
+			vle_compl_add_match(get_opt_full_name(opt), opt->descr);
 		}
 	}
 }
 
+/* Checks whether option matches the scope.  Returns non-zero if so, otherwise
+ * zero is returned. */
+static int
+option_matches(const opt_t *opt, OPT_SCOPE scope)
+{
+	return opt->scope == scope || scope == OPT_ANY;
+}
+
 /* Gets full name of the option.  Returns pointer to the name. */
 static const char *
 get_opt_full_name(opt_t *const opt)
@@ -1280,10 +1574,12 @@ complete_list_value(const opt_t *opt, const char beginning[])
 
 	len = strlen(beginning);
 
-	for(i = 0; i < opt->val_count; i++)
+	for(i = 0; i < opt->val_count; ++i)
 	{
-		if(strncmp(beginning, opt->vals[i], len) == 0)
-			vle_compl_add_match(opt->vals[i]);
+		if(strncmp(beginning, opt->vals[i][0], len) == 0)
+		{
+			vle_compl_add_match(opt->vals[i][0], opt->vals[i][1]);
+		}
 	}
 
 	return 0;
@@ -1294,15 +1590,12 @@ complete_list_value(const opt_t *opt, const char beginning[])
 static int
 complete_char_value(const opt_t *opt, const char beginning[])
 {
-	const char *vals = *opt->vals;
 	int i;
-
-	for(i = 0; i < opt->val_count; i++)
+	for(i = 1; i < opt->val_count; ++i)
 	{
-		if(strchr(beginning, vals[i]) == NULL)
+		if(strchr(beginning, *opt->vals[i][0]) == NULL)
 		{
-			const char char_str[] = { vals[i], '\0' };
-			vle_compl_add_match(char_str);
+			vle_compl_add_match(opt->vals[i][0], opt->vals[i][1]);
 		}
 	}
 
@@ -1310,4 +1603,4 @@ complete_char_value(const opt_t *opt, const char beginning[])
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
