@@ -22,12 +22,14 @@
 #include <ctype.h>
 #include <stddef.h> /* size_t */
 #include <stdio.h>
-#include <stdlib.h> /* free() */
+#include <stdlib.h> /* free() realloc() */
 #include <string.h>
 
+#include "../compat/reallocarray.h"
 #include "../utils/env.h"
 #include "../utils/macros.h"
 #include "../utils/str.h"
+#include "private/options.h"
 #include "completion.h"
 #include "parsing.h"
 #include "text_buffer.h"
@@ -36,13 +38,35 @@
 #define VAR_NAME_MAX 64
 #define VAL_LEN_MAX 2048
 
-typedef struct {
+/* Types of supported variables. */
+typedef enum
+{
+	VT_ENVVAR,        /* Environment variable. */
+	VT_ANY_OPTION,    /* Global and local options (if local exists). */
+	VT_GLOBAL_OPTION, /* Global option. */
+	VT_LOCAL_OPTION,  /* Local option. */
+}
+VariableType;
+
+/* Supported operations. */
+typedef enum
+{
+	VO_ASSIGN, /* Assigning a variable (=). */
+	VO_APPEND, /* Appending to a string (.=). */
+	VO_ADD,    /* Adding to numbers or composite values (+=). */
+	VO_SUB,    /* Subtracting from numbers or removing from composites (-=). */
+}
+VariableOperation;
+
+typedef struct
+{
 	char *name;
 	char *val;
 	char *initial;
 	int from_parent;
 	int removed;
-}envvar_t;
+}
+envvar_t;
 
 const char ENV_VAR_NAME_FIRST_CHAR[] = "abcdefghijklmnopqrstuvwxyz"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ_";
@@ -50,9 +74,20 @@ const char ENV_VAR_NAME_CHARS[] = "abcdefghijklmnopqrstuvwxyz"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_";
 
 static void init_var(const char *env);
+static int extract_name(const char **in, VariableType *type, size_t buf_len,
+		char buf[]);
+static int extract_op(const char **in, VariableOperation *vo);
+static int parse_name(const char **in, const char first[], const char other[],
+		size_t buf_len, char buf[]);
 static void report_parsing_error(ParsingErrors error);
+static int is_valid_op(const char name[], VariableType vt,
+		VariableOperation vo);
+static int perform_op(const char name[], VariableType vt,
+		VariableOperation vo, const char value[]);
 static void append_envvar(const char *name, const char *val);
 static void set_envvar(const char *name, const char *val);
+static int perform_opt_op(const char name[], VariableType vt,
+		VariableOperation vo, const char value[]);
 static envvar_t * get_record(const char *name);
 static char * skip_non_whitespace(const char str[]);
 static envvar_t * find_record(const char *name);
@@ -72,7 +107,7 @@ init_variables(void)
 	if(nvars > 0)
 		clear_variables();
 
-	/* count environment variables */
+	/* Count environment variables. */
 	env_count = 0;
 	while(environ[env_count] != NULL)
 		env_count++;
@@ -81,11 +116,11 @@ init_variables(void)
 	{
 		int i;
 
-		/* allocate memory for environment variables */
-		vars = malloc(sizeof(*vars)*env_count);
-		assert(vars != NULL);
+		/* Allocate memory for environment variables. */
+		vars = reallocarray(NULL, env_count, sizeof(*vars));
+		assert(vars != NULL && "Failed to allocate memory for env vars.");
 
-		/* initialize variable list */
+		/* Initialize variable list. */
 		i = 0;
 		while(environ[i] != NULL)
 		{
@@ -134,7 +169,7 @@ clear_variables(void)
 	size_t i;
 	assert(initialized);
 
-	/* free memory */
+	/* Free memory. */
 	for(i = 0U; i < nvars; ++i)
 	{
 		if(vars[i].name == NULL)
@@ -160,61 +195,30 @@ int
 let_variables(const char *cmd)
 {
 	char name[VAR_NAME_MAX + 1];
-	char *p;
-	int append = 0;
+	int error;
 	var_t res_var;
-	char *str_var;
+	char *str_val;
 	ParsingErrors parsing_error;
+	VariableType type;
+	VariableOperation op;
 
 	assert(initialized);
 
-	/* currently we support only environment variables */
-	if(*cmd != '$')
+	if(extract_name(&cmd, &type, sizeof(name), name) != 0)
 	{
-		vle_tb_append_line(vle_err, "Incorrect variable type");
-		return -1;
+		return 1;
 	}
-	cmd++;
-
-	/* copy variable name */
-	p = name;
-	while(*cmd != '\0' && char_is_one_of(ENV_VAR_NAME_CHARS, *cmd) &&
-			*cmd != '.' && *cmd != '=' && (size_t)(p - name) < sizeof(name) - 1)
-	{
-		if(*cmd != '_' && !isalnum(*cmd))
-		{
-			vle_tb_append_line(vle_err, "Incorrect variable name");
-			return -1;
-		}
-		*p++ = *cmd++;
-	}
-	/* test for empty variable name */
-	if(p == name)
-	{
-		vle_tb_append_linef(vle_err, "%s: %s", "Unsupported variable name",
-				"empty name");
-		return -1;
-	}
-	*p = '\0';
 
 	cmd = skip_whitespace(cmd);
 
-	/* check for dot and skip it */
-	if(*cmd == '.')
+	if(extract_op(&cmd, &op) != 0)
 	{
-		append = 1;
-		cmd++;
-	}
-
-	/* check for equal sign and skip it */
-	if(*cmd != '=')
-	{
-		vle_tb_append_linef(vle_err, "%s: %s", "Incorrect :let statement",
-				"'=' expected");
 		return -1;
 	}
 
-	parsing_error = parse(cmd + 1, &res_var);
+	cmd = skip_whitespace(cmd);
+
+	parsing_error = parse(cmd, &res_var);
 	if(parsing_error != PE_NO_ERROR)
 	{
 		report_parsing_error(parsing_error);
@@ -228,16 +232,132 @@ let_variables(const char *cmd)
 		return -1;
 	}
 
-	/* update environment variable */
-	str_var = var_to_string(res_var);
-	if(append)
-		append_envvar(name, str_var);
-	else
-		set_envvar(name, str_var);
-	free(str_var);
+	if(!is_valid_op(name, type, op))
+	{
+		vle_tb_append_linef(vle_err, "Wrong variable type for this operation");
+		return -1;
+	}
 
+	str_val = var_to_string(res_var);
+
+	error = perform_op(name, type, op, str_val);
+
+	free(str_val);
 	var_free(res_var);
 
+	return error;
+}
+
+/* Extracts name from the string.  Returns zero on success, otherwise non-zero
+ * is returned. */
+static int
+extract_name(const char **in, VariableType *type, size_t buf_len, char buf[])
+{
+	int error;
+
+	/* Copy variable name. */
+	if(**in == '$')
+	{
+		++*in;
+		error = (parse_name(in, ENV_VAR_NAME_FIRST_CHAR, ENV_VAR_NAME_CHARS,
+				buf_len, buf) != 0);
+		*type = VT_ENVVAR;
+	}
+	else if(**in == '&')
+	{
+		++*in;
+
+		*type = VT_ANY_OPTION;
+		if(((*in)[0] == 'l' || (*in)[0] == 'g') && (*in)[1] == ':')
+		{
+			*type = (*in[0] == 'l') ? VT_LOCAL_OPTION : VT_GLOBAL_OPTION;
+			*in += 2;
+		}
+
+		error = (parse_name(in, OPT_NAME_FIRST_CHAR, OPT_NAME_CHARS, buf_len,
+					buf) != 0);
+	}
+	else
+	{
+		/* Currently we support only environment variables and options. */
+		vle_tb_append_line(vle_err, "Incorrect variable type");
+		return -1;
+	}
+	if(error)
+	{
+		vle_tb_append_line(vle_err, "Incorrect variable name");
+		return -1;
+	}
+
+	/* Test for empty variable name. */
+	if(buf[0] == '\0')
+	{
+		vle_tb_append_linef(vle_err, "%s: %s", "Unsupported variable name",
+				"empty name");
+		return -1;
+	}
+
+	return 0;
+}
+
+/* Extracts operation from the string.  Sets operation advancing the
+ * pointer if needed.  Returns zero on success, otherwise non-zero returned. */
+static int
+extract_op(const char **in, VariableOperation *vo)
+{
+	/* Check first operator char and skip it. */
+	if(**in == '.')
+	{
+		++*in;
+		*vo = VO_APPEND;
+	}
+	else if(**in == '+')
+	{
+		*vo = VO_ADD;
+		++*in;
+	}
+	else if(**in == '-')
+	{
+		*vo = VO_SUB;
+		++*in;
+	}
+	else
+	{
+		*vo = VO_ASSIGN;
+	}
+
+	/* Check for equal sign and skip it. */
+	if(**in != '=')
+	{
+		vle_tb_append_linef(vle_err, "%s: '=' expected at %s",
+				"Incorrect :let statement", *in);
+		return 1;
+	}
+	++*in;
+
+	return 0;
+}
+
+/* Parses name of the form `first { other }`.  Returns zero on success,
+ * otherwise non-zero is returned. */
+static int
+parse_name(const char **in, const char first[], const char other[],
+		size_t buf_len, char buf[])
+{
+	if(buf_len == 0UL || !char_is_one_of(first, **in))
+	{
+		return 1;
+	}
+
+	buf[0] = '\0';
+
+	do
+	{
+		strcatch(buf, **in);
+		++*in;
+	}
+	while(--buf_len > 1UL && char_is_one_of(other, **in));
+
 	return 0;
 }
 
@@ -262,9 +382,85 @@ report_parsing_error(ParsingErrors error)
 			vle_tb_append_linef(vle_err, "%s: %s",
 					"Invalid :let expression (missing quote)", get_last_position());
 			break;
+		case PE_INTERNAL:
+			vle_tb_append_line(vle_err, "Internal error");
+			break;
 	}
 }
 
+/* Validates operation on a specific variable type.  Returns non-zero for valid
+ * operation, otherwise zero is returned. */
+static int
+is_valid_op(const char name[], VariableType vt, VariableOperation vo)
+{
+	opt_t *opt;
+
+	if(vt == VT_ENVVAR)
+	{
+		return (vo == VO_ASSIGN || vo == VO_APPEND);
+	}
+
+	opt = find_option(name, OPT_GLOBAL);
+	if(opt == NULL)
+	{
+		/* Let this error be handled somewhere else. */
+		return 1;
+	}
+
+	if(opt->type == OPT_BOOL)
+	{
+		return 0;
+	}
+
+	if(opt->type == OPT_STR)
+	{
+		return (vo == VO_ASSIGN || vo == VO_APPEND);
+	}
+
+	return (vo == VO_ASSIGN || vo == VO_ADD || vo == VO_SUB);
+}
+
+/* Performs operation on a value.  Returns zero on success, otherwise non-zero
+ * is returned. */
+static int
+perform_op(const char name[], VariableType vt, VariableOperation vo,
+		const char value[])
+{
+	if(vt == VT_ENVVAR)
+	{
+		/* Update environment variable. */
+		if(vo == VO_APPEND)
+		{
+			append_envvar(name, value);
+		}
+		else
+		{
+			set_envvar(name, value);
+		}
+		return 0;
+	}
+
+	/* Update an option. */
+
+	if(vt == VT_ANY_OPTION || vt == VT_LOCAL_OPTION)
+	{
+		if(perform_opt_op(name, vt, vo, value) != 0)
+		{
+			return 1;
+		}
+	}
+
+	if(vt == VT_ANY_OPTION || vt == VT_GLOBAL_OPTION)
+	{
+		if(perform_opt_op(name, VT_GLOBAL_OPTION, vo, value) != 0)
+		{
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 static void
 append_envvar(const char *name, const char *val)
 {
@@ -314,7 +510,55 @@ set_envvar(const char *name, const char *val)
 	env_set(name, val);
 }
 
-/* searches for variable and creates new record if it didn't existed */
+/* Performs operation on an option.  Returns zero on success, otherwise non-zero
+ * is returned. */
+static int
+perform_opt_op(const char name[], VariableType vt, VariableOperation vo,
+		const char value[])
+{
+	OPT_SCOPE scope = (vt == VT_ANY_OPTION || vt == VT_LOCAL_OPTION)
+	                ? OPT_LOCAL
+	                : OPT_GLOBAL;
+
+	opt_t *const opt = find_option(name, scope);
+	if(opt == NULL)
+	{
+		if(vt == VT_ANY_OPTION)
+		{
+			return 0;
+		}
+
+		vle_tb_append_linef(vle_err, "Unknown %s option name: %s",
+				(scope == OPT_LOCAL) ? "local" : "global", name);
+		return 1;
+	}
+
+	switch(vo)
+	{
+		case VO_ASSIGN:
+			if(set_set(opt, value) != 0)
+			{
+				return 1;
+			}
+			break;
+		case VO_ADD:
+		case VO_APPEND:
+			if(set_add(opt, value) != 0)
+			{
+				return 1;
+			}
+			break;
+		case VO_SUB:
+			if(set_remove(opt, value) != 0)
+			{
+				return 1;
+			}
+			break;
+	}
+	return 0;
+}
+
+/* Searches for variable and creates new record if it didn't existed. */
 static envvar_t *
 get_record(const char *name)
 {
@@ -369,13 +613,13 @@ unlet_variables(const char *cmd)
 		char *p;
 		int envvar = 1;
 
-		/* check if its environment variable */
+		/* Check if it's environment variable. */
 		if(*cmd != '$')
 			envvar = 0;
 		else
 			cmd++;
 
-		/* copy variable name */
+		/* Copy variable name. */
 		p = name;
 		while(*cmd != '\0' && char_is_one_of(ENV_VAR_NAME_CHARS, *cmd) &&
 				(size_t)(p - name) < sizeof(name) - 1)
@@ -393,7 +637,7 @@ unlet_variables(const char *cmd)
 
 		cmd = skip_whitespace(cmd);
 
-		/* currently we support only environment variables */
+		/* Currently we support only environment variables. */
 		if(!envvar)
 		{
 			vle_tb_append_linef(vle_err, "%s: %s", "Unsupported variable type", name);
@@ -403,7 +647,7 @@ unlet_variables(const char *cmd)
 			continue;
 		}
 
-		/* test for empty variable name */
+		/* Test for empty variable name. */
 		if(name[0] == '\0')
 		{
 			vle_tb_append_linef(vle_err, "%s: %s", "Unsupported variable name",
@@ -480,17 +724,17 @@ complete_variables(const char *cmd, const char **start)
 	size_t len;
 	assert(initialized && "Variables unit is not initialized.");
 
-	/* currently we support only environment variables */
+	/* Currently we support only environment variables. */
 	if(*cmd != '$')
 	{
 		*start = cmd;
-		vle_compl_add_match(cmd);
+		vle_compl_add_match(cmd, "");
 		return;
 	}
-	cmd++;
+	++cmd;
 	*start = cmd;
 
-	/* add all variables that start with given beginning */
+	/* Add all variables that start with given beginning. */
 	len = strlen(cmd);
 	for(i = 0U; i < nvars; ++i)
 	{
@@ -499,11 +743,11 @@ complete_variables(const char *cmd, const char **start)
 		if(vars[i].removed)
 			continue;
 		if(strnoscmp(vars[i].name, cmd, len) == 0)
-			vle_compl_add_match(vars[i].name);
+			vle_compl_add_match(vars[i].name, vars[i].val);
 	}
 	vle_compl_finish_group();
 	vle_compl_add_last_match(cmd);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
