@@ -27,6 +27,7 @@
 
 #include "ui/ui.h"
 #include "utils/test_helpers.h"
+#include "registers.h"
 
 /* Type of contiguous area of file list. */
 typedef enum
@@ -37,7 +38,7 @@ typedef enum
 FileListScope;
 
 /* Type of filter function for zapping list of entries.  Should return non-zero
- * if entry is to be keeped and zero otherwise. */
+ * if entry is to be kept and zero otherwise. */
 typedef int (*zap_filter)(FileView *view, const dir_entry_t *entry, void *arg);
 
 /* Initialization/termination functions. */
@@ -47,13 +48,17 @@ void init_filelists(void);
 /* Reinitializes views. */
 void reset_views(void);
 /* Loads view file list for the first time. */
-void load_initial_directory(FileView *view, const char *dir);
+void load_initial_directory(FileView *view, const char dir[]);
 
 /* Position related functions. */
 
 /* Find index of the file within list of currently visible files of the view.
  * Returns file entry index or -1, if file wasn't found. */
 int find_file_pos_in_list(const FileView *const view, const char file[]);
+/* Find index of the file within list of currently visible files of the view.
+ * Always matches file name and can optionally match directory if dir is not
+ * NULL.  Returns file entry index or -1, if file wasn't found. */
+int flist_find_entry(const FileView *view, const char file[], const char dir[]);
 /* Tries to move cursor by pos_delta positions.  A wrapper for
  * correct_list_pos_on_scroll_up() and correct_list_pos_on_scroll_down()
  * functions. */
@@ -82,6 +87,14 @@ void go_to_start_of_line(FileView *view);
 int get_start_of_line(const FileView *view);
 /* Returns position of the last file in current line. */
 int get_end_of_line(const FileView *view);
+/* Finds position of the next/previous group defined by primary sorting key.
+ * Returns determined position (might point to the last/first entry in corner
+ * cases). */
+int flist_find_group(FileView *view, int next);
+/* Finds position of the next/previous group defined by entries being files or
+ * directories.  Returns determined position (might point to the last/first
+ * entry in corner cases). */
+int flist_find_dir_group(FileView *view, int next);
 
 /* Appearance related functions. */
 
@@ -103,8 +116,10 @@ void navigate_to(FileView *view, const char path[]);
 void navigate_back(FileView *view);
 /* Changes current directory of the view to the dir if it's possible and in case
  * of success reloads filelist of the view and sets its cursor position on the
- * file trying to ensure that it's visible. */
-void navigate_to_file(FileView *view, const char dir[], const char file[]);
+ * file trying to ensure that it's visible.  If preserve_cv is set, tries to do
+ * not reset custom view when possible. */
+void navigate_to_file(FileView *view, const char dir[], const char file[],
+		int preserve_cv);
 /* The directory can either be relative to the current
  * directory - ../
  * or an absolute path - /usr/local/share
@@ -135,11 +150,12 @@ void clean_selected_files(FileView *view);
 void erase_selection(FileView *view);
 /* Inverts selection of files in the view. */
 void invert_selection(FileView *view);
+/* Reselects previously selected entries.  When reg is NULL, saved selection is
+ * restored, otherwise list of files to restore is taken from the register. */
+void flist_sel_restore(FileView *view, reg_t *reg);
 /* Counts number of selected files and writes saves the number in
  * view->selected_files. */
 void recount_selected_files(FileView *view);
-/* Frees memory from list of captured files. */
-void free_file_capture(FileView *view);
 /* Remove dot and regexp filters if it's needed to make file visible.  Returns
  * non-zero if file was found. */
 int ensure_file_is_selected(FileView *view, const char name[]);
@@ -152,22 +168,23 @@ void navigate_backward_in_history(FileView *view);
 /* Changes current directory of the view to next location forward in history, if
  * available. */
 void navigate_forward_in_history(FileView *view);
-void save_view_history(FileView *view, const char *path, const char *file,
+/* Adds new entry to directory history of the view or updates an existing entry.
+ * If path is NULL, current path is used.  If file is NULL current file is used.
+ * If pos is negative, current position is used.  Empty file name signifies
+ * visiting directory, which shouldn't reset name of previously active file in
+ * it. */
+void save_view_history(FileView *view, const char path[], const char file[],
 		int pos);
 int is_in_view_history(FileView *view, const char *path);
 void clean_positions_in_history(FileView *view);
+/* Looks up history in the source to update cursor position in the view. */
+void flist_hist_lookup(FileView *view, const FileView *source);
 
-/* Typed (with trailing slash for directories) file name functions. */
+/* Typed (with trailing slash for directories) file name function. */
 
-/* Gets typed filename (not path, just name) for current entry of the view.
- * Allocates memory, that should be freed by the caller. */
-char * get_typed_current_fpath(const FileView *view);
-/* Gets typed filename (not path, just name) for the entry.  Allocates memory,
- * that should be freed by the caller. */
-char * get_typed_entry_fname(const dir_entry_t *entry);
-/* Gets typed filename (not path, just name).  Allocates memory, that should be
+/* Gets typed path for the entry.  On return allocates memory, that should be
  * freed by the caller. */
-char * get_typed_fname(const char path[]);
+char * get_typed_entry_fpath(const dir_entry_t *entry);
 
 /* Custom file list functions. */
 
@@ -180,9 +197,11 @@ void flist_custom_start(FileView *view, const char title[]);
 void flist_custom_add(FileView *view, const char path[]);
 /* Finishes file list population, handles empty resulting list corner case.
  * Returns zero on success, otherwise (on empty list) non-zero is returned. */
-int flist_custom_finish(FileView *view);
+int flist_custom_finish(FileView *view, int very);
 /* Removes selected files from custom view. */
 void flist_custom_exclude(FileView *view);
+/* Clones list of files from from view to to view. */
+void flist_custom_clone(FileView *to, const FileView *from);
 
 /* Other functions. */
 
@@ -260,6 +279,12 @@ void check_marking(FileView *view, int count, const int indexes[]);
 void mark_files_at(FileView *view, int count, const int indexes[]);
 /* Marks selected files of the view. */
 void mark_selected(FileView *view);
+/* Same as mark_selected() function, but when selection is absent current file
+ * is marked. */
+void mark_selection_or_current(FileView *view);
+/* Counts number of marked files that can be processed (thus excluding parent
+ * directory, which is "..").  Returns the count. */
+int flist_count_marked(FileView *const view);
 /* Removes dead entries (those that refer to non-existing files) or those that
  * do not match local filter from the view.  Returns number of erased
  * entries. */
@@ -269,6 +294,12 @@ int zap_entries(FileView *view, dir_entry_t *entries, int *count,
  * the found entry or NULL. */
 dir_entry_t * entry_from_path(dir_entry_t *entries, int count,
 		const char path[]);
+/* Retrieves number of items in a directory specified by the entry.  Returns the
+ * number, which is zero for files. */
+uint64_t entry_get_nitems(FileView *view, const dir_entry_t *entry);
+/* Calculates number of items at path specified by the entry.  No check for file
+ * type is performed.  Returns the number, which is zero for files. */
+uint64_t entry_calc_nitems(const dir_entry_t *entry);
 /* Replaces all entries of the *entries with copy of with_entries elements. */
 void replace_dir_entries(FileView *view, dir_entry_t **entries, int *count,
 		const dir_entry_t *with_entries, int with_count);
@@ -280,6 +311,17 @@ int add_dir_entry(dir_entry_t **list, size_t *list_size,
 void free_dir_entry(const FileView *view, dir_entry_t *entry);
 /* Adds parent directory entry (..) to filelist. */
 void add_parent_dir(FileView *view);
+/* Loads list of paths (absolute or relative to the path) into custom view.
+ * Exists with error message on failed attempt. */
+void flist_set(FileView *view, const char title[], const char path[],
+		char *lines[], int nlines);
+/* Parses line to extract path and adds it to custom view or does nothing. */
+void flist_add_custom_line(FileView *view, const char line[]);
+/* A more high level version of flist_custom_finish(), which takes care of error
+ * handling and cursor position. */
+void flist_end_custom(FileView *view, int very);
+/* Changes name of a file entry, performing additional required updates. */
+void fentry_rename(FileView *view, dir_entry_t *entry, const char to[]);
 
 TSTATIC_DEFS(
 	TSTATIC void pick_cd_path(FileView *view, const char base_dir[],
@@ -289,4 +331,4 @@ TSTATIC_DEFS(
 #endif /* VIFM__FILELIST_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
