@@ -48,8 +48,25 @@ typedef enum
 }
 CopyMoveLikeOp;
 
+struct response_variant;
+
+/* Callback for returning edited filename. */
+typedef void (*fo_prompt_cb)(const char new_filename[]);
+
+/* Line completion function.  arg is user supplied value, which is passed
+ * through.  Should return completion offset. */
+typedef int (*fo_complete_cmd_func)(const char cmd[], void *arg);
+
+/* Function to request filename editing. */
+typedef void (*line_prompt_func)(const char prompt[], const char filename[],
+		fo_prompt_cb cb, fo_complete_cmd_func complete, int allow_ee);
+
+/* Function to choose an option.  Returns choice. */
+typedef char (*options_prompt_func)(const char title[], const char message[],
+		const struct response_variant *variants);
+
 /* Initializes file operations. */
-void init_fileops(void);
+void init_fileops(line_prompt_func line_func, options_prompt_func options_func);
 
 /* Removes marked files (optionally into trash directory) of the view to
  * specified register.  Returns new value for save_msg flag. */
@@ -59,14 +76,14 @@ int delete_files(FileView *view, int reg, int use_trash);
  * specified register.  Returns new value for save_msg flag. */
 int delete_files_bg(FileView *view, int use_trash);
 
-/* Yanks selected files of the view into register specified by its name via reg
+/* Yanks marked files of the view into register specified by its name via reg
  * parameter.  Returns new value for save_msg. */
 int yank_files(FileView *view, int reg);
 
 /* Renames single file under the cursor. */
 void rename_current_file(FileView *view, int name_only);
 
-/* Renames marked filese using names given in the list of length nlines (or
+/* Renames marked files using names given in the list of length nlines (or
  * filled in by the user, when the list is empty).  Recursively traverses
  * directories in selection when recursive flag is not zero.  Recursive
  * traversal is incompatible with list of names.  Returns new value for
@@ -78,6 +95,8 @@ int rename_files(FileView *view, char **list, int nlines, int recursive);
 int incdec_names(FileView *view, int k);
 
 #ifndef _WIN32
+/* Sets uid and or gid for marked files.  Non-zero u enables setting of uid,
+ * non-zero g of gid. */
 void chown_files(int u, int g, uid_t uid, gid_t gid);
 #endif
 
@@ -87,11 +106,17 @@ void change_group(void);
 
 int change_link(FileView *view);
 
-/* Returns new value for save_msg flag. */
+/* Puts files from specified register into current directory.  Returns new value
+ * for save_msg flag. */
 int put_files(FileView *view, int reg_name, int move);
 
+/* Starts background task that puts files from specified register into current
+ * directory.  Returns new value for save_msg flag. */
+int put_files_bg(FileView *view, int reg_name, int move);
+
 /* Clones marked files in the view.  Returns new value for save_msg flag. */
-int clone_files(FileView *view, char **list, int nlines, int force, int copies);
+int clone_files(FileView *view, char *list[], int nlines, int force,
+		int copies);
 
 /* Whether set of view files can be altered (renamed, deleted, but not added).
  * Returns non-zero if so, otherwise zero is returned. */
@@ -116,7 +141,7 @@ const char * substitute_in_name(const char name[], const char pattern[],
 
 /* Changes case of all letters in names of marked files of the view.  Returns
  * new value for save_msg flag. */
-int change_case(FileView *view, int toupper);
+int change_case(FileView *view, int to_upper);
 
 /* Performs copy/moves-like operation on marked files.  Returns new value for
  * save_msg flag. */
@@ -127,16 +152,18 @@ int cpmv_files(FileView *view, char **list, int nlines, CopyMoveLikeOp op,
  * value for save_msg flag. */
 int cpmv_files_bg(FileView *view, char **list, int nlines, int move, int force);
 
-/* Can modify strings in the names array. */
-void make_dirs(FileView *view, char **names, int count, int create_parent);
+/* Creates directories, possibly including intermediate ones.  Can modify
+ * strings in the names array.  Returns new value for save_msg flag. */
+int make_dirs(FileView *view, char *names[], int count, int create_parent);
 
 int make_files(FileView *view, char **names, int count);
 
 /* Returns new value for save_msg flag. */
 int restore_files(FileView *view);
 
-/* Calculates size of a directory specified by path.  Forcing disables using
- * previously cached values.  Returns the size. */
+/* Calculates size of a directory specified by path possibly using cache of
+ * known sizes.  Forcing disables using previously cached values.  Returns size
+ * of a directory or zero on error. */
 uint64_t calculate_dir_size(const char path[], int force);
 
 /* Initiates background calculation of directory sizes.  Forcing disables using
@@ -160,4 +187,4 @@ TSTATIC_DEFS(
 #endif /* VIFM__FILEOPS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
