@@ -69,8 +69,10 @@ void filters_dir_updated(FileView *view);
 
 /* Sets regular expression of the local filter for the view.  First call of this
  * function initiates filter set process, which should be ended by call to
- * local_filter_accept() or local_filter_cancel(). */
-void local_filter_set(FileView *view, const char filter[]);
+ * local_filter_accept() or local_filter_cancel().  Returns zero if not all
+ * files are filtered out, -1 if filter expression is incorrect and 1 if all
+ * files were filtered. */
+int local_filter_set(FileView *view, const char filter[]);
 
 /* Updates cursor position and top line of the view according to interactive
  * local filter in progress. */
@@ -99,4 +101,4 @@ int local_filter_matches(FileView *view, const dir_entry_t *entry);
 #endif /* VIFM__FILTERING_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
