@@ -24,10 +24,14 @@
 
 #include "iop.h"
 
+#ifndef _WIN32
+#include <sys/ioctl.h> /* ioctl() */
+#endif
 #include <sys/stat.h> /* stat */
 #include <sys/types.h> /* mode_t */
 #include <unistd.h> /* rmdir() symlink() unlink() */
 
+#include <assert.h> /* assert() */
 #include <errno.h> /* EEXIST ENOENT EISDIR errno */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* FILE fpos_t fclose() fgetpos() fread() fseek() fsetpos()
@@ -35,10 +39,10 @@
 #include <stdlib.h> /* free() */
 #include <string.h> /* strchr() strerror() */
 
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../ui/cancellation.h"
 #include "../utils/fs.h"
-#include "../utils/fs_limits.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
 #include "../utils/path.h"
@@ -53,12 +57,15 @@
 /* Amount of data to transfer at once. */
 #define BLOCK_SIZE 32*1024
 
+static int clone_file(int dest_fd, int src_fd);
 #ifdef _WIN32
 static DWORD CALLBACK win_progress_cb(LARGE_INTEGER total,
 		LARGE_INTEGER transferred, LARGE_INTEGER stream_size,
 		LARGE_INTEGER stream_transfered, DWORD stream_num, DWORD reason,
 		HANDLE src_file, HANDLE dst_file, LPVOID param);
 #endif
+static IoErrCbResult sig_err(io_args_t *const args, int *result,
+		const char path[], int error_code, const char msg[]);
 
 int
 iop_mkfile(io_args_t *const args)
@@ -160,27 +167,46 @@ iop_rmfile(io_args_t *const args)
 	size = get_file_size(path);
 
 #ifndef _WIN32
-	result = unlink(path);
-	if(result != 0)
+	do
 	{
-		(void)ioe_errlst_append(&args->result.errors, path, errno, strerror(errno));
+		result = unlink(path);
+		if(result != 0)
+		{
+			if(sig_err(args, &result, path, errno, strerror(errno)) == IO_ECR_RETRY)
+			{
+				continue;
+			}
+		}
+
+		break;
 	}
+	while(1);
 #else
 	{
 		wchar_t *const utf16_path = utf8_to_utf16(path);
 		const DWORD attributes = GetFileAttributesW(utf16_path);
-		if(attributes & FILE_ATTRIBUTE_READONLY)
-		{
-			SetFileAttributesW(utf16_path, attributes & ~FILE_ATTRIBUTE_READONLY);
-		}
-		result = !DeleteFileW(utf16_path);
 
-		if(result)
+		do
 		{
-			/* FIXME: use real system error message here. */
-			(void)ioe_errlst_append(&args->result.errors, path, IO_ERR_UNKNOWN,
-					"Directory removal failed");
+			if(attributes & FILE_ATTRIBUTE_READONLY)
+			{
+				SetFileAttributesW(utf16_path, attributes & ~FILE_ATTRIBUTE_READONLY);
+			}
+			result = (DeleteFileW(utf16_path) == FALSE);
+
+			if(result != 0)
+			{
+				/* FIXME: use real system error message here. */
+				if(sig_err(args, &result, path, IO_ERR_UNKNOWN,
+							"File removal failed") == IO_ECR_RETRY)
+				{
+					continue;
+				}
+			}
+
+			break;
 		}
+		while(1);
 
 		free(utf16_path);
 	}
@@ -201,22 +227,40 @@ iop_rmdir(io_args_t *const args)
 	ioeta_update(args->estim, path, path, 0, 0);
 
 #ifndef _WIN32
-	result = rmdir(path);
-	if(result != 0)
+	do
 	{
-		(void)ioe_errlst_append(&args->result.errors, path, errno, strerror(errno));
+		result = rmdir(path);
+		if(result != 0)
+		{
+			if(sig_err(args, &result, path, errno, strerror(errno)) == IO_ECR_RETRY)
+			{
+				continue;
+			}
+		}
+
+		break;
 	}
+	while(1);
 #else
 	{
 		wchar_t *const utf16_path = utf8_to_utf16(path);
-		result = RemoveDirectoryW(utf16_path) == 0;
 
-		if(result)
+		do
 		{
-			/* FIXME: use real system error message here. */
-			(void)ioe_errlst_append(&args->result.errors, path, IO_ERR_UNKNOWN,
-					"Directory removal failed");
+			result = (RemoveDirectoryW(utf16_path) == FALSE);
+			if(result != 0)
+			{
+				/* FIXME: use real system error message here. */
+				if(sig_err(args, &result, path, IO_ERR_UNKNOWN,
+							"Directory removal failed") == IO_ECR_RETRY)
+				{
+					continue;
+				}
+			}
+
+			break;
 		}
+		while(1);
 
 		free(utf16_path);
 	}
@@ -239,8 +283,10 @@ iop_cp(io_args_t *const args)
 
 	char block[BLOCK_SIZE];
 	FILE *in, *out;
-	size_t nread;
+	/* Suppress possible false-positive compiler warning. */
+	size_t nread = (size_t)-1;
 	int error;
+	int cloned;
 	struct stat src_st;
 	const char *open_mode = "wb";
 
@@ -283,6 +329,11 @@ iop_cp(io_args_t *const args)
 		free(utf16_src);
 		free(utf16_dst);
 
+		if(error == 0)
+		{
+			clone_timestamps(dst, src, NULL);
+		}
+
 		ioeta_update(args->estim, NULL, NULL, 1, 0);
 
 		return error;
@@ -424,7 +475,7 @@ iop_cp(io_args_t *const args)
 	}
 
 	/* Replicate socket or device file without even opening it. */
-	if(S_ISSOCK(st.st_mode) | S_ISBLK(st.st_mode) || S_ISCHR(st.st_mode))
+	if(S_ISSOCK(st.st_mode) || S_ISBLK(st.st_mode) || S_ISCHR(st.st_mode))
 	{
 		if(mknod(dst, st.st_mode & (S_IFMT | 07777), st.st_rdev) != 0)
 		{
@@ -449,6 +500,7 @@ iop_cp(io_args_t *const args)
 	}
 
 	error = 0;
+	cloned = 0;
 
 	if(crs == IO_CRS_APPEND_TO_FILES)
 	{
@@ -463,10 +515,17 @@ iop_cp(io_args_t *const args)
 			ioeta_update(args->estim, NULL, NULL, 0, get_file_size(dst));
 		}
 	}
+	else if(args->arg4.fast_file_cloning)
+	{
+		if(clone_file(fileno(out), fileno(in)) == 0)
+		{
+			cloned = 1;
+		}
+	}
 
 	/* TODO: use sendfile() if platform supports it. */
 
-	while((nread = fread(&block, 1, sizeof(block), in)) != 0U)
+	while(!cloned && (nread = fread(&block, 1, sizeof(block), in)) != 0U)
 	{
 		if(cancellable && ui_cancellation_requested())
 		{
@@ -484,7 +543,7 @@ iop_cp(io_args_t *const args)
 
 		ioeta_update(args->estim, NULL, NULL, 0, nread);
 	}
-	if(nread == 0U && !feof(in) && ferror(in))
+	if(!cloned && nread == 0U && !feof(in) && ferror(in))
 	{
 		(void)ioe_errlst_append(&args->result.errors, src, errno, strerror(errno));
 	}
@@ -508,11 +567,34 @@ iop_cp(io_args_t *const args)
 		}
 	}
 
+	if(error == 0)
+	{
+		clone_timestamps(dst, src, &st);
+	}
+
 	ioeta_update(args->estim, NULL, NULL, 1, 0);
 
 	return error;
 }
 
+/* Try to clone file fast on btrfs.  Returns 0 on success, otherwise non-zero is
+ * returned. */
+static int
+clone_file(int dst, int src)
+{
+#ifdef __linux__
+#undef BTRFS_IOCTL_MAGIC
+#define BTRFS_IOCTL_MAGIC 0x94
+#undef BTRFS_IOC_CLONE
+#define BTRFS_IOC_CLONE _IOW(BTRFS_IOCTL_MAGIC, 9, int)
+	return ioctl(dst, BTRFS_IOC_CLONE, src);
+#else
+	(void)dst;
+	(void)src;
+	return -1;
+#endif
+}
+
 #ifdef _WIN32
 
 static DWORD CALLBACK win_progress_cb(LARGE_INTEGER total,
@@ -565,12 +647,6 @@ iop_ln(io_args_t *const args)
 
 	int result;
 
-#ifdef _WIN32
-	char cmd[6 + PATH_MAX*2 + 1];
-	char *escaped_path, *escaped_target;
-	char base_dir[PATH_MAX + 2];
-#endif
-
 #ifndef _WIN32
 	result = symlink(path, target);
 	if(result != 0 && errno == EEXIST && overwrite && is_symlink(target))
@@ -597,6 +673,10 @@ iop_ln(io_args_t *const args)
 				strerror(errno));
 	}
 #else
+	char cmd[6 + PATH_MAX*2 + 1];
+	char *escaped_path, *escaped_target;
+	char base_dir[PATH_MAX + 2];
+
 	if(!overwrite && path_exists(target, DEREF))
 	{
 		(void)ioe_errlst_append(&args->result.errors, target, EEXIST,
@@ -611,8 +691,8 @@ iop_ln(io_args_t *const args)
 		return -1;
 	}
 
-	escaped_path = escape_filename(path, 0);
-	escaped_target = escape_filename(target, 0);
+	escaped_path = shell_like_escape(path, 0);
+	escaped_target = shell_like_escape(target, 0);
 	if(escaped_path == NULL || escaped_target == NULL)
 	{
 		(void)ioe_errlst_append(&args->result.errors, target, IO_ERR_UNKNOWN,
@@ -649,5 +729,36 @@ iop_ln(io_args_t *const args)
 	return result;
 }
 
+/* Error handler that calls external code to figure out what to do and also logs
+ * the error when needed.  Returns the response. */
+static IoErrCbResult
+sig_err(io_args_t *const args, int *result, const char path[], int error_code,
+		const char msg[])
+{
+	ioerr_cb errors = args->result.errors_cb;
+
+	ioe_err_t err = {
+		.path = (char*)path, .error_code = errno, .msg = (char*)msg,
+	};
+
+	assert(*result != 0 && "The function should be called on error path only.");
+
+	switch((errors == NULL) ? IO_ECR_BREAK : errors(args, &err))
+	{
+		case IO_ECR_RETRY:
+			return IO_ECR_RETRY;
+		case IO_ECR_IGNORE:
+			*result = 0;
+			return IO_ECR_IGNORE;
+		case IO_ECR_BREAK:
+			(void)ioe_errlst_append(&args->result.errors, path, errno, msg);
+			return IO_ECR_BREAK;
+
+		default:
+			assert(0 && "Unknown error handling result.");
+			return IO_ECR_BREAK;
+	}
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
