@@ -23,17 +23,18 @@
 
 #include <errno.h> /* EEXIST EISDIR ENOTEMPTY EXDEV errno */
 #include <stddef.h> /* NULL */
-#include <stdio.h> /* removee() snprintf() */
+#include <stdio.h> /* remove() snprintf() */
 #include <stdlib.h> /* free() */
 #include <string.h> /* strerror() strlen() */
 
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../ui/cancellation.h"
 #include "../utils/fs.h"
-#include "../utils/fs_limits.h"
 #include "../utils/log.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
+#include "../utils/utils.h"
 #include "../background.h"
 #include "private/ioe.h"
 #include "private/ioeta.h"
@@ -88,7 +89,7 @@ rm_visitor(const char full_path[], VisitAction action, void *param)
 					.result = rm_args->result,
 				};
 
-				result = iop_rmfile(&args);
+				result = (iop_rmfile(&args) == 0) ? VR_OK : VR_ERROR;
 				rm_args->result = args.result;
 				break;
 			}
@@ -103,7 +104,7 @@ rm_visitor(const char full_path[], VisitAction action, void *param)
 					.result = rm_args->result,
 				};
 
-				result = iop_rmdir(&args);
+				result = (iop_rmdir(&args) == 0) ? VR_OK : VR_ERROR;
 				rm_args->result = args.result;
 				break;
 			}
@@ -232,6 +233,11 @@ ior_mv(io_args_t *const args)
 		case EISDIR:
 		case ENOTEMPTY:
 		case EEXIST:
+#ifdef _WIN32
+		/* For MXE builds running in Wine. */
+		case EPERM:
+		case EACCES:
+#endif
 			if(crs == IO_CRS_REPLACE_ALL)
 			{
 				int error;
@@ -367,10 +373,6 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 				result = (iop_mkdir(&args) == 0) ? VR_OK : VR_ERROR;
 				cp_args->result = args.result;
 			}
-			else if(cp)
-			{
-				result = VR_SKIP_DIR_LEAVE;
-			}
 			break;
 		case VA_FILE:
 			{
@@ -378,6 +380,8 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 					.arg1.src = full_path,
 					.arg2.dst = dst_full_path,
 					.arg3.crs = cp_args->arg3.crs,
+					/* It's safe to always use fast file cloning on moving files. */
+					.arg4.fast_file_cloning = cp ? cp_args->arg4.fast_file_cloning : 1,
 
 					.cancellable = cp_args->cancellable,
 					.confirm = cp_args->confirm,
@@ -417,6 +421,7 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 						(void)ioe_errlst_append(&cp_args->result.errors, dst_full_path,
 								errno, strerror(errno));
 					}
+					clone_timestamps(dst_full_path, full_path, &st);
 				}
 				else
 				{
@@ -434,4 +439,4 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
