@@ -27,9 +27,9 @@
 #include <string.h> /* memset() strncat() strcpy() strlen() strdup() */
 
 #include "cfg/config.h"
+#include "compat/fs_limits.h"
 #include "modes/dialogs/msg_dialog.h"
 #include "ui/ui.h"
-#include "utils/fs_limits.h"
 #include "utils/path.h"
 #include "utils/str.h"
 #include "utils/test_helpers.h"
@@ -48,6 +48,14 @@ typedef enum
 }
 PathType;
 
+/* Should return the same character if processing of the macro is allowed or
+ * '\0' if it's not allowed. */
+typedef char (*macro_filter_func)(int *quoted, char c, char data);
+
+static char filter_all(int *quoted, char c, char data);
+static char filter_single(int *quoted, char c, char data);
+static char * expand_macros_i(const char command[], const char args[],
+		MacroFlags *flags, int for_shell, macro_filter_func filter);
 static void set_flags(MacroFlags *flags, MacroFlags value);
 TSTATIC char * append_selected_files(FileView *view, char expanded[],
 		int under_cursor, int quotes, const char mod[], int for_shell);
@@ -68,6 +76,65 @@ static char * add_missing_macros(char expanded[], size_t len, size_t nmacros,
 char *
 expand_macros(const char command[], const char args[], MacroFlags *flags,
 		int for_shell)
+{
+	return expand_macros_i(command, args, flags, for_shell, &filter_all);
+}
+
+/* macro_filter_func instantiation that allows all macros.  Returns the
+ * argument. */
+static char
+filter_all(int *quoted, char c, char data)
+{
+	return c;
+}
+
+char *
+ma_expand_single(const char command[])
+{
+	char *const res = expand_macros_i(command, NULL, NULL, 0, &filter_single);
+	unescape(res, 0);
+	return res;
+}
+
+/* macro_filter_func instantiation that filters out non-single element macros.
+ * Returns the argument on allowed macro and '\0' otherwise. */
+static char
+filter_single(int *quoted, char c, char data)
+{
+	if(strchr("cCdD", c) != NULL)
+	{
+		*quoted = 0;
+		return c;
+	}
+
+	if(c == 'f' && curr_view->selected_files <= 1)
+	{
+		return c;
+	}
+
+	if(c == 'F' && other_view->selected_files <= 1)
+	{
+		return c;
+	}
+
+	if(c == 'r')
+	{
+		reg_t *reg = regs_find(tolower(data));
+		if(reg != NULL && reg->nfiles == 1)
+		{
+			return c;
+		}
+	}
+
+	return '\0';
+}
+
+/* args and flags parameters can equal NULL. The string returned needs to be
+ * freed in the calling function. After executing flags is one of MF_*
+ * values. */
+static char *
+expand_macros_i(const char command[], const char args[], MacroFlags *flags,
+		int for_shell, macro_filter_func filter)
 {
 	/* TODO: refactor this function expand_macros() */
 	/* FIXME: repetitive len = strlen(expanded) could be optimized. */
@@ -106,9 +173,10 @@ expand_macros(const char command[], const char args[], MacroFlags *flags,
 		if(command[x] == '"' && char_is_one_of(MACROS_WITH_QUOTING, command[x + 1]))
 		{
 			quotes = 1;
-			x++;
+			++x;
 		}
-		switch(command[x])
+		switch(filter(&quotes, command[x],
+					command[x] == '\0' ? '\0' : command[x + 1]))
 		{
 			case 'a': /* user arguments */
 				if(args != NULL)
@@ -191,10 +259,16 @@ expand_macros(const char command[], const char args[], MacroFlags *flags,
 					}
 				}
 				break;
-			case 'p': /* preview pane properties */
+			case 'p': /* Preview pane properties. */
 				{
 					int well_formed;
-					expanded = expand_preview(expanded, command[x + 1], &well_formed);
+					const char key = command[x + 1];
+					if(key == 'c')
+					{
+						return expanded;
+					}
+
+					expanded = expand_preview(expanded, key, &well_formed);
 					len = strlen(expanded);
 					if(well_formed)
 					{
@@ -207,7 +281,11 @@ expand_macros(const char command[], const char args[], MacroFlags *flags,
 				len = strlen(expanded);
 				break;
 
-			default:
+			case '\0':
+				if(char_is_one_of("pr", command[x]) && command[x + 1] != '\0')
+				{
+					++x;
+				}
 				break;
 		}
 		if(command[x] != '\0')
@@ -356,7 +434,7 @@ expand_directory_path(FileView *view, char *expanded, int quotes,
 }
 
 /* Expands content of a register specified by the key argument considering
- * filename-modifiers.  If key is unknown, fallbacks to the default register.
+ * filename-modifiers.  If key is unknown, falls back to the default register.
  * Sets *well_formed to non-zero for valid value of the key.  Reallocates the
  * expanded string and returns result (possibly NULL). */
 static char *
@@ -364,23 +442,24 @@ expand_register(const char curr_dir[], char expanded[], int quotes,
 		const char mod[], int key, int *well_formed, int for_shell)
 {
 	int i;
+	reg_t *reg;
 
 	*well_formed = 1;
-	registers_t *reg = find_register(tolower(key));
+	reg = regs_find(tolower(key));
 	if(reg == NULL)
 	{
 		*well_formed = 0;
-		reg = find_register(DEFAULT_REG_NAME);
+		reg = regs_find(DEFAULT_REG_NAME);
 		assert(reg != NULL);
 		mod--;
 	}
 
-	for(i = 0; i < reg->num_files; i++)
+	for(i = 0; i < reg->nfiles; ++i)
 	{
 		const char *const modified = apply_mods(reg->files[i], curr_dir, mod,
 				for_shell);
 		expanded = append_path_to_expanded(expanded, quotes, modified);
-		if(i != reg->num_files - 1)
+		if(i != reg->nfiles - 1)
 		{
 			expanded = append_to_expanded(expanded, " ");
 		}
@@ -407,6 +486,7 @@ expand_preview(char expanded[], int key, int *well_formed)
 	char num_str[32];
 	int h, w, x, y;
 	int param;
+	const int with_margin = (curr_stats.clear_preview == 0) && cfg.extra_padding;
 
 	if(!char_is_one_of("hwxy", key))
 	{
@@ -423,10 +503,10 @@ expand_preview(char expanded[], int key, int *well_formed)
 
 	switch(key)
 	{
-		case 'h': param = h - 2; break;
-		case 'w': param = w - 2; break;
-		case 'x': param = x + 1; break;
-		case 'y': param = y + 1; break;
+		case 'h': param = h - 2*with_margin; break;
+		case 'w': param = w - 2*with_margin; break;
+		case 'x': param = x + 1*with_margin; break;
+		case 'y': param = y + 1*with_margin; break;
 
 		default:
 			assert(0 && "Unhandled preview property type");
@@ -473,7 +553,7 @@ append_path_to_expanded(char expanded[], int quotes, const char path[])
 	}
 	else
 	{
-		char *const escaped = escape_filename(path, 0);
+		char *const escaped = shell_like_escape(path, 0);
 		if(escaped == NULL)
 		{
 			show_error_msg("Memory Error", "Unable to allocate enough memory");
@@ -507,6 +587,20 @@ append_to_expanded(char expanded[], const char str[])
 	return t;
 }
 
+const char *
+ma_get_clean_cmd(const char cmd[])
+{
+	const char *clean_cmd;
+	const char *const break_point = strstr(cmd, "%pc");
+	if(break_point == NULL)
+	{
+		return NULL;
+	}
+
+	clean_cmd = break_point + 3;
+	return is_null_or_empty(clean_cmd) ? NULL : clean_cmd;
+}
+
 char *
 expand_custom_macros(const char pattern[], size_t nmacros,
 		custom_macro_t macros[])
@@ -537,6 +631,7 @@ expand_custom_macros(const char pattern[], size_t nmacros,
 			{
 				expanded = extend_string(expanded, macros[i].value, &len);
 				--macros[i].uses_left;
+				macros[i].explicit_use = 1;
 			}
 		}
 		pattern++;
@@ -574,8 +669,12 @@ add_missing_macros(char expanded[], size_t len, size_t nmacros,
 		int *const uses_left = (group >= 0) ? &groups[group] : &macro->uses_left;
 		while(*uses_left > 0)
 		{
-			expanded = extend_string(expanded, " ", &len);
-			expanded = extend_string(expanded, macro->value, &len);
+			/* Make sure we don't add spaces for nothing. */
+			if(macro->value[0] != '\0')
+			{
+				expanded = extend_string(expanded, " ", &len);
+				expanded = extend_string(expanded, macro->value, &len);
+			}
 			--*uses_left;
 		}
 	}
@@ -605,4 +704,4 @@ macros_to_str(MacroFlags flags)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
