@@ -49,6 +49,7 @@ typedef struct
 	const char *value; /* A value to replace macro with. */
 	int uses_left;     /* Number of mandatory uses of the macro for group head. */
 	int group;         /* Index of macro group head or -1. */
+	int explicit_use;  /* Set to non-zero on explicit expansion. */
 }
 custom_macro_t;
 
@@ -58,6 +59,14 @@ custom_macro_t;
 char * expand_macros(const char command[], const char args[], MacroFlags *flags,
 		int for_shell);
 
+/* Like expand_macros(), but expands only single element macros and aims for
+ * single string, so escaping is disabled. */
+char * ma_expand_single(const char command[]);
+
+/* Gets clean part of the viewer.  Returns NULL if there is none, otherwise
+ * pointer inside the cmd string is returned. */
+const char * ma_get_clean_cmd(const char cmd[]);
+
 /* Expands macros of form %x in the pattern (%% is expanded to %) according to
  * macros specification. */
 char * expand_custom_macros(const char pattern[], size_t nmacros,
@@ -79,4 +88,4 @@ TSTATIC_DEFS(
 #endif /* VIFM__MACROS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
