@@ -22,12 +22,12 @@
 #include <stdlib.h> /* free() qsort() */
 #include <string.h> /* strdup() strcmp() */
 
+#include "../compat/fs_limits.h"
 #include "../cfg/config.h"
+#include "../ui/color_scheme.h"
 #include "../ui/ui.h"
-#include "../utils/fs_limits.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
-#include "../color_scheme.h"
 #include "menus.h"
 
 static int sorter(const void *first, const void *second);
@@ -37,11 +37,11 @@ int
 show_colorschemes_menu(FileView *view)
 {
 	static menu_info m;
-	init_menu_info(&m, COLORSCHEME_MENU, strdup("No color schemes found"));
-	m.title = strdup(" Choose the default Color Scheme ");
+	init_menu_info(&m, strdup("Choose the default Color Scheme"),
+			strdup("No color schemes found"));
 	m.execute_handler = &execute_colorscheme_cb;
 
-	m.items = list_color_schemes(&m.len);
+	m.items = cs_list(&m.len);
 
 	qsort(m.items, m.len, sizeof(*m.items), &sorter);
 
@@ -70,9 +70,9 @@ sorter(const void *first, const void *second)
 static int
 execute_colorscheme_cb(FileView *view, menu_info *m)
 {
-	load_primary_color_scheme(m->items[m->pos]);
+	cs_load_primary(m->items[m->pos]);
 	return 0;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
