@@ -22,16 +22,18 @@
 #include <assert.h> /* assert() */
 #include <stddef.h> /* NULL size_t wchar_t */
 #include <stdio.h> /* snprintf() */
-#include <stdlib.h> /* malloc() free() */
+#include <stdlib.h> /* free() */
 #include <string.h> /* strdup() strlen() */
 #include <wchar.h> /* wcscmp() */
 
+#include "../compat/reallocarray.h"
 #include "../engine/cmds.h"
+#include "../modes/cmdline.h"
 #include "../modes/menu.h"
 #include "../ui/ui.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
-#include "../commands.h"
+#include "../cmd_core.h"
 #include "menus.h"
 
 /* Minimal length of command name column. */
@@ -48,12 +50,11 @@ show_commands_menu(FileView *view)
 	size_t cmdname_width = CMDNAME_COLUMN_MIN_WIDTH;
 
 	static menu_info m;
-	init_menu_info(&m, COMMANDS_MENU, strdup("No commands set"));
+	init_menu_info(&m, strdup("Command ------ Action"),
+			strdup("No commands set"));
 	m.execute_handler = &execute_commands_cb;
 	m.key_handler = &commands_khandler;
 
-	m.title = strdup(" Command ------ Action ");
-
 	list = list_udf();
 
 	m.len = -1;
@@ -70,7 +71,7 @@ show_commands_menu(FileView *view)
 	}
 	m.len /= 2;
 
-	m.items = (m.len != 0) ? malloc(sizeof(char *)*m.len) : NULL;
+	m.items = (m.len != 0) ? reallocarray(NULL, m.len, sizeof(char *)) : NULL;
 	for(i = 0; i < m.len; i++)
 	{
 		m.items[i] = format_str("%-*s %s", (int)cmdname_width, list[i*2],
@@ -97,8 +98,9 @@ execute_commands_cb(FileView *view, menu_info *m)
 static KHandlerResponse
 commands_khandler(menu_info *m, const wchar_t keys[])
 {
-	if(wcscmp(keys, L"dd") == 0) /* Remove element. */
+	if(wcscmp(keys, L"dd") == 0)
 	{
+		/* Remove element. */
 		char cmd_buf[512];
 
 		break_at(m->items[m->pos], ' ');
@@ -108,8 +110,31 @@ commands_khandler(menu_info *m, const wchar_t keys[])
 		remove_current_item(m);
 		return KHR_REFRESH_WINDOW;
 	}
+	else if(wcscmp(keys, L"c") == 0)
+	{
+		const char *const rhs = skip_whitespace(after_first(m->items[m->pos], ' '));
+		/* Insert command RHS. */
+		if(rhs[0] == ':')
+		{
+			menu_morph_into_cmdline(CLS_COMMAND, skip_whitespace(rhs + 1), 0);
+		}
+		else if(rhs[0] == '/')
+		{
+			menu_morph_into_cmdline(CLS_FSEARCH, rhs + 1, 0);
+		}
+		else if(rhs[0] == '=')
+		{
+			menu_morph_into_cmdline(CLS_FILTER, rhs + 1, 0);
+		}
+		else
+		{
+			/* filter commands go here. */
+			menu_morph_into_cmdline(CLS_COMMAND, rhs, (rhs[0] != '!'));
+		}
+		return KHR_MORPHED_MENU;
+	}
 	return KHR_UNHANDLED;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
