@@ -39,9 +39,9 @@ show_history_menu(FileView *view)
 	int need_cleanup;
 
 	static menu_info m;
-	init_menu_info(&m, DIRHISTORY_MENU, strdup("History disabled or empty"));
+	init_menu_info(&m, strdup("Directory History"),
+			strdup("History disabled or empty"));
 
-	m.title = strdup(" Directory History ");
 	m.execute_handler = &execute_dirhistory_cb;
 
 	need_cleanup = 0;
@@ -129,4 +129,4 @@ execute_dirhistory_cb(FileView *view, menu_info *m)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
