@@ -19,20 +19,22 @@
 
 #include "filetypes_menu.h"
 
+#include <stddef.h> /* NULL */
 #include <stdio.h> /* snprintf() */
 #include <stdlib.h> /* free() */
 #include <string.h> /* strdup() strlen() */
 
+#include "../compat/fs_limits.h"
+#include "../int/file_magic.h"
+#include "../modes/cmdline.h"
 #include "../modes/menu.h"
+#include "../ui/fileview.h"
 #include "../ui/ui.h"
-#include "../utils/fs_limits.h"
 #include "../utils/macros.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
-#include "../file_magic.h"
 #include "../filelist.h"
 #include "../filetype.h"
-#include "../fileview.h"
 #include "../running.h"
 #include "../types.h"
 #include "menus.h"
@@ -41,6 +43,7 @@ static const char * form_filetype_menu_entry(assoc_record_t prog,
 		int descr_width);
 static const char * form_filetype_data_entry(assoc_record_t prog);
 static int execute_filetype_cb(FileView *view, menu_info *m);
+static KHandlerResponse filetypes_khandler(menu_info *m, const wchar_t keys[]);
 static void fill_menu_from_records(menu_info *m,
 		const assoc_records_t *records);
 static int max_desc_len(const assoc_records_t *records);
@@ -53,15 +56,16 @@ show_file_menu(FileView *view, int background)
 	int i;
 	int max_len;
 
-	char *const typed_name = get_typed_current_fpath(view);
+	char *const typed_name = get_typed_entry_fpath(get_current_entry(view));
 	assoc_records_t ft = ft_get_all_programs(typed_name);
 	assoc_records_t magic = get_magic_handlers(typed_name);
 	free(typed_name);
 
-	init_menu_info(&m, FILE_MENU, strdup("No programs set for this filetype"));
+	init_menu_info(&m, strdup("Filetype associated commands"),
+			strdup("No programs set for this filetype"));
 
-	m.title = strdup(" Filetype associated commands ");
 	m.execute_handler = &execute_filetype_cb;
+	m.key_handler = &filetypes_khandler;
 	m.extra_data = (background ? 1 : 0);
 
 	max_len = MAX(max_desc_len(&ft), max_desc_len(&magic));
@@ -127,9 +131,9 @@ form_filetype_data_entry(assoc_record_t prog)
 	return result;
 }
 
-/* Callback that is called when menu item is selected.  Should return non-zero
+/* Callback that is invoked when menu item is selected.  Should return non-zero
  * to stay in menu mode. */
-int
+static int
 execute_filetype_cb(FileView *view, menu_info *m)
 {
 	if(view->dir_entry[view->list_pos].type == FT_DIR && m->pos == 0)
@@ -151,6 +155,23 @@ execute_filetype_cb(FileView *view, menu_info *m)
 	return 0;
 }
 
+/* Menu-specific shortcut handler.  Returns code that specifies both taken
+ * actions and what should be done next. */
+static KHandlerResponse
+filetypes_khandler(menu_info *m, const wchar_t keys[])
+{
+	if(wcscmp(keys, L"c") == 0)
+	{
+		const char *prog_str = after_first(m->data[m->pos], '|');
+		if(prog_str[0] != '\0')
+		{
+			menu_morph_into_cmdline(CLS_COMMAND, prog_str, 1);
+			return KHR_MORPHED_MENU;
+		}
+	}
+	return KHR_UNHANDLED;
+}
+
 int
 show_fileprograms_menu(FileView *view, const char fname[])
 {
@@ -158,11 +179,9 @@ show_fileprograms_menu(FileView *view, const char fname[])
 
 	assoc_records_t file_programs;
 
-	init_menu_info(&m, FILEPROGRAMS_MENU,
+	init_menu_info(&m, format_str("Programs that match %s", fname),
 			format_str("No programs match %s", fname));
 
-	m.title = format_str(" Programs that match %s ", fname);
-
 	file_programs = ft_get_all_programs(fname);
 	fill_menu_from_records(&m, &file_programs);
 	ft_assoc_records_free(&file_programs);
@@ -177,11 +196,9 @@ show_fileviewers_menu(FileView *view, const char fname[])
 
 	assoc_records_t file_viewers;
 
-	init_menu_info(&m, FILEVIEWERS_MENU,
+	init_menu_info(&m, format_str("Viewers that match %s", fname),
 			format_str("No viewers match %s", fname));
 
-	m.title = format_str(" Viewers that match %s ", fname);
-
 	file_viewers = ft_get_all_viewers(fname);
 	fill_menu_from_records(&m, &file_viewers);
 	ft_assoc_records_free(&file_viewers);
@@ -218,4 +235,4 @@ max_desc_len(const assoc_records_t *records)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
