@@ -37,16 +37,21 @@ static int execute_grep_cb(FileView *view, menu_info *m);
 int
 show_grep_menu(FileView *view, const char args[], int invert)
 {
+	enum { M_i, M_a, M_s, M_A, M_u, M_U, };
+
 	char *targets;
 	int save_msg;
 	char *cmd;
 	char *escaped_args = NULL;
 
 	custom_macro_t macros[] = {
-		{ .letter = 'i', .value = NULL, .uses_left = 1, .group = -1 },
-		{ .letter = 'a', .value = NULL, .uses_left = 1, .group =  1 },
-		{ .letter = 's', .value = NULL, .uses_left = 1, .group = -1 },
-		{ .letter = 'A', .value = NULL, .uses_left = 0, .group =  1 },
+		[M_i] = { .letter = 'i', .value = NULL, .uses_left = 1, .group = -1 },
+		[M_a] = { .letter = 'a', .value = NULL, .uses_left = 1, .group =  1 },
+		[M_s] = { .letter = 's', .value = NULL, .uses_left = 1, .group = -1 },
+		[M_A] = { .letter = 'A', .value = NULL, .uses_left = 0, .group =  1 },
+
+		[M_u] = { .letter = 'u', .value = "",   .uses_left = 1, .group = -1 },
+		[M_U] = { .letter = 'U', .value = "",   .uses_left = 1, .group = -1 },
 	};
 
 	static menu_info m;
@@ -58,20 +63,20 @@ show_grep_menu(FileView *view, const char args[], int invert)
 		return 0;
 	}
 
-	init_menu_info(&m, GREP_MENU, format_str("No matches found: %s", args));
+	init_menu_info(&m, format_str("Grep %s", args),
+			format_str("No matches found: %s", args));
 
-	m.title = format_str(" Grep %s ", args);
 	m.execute_handler = &execute_grep_cb;
 	m.key_handler = &filelist_khandler;
 
-	macros[0].value = invert ? "-v" : "";
-	macros[1].value = args;
-	macros[2].value = targets;
-	macros[3].value = args;
+	macros[M_i].value = invert ? "-v" : "";
+	macros[M_a].value = args;
+	macros[M_s].value = targets;
+	macros[M_A].value = args;
 	if(args[0] != '-')
 	{
-		escaped_args = escape_filename(args, 0);
-		macros[1].value = escaped_args;
+		escaped_args = shell_like_escape(args, 0);
+		macros[M_a].value = escaped_args;
 	}
 
 	cmd = expand_custom_macros(cfg.grep_prg, ARRAY_LEN(macros), macros);
@@ -80,7 +85,8 @@ show_grep_menu(FileView *view, const char args[], int invert)
 	free(targets);
 
 	status_bar_message("grep...");
-	save_msg = capture_output_to_menu(view, cmd, 0, &m);
+	save_msg = capture_output(view, cmd, 0, &m, macros[M_u].explicit_use,
+			macros[M_U].explicit_use);
 	free(cmd);
 
 	return save_msg;
@@ -91,9 +97,9 @@ show_grep_menu(FileView *view, const char args[], int invert)
 static int
 execute_grep_cb(FileView *view, menu_info *m)
 {
-	goto_selected_file(view, m->items[m->pos], 1);
+	(void)goto_selected_file(view, m->items[m->pos], 1);
 	return 1;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
