@@ -38,8 +38,8 @@ show_jobs_menu(FileView *view)
 	int i;
 
 	static menu_info m;
-	init_menu_info(&m, JOBS_MENU, strdup("No jobs currently running"));
-	m.title = strdup(" Pid --- Command ");
+	init_menu_info(&m, strdup("Pid --- Command"),
+			strdup("No jobs currently running"));
 	m.execute_handler = &execute_jobs_cb;
 
 	check_background_jobs();
@@ -58,7 +58,8 @@ show_jobs_menu(FileView *view)
 
 			if(p->type == BJT_COMMAND)
 			{
-				snprintf(info_buf, sizeof(info_buf), PRINTF_PID_T, p->pid);
+				snprintf(info_buf, sizeof(info_buf), "%" PRINTF_ULL,
+						(unsigned long long)p->pid);
 			}
 			else if(p->bg_op.total == BG_UNDEFINED_TOTAL)
 			{
@@ -94,4 +95,4 @@ execute_jobs_cb(FileView *view, menu_info *m)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
