@@ -24,13 +24,12 @@
 
 #include "../ui/ui.h"
 
-/* Displays menu of mappings that have specified prefix.  Frees the list array
- * and all of its strings.  Returns non-zero if status bar message should be
- * saved. */
-int show_map_menu(FileView *view, const char mode_str[], wchar_t *list[],
+/* Displays menu of mappings that have specified prefix.  Returns non-zero if
+ * status bar message should be saved. */
+int show_map_menu(FileView *view, const char mode_str[], int mode,
 		const wchar_t start[]);
 
 #endif /* VIFM__MENUS__MAP_MENU_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
