@@ -24,53 +24,12 @@
 
 #include "../ui/ui.h"
 
-enum
-{
-	NONE,
-	UP,
-	DOWN
-};
-
-/* List of menu identifiers. */
-enum
-{
-	APROPOS_MENU,
-	BOOKMARK_MENU,
-	CABBREVS_MENU,
-	CMDHISTORY_MENU,
-	FSEARCHHISTORY_MENU,
-	BSEARCHHISTORY_MENU,
-	PROMPTHISTORY_MENU,
-	FILTERHISTORY_MENU,
-	COLORSCHEME_MENU,
-	COMMANDS_MENU,
-	DIRSTACK_MENU,
-	FILE_MENU,
-	FILEPROGRAMS_MENU,
-	FILEVIEWERS_MENU,
-	FIND_MENU,
-	DIRHISTORY_MENU,
-	JOBS_MENU,
-	LOCATE_MENU,
-	MAP_MENU,
-	REGISTER_MENU,
-	UNDOLIST_MENU,
-	USER_MENU,
-	USER_NAVIGATE_MENU,
-	VIFM_MENU,
-	GREP_MENU,
-	TRASH_MENU,
-	TRASHES_MENU,
-#ifdef _WIN32
-	VOLUMES_MENU,
-#endif
-};
-
 /* Result of handling key sequence by menu-specific shortcut handler. */
 typedef enum
 {
 	KHR_REFRESH_WINDOW, /* Menu window refresh is needed. */
 	KHR_CLOSE_MENU,     /* Menu mode should be left. */
+	KHR_MORPHED_MENU,   /* Menu was morphed menu_morph_into_cmdline. */
 	KHR_UNHANDLED,      /* Passed key wasn't handled. */
 }
 KHandlerResponse;
@@ -83,11 +42,14 @@ typedef struct menu_info
 	int pos; /* Menu item under the cursor. */
 	int hor_pos;
 	int win_rows;
-	int type;
-	int match_dir;
+	int backward_search; /* Search direction. */
 	/* Number of menu entries that actually match the regexp. */
 	int matching_entries;
-	int *matches;
+	/* Whether search highlight matches are currently highlighted. */
+	int search_highlight;
+	/* Start and end positions of search match.  If there is no match, values are
+	 * equal to -1. */
+	short int (*matches)[2];
 	char *regexp;
 	char *title;
 	char *args;
@@ -106,14 +68,17 @@ typedef struct menu_info
 	/* Text displayed by display_menu() function in case menu is empty, it can be
 	 * NULL if this cannot happen and will be freed by reset_popup_menu(). */
 	char *empty_msg;
+	/* Number of times to repeat search. */
+	int search_repeat;
 }
 menu_info;
 
 /* Fills fields of menu_info structure with some safe values.  empty_msg is
  * text displayed by display_menu() function in case menu is empty, it can be
  * NULL if this cannot happen and will be freed by reset_popup_menu(). */
-void init_menu_info(menu_info *m, int menu_type, char empty_msg[]);
+void init_menu_info(menu_info *m, char title[], char empty_msg[]);
 
+/* Frees resources associated with the menu and clears menu window. */
 void reset_popup_menu(menu_info *m);
 
 void setup_menu(void);
@@ -130,8 +95,10 @@ void redraw_menu(menu_info *m);
 void draw_menu(menu_info *m);
 
 /* Navigates to/open path specification.  Specification can contain colon
- * followed by a line number when try_open is not zero. */
-void goto_selected_file(FileView *view, const char spec[], int try_open);
+ * followed by a line number when try_open is not zero.  Returns zero on
+ * successful parsing and performed try to handle the file otherwise non-zero is
+ * returned. */
+int goto_selected_file(FileView *view, const char spec[], int try_open);
 
 /* Navigates to directory from a menu. */
 void goto_selected_directory(FileView *view, const char path[]);
@@ -158,9 +125,30 @@ KHandlerResponse filelist_khandler(menu_info *m, const wchar_t keys[]);
 
 /* Moves menu items into custom view.  Returns zero on success, otherwise
  * non-zero is returned. */
-int menu_to_custom_view(menu_info *m, FileView *view);
+int menu_to_custom_view(menu_info *m, FileView *view, int very);
+
+/* Either makes a menu or custom view out of command output.  Returns non-zero
+ * if status bar message should be saved. */
+int capture_output(FileView *view, const char cmd[], int user_sh, menu_info *m,
+		int custom_view, int very_custom_view);
+
+/* Performs search in requested direction.  Either continues the previous one or
+ * restarts it. */
+void menus_search(menu_info *m, int backward);
+
+/* Performs search of pattern among menu items.  NULL pattern requests use of
+ * the last used pattern.  Returns new value for save_msg flag, but when
+ * print_errors isn't requested can return -1 to indicate issues with the
+ * pattern. */
+int search_menu_list(const char pattern[], menu_info *m, int print_errors);
+
+/* Prints results or error message about search operation to the user. */
+void menu_print_search_msg(const menu_info *m);
+
+/* Reset search highlight of a menu. */
+void menus_reset_search_highlight(menu_info *m);
 
 #endif /* VIFM__MENUS__MENUS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
