@@ -29,15 +29,16 @@ int
 show_register_menu(FileView *view, const char registers[])
 {
 	static menu_info m;
-	init_menu_info(&m, REGISTER_MENU, strdup("Registers are empty"));
-	m.title = strdup(" Registers ");
+	init_menu_info(&m, strdup("Registers"), strdup("Registers are empty"));
 
-	m.items = list_registers_content(registers);
+	m.items = regs_list(registers);
 	while(m.items[m.len] != NULL)
-		m.len++;
+	{
+		++m.len;
+	}
 
 	return display_menu(&m, view);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
