@@ -18,7 +18,7 @@
 
 #include "trashes_menu.h"
 
-#include <string.h> /* strdup() */
+#include <string.h> /* strchr() strdup() */
 
 #include "../ui/ui.h"
 #include "../utils/str.h"
@@ -30,6 +30,7 @@
 
 static char * format_item(const char trash_dir[], int calc_size);
 static int execute_trashes_cb(FileView *view, menu_info *m);
+static KHandlerResponse trashes_khandler(menu_info *m, const wchar_t keys[]);
 
 int
 show_trashes_menu(FileView *view, int calc_size)
@@ -39,12 +40,12 @@ show_trashes_menu(FileView *view, int calc_size)
 	int i;
 
 	static menu_info m;
-	init_menu_info(&m, TRASHES_MENU,
+	init_menu_info(&m,
+			format_str("%sNon-empty trash directories", calc_size ? "[  size] " : ""),
 			strdup("No non-empty trash directories found"));
 
-	m.title = format_str(" %sNon-empty trash directories ",
-			calc_size ? "[  size] " : "");
 	m.execute_handler = &execute_trashes_cb;
+	m.key_handler = &trashes_khandler;
 	m.extra_data = calc_size;
 
 	trashes = list_trashes(&ntrashes);
@@ -96,5 +97,21 @@ execute_trashes_cb(FileView *view, menu_info *m)
 	return 0;
 }
 
+/* Menu-specific shortcut handler.  Returns code that specifies both taken
+ * actions and what should be done next. */
+static KHandlerResponse
+trashes_khandler(menu_info *m, const wchar_t keys[])
+{
+	if(wcscmp(keys, L"dd") == 0)
+	{
+		const char *const item = m->items[m->pos];
+		const char *const trash_dir = m->extra_data ? strchr(item, ']') + 2 : item;
+		trash_empty(trash_dir);
+		remove_current_item(m);
+		return KHR_REFRESH_WINDOW;
+	}
+	return KHR_UNHANDLED;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
