@@ -34,10 +34,9 @@ show_undolist_menu(FileView *view, int with_details)
 	char **p;
 
 	static menu_info m;
-	init_menu_info(&m, UNDOLIST_MENU, strdup("Undolist is empty"));
+	init_menu_info(&m, strdup("Undolist"), strdup("Undolist is empty"));
 	m.current = get_undolist_pos(with_details) + 1;
 	m.pos = m.current - 1;
-	m.title = strdup(" Undolist ");
 
 	m.items = undolist(with_details);
 	p = m.items;
@@ -61,4 +60,4 @@ show_undolist_menu(FileView *view, int with_details)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
