@@ -20,9 +20,9 @@
 #include "vifm_menu.h"
 
 #include <stddef.h> /* NULL */
-#include <stdlib.h> /* malloc() */
 #include <string.h> /* strdup() */
 
+#include "../compat/reallocarray.h"
 #include "../ui/ui.h"
 #include "../version.h"
 #include "menus.h"
@@ -33,15 +33,14 @@ show_vifm_menu(FileView *view)
 	static menu_info m;
 	int len;
 	/* Version information menu always contains at least one item. */
-	init_menu_info(&m, VIFM_MENU, NULL);
-	m.title = strdup(" vifm information ");
+	init_menu_info(&m, strdup("Vifm Information"), NULL);
 
 	len = fill_version_info(NULL);
-	m.items = malloc(sizeof(char*)*len);
+	m.items = reallocarray(NULL, len, sizeof(char *));
 	m.len = fill_version_info(m.items);
 
 	return display_menu(&m, view);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
