@@ -42,8 +42,7 @@ show_volumes_menu(FileView *view)
 	char file_buf[MAX_PATH];
 
 	static menu_info m;
-	init_menu_info(&m, VOLUMES_MENU, strdup("No volumes mounted"));
-	m.title = strdup(" Mounted Volumes ");
+	init_menu_info(&m, strdup("Mounted Volumes"), strdup("No volumes mounted"));
 	m.execute_handler = &execute_volumes_cb;
 
 	for(c = 'a'; c <= 'z'; ++c)
@@ -81,4 +80,4 @@ execute_volumes_cb(FileView *view, menu_info *m)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
