@@ -20,11 +20,6 @@
 #ifndef VIFM__MODES__CMDLINE_H__
 #define VIFM__MODES__CMDLINE_H__
 
-#include <stddef.h> /* wchar_t */
-
-#ifdef TEST
-#include "../utils/fs_limits.h"
-#endif
 #include "../utils/test_helpers.h"
 
 /* Submodes of command-line mode. */
@@ -45,35 +40,52 @@ typedef enum
 }
 CmdLineSubmode;
 
-typedef void (*prompt_cb)(const char renponse[]);
+/* Callback for prompt input. */
+typedef void (*prompt_cb)(const char response[]);
 
 /* Custom prompt line completion function.  arg is user supplied value, which is
- * passed through. */
+ * passed through.  Should return completion offset. */
 typedef int (*complete_cmd_func)(const char cmd[], void *arg);
 
 /* Initializes command-line mode. */
 void init_cmdline_mode(void);
 
-void enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const wchar_t *cmd,
+/* Enters command-line editing mode with specified submode.  cmd specifies
+ * initial value, ptr is submode-specific data to be passed back. */
+void enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const char cmd[],
 		void *ptr);
 
 /* Enters command-line editing mode with prompt submode activated.  cmd
- * specifies initial value, cb - callback called on success, complete -
- * completion function, allow_ee - whether issuing external editor is
+ * specifies initial value, cb is callback called on success, complete is
+ * completion function, allow_ee specifies whether issuing external editor is
  * allowed. */
-void enter_prompt_mode(const wchar_t prompt[], const char cmd[], prompt_cb cb,
+void enter_prompt_mode(const char prompt[], const char cmd[], prompt_cb cb,
 		complete_cmd_func complete, int allow_ee);
 
+/* Redraws UI elements of the command-line mode. */
 void redraw_cmdline(void);
 
 #ifdef TEST
+#include <stddef.h> /* size_t wchar_t */
+
+#include "../cfg/hist.h"
+#include "../compat/fs_limits.h"
 
 typedef enum
 {
 	HIST_NONE,
 	HIST_GO,
 	HIST_SEARCH
-}HIST;
+}
+HIST;
+
+typedef enum
+{
+	PS_NORMAL,
+	PS_WRONG_PATTERN,
+	PS_NO_MATCH,
+}
+PromptState;
 
 /* Holds state of the command-line editing mode. */
 typedef struct
@@ -101,15 +113,17 @@ typedef struct
 	int line_edited;          /* Cache for whether input line changed flag. */
 	int entered_by_mapping;   /* The mode was entered by a mapping. */
 	int expanding_abbrev;     /* Abbreviation expansion is in progress. */
+	PromptState state;        /* Prompt state with regard to current input. */
 }
 line_stats_t;
 #endif
 TSTATIC_DEFS(
 	int line_completion(line_stats_t *stat);
 	const wchar_t * extract_abbrev(line_stats_t *stat, int *pos, int *no_remap);
+	void hist_prev(line_stats_t *stat, const hist_t *hist, size_t len);
 )
 
 #endif /* VIFM__MODES__CMDLINE_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
