@@ -31,21 +31,23 @@
 #include <stdio.h> /* snprintf() */
 #include <string.h> /* strncat() strlen() */
 
+#include "../../compat/fs_limits.h"
 #include "../../engine/keys.h"
 #include "../../engine/mode.h"
 #include "../../modes/dialogs/msg_dialog.h"
 #include "../../ui/cancellation.h"
+#include "../../ui/fileview.h"
 #include "../../ui/ui.h"
 #include "../../utils/fs.h"
-#include "../../utils/fs_limits.h"
 #include "../../utils/macros.h"
 #include "../../utils/path.h"
+#include "../../utils/test_helpers.h"
 #include "../../filelist.h"
-#include "../../fileview.h"
 #include "../../ops.h"
 #include "../../status.h"
 #include "../../undo.h"
 #include "../modes.h"
+#include "../wk.h"
 
 static const char * get_title(void);
 static int is_one_file_selected(int first_file_index);
@@ -53,9 +55,9 @@ static int get_first_file_index(void);
 static int get_selection_size(int first_file_index);
 static void leave_attr_mode(void);
 static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
-static void cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info);
-static void set_perm_string(FileView *view, const int *perms,
-		const int *origin_perms);
+static void cmd_return(key_info_t key_info, keys_info_t *keys_info);
+TSTATIC void set_perm_string(FileView *view, const int perms[13],
+		const int origin_perms[13], int adv_perms[3]);
 static void chmod_file_in_list(FileView *view, int pos, const char *mode,
 		const char *inv_mode, int recurse_dirs);
 static void file_chmod(char *path, const char *mode, const char *inv_mode,
@@ -76,35 +78,33 @@ static int step;
 static int col;
 static int changed;
 static int file_is_dir;
-static int perms[13] = {0,0,0,0,0,0,0,0,0,0,0,0,0};
+static int perms[13];
 static int origin_perms[13];
 static int adv_perms[3];
 
 static keys_add_info_t builtin_cmds[] = {
-	{L"\x03", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	/* return */
-	{L"\x0d", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_m}}},
-	{L"\x0e", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"\x10", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	/* escape */
-	{L"\x1b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L" ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_space}}},
-	{L"G", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_G}}},
-	{L"ZQ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"ZZ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"gg", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_gg}}},
-	{L"h", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_space}}},
-	{L"j", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"k", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{L"l", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_m}}},
-	{L"q", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"t", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_space}}},
+	{WK_C_c,    {{&cmd_ctrl_c}, .descr = "close the dialog"}},
+	{WK_CR,     {{&cmd_return}, .descr = "update permissions"}},
+	{WK_C_n,    {{&cmd_j},      .descr = "go to item below"}},
+	{WK_C_p,    {{&cmd_k},      .descr = "go to item above"}},
+	{WK_ESC,    {{&cmd_ctrl_c}, .descr = "close the dialog"}},
+	{WK_SPACE,  {{&cmd_space},  .descr = "toggle current item"}},
+	{WK_G,      {{&cmd_G},      .descr = "go to the last item"}},
+	{WK_Z WK_Q, {{&cmd_ctrl_c}, .descr = "close the dialog"}},
+	{WK_Z WK_Z, {{&cmd_ctrl_c}, .descr = "close the dialog"}},
+	{WK_g WK_g, {{&cmd_gg},     .descr = "go to the first item"}},
+	{WK_h,      {{&cmd_space},  .descr = "toggle current item"}},
+	{WK_j,      {{&cmd_j},      .descr = "go to item below"}},
+	{WK_k,      {{&cmd_k},      .descr = "go to item above"}},
+	{WK_l,      {{&cmd_return}, .descr = "update permissions"}},
+	{WK_q,      {{&cmd_ctrl_c}, .descr = "close the dialog"}},
+	{WK_t,      {{&cmd_space},  .descr = "toggle current item"}},
 #ifdef ENABLE_EXTENDED_KEYS
-	{{KEY_HOME}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_gg}}},
-	{{KEY_END}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_G}}},
-	{{KEY_UP}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{{KEY_DOWN}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{{KEY_RIGHT}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_m}}},
+	{{KEY_HOME},  {{&cmd_gg},     .descr = "go to the first item"}},
+	{{KEY_END},   {{&cmd_G},      .descr = "go to the last item"}},
+	{{KEY_UP},    {{&cmd_k},      .descr = "go to item above"}},
+	{{KEY_DOWN},  {{&cmd_j},      .descr = "go to item below"}},
+	{{KEY_RIGHT}, {{&cmd_return}, .descr = "update permissions"}},
 #endif /* ENABLE_EXTENDED_KEYS */
 };
 
@@ -113,7 +113,7 @@ init_attr_dialog_mode(void)
 {
 	int ret_code;
 
-	ret_code = add_cmds(builtin_cmds, ARRAY_LEN(builtin_cmds), ATTR_MODE);
+	ret_code = vle_keys_add(builtin_cmds, ARRAY_LEN(builtin_cmds), ATTR_MODE);
 	assert(ret_code == 0);
 
 	(void)ret_code;
@@ -301,7 +301,7 @@ redraw_attr_dialog(void)
 		waddstr(change_win, "...");
 	}
 
-	curs_set(TRUE);
+	curs_set(1);
 	checked_wmove(change_win, curr, col);
 	wrefresh(change_win);
 }
@@ -311,16 +311,21 @@ redraw_attr_dialog(void)
 static const char *
 get_title(void)
 {
-	static char title[64];
+	static char title[NAME_MAX];
 
 	const int first_file_index = get_first_file_index();
+
 	if(is_one_file_selected(first_file_index))
 	{
-		return view->dir_entry[first_file_index].name;
+		snprintf(title, sizeof(title), " %s ",
+				view->dir_entry[first_file_index].name);
+	}
+	else
+	{
+		snprintf(title, sizeof(title), " %d files ",
+				get_selection_size(first_file_index));
 	}
 
-	snprintf(title, sizeof(title), "%d files",
-			get_selection_size(first_file_index));
 	return title;
 }
 
@@ -376,7 +381,7 @@ static void
 leave_attr_mode(void)
 {
 	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
-	curs_set(FALSE);
+	curs_set(0);
 	curr_stats.use_input_bar = 1;
 
 	clean_selected_files(view);
@@ -393,7 +398,7 @@ cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 }
 
 static void
-cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info)
+cmd_return(key_info_t key_info, keys_info_t *keys_info)
 {
 	char path[PATH_MAX];
 
@@ -403,13 +408,14 @@ cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info)
 	}
 
 	get_current_full_path(view, sizeof(path), path);
-	set_perm_string(view, perms, origin_perms);
+	set_perm_string(view, perms, origin_perms, adv_perms);
 
 	leave_attr_mode();
 }
 
-static void
-set_perm_string(FileView *view, const int *perms, const int *origin_perms)
+TSTATIC void
+set_perm_string(FileView *view, const int perms[13], const int origin_perms[13],
+		int adv_perms[3])
 {
 	int i = 0;
 	char *add_perm[] = {"u+r", "u+w", "u+x", "u+s", "g+r", "g+w", "g+x", "g+s",
@@ -425,10 +431,10 @@ set_perm_string(FileView *view, const int *perms, const int *origin_perms)
 		adv_perms[0] = -1;
 		adv_perms[1] = -1;
 		adv_perms[2] = -1;
-	}
 
-	perm_str_len += snprintf(perm_str + perm_str_len,
-			sizeof(perm_str) - perm_str_len, "a-x+X,");
+		perm_str_len += snprintf(perm_str + perm_str_len,
+				sizeof(perm_str) - perm_str_len, "a-x+X,");
+	}
 
 	for(i = 0; i < 12; i++)
 	{
@@ -696,4 +702,4 @@ dec_curr(void)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
