@@ -36,6 +36,7 @@
 #include "../../utils/utf8.h"
 #include "../../event_loop.h"
 #include "../../status.h"
+#include "../wk.h"
 
 /* Kinds of dialogs. */
 typedef enum
@@ -76,17 +77,18 @@ static void redraw_error_msg(const char title_arg[], const char message_arg[],
 static const char * get_control_msg(Dialog msg_kind, int global_skip);
 static const char * get_custom_control_msg(const response_variant responses[]);
 static void draw_msg(const char title[], const char msg[],
-		const char ctrl_msg[], int centered);
+		const char ctrl_msg[], int centered, int recommended_width);
+static size_t measure_sub_lines(const char msg[], size_t *max_len);
 static size_t determine_width(const char msg[]);
 
 /* List of builtin key bindings. */
 static keys_add_info_t builtin_cmds[] = {
-	{L"\x03", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"\x0c", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_l}}},
-	{L"\x0d", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_m}}},
-	{L"\x1b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"n", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_n}}},
-	{L"y", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_y}}},
+	{WK_C_c, {{&cmd_ctrl_c}, .descr = "cancel"}},
+	{WK_C_l, {{&cmd_ctrl_l}, .descr = "redraw"}},
+	{WK_C_m, {{&cmd_ctrl_m}, .descr = "agree to the query"}},
+	{WK_ESC, {{&cmd_ctrl_c}, .descr = "cancel"}},
+	{WK_n,   {{&cmd_n},      .descr = "deny the query"}},
+	{WK_y,   {{&cmd_y},      .descr = "confirm the query"}},
 };
 
 /* Mode that was active before the dialog. */
@@ -112,9 +114,9 @@ init_msg_dialog_mode(void)
 {
 	int ret_code;
 
-	set_def_handler(MSG_MODE, def_handler);
+	vle_keys_set_def_handler(MSG_MODE, def_handler);
 
-	ret_code = add_cmds(builtin_cmds, ARRAY_LEN(builtin_cmds), MSG_MODE);
+	ret_code = vle_keys_add(builtin_cmds, ARRAY_LEN(builtin_cmds), MSG_MODE);
 	assert(ret_code == 0 && "Failed to register msg dialog keys.");
 
 	(void)ret_code;
@@ -182,7 +184,7 @@ handle_response(Result r)
 	/* Map result to corresponding input key to omit branching per handler. */
 	static const char r_to_c[] = {
 		[R_OK]     = '\r',
-		[R_CANCEL] = '\x03',
+		[R_CANCEL] = NC_C_c,
 		[R_YES]    = 'y',
 		[R_NO]     = 'n',
 	};
@@ -213,7 +215,7 @@ leave(Result r)
 void
 redraw_msg_dialog(int lazy)
 {
-	redraw_error_msg(NULL, NULL, 0, 1);
+	redraw_error_msg(NULL, NULL, 0, lazy);
 }
 
 void
@@ -383,7 +385,7 @@ redraw_error_msg(const char title_arg[], const char message_arg[],
 	}
 
 	ctrl_msg = get_control_msg(msg_kind, ctrl_c);
-	draw_msg(title, message, ctrl_msg, centered);
+	draw_msg(title, message, ctrl_msg, centered, 0);
 
 	if(lazy)
 	{
@@ -429,10 +431,13 @@ get_custom_control_msg(const response_variant responses[])
 	msg_buf[0] = '\0';
 	while(response != NULL && response->key != '\0')
 	{
-		(void)sstrappend(msg_buf, &len, sizeof(msg_buf), response->descr);
-		if(response[1].key != '\0')
+		if(response->descr[0] != '\0')
 		{
-			(void)sstrappendch(msg_buf, &len, sizeof(msg_buf), '/');
+			(void)sstrappend(msg_buf, &len, sizeof(msg_buf), response->descr);
+			if(response[1].key != '\0')
+			{
+				(void)sstrappendch(msg_buf, &len, sizeof(msg_buf), '/');
+			}
 		}
 
 		++response;
@@ -442,7 +447,8 @@ get_custom_control_msg(const response_variant responses[])
 }
 
 void
-draw_msgf(const char title[], const char ctrl_msg[], const char format[], ...)
+draw_msgf(const char title[], const char ctrl_msg[], int recommended_width,
+		const char format[], ...)
 {
 	char msg[8192];
 	va_list pa;
@@ -451,7 +457,7 @@ draw_msgf(const char title[], const char ctrl_msg[], const char format[], ...)
 	vsnprintf(msg, sizeof(msg), format, pa);
 	va_end(pa);
 
-	draw_msg(title, msg, ctrl_msg, 0);
+	draw_msg(title, msg, ctrl_msg, 0, recommended_width);
 	touch_all_windows();
 	wrefresh(error_win);
 }
@@ -460,20 +466,28 @@ draw_msgf(const char title[], const char ctrl_msg[], const char format[], ...)
  * message on error_win. */
 static void
 draw_msg(const char title[], const char msg[], const char ctrl_msg[],
-		int centered)
+		int centered, int recommended_width)
 {
 	enum { margin = 1 };
 
 	int sw, sh;
 	int w, h;
 	int len;
+	size_t ctrl_msg_n;
+	size_t wctrl_msg;
+	size_t i;
 
-	curs_set(FALSE);
+	curs_set(0);
 
 	getmaxyx(stdscr, sh, sw);
 
-	h = sh - 3 + !cfg.display_statusline;
-	w = MIN(sw - 2, MAX(sw/3, (int)determine_width(msg) + 4));
+	ctrl_msg_n = MAX(measure_sub_lines(ctrl_msg, &wctrl_msg), 1U);
+
+	h = sh - 2 - ctrl_msg_n + !cfg.display_statusline;
+	/* The outermost condition is for VLA below (to calm static analyzers). */
+	w = MAX(2 + 2*margin, MIN(sw - 2,
+	        MAX(MAX(recommended_width, sw/3),
+	            (int)MAX(wctrl_msg, determine_width(msg)) + 4)));
 	wresize(error_win, h, w);
 
 	werase(error_win);
@@ -481,7 +495,7 @@ draw_msg(const char title[], const char msg[], const char ctrl_msg[],
 	len = strlen(msg);
 	if(len <= w - 2 && strchr(msg, '\n') == NULL)
 	{
-		h = 6;
+		h = 5 + ctrl_msg_n;
 		wresize(error_win, h, w);
 		mvwin(error_win, (sh - h)/2, (sw - w)/2);
 		checked_wmove(error_win, 2, (w - len)/2);
@@ -516,7 +530,7 @@ draw_msg(const char title[], const char msg[], const char ctrl_msg[],
 			wresize(error_win, h, w);
 			mvwin(error_win, (sh - h)/2, (sw - w)/2);
 
-			cx = centered ? (w - get_screen_string_length(buf))/2 : (1 + margin);
+			cx = centered ? (w - utf8_strsw(buf))/2 : (1 + margin);
 			checked_wmove(error_win, cy++, cx);
 			wprint(error_win, buf);
 		}
@@ -527,7 +541,35 @@ draw_msg(const char title[], const char msg[], const char ctrl_msg[],
 	{
 		mvwprintw(error_win, 0, (w - strlen(title) - 2)/2, " %s ", title);
 	}
-	mvwaddstr(error_win, h - 2, (w - strlen(ctrl_msg))/2, ctrl_msg);
+
+	/* Print control message line by line. */
+	for(i = ctrl_msg_n; i > 0U; --i)
+	{
+		const size_t len = strcspn(ctrl_msg, "\n");
+		mvwaddnstr(error_win, h - i - 1, MAX(0, (w - (int)len)/2), ctrl_msg, len);
+		ctrl_msg = skip_char(ctrl_msg + len + 1U, '/');
+	}
+}
+
+/* Counts number of sub-lines (separated by new-line character in the msg.  Sets
+ * *max_len to the length of the longest sub-line.  Returns total number of
+ * sub-lines, which can be zero is msg is an empty line. */
+static size_t
+measure_sub_lines(const char msg[], size_t *max_len)
+{
+	size_t nlines = 0U;
+	*max_len = 0U;
+	while(*msg != '\0')
+	{
+		const size_t len = strcspn(msg, "\n");
+		if(len > *max_len)
+		{
+			*max_len = len;
+		}
+		++nlines;
+		msg += len + (msg[len] == '\n' ? 1U : 0U);
+	}
+	return nlines;
 }
 
 /* Determines maximum width of line in the message.  Returns the width. */
@@ -560,13 +602,16 @@ determine_width(const char msg[])
 }
 
 int
-confirm_deletion(int use_trash)
+confirm_deletion(int nfiles, int use_trash)
 {
 	curr_stats.confirmed = 0;
-	if(!use_trash && cfg.confirm)
+	if(cfg_confirm_delete(use_trash))
 	{
-		const int proceed = prompt_msg("Permanent deletion",
-				"Are you sure you want to delete files permanently?");
+		const char *const title = use_trash ? "Deletion" : "Permanent deletion";
+		char *const msg = format_str("Are you sure you want to delete %d file%s?",
+				nfiles, (nfiles == 1) ? "" : "s");
+		const int proceed = prompt_msg(title, msg);
+		free(msg);
 
 		if(!proceed)
 		{
@@ -611,4 +656,4 @@ show_errors_from_file(FILE *ef, const char title[])
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
