@@ -28,27 +28,30 @@
 #include <stddef.h> /* size_t */
 #include <stdint.h> /* uint64_t */
 #include <stdio.h>
+#include <stdlib.h> /* free() */
 #include <string.h> /* strlen() */
 #include <time.h> /* tm localtime() strftime() */
 
 #include "../cfg/config.h"
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../engine/keys.h"
 #include "../engine/mode.h"
+#include "../int/file_magic.h"
 #include "../menus/menus.h"
 #include "../ui/ui.h"
 #include "../utils/fs.h"
-#include "../utils/fs_limits.h"
+#include "../utils/fsdata.h"
 #include "../utils/macros.h"
+#include "../utils/path.h"
 #include "../utils/str.h"
-#include "../utils/tree.h"
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
 #include "../filelist.h"
-#include "../file_magic.h"
 #include "../status.h"
 #include "../types.h"
 #include "modes.h"
+#include "wk.h"
 
 static void leave_file_info_mode(void);
 static int print_item(const char label[], const char path[], int curr_y);
@@ -62,15 +65,13 @@ static FileView *view;
 static int was_redraw;
 
 static keys_add_info_t builtin_cmds[] = {
-	{L"\x03", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"\x0c", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_l}}},
-	/* return */
-	{L"\x0d", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	/* escape */
-	{L"\x1b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"ZQ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"ZZ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"q", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
+	{WK_C_c,    {{&cmd_ctrl_c}, .descr = "hide file info"}},
+	{WK_C_l,    {{&cmd_ctrl_l}, .descr = "redraw"}},
+	{WK_CR,     {{&cmd_ctrl_c}, .descr = "hide file info"}},
+	{WK_ESC,    {{&cmd_ctrl_c}, .descr = "hide file info"}},
+	{WK_Z WK_Q, {{&cmd_ctrl_c}, .descr = "hide file info"}},
+	{WK_Z WK_Z, {{&cmd_ctrl_c}, .descr = "hide file info"}},
+	{WK_q,      {{&cmd_ctrl_c}, .descr = "hide file info"}},
 };
 
 void
@@ -78,7 +79,8 @@ init_file_info_mode(void)
 {
 	int ret_code;
 
-	ret_code = add_cmds(builtin_cmds, ARRAY_LEN(builtin_cmds), FILE_INFO_MODE);
+	ret_code = vle_keys_add(builtin_cmds, ARRAY_LEN(builtin_cmds),
+			FILE_INFO_MODE);
 	assert(ret_code == 0);
 
 	(void)ret_code;
@@ -135,15 +137,13 @@ redraw_file_info_dialog(void)
 
 	entry = &view->dir_entry[view->list_pos];
 
-	size = 0;
+	size = DCACHE_UNKNOWN;
 	if(entry->type == FT_DIR)
 	{
-		char full_path[PATH_MAX];
-		get_current_full_path(view, sizeof(full_path), full_path);
-		tree_get_data(curr_stats.dirsize_cache, full_path, &size);
+		dcache_get_of(entry, &size, NULL);
 	}
 
-	if(size == 0)
+	if(size == DCACHE_UNKNOWN)
 	{
 		size = entry->size;
 	}
@@ -213,7 +213,7 @@ static int
 print_item(const char label[], const char path[], int curr_y)
 {
 	const int max_width = getmaxx(menu_win) - strlen(label) - 2;
-	const size_t print_len = get_normal_utf8_string_widthn(path, max_width);
+	const size_t print_len = utf8_nstrsnlen(path, max_width);
 
 	mvwaddstr(menu_win, curr_y, 2, label);
 	if(path[print_len] == '\0')
@@ -274,11 +274,14 @@ show_file_type(FileView *view, int curr_y)
 		FILE *pipe;
 		char command[1024];
 		char buf[NAME_MAX];
+		char *escaped_full_path;
 
 		get_current_full_path(view, sizeof(full_path), full_path);
 
 		/* Use the file command to get file information. */
-		snprintf(command, sizeof(command), "file \"%s\" -b", full_path);
+		escaped_full_path = shell_like_escape(full_path, 0);
+		snprintf(command, sizeof(command), "file %s -b", escaped_full_path);
+		free(escaped_full_path);
 
 		if((pipe = popen(command, "r")) == NULL)
 		{
@@ -353,7 +356,7 @@ static int
 show_mime_type(FileView *view, int curr_y)
 {
 	char full_path[PATH_MAX];
-	const char* mimetype = NULL;
+	const char *mimetype = NULL;
 
 	get_current_full_path(view, sizeof(full_path), full_path);
 	mimetype = get_mimetype(full_path);
@@ -361,7 +364,9 @@ show_mime_type(FileView *view, int curr_y)
 	mvwaddstr(menu_win, curr_y, 2, "Mime Type: ");
 
 	if(mimetype == NULL)
+	{
 		mimetype = "Unknown";
+	}
 
 	mvwaddstr(menu_win, curr_y, 13, mimetype);
 
@@ -395,4 +400,4 @@ cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
