@@ -19,39 +19,43 @@
 
 #include "menu.h"
 
-#include <regex.h>
-
 #include <curses.h>
 
 #include <assert.h> /* assert() */
+#include <errno.h> /* errno */
 #include <stddef.h> /* NULL wchar_t */
-#include <stdlib.h> /* free() malloc() */
-#include <string.h>
+#include <stdio.h> /* pclose() popen() */
+#include <stdlib.h> /* free() */
+#include <string.h> /* strerror() */
 
 #include "../cfg/config.h"
+#include "../compat/reallocarray.h"
 #include "../engine/cmds.h"
 #include "../engine/keys.h"
 #include "../engine/mode.h"
 #include "../menus/menus.h"
 #include "../modes/dialogs/msg_dialog.h"
+#include "../ui/fileview.h"
 #include "../ui/statusbar.h"
 #include "../ui/ui.h"
 #include "../utils/macros.h"
+#include "../utils/path.h"
+#include "../utils/str.h"
+#include "../utils/string_array.h"
 #include "../utils/utils.h"
-#include "../commands.h"
+#include "../cmd_core.h"
+#include "../cmd_completion.h"
 #include "../filelist.h"
-#include "../fileview.h"
 #include "../status.h"
 #include "cmdline.h"
 #include "modes.h"
+#include "wk.h"
 
 static const int SCROLL_GAP = 2;
 
-static int complete_args(int id, const cmd_info_t *cmd_info, int arg_pos,
-		void *extra_arg);
 static int swap_range(void);
 static int resolve_mark(char mark);
-static char * menu_expand_macros(const char *str, int for_shell, int *usr1,
+static char * menu_expand_macros(const char str[], int for_shell, int *usr1,
 		int *usr2);
 static char * menu_expand_envvars(const char *str);
 static void post(int id);
@@ -59,7 +63,6 @@ static void menu_select_range(int id, const cmd_info_t *cmd_info);
 static int skip_at_beginning(int id, const char *args);
 
 static int key_handler(wchar_t key);
-static void leave_menu_mode(void);
 static void cmd_ctrl_b(key_info_t key_info, keys_info_t *keys_info);
 static int can_scroll_menu_up(const menu_info *menu);
 static void cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info);
@@ -69,20 +72,22 @@ static void cmd_ctrl_f(key_info_t key_info, keys_info_t *keys_info);
 static int can_scroll_menu_down(const menu_info *menu);
 static void change_menu_top(menu_info *const menu, int delta);
 static void cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info);
-static void cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_return(key_info_t key_info, keys_info_t *keys_info);
 static void update_ui_on_leaving(void);
 static void cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info);
 static int get_effective_menu_scroll_offset(const menu_info *menu);
 static void cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_slash(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_colon(key_info_t key_info, keys_info_t *keys_info);
-static void cmd_question(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_qmark(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_B(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_G(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_H(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_L(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_M(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_N(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_b(key_info_t key_info, keys_info_t *keys_info);
+static void dump_into_custom_view(int very);
 static void cmd_dd(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gf(key_info_t key_info, keys_info_t *keys_info);
 static int pass_combination_to_khandler(const wchar_t keys[]);
@@ -90,7 +95,7 @@ static void cmd_gg(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_j(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_k(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_n(key_info_t key_info, keys_info_t *keys_info);
-static void search(int backward);
+static void cmd_v(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zb(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zH(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zL(key_info_t key_info, keys_info_t *keys_info);
@@ -99,103 +104,112 @@ static void cmd_zl(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zt(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_zz(key_info_t key_info, keys_info_t *keys_info);
 static int all_lines_visible(const menu_info *const menu);
-
 static int goto_cmd(const cmd_info_t *cmd_info);
+static int nohlsearch_cmd(const cmd_info_t *cmd_info);
 static int quit_cmd(const cmd_info_t *cmd_info);
-
-static int search_menu(menu_info *m, int start_pos);
-static int search_menu_forwards(menu_info *m, int start_pos);
-static int search_menu_backwards(menu_info *m, int start_pos);
+static int write_cmd(const cmd_info_t *cmd_info);
+static void leave_menu_mode(int reset_selection);
 
 static FileView *view;
 static menu_info *menu;
 static int last_search_backward;
 static int was_redraw;
 static int saved_top, saved_pos;
-static int search_repeat;
 
 static keys_add_info_t builtin_cmds[] = {
-	{L"\x02", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_b}}},
-	{L"\x03", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"\x04", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_d}}},
-	{L"\x05", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_e}}},
-	{L"\x06", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_f}}},
-	{L"\x0c", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_l}}},
-	/* return */
-	{L"\x0d", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_m}}},
-	{L"\x0e", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"\x10", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{L"\x15", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_u}}},
-	{L"\x19", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_y}}},
-	/* escape */
-	{L"\x1b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"/", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_slash}}},
-	{L":", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_colon}}},
-	{L"?", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_question}}},
-	{L"G", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_G}}},
-	{L"H", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_H}}},
-	{L"L", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_L}}},
-	{L"M", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_M}}},
-	{L"N", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_N}}},
-	{L"ZZ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"ZQ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_b}}},
-	{L"dd", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_dd}}},
-	{L"gf", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_gf}}},
-	{L"gg", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_gg}}},
-	{L"j", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"k", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{L"l", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_m}}},
-	{L"n", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_n}}},
-	{L"q", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_c}}},
-	{L"zb", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zb}}},
-	{L"zH", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zH}}},
-	{L"zL", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zL}}},
-	{L"zh", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zh}}},
-	{L"zl", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zl}}},
-	{L"zt", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zt}}},
-	{L"zz", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zz}}},
+	{WK_C_b,    {{&cmd_ctrl_b}, .descr = "scroll page up"}},
+	{WK_C_c,    {{&cmd_ctrl_c}, .descr = "leave menu mode"}},
+	{WK_C_d,    {{&cmd_ctrl_d}, .descr = "scroll half-page down"}},
+	{WK_C_e,    {{&cmd_ctrl_e}, .descr = "scroll one line down"}},
+	{WK_C_f,    {{&cmd_ctrl_f}, .descr = "scroll page down"}},
+	{WK_C_l,    {{&cmd_ctrl_l}, .descr = "redraw"}},
+	{WK_CR,     {{&cmd_return}, .descr = "pick current item"}},
+	{WK_C_n,    {{&cmd_j},      .descr = "go to item below"}},
+	{WK_C_p,    {{&cmd_k},      .descr = "go to item above"}},
+	{WK_C_u,    {{&cmd_ctrl_u}, .descr = "scroll half-page up"}},
+	{WK_C_y,    {{&cmd_ctrl_y}, .descr = "scroll one line up"}},
+	{WK_ESC,    {{&cmd_ctrl_c}, .descr = "leave menu mode"}},
+	{WK_SLASH,  {{&cmd_slash},  .descr = "search forward"}},
+	{WK_COLON,  {{&cmd_colon},  .descr = "go to cmdline mode"}},
+	{WK_QM,     {{&cmd_qmark},  .descr = "search backward"}},
+	{WK_B,      {{&cmd_B},      .descr = "make unsorted custom view"}},
+	{WK_G,      {{&cmd_G},      .descr = "go to the last item"}},
+	{WK_H,      {{&cmd_H},      .descr = "go to top of viewport"}},
+	{WK_L,      {{&cmd_L},      .descr = "go to bottom of viewport"}},
+	{WK_M,      {{&cmd_M},      .descr = "go to middle of viewport"}},
+	{WK_N,      {{&cmd_N},      .descr = "go to previous search match"}},
+	{WK_Z WK_Z, {{&cmd_ctrl_c}, .descr = "leave menu mode"}},
+	{WK_Z WK_Q, {{&cmd_ctrl_c}, .descr = "leave menu mode"}},
+	{WK_b,      {{&cmd_b},      .descr = "make custom view"}},
+	{WK_d WK_d, {{&cmd_dd},     .descr = "remove files"}},
+	{WK_g WK_f, {{&cmd_gf},     .descr = "navigate to file location"}},
+	{WK_g WK_g, {{&cmd_gg},     .descr = "go to the first item"}},
+	{WK_j,      {{&cmd_j},      .descr = "go to item below"}},
+	{WK_k,      {{&cmd_k},      .descr = "go to item above"}},
+	{WK_l,      {{&cmd_return}, .descr = "pick current item"}},
+	{WK_n,      {{&cmd_n},      .descr = "go to next search match"}},
+	{WK_q,      {{&cmd_ctrl_c}, .descr = "leave menu mode"}},
+	{WK_v,      {{&cmd_v},      .descr = "use items as Vim quickfix list"}},
+	{WK_z WK_b, {{&cmd_zb},     .descr = "push cursor to the bottom"}},
+	{WK_z WK_H, {{&cmd_zH},     .descr = "scroll page left"}},
+	{WK_z WK_L, {{&cmd_zL},     .descr = "scroll page right"}},
+	{WK_z WK_h, {{&cmd_zh},     .descr = "scroll one column left"}},
+	{WK_z WK_l, {{&cmd_zl},     .descr = "scroll one column right"}},
+	{WK_z WK_t, {{&cmd_zt},     .descr = "push cursor to the top"}},
+	{WK_z WK_z, {{&cmd_zz},     .descr = "center cursor position"}},
 #ifdef ENABLE_EXTENDED_KEYS
-	{{KEY_PPAGE}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_b}}},
-	{{KEY_NPAGE}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_f}}},
-	{{KEY_UP}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{{KEY_DOWN}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{{KEY_RIGHT}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_m}}},
-	{{KEY_HOME}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_gg}}},
-	{{KEY_END}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_G}}},
-	{{L'z', KEY_LEFT}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zh}}},
-	{{L'z', KEY_RIGHT}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_zl}}},
+	{{KEY_PPAGE},       {{&cmd_ctrl_b}, .descr = "scroll page up"}},
+	{{KEY_NPAGE},       {{&cmd_ctrl_f}, .descr = "scroll page down"}},
+	{{KEY_UP},          {{&cmd_k},      .descr = "go to item above"}},
+	{{KEY_DOWN},        {{&cmd_j},      .descr = "go to item below"}},
+	{{KEY_RIGHT},       {{&cmd_return}, .descr = "pick current item"}},
+	{{KEY_HOME},        {{&cmd_gg},     .descr = "go to the first item"}},
+	{{KEY_END},         {{&cmd_G},      .descr = "go to the last item"}},
+	{{WC_z, KEY_LEFT},  {{&cmd_zh},     .descr = "scroll one column left"}},
+	{{WC_z, KEY_RIGHT}, {{&cmd_zl},     .descr = "scroll one column right"}},
 #endif /* ENABLE_EXTENDED_KEYS */
 };
 
+/* Specification of builtin commands. */
 static const cmd_add_t commands[] = {
-	{ .name = "",                 .abbr = NULL,    .emark = 0,  .id = -1,              .range = 1,    .bg = 0, .quote = 0, .regexp = 0,
-		.handler = goto_cmd,        .qmark = 0,      .expand = 0, .cust_sep = 0,         .min_args = 0, .max_args = 0,       .select = 0, },
-	{ .name = "exit",             .abbr = "exi",   .emark = 1,  .id = -1,              .range = 0,    .bg = 0, .quote = 0, .regexp = 0,
-		.handler = quit_cmd,        .qmark = 0,      .expand = 0, .cust_sep = 0,         .min_args = 0, .max_args = 0,       .select = 0, },
-	{ .name = "quit",             .abbr = "q",     .emark = 1,  .id = -1,              .range = 0,    .bg = 0, .quote = 0, .regexp = 0,
-		.handler = quit_cmd,        .qmark = 0,      .expand = 0, .cust_sep = 0,         .min_args = 0, .max_args = 0,       .select = 0, },
-	{ .name = "xit",              .abbr = "x",     .emark = 1,  .id = -1,              .range = 0,    .bg = 0, .quote = 0, .regexp = 0,
-		.handler = quit_cmd,        .qmark = 0,      .expand = 0, .cust_sep = 0,         .min_args = 0, .max_args = 0,       .select = 0, },
+	{ .name = "",                  .abbr = NULL,    .id = -1,
+	  .descr = "navigate to specific line",
+	  .flags = HAS_RANGE,
+	  .handler = &goto_cmd,        .min_args = 0,   .max_args = 0, },
+	{ .name = "exit",              .abbr = "exi",   .id = -1,
+	  .descr = "exit the application",
+	  .flags = 0,
+	  .handler = &quit_cmd,        .min_args = 0,   .max_args = 0, },
+	{ .name = "nohlsearch",        .abbr = "noh",   .id = -1,
+	  .descr = "reset highlighting of search matches",
+	  .flags = 0,
+	  .handler = &nohlsearch_cmd,  .min_args = 0,   .max_args = 0, },
+	{ .name = "quit",              .abbr = "q",     .id = -1,
+	  .descr = "exit the application",
+	  .flags = 0,
+	  .handler = &quit_cmd,        .min_args = 0,   .max_args = 0, },
+	{ .name = "write",             .abbr = "w",     .id = COM_MENU_WRITE,
+	  .descr = "write all menu lines into a file",
+	  .flags = HAS_QUOTED_ARGS,
+	  .handler = &write_cmd,       .min_args = 1,   .max_args = 1, },
+	{ .name = "xit",               .abbr = "x",     .id = -1,
+	  .descr = "exit the application",
+	  .flags = 0,
+	  .handler = &quit_cmd,        .min_args = 0,   .max_args = 0, },
 };
 
+/* Settings for the cmds unit. */
 static cmds_conf_t cmds_conf = {
-	.complete_args = complete_args,
-	.swap_range = swap_range,
-	.resolve_mark = resolve_mark,
-	.expand_macros = menu_expand_macros,
-	.expand_envvars = menu_expand_envvars,
-	.post = post,
-	.select_range = menu_select_range,
-	.skip_at_beginning = skip_at_beginning,
+	.complete_args = &complete_args,
+	.swap_range = &swap_range,
+	.resolve_mark = &resolve_mark,
+	.expand_macros = &menu_expand_macros,
+	.expand_envvars = &menu_expand_envvars,
+	.post = &post,
+	.select_range = &menu_select_range,
+	.skip_at_beginning = &skip_at_beginning,
 };
 
-static int
-complete_args(int id, const cmd_info_t *cmd_info, int arg_pos, void *extra_arg)
-{
-	return 0;
-}
-
 static int
 swap_range(void)
 {
@@ -208,8 +222,10 @@ resolve_mark(char mark)
 	return -1;
 }
 
+/* Implementation of macros expansion callback for cmds unit.  Returns newly
+ * allocated memory. */
 static char *
-menu_expand_macros(const char *str, int for_shell, int *usr1, int *usr2)
+menu_expand_macros(const char str[], int for_shell, int *usr1, int *usr2)
 {
 	return strdup(str);
 }
@@ -241,12 +257,12 @@ init_menu_mode(void)
 {
 	int ret_code;
 
-	ret_code = add_cmds(builtin_cmds, ARRAY_LEN(builtin_cmds), MENU_MODE);
+	ret_code = vle_keys_add(builtin_cmds, ARRAY_LEN(builtin_cmds), MENU_MODE);
 	assert(ret_code == 0);
 
 	(void)ret_code;
 
-	set_def_handler(MENU_MODE, key_handler);
+	vle_keys_set_def_handler(MENU_MODE, key_handler);
 
 	init_cmds(0, &cmds_conf);
 	add_builtin_commands((const cmd_add_t *)&commands, ARRAY_LEN(commands));
@@ -260,7 +276,7 @@ key_handler(wchar_t key)
 	if(pass_combination_to_khandler(shortcut) && menu->len == 0)
 	{
 		show_error_msg("No more items in the menu", "Menu will be closed");
-		leave_menu_mode();
+		leave_menu_mode(1);
 	}
 
 	return 0;
@@ -310,19 +326,6 @@ menu_redraw(void)
 	redraw_menu(menu);
 }
 
-static void
-leave_menu_mode(void)
-{
-	reset_popup_menu(menu);
-
-	clean_selected_files(view);
-	redraw_view(view);
-
-	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
-
-	update_ui_on_leaving();
-}
-
 static void
 cmd_ctrl_b(key_info_t key_info, keys_info_t *keys_info)
 {
@@ -349,7 +352,7 @@ can_scroll_menu_up(const menu_info *menu)
 static void
 cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 {
-	leave_menu_mode();
+	leave_menu_mode(1);
 }
 
 static void
@@ -423,7 +426,7 @@ cmd_ctrl_l(key_info_t key_info, keys_info_t *keys_info)
 }
 
 static void
-cmd_ctrl_m(key_info_t key_info, keys_info_t *keys_info)
+cmd_return(key_info_t key_info, keys_info_t *keys_info)
 {
 	static menu_info *saved_menu;
 
@@ -504,12 +507,12 @@ cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_slash(key_info_t key_info, keys_info_t *keys_info)
 {
-	search_repeat = (key_info.count == NO_COUNT_GIVEN) ? 1 : key_info.count;
+	menu->search_repeat = def_count(key_info.count);
 	last_search_backward = 0;
-	menu->match_dir = NONE;
+	menu->backward_search = 0;
 	free(menu->regexp);
 	menu->regexp = NULL;
-	enter_cmdline_mode(CLS_MENU_FSEARCH, L"", menu);
+	enter_cmdline_mode(CLS_MENU_FSEARCH, "", menu);
 }
 
 static void
@@ -518,17 +521,24 @@ cmd_colon(key_info_t key_info, keys_info_t *keys_info)
 	cmds_conf.begin = 1;
 	cmds_conf.current = menu->pos;
 	cmds_conf.end = menu->len;
-	enter_cmdline_mode(CLS_MENU_COMMAND, L"", menu);
+	enter_cmdline_mode(CLS_MENU_COMMAND, "", menu);
 }
 
 static void
-cmd_question(key_info_t key_info, keys_info_t *keys_info)
+cmd_qmark(key_info_t key_info, keys_info_t *keys_info)
 {
-	search_repeat = (key_info.count == NO_COUNT_GIVEN) ? 1 : key_info.count;
+	menu->search_repeat = def_count(key_info.count);
 	last_search_backward = 1;
-	menu->match_dir = NONE;
+	menu->backward_search = 1;
 	free(menu->regexp);
-	enter_cmdline_mode(CLS_MENU_BSEARCH, L"", menu);
+	enter_cmdline_mode(CLS_MENU_BSEARCH, "", menu);
+}
+
+/* Populates very custom (unsorted) view with list of files. */
+static void
+cmd_B(key_info_t key_info, keys_info_t *keys_info)
+{
+	dump_into_custom_view(1);
 }
 
 static void
@@ -587,41 +597,50 @@ cmd_L(key_info_t key_info, keys_info_t *keys_info)
 	wrefresh(menu_win);
 }
 
+/* Moves cursor to the middle of the window. */
 static void
 cmd_M(key_info_t key_info, keys_info_t *keys_info)
 {
 	int new_pos;
 	if(menu->len < menu->win_rows)
-		new_pos = menu->len/2;
+		new_pos = DIV_ROUND_UP(menu->len, 2);
 	else
-		new_pos = menu->top + (menu->win_rows - 3)/2;
+		new_pos = menu->top + DIV_ROUND_UP(menu->win_rows - 3, 2);
 
 	clean_menu_position(menu);
-	move_to_menu_pos(new_pos, menu);
+	move_to_menu_pos(MAX(0, new_pos - 1), menu);
 	wrefresh(menu_win);
 }
 
 static void
 cmd_N(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(key_info.count == NO_COUNT_GIVEN)
-		key_info.count = 1;
+	key_info.count = def_count(key_info.count);
 	while(key_info.count-- > 0)
-		search(!last_search_backward);
+	{
+		menus_search(menu, !last_search_backward);
+	}
 }
 
-/* Populates view with list of files. */
+/* Populates custom view with list of files. */
 static void
 cmd_b(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(menu_to_custom_view(menu, view) != 0)
+	dump_into_custom_view(0);
+}
+
+/* Makes custom view of specified type out of menu items. */
+static void
+dump_into_custom_view(int very)
+{
+	if(menu_to_custom_view(menu, view, very) != 0)
 	{
 		show_error_msg("Menu transformation",
 				"No valid paths discovered in menu content");
 		return;
 	}
 
-	leave_menu_mode();
+	leave_menu_mode(1);
 }
 
 static void
@@ -630,7 +649,7 @@ cmd_dd(key_info_t key_info, keys_info_t *keys_info)
 	if(pass_combination_to_khandler(L"dd") && menu->len == 0)
 	{
 		show_error_msg("Menu is closing", "No more items in the menu");
-		leave_menu_mode();
+		leave_menu_mode(1);
 	}
 }
 
@@ -661,7 +680,10 @@ pass_combination_to_khandler(const wchar_t keys[])
 			wrefresh(menu_win);
 			return 1;
 		case KHR_CLOSE_MENU:
-			leave_menu_mode();
+			leave_menu_mode(1);
+			return 1;
+		case KHR_MORPHED_MENU:
+			assert(!vle_mode_is(MENU_MODE) && "Wrong use of KHR_MORPHED_MENU.");
 			return 1;
 		case KHR_UNHANDLED:
 			return 0;
@@ -714,31 +736,73 @@ cmd_k(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_n(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(key_info.count == NO_COUNT_GIVEN)
-		key_info.count = 1;
+	key_info.count = def_count(key_info.count);
 	while(key_info.count-- > 0)
-		search(last_search_backward);
+	{
+		menus_search(menu, last_search_backward);
+	}
 }
 
+/* Handles current content of the menu to Vim as quickfix list. */
 static void
-search(int backward)
+cmd_v(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(menu->regexp != NULL)
-	{
-		menu->match_dir = backward ? UP : DOWN;
-		(void)search_menu_list(NULL, menu);
-		wrefresh(menu_win);
+	int bg;
+	const char *vi_cmd;
+	FILE *vim_stdin;
+	char *cmd;
+	int i;
+	int qf = 1;
 
-		if(menu->matching_entries > 0)
-		{
-			status_bar_messagef("%c%s", backward ? '?' : '/', menu->regexp);
-		}
+	/* If both first and last lines do not contain colons, treat lines as list of
+	 * file names. */
+	if(strchr(menu->items[0], ':') == NULL &&
+			strchr(menu->items[menu->len - 1], ':') == NULL)
+	{
+		qf = 0;
+	}
+
+	endwin();
+	curr_stats.need_update = UT_FULL;
+
+	vi_cmd = cfg_get_vicmd(&bg);
+	if(!qf)
+	{
+		char *const arg = shell_like_escape("+exe 'bd!|args' "
+				"join(map(getline('1','$'),'fnameescape(v:val)'))", 0);
+		cmd = format_str("%s %s +argument%d -", vi_cmd, arg, menu->pos + 1);
+		free(arg);
+	}
+	else if(menu->pos == 0)
+	{
+		/* For some reason +cc1 causes noisy messages on status line, so handle this
+		 * case separately. */
+		cmd = format_str("%s +cgetbuffer +bd! +cfirst -", vi_cmd);
 	}
 	else
 	{
-		status_bar_error("No search pattern set");
+		cmd = format_str("%s +cgetbuffer +bd! +cfirst +cc%d -", vi_cmd,
+				menu->pos + 1);
 	}
-	curr_stats.save_msg = 1;
+
+	vim_stdin = popen(cmd, "w");
+	free(cmd);
+
+	if(vim_stdin == NULL)
+	{
+		recover_after_shellout();
+		show_error_msg("Vim QuickFix", "Failed to send list of files to editor.");
+		return;
+	}
+
+	for(i = 0; i < menu->len; ++i)
+	{
+		fputs(menu->items[i], vim_stdin);
+		putc('\n', vim_stdin);
+	}
+
+	pclose(vim_stdin);
+	recover_after_shellout();
 }
 
 static void
@@ -845,10 +909,31 @@ goto_cmd(const cmd_info_t *cmd_info)
 	return 0;
 }
 
+/* Disables highlight of search result matches. */
+static int
+nohlsearch_cmd(const cmd_info_t *cmd_info)
+{
+	menus_reset_search_highlight(menu);
+	return 0;
+}
+
 static int
 quit_cmd(const cmd_info_t *cmd_info)
 {
-	leave_menu_mode();
+	leave_menu_mode(1);
+	return 0;
+}
+
+/* Writes all menu lines into file specified as argument. */
+static int
+write_cmd(const cmd_info_t *cmd_info)
+{
+	char *const no_tilde = expand_tilde(cmd_info->argv[0]);
+	if(write_file_of_lines(no_tilde, menu->items, menu->len) != 0)
+	{
+		show_error_msg("Failed to open output file", strerror(errno));
+	}
+	free(no_tilde);
 	return 0;
 }
 
@@ -866,194 +951,54 @@ load_menu_pos(void)
 	menu->pos = saved_pos;
 }
 
-int
-search_menu_list(const char pattern[], menu_info *m)
+void
+menu_morph_into_cmdline(CmdLineSubmode submode, const char input[],
+		int external)
 {
-	int save = 0;
-	int i;
+	/* input might point to part of menu data. */
+	char *input_copy;
 
-	if(pattern != NULL)
+	if(input[0] == '\0')
 	{
-		m->regexp = strdup(pattern);
-		if(search_menu(m, m->pos) != 0)
-		{
-			draw_menu(m);
-			move_to_menu_pos(m->pos, m);
-			return 1;
-		}
-		draw_menu(m);
+		show_error_msg("Command insertion", "Ignoring empty command");
+		return;
 	}
 
-	for(i = 0; i < search_repeat; ++i)
+	input_copy = external ? format_str("!%s", input) : strdup(input);
+	if(input_copy == NULL)
 	{
-		switch(m->match_dir)
-		{
-			case NONE:
-			case DOWN:
-				save = search_menu_forwards(m, m->pos + 1);
-				break;
-			case UP:
-				save = search_menu_backwards(m, m->pos - 1);
-				break;
-			default:
-				break;
-		}
+		show_error_msg("Error", "Not enough memory");
+		return;
 	}
-	return save;
+
+	leave_menu_mode(0);
+	enter_cmdline_mode(submode, input_copy, NULL);
+
+	free(input_copy);
 }
 
-/* Returns non-zero on error */
-static int
-search_menu(menu_info *m, int start_pos)
+/* Leaves menu mode, possibly resetting selection.  Does nothing if current mode
+ * isn't menu mode. */
+static void
+leave_menu_mode(int reset_selection)
 {
-	int cflags;
-	regex_t re;
-	int err;
-
-	if(m->matches == NULL)
-		m->matches = malloc(sizeof(int)*m->len);
-
-	memset(m->matches, 0, sizeof(int)*m->len);
-	m->matching_entries = 0;
-
-	if(m->regexp[0] == '\0')
-		return 0;
-
-	cflags = get_regexp_cflags(m->regexp);
-	if((err = regcomp(&re, m->regexp, cflags)) == 0)
+	/* Some menu implementation could have switched mode from one of handlers. */
+	if(!vle_mode_is(MENU_MODE))
 	{
-		int x;
-		for(x = 0; x < m->len; x++)
-		{
-			if(regexec(&re, m->items[x], 0, NULL, 0) != 0)
-				continue;
-			m->matches[x] = 1;
-
-			m->matching_entries++;
-		}
-		regfree(&re);
-		return 0;
-	}
-	else
-	{
-		status_bar_errorf("Regexp error: %s", get_regexp_error(err, &re));
-		regfree(&re);
-		return -1;
-	}
-}
-
-static int
-search_menu_forwards(menu_info *m, int start_pos)
-{
-	/* FIXME: code duplication with search_menu_backwards. */
-
-	int match_up = -1;
-	int match_down = -1;
-	int x;
-
-	for(x = 0; x < m->len; x++)
-	{
-		if(!m->matches[x])
-			continue;
-
-		if(match_up < 0)
-		{
-			if(x < start_pos)
-				match_up = x;
-		}
-		if(match_down < 0)
-		{
-			if(x >= start_pos)
-				match_down = x;
-		}
+		return;
 	}
 
-	if(match_up > -1 || match_down > -1)
+	reset_popup_menu(menu);
+
+	if(reset_selection)
 	{
-		int pos;
-
-		if(!cfg.wrap_scan && match_down <= -1)
-		{
-			status_bar_errorf("Search hit BOTTOM without match for: %s", m->regexp);
-			return 1;
-		}
-
-		pos = (match_down > -1) ? match_down : match_up;
-
-		clean_menu_position(m);
-		move_to_menu_pos(pos, m);
-		menu_print_search_msg(m);
-	}
-	else
-	{
-		move_to_menu_pos(m->pos, m);
-		if(!cfg.wrap_scan)
-		{
-			status_bar_errorf("Search hit BOTTOM without match for: %s", m->regexp);
-		}
-		else
-		{
-			menu_print_search_msg(m);
-		}
-	}
-	return 1;
-}
-
-static int
-search_menu_backwards(menu_info *m, int start_pos)
-{
-	/* FIXME: code duplication with search_menu_forwards. */
-
-	int match_up = -1;
-	int match_down = -1;
-	int x;
-
-	for(x = m->len - 1; x > -1; x--)
-	{
-		if(!m->matches[x])
-			continue;
-
-		if(match_up < 0)
-		{
-			if(x <= start_pos)
-				match_up = x;
-		}
-		if(match_down < 0)
-		{
-			if(x > start_pos)
-				match_down = x;
-		}
+		clean_selected_files(view);
+		redraw_view(view);
 	}
 
-	if(match_up > -1 || match_down > -1)
-	{
-		int pos;
+	vle_mode_set(NORMAL_MODE, VMT_PRIMARY);
 
-		if(!cfg.wrap_scan && match_up <= -1)
-		{
-			status_bar_errorf("Search hit TOP without match for: %s", m->regexp);
-			return 1;
-		}
-
-		pos = (match_up > -1) ? match_up : match_down;
-
-		clean_menu_position(m);
-		move_to_menu_pos(pos, m);
-		menu_print_search_msg(m);
-	}
-	else
-	{
-		move_to_menu_pos(m->pos, m);
-		if(!cfg.wrap_scan)
-		{
-			status_bar_errorf("Search hit TOP without match for: %s", m->regexp);
-		}
-		else
-		{
-			menu_print_search_msg(m);
-		}
-	}
-	return 1;
+	update_ui_on_leaving();
 }
 
 void
@@ -1066,42 +1011,5 @@ execute_cmdline_command(const char cmd[])
 	init_cmds(0, &cmds_conf);
 }
 
-void
-menu_print_search_msg(const menu_info *m)
-{
-	int cflags;
-	regex_t re;
-	int err;
-
-	/* Can be NULL after regex compilation failure. */
-	if(m->regexp == NULL)
-	{
-		return;
-	}
-
-	cflags = get_regexp_cflags(m->regexp);
-	err = regcomp(&re, m->regexp, cflags);
-
-	if(err != 0)
-	{
-		status_bar_errorf("Regexp (%s) error: %s", m->regexp,
-				get_regexp_error(err, &re));
-		regfree(&re);
-		return;
-	}
-
-	regfree(&re);
-
-	if(m->matching_entries > 0)
-	{
-		status_bar_messagef("%d %s", m->matching_entries,
-				(m->matching_entries == 1) ? "match" : "matches");
-	}
-	else
-	{
-		status_bar_errorf("No matches for: %s", m->regexp);
-	}
-}
-
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
