@@ -22,8 +22,9 @@
 
 #include "../ui/ui.h"
 #include "../menus/menus.h"
+#include "cmdline.h"
 
-/* Initiailizes menu mode. */
+/* Initializes menu mode. */
 void init_menu_mode(void);
 
 void enter_menu_mode(menu_info *m, FileView *active_view);
@@ -43,22 +44,21 @@ void save_menu_pos(void);
 
 void load_menu_pos(void);
 
-/* Performs search of pattern among menu items.  NULL pattern requests use of
- * results of the last used pattern.  Returns new value for save_msg flag. */
-int search_menu_list(const char pattern[], menu_info *m);
+/* Leaves menu and starts command-line mode.  external flag shows whether
+ * cmd should be prepended with ":!".  To be used from keyboard handlers, which
+ * in this case must return KHR_MORPHED_MENU. */
+void menu_morph_into_cmdline(CmdLineSubmode submode, const char input[],
+		int external);
 
 /* Allows running regular command-line mode commands from menu mode. */
 void execute_cmdline_command(const char cmd[]);
 
 /* Returns index of last visible line in the menu.  Value returned may be
  * greater than or equal to number of lines in the menu, which should be
- * threated correctly. */
+ * treated correctly. */
 int get_last_visible_line(const menu_info *m);
 
-/* Prints results or error message about search operation to the user. */
-void menu_print_search_msg(const menu_info *m);
-
 #endif /* VIFM__MODES__MENU_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
