@@ -32,6 +32,7 @@ void normal_cmd_zt(key_info_t key_info, keys_info_t *keys_info);
 
 void normal_cmd_zz(key_info_t key_info, keys_info_t *keys_info);
 
+/* Centers the splitter. */
 void normal_cmd_ctrl_wequal(key_info_t key_info, keys_info_t *keys_info);
 
 void normal_cmd_ctrl_wless(key_info_t key_info, keys_info_t *keys_info);
@@ -46,14 +47,14 @@ void normal_cmd_ctrl_wpipe(key_info_t key_info, keys_info_t *keys_info);
 
 int ffind(int ch, int backward, int wrap);
 
-int cmd_paren(int lb, int ub, int inc);
-
 /* Kind of callback to allow starting searches from the module and rely on other
- * modules.  Returns new value for status bar message flag. */
+ * modules.  Returns new value for status bar message flag, but when
+ * print_errors isn't requested can return -1 to indicate issues with the
+ * pattern. */
 int find_npattern(FileView *view, const char pattern[], int backward,
-		int interactive);
+		int print_errors);
 
 #endif /* VIFM__MODES__NORMAL_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
