@@ -21,46 +21,48 @@
 #include <curses.h>
 
 #include <regex.h>
+#include <unistd.h> /* usleep() */
 
 #include <assert.h> /* assert() */
 #include <stddef.h> /* ptrdiff_t size_t */
 #include <string.h> /* memset() strdup() */
 #include <stdio.h>  /* fclose() snprintf() */
-#include <stdlib.h> /* free() malloc() */
+#include <stdlib.h> /* free() */
 
 #include "../cfg/config.h"
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
+#include "../compat/reallocarray.h"
 #include "../engine/keys.h"
 #include "../engine/mode.h"
+#include "../int/vim.h"
 #include "../modes/dialogs/msg_dialog.h"
+#include "../ui/cancellation.h"
+#include "../ui/color_manager.h"
+#include "../ui/colors.h"
+#include "../ui/escape.h"
+#include "../ui/fileview.h"
+#include "../ui/quickview.h"
 #include "../ui/statusbar.h"
 #include "../ui/ui.h"
 #include "../utils/filemon.h"
 #include "../utils/fs.h"
-#include "../utils/fs_limits.h"
 #include "../utils/macros.h"
 #include "../utils/path.h"
+#include "../utils/regexp.h"
 #include "../utils/str.h"
 #include "../utils/string_array.h"
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
-#include "../color_manager.h"
-#include "../colors.h"
-#include "../escape.h"
 #include "../filelist.h"
 #include "../filetype.h"
-#include "../fileview.h"
-#include "../quickview.h"
 #include "../running.h"
 #include "../status.h"
 #include "../types.h"
-#include "../vim.h"
 #include "cmdline.h"
 #include "modes.h"
 #include "normal.h"
-
-/* Column at which view content should be displayed. */
-#define COL 1
+#include "wk.h"
 
 /* Named boolean values of "silent" parameter for better readability. */
 enum
@@ -69,28 +71,37 @@ enum
 	SILENT,   /* Do not display error message dialog. */
 };
 
+/* Describes view state and its properties. */
 typedef struct
 {
-	char **lines;
-	int (*widths)[2];
-	int nlines;
-	int nlinesv;
-	int line;
-	int linev;
-	int win_size; /* Scroll window size. */
-	int half_win;
-	int width;
-	FileView *view;
-	regex_t re;
-	int last_search_backward; /* Value -1 means no search was performed. */
-	int search_repeat; /* Saved count prefix of search commands. */
-	int wrap;
-	int abandoned; /* Shows whether view mode was abandoned. */
-	char *filename;
-	int graphics; /* Whether viewer presumably displays graphics. */
+	/* Data of the view. */
+	char **lines;     /* List of real lines. */
+	int (*widths)[2]; /* (virtual line, screen width) pair per real line. */
+	int nlines;       /* Number of real lines. */
+	int nlinesv;      /* Number of virtual (possibly wrapped) lines. */
+	int line;         /* Current real line number. */
+	int linev;        /* Current virtual line number. */
 
+	/* Dimensions, units of actions. */
+	int win_size; /* Scroll window size. */
+	int half_win; /* Height of a "page" (can be changed). */
+	int width;    /* Last width used for breaking lines. */
+
+	/* Monitoring of changes for automatic forwarding. */
 	int auto_forward;   /* Whether auto forwarding (tail -F) is enabled. */
 	filemon_t file_mon; /* File monitor for auto forwarding mode. */
+
+	/* Related to search. */
+	regex_t re;               /* Search regular expression. */
+	int last_search_backward; /* Value -1 means no search was performed. */
+	int search_repeat;        /* Saved count prefix of search commands. */
+
+	/* The rest of the state. */
+	FileView *view; /* File view association with the view. */
+	char *filename; /* Full path to the file being viewed. */
+	int abandoned;  /* Whether view mode was abandoned. */
+	int graphics;   /* Whether viewer presumably displays graphics. */
+	int wrap;       /* Whether lines are wrapped. */
 }
 view_info_t;
 
@@ -176,109 +187,108 @@ static int forward_if_changed(view_info_t *vi);
 static int scroll_to_bottom(view_info_t *vi);
 static void reload_view(view_info_t *vi, int silent);
 
-view_info_t view_info[VI_COUNT];
-view_info_t* vi = &view_info[VI_QV];
+static view_info_t view_info[VI_COUNT];
+static view_info_t* vi = &view_info[VI_QV];
 
 static keys_add_info_t builtin_cmds[] = {
-	{L"\x02", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_b}}},
-	{L"\x04", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_d}}},
-	{L"\x05", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"\x06", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_f}}},
-	{L"\x09", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_tab}}},
-	{L"\x0b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{L"\x0c", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_l}}},
-	{L"\x0d", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"\x0e", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"\x10", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{L"\x12", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_l}}},
-	{L"\x15", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_u}}},
-	{L"\x16", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_f}}},
-	{L"\x17H", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wH}}},
-	{L"\x17J", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wJ}}},
-	{L"\x17K", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wK}}},
-	{L"\x17L", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wL}}},
-	{L"\x17"L"b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wb}}},
-	{L"\x17\x08", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wh}}},
-	{L"\x17h", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wh}}},
-	{L"\x17\x09", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wj}}},
-	{L"\x17j", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wj}}},
-	{L"\x17\x0b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wk}}},
-	{L"\x17k", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wk}}},
-	{L"\x17\x0c", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wl}}},
-	{L"\x17l", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wl}}},
-	{L"\x17\x0f", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wo}}},
-	{L"\x17o", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wo}}},
-	{L"\x17\x10", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_ww}}},
-	{L"\x17p", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_ww}}},
-	{L"\x17\x14", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wt}}},
-	{L"\x17t", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wt}}},
-	{L"\x17\x17", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_ww}}},
-	{L"\x17w", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_ww}}},
-	{L"\x17\x18", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wx}}},
-	{L"\x17x", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wx}}},
-	{L"\x17\x1a", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wz}}},
-	{L"\x17z", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wz}}},
-	{L"\x17\x13", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_ws}}},
-	{L"\x17s", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_ws}}},
-	{L"\x17\x16", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wv}}},
-	{L"\x17v", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_wv}}},
-	{L"\x17=", {BUILTIN_NIM_KEYS, FOLLOWED_BY_NONE, {.handler = normal_cmd_ctrl_wequal}}},
-	{L"\x17<", {BUILTIN_NIM_KEYS, FOLLOWED_BY_NONE, {.handler = normal_cmd_ctrl_wless}}},
-	{L"\x17>", {BUILTIN_NIM_KEYS, FOLLOWED_BY_NONE, {.handler = normal_cmd_ctrl_wgreater}}},
-	{L"\x17+", {BUILTIN_NIM_KEYS, FOLLOWED_BY_NONE, {.handler = normal_cmd_ctrl_wplus}}},
-	{L"\x17-", {BUILTIN_NIM_KEYS, FOLLOWED_BY_NONE, {.handler = normal_cmd_ctrl_wminus}}},
-	{L"\x17|", {BUILTIN_NIM_KEYS, FOLLOWED_BY_NONE, {.handler = normal_cmd_ctrl_wpipe}}},
-	{L"\x17_", {BUILTIN_NIM_KEYS, FOLLOWED_BY_NONE, {.handler = normal_cmd_ctrl_wpipe}}},
-	{L"\x19", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-#ifdef ENABLE_EXTENDED_KEYS
-	{{KEY_BTAB, 0}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_q}}},
+	{WK_C_b,           {{&cmd_b},      .descr = "scroll page up"}},
+	{WK_C_d,           {{&cmd_d},      .descr = "scroll half-page down"}},
+	{WK_C_e,           {{&cmd_j},      .descr = "scroll one line down"}},
+	{WK_C_f,           {{&cmd_f},      .descr = "scroll page down"}},
+	{WK_C_i,           {{&cmd_tab},    .descr = "quit preview/switch pane"}},
+	{WK_C_k,           {{&cmd_k},      .descr = "scroll one line up"}},
+	{WK_C_l,           {{&cmd_ctrl_l}, .descr = "redraw"}},
+	{WK_CR,            {{&cmd_j},      .descr = "scroll one line down"}},
+	{WK_C_n,           {{&cmd_j},      .descr = "scroll one line down"}},
+	{WK_C_p,           {{&cmd_k},      .descr = "scroll one line up"}},
+	{WK_C_r,           {{&cmd_ctrl_l}, .descr = "redraw"}},
+	{WK_C_u,           {{&cmd_u},      .descr = "undo file operation"}},
+	{WK_C_v,           {{&cmd_f},      .descr = "scroll page down"}},
+	{WK_C_w WK_H,      {{&cmd_ctrl_wH}, .descr = "move window to the left"}},
+	{WK_C_w WK_J,      {{&cmd_ctrl_wJ}, .descr = "move window to the bottom"}},
+	{WK_C_w WK_K,      {{&cmd_ctrl_wK}, .descr = "move window to the top"}},
+	{WK_C_w WK_L,      {{&cmd_ctrl_wL}, .descr = "move window to the right"}},
+	{WK_C_w WK_b,      {{&cmd_ctrl_wb}, .descr = "go to bottom-right window"}},
+	{WK_C_w WK_C_h,    {{&cmd_ctrl_wh}, .descr = "go to left window"}},
+	{WK_C_w WK_h,      {{&cmd_ctrl_wh}, .descr = "go to left window"}},
+	{WK_C_w WK_C_j,    {{&cmd_ctrl_wj}, .descr = "go to bottom window"}},
+	{WK_C_w WK_j,      {{&cmd_ctrl_wj}, .descr = "go to bottom window"}},
+	{WK_C_w WK_C_k,    {{&cmd_ctrl_wk}, .descr = "go to top window"}},
+	{WK_C_w WK_k,      {{&cmd_ctrl_wk}, .descr = "go to top window"}},
+	{WK_C_w WK_C_l,    {{&cmd_ctrl_wl}, .descr = "go to right window"}},
+	{WK_C_w WK_l,      {{&cmd_ctrl_wl}, .descr = "go to right window"}},
+	{WK_C_w WK_C_o,    {{&cmd_ctrl_wo}, .descr = "single-pane mode"}},
+	{WK_C_w WK_o,      {{&cmd_ctrl_wo}, .descr = "single-pane mode"}},
+	{WK_C_w WK_C_p,    {{&cmd_ctrl_ww}, .descr = "go to other pane"}},
+	{WK_C_w WK_p,      {{&cmd_ctrl_ww}, .descr = "go to other pane"}},
+	{WK_C_w WK_C_t,    {{&cmd_ctrl_wt}, .descr = "go to top-left window"}},
+	{WK_C_w WK_t,      {{&cmd_ctrl_wt}, .descr = "go to top-left window"}},
+	{WK_C_w WK_C_w,    {{&cmd_ctrl_ww}, .descr = "go to other pane"}},
+	{WK_C_w WK_w,      {{&cmd_ctrl_ww}, .descr = "go to other pane"}},
+	{WK_C_w WK_C_x,    {{&cmd_ctrl_wx}, .descr = "exchange panes"}},
+	{WK_C_w WK_x,      {{&cmd_ctrl_wx}, .descr = "exchange panes"}},
+	{WK_C_w WK_C_z,    {{&cmd_ctrl_wz}, .descr = "exit preview/view modes"}},
+	{WK_C_w WK_z,      {{&cmd_ctrl_wz}, .descr = "exit preview/view modes"}},
+	{WK_C_w WK_C_s,    {{&cmd_ctrl_ws}, .descr = "horizontal split layout"}},
+	{WK_C_w WK_s,      {{&cmd_ctrl_ws}, .descr = "horizontal split layout"}},
+	{WK_C_w WK_C_v,    {{&cmd_ctrl_wv}, .descr = "vertical split layout"}},
+	{WK_C_w WK_v,      {{&cmd_ctrl_wv}, .descr = "vertical split layout"}},
+	{WK_C_w WK_EQUALS, {{&normal_cmd_ctrl_wequal},   .nim = 1, .descr = "size panes equally"}},
+	{WK_C_w WK_LT,     {{&normal_cmd_ctrl_wless},    .nim = 1, .descr = "decrease pane size by one"}},
+	{WK_C_w WK_GT,     {{&normal_cmd_ctrl_wgreater}, .nim = 1, .descr = "increase pane size by one"}},
+	{WK_C_w WK_PLUS,   {{&normal_cmd_ctrl_wplus},    .nim = 1, .descr = "increase pane size by one"}},
+	{WK_C_w WK_MINUS,  {{&normal_cmd_ctrl_wminus},   .nim = 1, .descr = "decrease pane size by one"}},
+	{WK_C_w WK_BAR,    {{&normal_cmd_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
+	{WK_C_w WK_USCORE, {{&normal_cmd_ctrl_wpipe},    .nim = 1, .descr = "maximize pane size"}},
+	{WK_C_y,           {{&cmd_k},     .descr = "scroll one line up"}},
+	{WK_SLASH,         {{&cmd_slash}, .descr = "search forward"}},
+	{WK_QM,            {{&cmd_qmark}, .descr = "search backward"}},
+	{WK_LT,            {{&cmd_g}, .descr = "scroll to the beginning"}},
+	{WK_GT,            {{&cmd_G}, .descr = "scroll to the end"}},
+	{WK_SPACE,         {{&cmd_f}, .descr = "scroll page down"}},
+	{WK_ALT WK_LT,     {{&cmd_g}, .descr = "scroll to the beginning"}},
+	{WK_ALT WK_GT,     {{&cmd_G}, .descr = "scroll to the end"}},
+	{WK_ALT WK_SPACE,  {{&cmd_meta_space}, .descr = "scroll down one window"}},
+	{WK_PERCENT,       {{&cmd_percent},    .descr = "to [count]% position"}},
+	{WK_F,             {{&cmd_F}, .descr = "toggle automatic forward scroll"}},
+	{WK_G,             {{&cmd_G}, .descr = "scroll to the end"}},
+	{WK_N,             {{&cmd_N}, .descr = "go to previous search match"}},
+	{WK_Q,             {{&cmd_q}, .descr = "leave view mode"}},
+	{WK_R,             {{&cmd_R}, .descr = "reload view contents"}},
+	{WK_Z WK_Q,        {{&cmd_q}, .descr = "leave view mode"}},
+	{WK_Z WK_Z,        {{&cmd_q}, .descr = "leave view mode"}},
+	{WK_b,             {{&cmd_b}, .descr = "scroll page up"}},
+	{WK_d,             {{&cmd_d}, .descr = "scroll half-page down"}},
+	{WK_f,             {{&cmd_f}, .descr = "scroll page down"}},
+	{WK_g,             {{&cmd_g}, .descr = "scroll to the beginning"}},
+	{WK_e,             {{&cmd_j}, .descr = "scroll one line down"}},
+	{WK_j,             {{&cmd_j}, .descr = "scroll one line down"}},
+	{WK_k,             {{&cmd_k}, .descr = "scroll one line up"}},
+	{WK_n,             {{&cmd_n}, .descr = "go to next search match"}},
+	{WK_p,             {{&cmd_percent}, .descr = "to [count]% position"}},
+	{WK_q,             {{&cmd_q},       .descr = "leave view mode"}},
+	{WK_r,             {{&cmd_ctrl_l},  .descr = "redraw"}},
+	{WK_u,             {{&cmd_u},       .descr = "undo file operation"}},
+	{WK_v,             {{&cmd_v},       .descr = "edit file at current line"}},
+	{WK_y,             {{&cmd_k},       .descr = "scroll one line up"}},
+	{WK_w,             {{&cmd_w},       .descr = "scroll backward one window"}},
+	{WK_z,             {{&cmd_z},       .descr = "scroll forward one window"}},
+#ifndef __PDCURSES__
+	{WK_ALT WK_v,      {{&cmd_b}, .descr = "scroll page up"}},
 #else
-	{L"\033"L"[Z", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_q}}},
+	{{ALT_V},          {{&cmd_b}, .descr = "scroll page up"}},
 #endif
-	{L"/", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_slash}}},
-	{L"?", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_qmark}}},
-	{L"<", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_g}}},
-	{L">", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_G}}},
-	{L" ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_f}}},
-	{L"\033"L"<", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_g}}},
-	{L"\033"L">", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_G}}},
-	{L"\033"L" ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_meta_space}}},
-#ifndef _WIN32
-	{L"\033"L"v", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_b}}},
-#else
-	{{ALT_V}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_b}}},
-#endif
-	{L"%", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_percent}}},
-	{L"F", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_F}}},
-	{L"G", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_G}}},
-	{L"N", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_N}}},
-	{L"Q", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_q}}},
-	{L"R", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_R}}},
-	{L"ZQ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_q}}},
-	{L"ZZ", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_q}}},
-	{L"b", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_b}}},
-	{L"d", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_d}}},
-	{L"f", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_f}}},
-	{L"g", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_g}}},
-	{L"e", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"j", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{L"k", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{L"n", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_n}}},
-	{L"p", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_percent}}},
-	{L"q", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_q}}},
-	{L"r", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_ctrl_l}}},
-	{L"u", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_u}}},
-	{L"v", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_v}}},
-	{L"y", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{L"w", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_w}}},
-	{L"z", {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_z}}},
 #ifdef ENABLE_EXTENDED_KEYS
-	{{KEY_PPAGE}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_b}}},
-	{{KEY_NPAGE}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_f}}},
-	{{KEY_DOWN}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_j}}},
-	{{KEY_UP}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_k}}},
-	{{KEY_HOME}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_g}}},
-	{{KEY_END}, {BUILTIN_KEYS, FOLLOWED_BY_NONE, {.handler = cmd_G}}},
+	{{WC_C_w, KEY_BACKSPACE}, {{&cmd_ctrl_wh}, .descr = "go to left window"}},
+	{{KEY_PPAGE},             {{&cmd_b}, .descr = "scroll page up"}},
+	{{KEY_NPAGE},             {{&cmd_f}, .descr = "scroll page down"}},
+	{{KEY_DOWN},              {{&cmd_j}, .descr = "scroll one line down"}},
+	{{KEY_UP},                {{&cmd_k}, .descr = "scroll one line up"}},
+	{{KEY_HOME},              {{&cmd_g}, .descr = "scroll to the beginning"}},
+	{{KEY_END},               {{&cmd_G}, .descr = "scroll to the end"}},
+	{{KEY_BTAB},              {{&cmd_q}, .descr = "leave view mode"}},
+#else
+	{WK_ESCAPE L"[Z",         {{&cmd_q}, .descr = "leave view mode"}},
 #endif /* ENABLE_EXTENDED_KEYS */
 };
 
@@ -287,7 +297,7 @@ init_view_mode(void)
 {
 	int ret_code;
 
-	ret_code = add_cmds(builtin_cmds, ARRAY_LEN(builtin_cmds), VIEW_MODE);
+	ret_code = vle_keys_add(builtin_cmds, ARRAY_LEN(builtin_cmds), VIEW_MODE);
 	assert(ret_code == 0);
 
 	(void)ret_code;
@@ -522,13 +532,13 @@ redraw(void)
 static void
 calc_vlines(void)
 {
-	if((int)vi->view->window_width - 1 == vi->width &&
-			vi->wrap == cfg.wrap_quick_view)
+	/* Skip the recalculation if window size and wrapping options are the same. */
+	if(ui_qv_width(vi->view) == vi->width && vi->wrap == cfg.wrap_quick_view)
 	{
 		return;
 	}
 
-	vi->width = vi->view->window_width - 1;
+	vi->width = ui_qv_width(vi->view);
 	vi->wrap = cfg.wrap_quick_view;
 
 	if(vi->wrap)
@@ -574,15 +584,18 @@ draw(void)
 {
 	int l, vl;
 	const col_scheme_t *cs = ui_view_get_cs(vi->view);
-	const int height = vi->view->window_rows - 1;
-	const int width = vi->view->window_width - 1;
+	const int height = ui_qv_height(vi->view);
+	const int width = ui_qv_width(vi->view);
 	const int max_l = MIN(vi->line + height, vi->nlines);
 	const int searched = (vi->last_search_backward != -1);
 	esc_state state;
 
 	if(vi->graphics)
 	{
-		ui_view_wipe(vi->view);
+		const char *cmd = qv_get_viewer(vi->filename);
+		cmd = (cmd != NULL) ? ma_get_clean_cmd(cmd) : NULL;
+		qv_cleanup(vi->view, cmd);
+
 		free_string_array(vi->lines, vi->nlines);
 		(void)get_view_data(vi, vi->filename);
 		return;
@@ -595,17 +608,18 @@ draw(void)
 	for(vl = 0, l = vi->line; l < max_l && vl < height; ++l)
 	{
 		int offset = 0;
-		int t = 0;
+		int processed = 0;
 		char *const line = vi->lines[l];
 		char *p = searched ? esc_highlight_pattern(line, &vi->re) : line;
 		do
 		{
 			int printed;
-			int vis = l != vi->line || vl + t >= vi->linev - vi->widths[vi->line][0];
-			offset += esc_print_line(p + offset, vi->view->win, COL, 1 + vl, width,
-					!vis, &state, &printed);
+			const int vis = l != vi->line
+			             || vl + processed >= vi->linev - vi->widths[vi->line][0];
+			offset += esc_print_line(p + offset, vi->view->win, ui_qv_left(vi->view),
+					ui_qv_top(vi->view) + vl, width, !vis, &state, &printed);
 			vl += vis;
-			t++;
+			++processed;
 		}
 		while(vi->wrap && p[offset] != '\0' && vl < height);
 		if(searched)
@@ -913,14 +927,14 @@ static void
 cmd_slash(key_info_t key_info, keys_info_t *keys_info)
 {
 	vi->search_repeat = key_info.count;
-	enter_cmdline_mode(CLS_VWFSEARCH, L"", NULL);
+	enter_cmdline_mode(CLS_VWFSEARCH, "", NULL);
 }
 
 static void
 cmd_qmark(key_info_t key_info, keys_info_t *keys_info)
 {
 	vi->search_repeat = key_info.count;
-	enter_cmdline_mode(CLS_VWBSEARCH, L"", NULL);
+	enter_cmdline_mode(CLS_VWBSEARCH, "", NULL);
 }
 
 /* Toggles automatic forwarding of file. */
@@ -986,9 +1000,6 @@ load_view_data(view_info_t *vi, const char action[], const char file_to_view[],
 		case 0:
 			break;
 
-		case 1:
-			show_error_msg(action, "Can't explore a directory");
-			return 1;
 		case 2:
 			show_error_msg(action, "Can't open file for reading");
 			return 1;
@@ -1004,7 +1015,7 @@ load_view_data(view_info_t *vi, const char action[], const char file_to_view[],
 			return 1;
 	}
 
-	vi->widths = malloc(sizeof(*vi->widths)*vi->nlines);
+	vi->widths = reallocarray(NULL, vi->nlines, sizeof(*vi->widths));
 	if(vi->widths == NULL)
 	{
 		free_string_array(vi->lines, vi->nlines);
@@ -1017,26 +1028,25 @@ load_view_data(view_info_t *vi, const char action[], const char file_to_view[],
 	return 0;
 }
 
-/* Reads data to be displayed handling error cases.  Returns zero on success, 1
- * if file is a directory, 2 on file reading error, 3 on issues with viewer or
- * 4 on empty input. */
+/* Reads data to be displayed handling error cases.  Returns zero on success, 2
+ * on file reading error, 3 on issues with viewer or 4 on empty input. */
 static int
 get_view_data(view_info_t *vi, const char file_to_view[])
 {
 	FILE *fp;
-
-	char *const typed_fname = get_typed_fname(file_to_view);
-	const char *const viewer = ft_get_viewer(typed_fname);
-	free(typed_fname);
+	const char *const viewer = qv_get_viewer(file_to_view);
 
 	if(is_null_or_empty(viewer))
 	{
 		if(is_dir(file_to_view))
 		{
-			return 1;
+			fp = qv_view_dir(file_to_view);
+		}
+		else
+		{
+			fp = os_fopen(file_to_view, "rb");
 		}
 
-		fp = os_fopen(file_to_view, "rb");
 		if(fp == NULL)
 		{
 			return 2;
@@ -1046,11 +1056,18 @@ get_view_data(view_info_t *vi, const char file_to_view[])
 	}
 	else
 	{
+		const int graphics = is_graphics_viewer(viewer);
 		FileView *const curr = curr_view;
 		curr_view = curr_stats.view ? curr_view
 		          : (vi->view != NULL) ? vi->view : curr_view;
 		curr_stats.preview_hint = vi->view;
 
+		if(graphics)
+		{
+			/* Wait a bit to let terminal emulator do actual refresh (at least some
+			 * of them need this). */
+			usleep(50000);
+		}
 		fp = use_info_prog(viewer);
 
 		curr_view = curr;
@@ -1061,12 +1078,15 @@ get_view_data(view_info_t *vi, const char file_to_view[])
 			return 3;
 		}
 
-		if(is_graphics_viewer(viewer))
+		if(graphics)
 		{
 			vi->graphics = 1;
 		}
 
-		vi->lines = read_stream_lines(fp, &vi->nlines);
+		ui_cancellation_reset();
+		ui_cancellation_enable();
+		vi->lines = read_stream_lines(fp, &vi->nlines, 0);
+		ui_cancellation_disable();
 	}
 
 	fclose(fp);
@@ -1143,8 +1163,7 @@ cmd_g(key_info_t key_info, keys_info_t *keys_info)
 	if(key_info.count == NO_COUNT_GIVEN)
 		key_info.count = 1;
 
-	key_info.count = MIN(vi->nlinesv - (vi->view->window_rows - 1),
-			key_info.count);
+	key_info.count = MIN(vi->nlinesv - ui_qv_height(vi->view), key_info.count);
 	key_info.count = MAX(1, key_info.count);
 
 	if(vi->linev == vi->widths[key_info.count - 1][0])
@@ -1159,7 +1178,7 @@ cmd_j(key_info_t key_info, keys_info_t *keys_info)
 {
 	if(key_info.reg == NO_REG_GIVEN)
 	{
-		if((vi->linev + 1) + (vi->view->window_rows - 1) > vi->nlinesv)
+		if((vi->linev + 1) + ui_qv_height(vi->view) > vi->nlinesv)
 			return;
 	}
 	else
@@ -1172,14 +1191,13 @@ cmd_j(key_info_t key_info, keys_info_t *keys_info)
 		key_info.count = 1;
 	if(key_info.reg == NO_REG_GIVEN)
 		key_info.count = MIN(key_info.count,
-				vi->nlinesv - (vi->view->window_rows - 1) - vi->linev);
+				vi->nlinesv - ui_qv_height(vi->view) - vi->linev);
 	else
 		key_info.count = MIN(key_info.count, vi->nlinesv - vi->linev - 1);
 
 	while(key_info.count-- > 0)
 	{
-		int height = (vi->widths[vi->line][1] + vi->width - 1)/vi->width;
-		height = MAX(height, 1);
+		const int height = MAX(DIV_ROUND_UP(vi->widths[vi->line][1], vi->width), 1);
 		if(vi->linev + 1 >= vi->widths[vi->line][0] + height)
 			++vi->line;
 
@@ -1257,7 +1275,7 @@ find_previous(int vline_offset)
 {
 	int i;
 	int offset = 0;
-	char buf[(vi->view->window_width - 1)*4];
+	char buf[ui_qv_width(vi->view)*4];
 	int vl, l;
 
 	vl = vi->linev - vline_offset;
@@ -1267,7 +1285,7 @@ find_previous(int vline_offset)
 		l--;
 
 	for(i = 0; i <= vl - vi->widths[l][0]; i++)
-		offset = get_part(vi->lines[l], offset, vi->view->window_width - 1, buf);
+		offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
 
 	/* Don't stop until we go above first virtual line of the first line. */
 	while(l >= 0 && vl >= 0)
@@ -1283,11 +1301,10 @@ find_previous(int vline_offset)
 			l--;
 			offset = 0;
 			for(i = 0; i <= vl - 1 - vi->widths[l][0]; i++)
-				offset = get_part(vi->lines[l], offset, vi->view->window_width - 1,
-						buf);
+				offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
 		}
 		else
-			offset = get_part(vi->lines[l], offset, vi->view->window_width - 1, buf);
+			offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
 		vl--;
 	}
 	draw();
@@ -1302,7 +1319,7 @@ find_next(void)
 {
 	int i;
 	int offset = 0;
-	char buf[(vi->view->window_width - 1)*4];
+	char buf[ui_qv_width(vi->view)*4];
 	int vl, l;
 
 	vl = vi->linev + 1;
@@ -1312,7 +1329,7 @@ find_next(void)
 		l++;
 
 	for(i = 0; i <= vl - vi->widths[l][0]; i++)
-		offset = get_part(vi->lines[l], offset, vi->view->window_width - 1, buf);
+		offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
 
 	while(l < vi->nlines)
 	{
@@ -1330,7 +1347,7 @@ find_next(void)
 			l++;
 			offset = 0;
 		}
-		offset = get_part(vi->lines[l], offset, vi->view->window_width - 1, buf);
+		offset = get_part(vi->lines[l], offset, ui_qv_width(vi->view), buf);
 		vl++;
 	}
 	draw();
@@ -1386,7 +1403,7 @@ update_with_half_win(key_info_t *const key_info)
 	{
 		key_info->count = (vi->half_win > 0)
 			? vi->half_win
-			: (vi->view->window_rows - 1)/2;
+			: ui_qv_height(vi->view)/2;
 	}
 	else
 	{
@@ -1402,7 +1419,7 @@ cmd_v(key_info_t key_info, keys_info_t *keys_info)
 {
 	char path[PATH_MAX];
 	get_current_full_path(curr_view, sizeof(path), path);
-	(void)vim_view_file(path, vi->line + (vi->view->window_rows - 1)/2, -1, 1);
+	(void)vim_view_file(path, vi->line + ui_qv_height(vi->view)/2, -1, 1);
 	/* In some cases two redraw operations are needed, otherwise TUI is not fully
 	 * redrawn. */
 	update_screen(UT_REDRAW);
@@ -1444,7 +1461,7 @@ set_from_default_win(key_info_t *const key_info)
 {
 	key_info->count = (vi->win_size > 0)
 		? vi->win_size
-		: (vi->view->window_rows - 2);
+		: (ui_qv_height(vi->view) - 1);
 }
 
 int
@@ -1560,12 +1577,12 @@ forward_if_changed(view_info_t *vi)
 static int
 scroll_to_bottom(view_info_t *vi)
 {
-	if(vi->linev + 1 + vi->view->window_rows - 1 > vi->nlinesv)
+	if(vi->linev + 1 + ui_qv_height(vi->view) > vi->nlinesv)
 	{
 		return 0;
 	}
 
-	vi->linev = vi->nlinesv - (vi->view->window_rows - 1);
+	vi->linev = vi->nlinesv - ui_qv_height(vi->view);
 	for(vi->line = 0; vi->line < vi->nlines - 1; ++vi->line)
 	{
 		if(vi->linev < vi->widths[vi->line + 1][0])
@@ -1597,4 +1614,4 @@ reload_view(view_info_t *vi, int silent)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
