@@ -36,30 +36,38 @@
 #include <string.h> /* strdup() */
 
 #include "cfg/config.h"
+#include "compat/fs_limits.h"
 #include "compat/os.h"
-#include "engine/text_buffer.h"
 #include "io/ioeta.h"
 #include "io/iop.h"
 #include "io/ior.h"
 #include "modes/dialogs/msg_dialog.h"
 #include "ui/cancellation.h"
 #include "utils/fs.h"
-#include "utils/fs_limits.h"
 #include "utils/log.h"
 #include "utils/macros.h"
 #include "utils/path.h"
 #include "utils/str.h"
 #include "utils/utils.h"
 #include "background.h"
+#include "bmarks.h"
 #include "status.h"
 #include "trash.h"
 #include "undo.h"
 
 #ifdef SUPPORT_NO_CLOBBER
 #define NO_CLOBBER "-n"
-#else /* SUPPORT_NO_CLOBBER */
+#else
 #define NO_CLOBBER ""
-#endif /* SUPPORT_NO_CLOBBER */
+#endif
+
+/* Enable O(1) file cloning if it's available in installed version of
+ * coreutils. */
+#ifdef SUPPORT_REFLINK_AUTO
+#define REFLINK_AUTO "--reflink=auto"
+#else
+#define REFLINK_AUTO ""
+#endif
 
 #ifdef GNU_TOOLCHAIN
 #define PRESERVE_FLAGS "--preserve=mode,timestamps"
@@ -109,10 +117,12 @@ static int op_rmdir(ops_t *ops, void *data, const char *src, const char *dst);
 static int op_mkfile(ops_t *ops, void *data, const char *src, const char *dst);
 static int exec_io_op(ops_t *ops, int (*func)(io_args_t *const),
 		io_args_t *const args);
-static char * err_list_to_str(const ioe_errlst_t *elist);
 static int confirm_overwrite(io_args_t *args, const char src[],
 		const char dst[]);
 static char * pretty_dir_path(const char path[]);
+static IoErrCbResult dispatch_error(io_args_t *args, const ioe_err_t *err);
+static char prompt_user(const io_args_t *args, const char title[],
+		const char msg[], const response_variant variants[]);
 
 /* List of functions that implement operations. */
 static op_func op_funcs[] = {
@@ -151,7 +161,7 @@ ARRAY_GUARD(op_funcs, OP_COUNT);
 static ops_t *curr_ops;
 
 ops_t *
-ops_alloc(OPS main_op, const char descr[], const char base_dir[],
+ops_alloc(OPS main_op, int bg, const char descr[], const char base_dir[],
 		const char target_dir[])
 {
 	ops_t *const ops = calloc(1, sizeof(*ops));
@@ -159,6 +169,7 @@ ops_alloc(OPS main_op, const char descr[], const char base_dir[],
 	ops->descr = descr;
 	ops->base_dir = strdup(base_dir);
 	ops->target_dir = strdup(target_dir);
+	ops->bg = bg;
 	return ops;
 }
 
@@ -261,11 +272,12 @@ op_none(ops_t *ops, void *data, const char *src, const char *dst)
 static int
 op_remove(ops_t *ops, void *data, const char *src, const char *dst)
 {
-	if(cfg.confirm && !curr_stats.confirmed)
+	if(cfg_confirm_delete(0) && !curr_stats.confirmed &&
+			(ops == NULL || !ops->bg))
 	{
 		curr_stats.confirmed = prompt_msg("Permanent deletion",
-				"Are you sure? If you undoing a command and want to see file names, "
-				"use :undolist! command");
+				"Are you sure?  If you're undoing a command and want to see file "
+				"names, use :undolist! command.");
 		if(!curr_stats.confirmed)
 			return SKIP_UNDO_REDO_OPERATION;
 	}
@@ -276,6 +288,33 @@ op_remove(ops_t *ops, void *data, const char *src, const char *dst)
 static int
 op_removesl(ops_t *ops, void *data, const char *src, const char *dst)
 {
+	if(cfg.delete_prg[0] != '\0')
+	{
+#ifndef _WIN32
+		char *escaped;
+		char cmd[2*PATH_MAX + 1];
+		const int cancellable = (data == NULL);
+
+		escaped = shell_like_escape(src, 0);
+		if(escaped == NULL)
+		{
+			return -1;
+		}
+
+		snprintf(cmd, sizeof(cmd), "%s %s", cfg.delete_prg, escaped);
+		free(escaped);
+
+		LOG_INFO_MSG("Running trash command: \"%s\"", cmd);
+		return background_and_wait_for_errors(cmd, cancellable);
+#else
+		char cmd[PATH_MAX*2 + 1];
+		snprintf(cmd, sizeof(cmd), "%s \"%s\"", cfg.delete_prg, src);
+		to_back_slash(cmd);
+
+		return os_system(cmd);
+#endif
+	}
+
 	if(!cfg.use_system_calls)
 	{
 #ifndef _WIN32
@@ -284,7 +323,7 @@ op_removesl(ops_t *ops, void *data, const char *src, const char *dst)
 		int result;
 		const int cancellable = data == NULL;
 
-		escaped = escape_filename(src, 0);
+		escaped = shell_like_escape(src, 0);
 		if(escaped == NULL)
 			return -1;
 
@@ -305,8 +344,7 @@ op_removesl(ops_t *ops, void *data, const char *src, const char *dst)
 
 			wchar_t *const utf16_path = utf8_to_utf16(path);
 
-			SHFILEOPSTRUCTW fo =
-			{
+			SHFILEOPSTRUCTW fo = {
 				.hwnd = NULL,
 				.wFunc = FO_DELETE,
 				.pFrom = utf16_path,
@@ -387,10 +425,10 @@ op_cp(ops_t *ops, void *data, const char src[], const char dst[],
 		char *escaped_src, *escaped_dst;
 		char cmd[6 + PATH_MAX*2 + 1];
 		int result;
-		const int cancellable = data == NULL;
+		const int cancellable = (data == NULL);
 
-		escaped_src = escape_filename(src, 0);
-		escaped_dst = escape_filename(dst, 0);
+		escaped_src = shell_like_escape(src, 0);
+		escaped_dst = shell_like_escape(dst, 0);
 		if(escaped_src == NULL || escaped_dst == NULL)
 		{
 			free(escaped_dst);
@@ -399,8 +437,9 @@ op_cp(ops_t *ops, void *data, const char src[], const char dst[],
 		}
 
 		snprintf(cmd, sizeof(cmd),
-				"cp %s -R " PRESERVE_FLAGS " %s %s",
+				"cp %s %s -R " PRESERVE_FLAGS " %s %s",
 				(conflict_action == CA_FAIL) ? NO_CLOBBER : "",
+				cfg.fast_file_cloning ? REFLINK_AUTO : "",
 				escaped_src, escaped_dst);
 		LOG_INFO_MSG("Running cp command: \"%s\"", cmd);
 		result = background_and_wait_for_errors(cmd, cancellable);
@@ -443,6 +482,7 @@ op_cp(ops_t *ops, void *data, const char src[], const char dst[],
 		.arg1.src = src,
 		.arg2.dst = dst,
 		.arg3.crs = ca_to_crs(conflict_action),
+		.arg4.fast_file_cloning = cfg.fast_file_cloning,
 
 		.cancellable = data == NULL,
 	};
@@ -490,13 +530,14 @@ op_mv(ops_t *ops, void *data, const char src[], const char dst[],
 		char cmd[6 + PATH_MAX*2 + 1];
 		const int cancellable = data == NULL;
 
-		if(conflict_action == CA_FAIL && os_lstat(dst, &st) == 0)
+		if(conflict_action == CA_FAIL && os_lstat(dst, &st) == 0 &&
+				!is_case_change(src, dst))
 		{
 			return -1;
 		}
 
-		escaped_src = escape_filename(src, 0);
-		escaped_dst = escape_filename(dst, 0);
+		escaped_src = shell_like_escape(src, 0);
+		escaped_dst = shell_like_escape(dst, 0);
 		if(escaped_src == NULL || escaped_dst == NULL)
 		{
 			free(escaped_dst);
@@ -543,6 +584,8 @@ op_mv(ops_t *ops, void *data, const char src[], const char dst[],
 			.arg1.src = src,
 			.arg2.dst = dst,
 			.arg3.crs = ca_to_crs(conflict_action),
+			/* It's safe to always use fast file cloning on moving files. */
+			.arg4.fast_file_cloning = 1,
 
 			.cancellable = data == NULL,
 		};
@@ -551,14 +594,8 @@ op_mv(ops_t *ops, void *data, const char src[], const char dst[],
 
 	if(result == 0)
 	{
-		if(is_under_trash(dst))
-		{
-			add_to_trash(src, dst);
-		}
-		else if(is_under_trash(src))
-		{
-			remove_from_trash(src);
-		}
+		trash_file_moved(src, dst);
+		bmarks_file_moved(src, dst);
 	}
 
 	return result;
@@ -587,7 +624,7 @@ op_chown(ops_t *ops, void *data, const char *src, const char *dst)
 	char *escaped;
 	uid_t uid = (uid_t)(long)data;
 
-	escaped = escape_filename(src, 0);
+	escaped = shell_like_escape(src, 0);
 	snprintf(cmd, sizeof(cmd), "chown -fR %u %s", uid, escaped);
 	free(escaped);
 
@@ -606,7 +643,7 @@ op_chgrp(ops_t *ops, void *data, const char *src, const char *dst)
 	char *escaped;
 	gid_t gid = (gid_t)(long)data;
 
-	escaped = escape_filename(src, 0);
+	escaped = shell_like_escape(src, 0);
 	snprintf(cmd, sizeof(cmd), "chown -fR :%u %s", gid, escaped);
 	free(escaped);
 
@@ -624,7 +661,7 @@ op_chmod(ops_t *ops, void *data, const char *src, const char *dst)
 	char cmd[128 + PATH_MAX];
 	char *escaped;
 
-	escaped = escape_filename(src, 0);
+	escaped = shell_like_escape(src, 0);
 	snprintf(cmd, sizeof(cmd), "chmod %s %s", (char *)data, escaped);
 	free(escaped);
 
@@ -638,11 +675,12 @@ op_chmodr(ops_t *ops, void *data, const char *src, const char *dst)
 	char cmd[128 + PATH_MAX];
 	char *escaped;
 
-	escaped = escape_filename(src, 0);
+	escaped = shell_like_escape(src, 0);
 	snprintf(cmd, sizeof(cmd), "chmod -R %s %s", (char *)data, escaped);
 	free(escaped);
-	start_background_job(cmd, 0);
-	return 0;
+
+	LOG_INFO_MSG("Running chmodr command: \"%s\"", cmd);
+	return background_and_wait_for_errors(cmd, 1);
 }
 #else
 static int
@@ -702,8 +740,8 @@ op_symlink(ops_t *ops, void *data, const char *src, const char *dst)
 		char exe_dir[PATH_MAX + 2];
 #endif
 
-		escaped_src = escape_filename(src, 0);
-		escaped_dst = escape_filename(dst, 0);
+		escaped_src = shell_like_escape(src, 0);
+		escaped_dst = shell_like_escape(dst, 0);
 		if(escaped_src == NULL || escaped_dst == NULL)
 		{
 			free(escaped_dst);
@@ -750,7 +788,7 @@ op_mkdir(ops_t *ops, void *data, const char *src, const char *dst)
 		char cmd[128 + PATH_MAX];
 		char *escaped;
 
-		escaped = escape_filename(src, 0);
+		escaped = shell_like_escape(src, 0);
 		snprintf(cmd, sizeof(cmd), "mkdir %s %s", (data == NULL) ? "" : "-p",
 				escaped);
 		free(escaped);
@@ -766,32 +804,25 @@ op_mkdir(ops_t *ops, void *data, const char *src, const char *dst)
 		}
 		else
 		{
-			char *p;
-			char t;
+			char *const partial_path = strdup(src);
+			char *part = partial_path + (is_path_absolute(src) ? 2 : 0);
+			char *state = NULL;
 
-			p = strchr(src + 2, '/');
-			do
+			while((part = split_and_get(part, '/', &state)) != NULL)
 			{
-				t = *p;
-				*p = '\0';
-
-				if(!is_dir(src))
+				if(!is_dir(partial_path))
 				{
-					wchar_t *const utf16_path = utf8_to_utf16(src);
+					wchar_t *const utf16_path = utf8_to_utf16(partial_path);
 					if(!CreateDirectoryW(utf16_path, NULL))
 					{
 						free(utf16_path);
-						*p = t;
+						free(partial_path);
 						return -1;
 					}
 					free(utf16_path);
 				}
-
-				*p = t;
-				if((p = strchr(p + 1, '/')) == NULL)
-					p = (char *)src + strlen(src);
 			}
-			while(t != '\0');
+			free(partial_path);
 			return 0;
 		}
 #endif
@@ -814,7 +845,7 @@ op_rmdir(ops_t *ops, void *data, const char *src, const char *dst)
 		char cmd[128 + PATH_MAX];
 		char *escaped;
 
-		escaped = escape_filename(src, 0);
+		escaped = shell_like_escape(src, 0);
 		snprintf(cmd, sizeof(cmd), "rmdir %s", escaped);
 		free(escaped);
 		LOG_INFO_MSG("Running rmdir command: \"%s\"", cmd);
@@ -842,7 +873,7 @@ op_mkfile(ops_t *ops, void *data, const char *src, const char *dst)
 		char cmd[128 + PATH_MAX];
 		char *escaped;
 
-		escaped = escape_filename(src, 0);
+		escaped = shell_like_escape(src, 0);
 		snprintf(cmd, sizeof(cmd), "touch %s", escaped);
 		free(escaped);
 		LOG_INFO_MSG("Running touch command: \"%s\"", cmd);
@@ -878,10 +909,15 @@ exec_io_op(ops_t *ops, int (*func)(io_args_t *const), io_args_t *const args)
 	int result;
 
 	args->estim = (ops == NULL) ? NULL : ops->estim;
-	args->confirm = &confirm_overwrite;
 
 	if(ops != NULL)
 	{
+		if(!ops->bg)
+		{
+			args->confirm = &confirm_overwrite;
+			args->result.errors_cb = &dispatch_error;
+		}
+
 		ioe_errlst_init(&args->result.errors);
 	}
 
@@ -901,37 +937,18 @@ exec_io_op(ops_t *ops, int (*func)(io_args_t *const), io_args_t *const args)
 
 	if(ops != NULL)
 	{
-		ops->errors = err_list_to_str(&args->result.errors);
+		size_t len = (ops->errors == NULL) ? 0U : strlen(ops->errors);
+		char *const suffix = ioe_errlst_to_str(&args->result.errors);
+
+		(void)strappend(&ops->errors, &len, suffix);
+
+		free(suffix);
 		ioe_errlst_free(&args->result.errors);
 	}
 
 	return result;
 }
 
-/* Converts list of errors from I/O module into multi-line string.  Returns
- * newly allocated string. */
-static char *
-err_list_to_str(const ioe_errlst_t *elist)
-{
-	/* TODO: maybe move this to io/ioe unit. */
-
-	size_t i;
-
-	vle_textbuf *str = vle_tb_create();
-	if(str == NULL)
-	{
-		return NULL;
-	}
-
-	for(i = 0U; i < elist->error_count; ++i)
-	{
-		const ioe_err_t *const err = &elist->errors[i];
-		vle_tb_append_linef(str, "%s: %s", replace_home_part(err->path), err->msg);
-	}
-
-	return vle_tb_release(str);
-}
-
 /* Asks user to confirm file overwrite.  Returns non-zero on positive user
  * answer, otherwise zero is returned. */
 static int
@@ -946,6 +963,7 @@ confirm_overwrite(io_args_t *args, const char src[], const char dst[])
 		{ },
 	};
 
+	char *title;
 	char *msg;
 	char response;
 	char *src_dir, *dst_dir;
@@ -959,25 +977,17 @@ confirm_overwrite(io_args_t *args, const char src[], const char dst[])
 	src_dir = pretty_dir_path(src);
 	dst_dir = pretty_dir_path(dst);
 
+	title = format_str("File overwrite while %s", curr_ops->descr);
 	msg = format_str("Overwrite \"%s\" in\n%s\nwith \"%s\" from\n%s\n?", fname,
 			dst_dir, fname, src_dir);
 
 	free(dst_dir);
 	free(src_dir);
 
-	/* Active cancellation conflicts with input processing by putting terminal in
-	 * a cocked mode. */
-	if(args->cancellable)
-	{
-		raw();
-	}
-	response = prompt_msg_custom("File overwrite", msg, responses);
-	if(args->cancellable)
-	{
-		noraw();
-	}
+	response = prompt_user(args, title, msg, responses);
 
 	free(msg);
+	free(title);
 
 	switch(response)
 	{
@@ -1009,5 +1019,72 @@ pretty_dir_path(const char path[])
 	return strdup(canonic);
 }
 
+/* Asks user what to do with encountered error.  Returns the response. */
+static IoErrCbResult
+dispatch_error(io_args_t *args, const ioe_err_t *err)
+{
+	static const response_variant responses[] = {
+		{ .key = 'r', .descr = "[r]etry", },
+		{ .key = 'i', .descr = "[i]gnore", },
+		{ .key = 'I', .descr = "[I]gnore for all", },
+		{ .key = 'a', .descr = "[a]bort", },
+		{ },
+	};
+
+	char *title;
+	char *msg;
+	char response;
+
+	if(curr_ops->erp == ERP_IGNORE_ALL)
+	{
+		return IO_ECR_IGNORE;
+	}
+
+	title = format_str("Error while %s", curr_ops->descr);
+	msg = format_str("%s: %s", replace_home_part(err->path), err->msg);
+
+	response = prompt_user(args, title, msg, responses);
+
+	free(msg);
+	free(title);
+
+	switch(response)
+	{
+		case 'r': return IO_ECR_RETRY;
+
+		case 'I': curr_ops->erp = ERP_IGNORE_ALL; /* Fall through. */
+		case 'i': return IO_ECR_IGNORE;
+
+		case 'a': return IO_ECR_BREAK;
+
+		default:
+			assert(0 && "Unexpected response.");
+			return 0;
+	}
+}
+
+/* prompt_msg_custom() wrapper that takes care of interaction if cancellation is
+ * active. */
+static char
+prompt_user(const io_args_t *args, const char title[], const char msg[],
+		const response_variant variants[])
+{
+	char response;
+
+	/* Active cancellation conflicts with input processing by putting terminal in
+	 * a cooked mode. */
+	if(args->cancellable)
+	{
+		raw();
+	}
+	response = prompt_msg_custom(title, msg, variants);
+	if(args->cancellable)
+	{
+		noraw();
+	}
+
+	return response;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
