@@ -19,16 +19,14 @@
 
 #include "registers.h"
 
-#include <sys/stat.h>
-
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() */
-#include <stdlib.h> /* free() realloc() */
 #include <string.h>
 
-#include "compat/os.h"
+#include "compat/reallocarray.h"
 #include "utils/fs.h"
 #include "utils/macros.h"
+#include "utils/path.h"
 #include "utils/str.h"
 #include "utils/string_array.h"
 #include "utils/utils.h"
@@ -36,7 +34,7 @@
 
 /* Name of the "unnamed" (the default) register. */
 #define UNNAMED_REG_NAME '"'
-/* Name of the "blackhole" register. */
+/* Name of the "black hole" register. */
 #define BLACKHOLE_REG_NAME '_'
 /* Number of registers named as alphabet letters. */
 #define NUM_LETTER_REGISTERS 26
@@ -44,7 +42,7 @@
 #define NUM_REGISTERS (2 + NUM_LETTER_REGISTERS)
 
 /* Data of all registers. */
-static registers_t registers[NUM_REGISTERS];
+static reg_t registers[NUM_REGISTERS];
 
 /* Names of registers + names of 26 uppercase register names + termination null
  * character. */
@@ -61,24 +59,24 @@ const char valid_registers[] = {
 ARRAY_GUARD(valid_registers, NUM_REGISTERS + NUM_LETTER_REGISTERS + 1);
 
 void
-init_registers(void)
+regs_init(void)
 {
 	int i;
 	for(i = 0; i < NUM_REGISTERS; i++)
 	{
 		registers[i].name = valid_registers[i];
-		registers[i].num_files = 0;
+		registers[i].nfiles = 0;
 		registers[i].files = NULL;
 	}
 }
 
 int
-register_exists(int key)
+regs_exists(int reg_name)
 {
 	int i;
-	for(i = 0; i < NUM_REGISTERS; i++)
+	for(i = 0; i < NUM_REGISTERS; ++i)
 	{
-		if(valid_registers[i] == key)
+		if(valid_registers[i] == reg_name)
 		{
 			return 1;
 		}
@@ -86,45 +84,48 @@ register_exists(int key)
 	return 0;
 }
 
-registers_t *
-find_register(int key)
+reg_t *
+regs_find(int reg_name)
 {
 	int i;
-	for(i = 0; i < NUM_REGISTERS; i++)
+	for(i = 0; i < NUM_REGISTERS; ++i)
 	{
-		if(registers[i].name == key)
+		if(registers[i].name == reg_name)
+		{
 			return &registers[i];
+		}
 	}
 	return NULL;
 }
 
 static int
-check_for_duplicate_file_names(registers_t *reg, const char file[])
+check_for_duplicate_file_names(reg_t *reg, const char file[])
 {
-	int x;
-	for(x = 0; x < reg->num_files; x++)
+	int i;
+	for(i = 0; i < reg->nfiles; ++i)
 	{
-		if(stroscmp(file, reg->files[x]) == 0)
+		if(stroscmp(file, reg->files[i]) == 0)
+		{
 			return 1;
+		}
 	}
 	return 0;
 }
 
 int
-append_to_register(int key, const char file[])
+regs_append(int reg_name, const char file[])
 {
-	registers_t *reg;
-	struct stat st;
+	reg_t *reg;
 
-	if(key == BLACKHOLE_REG_NAME)
+	if(reg_name == BLACKHOLE_REG_NAME)
 	{
 		return 0;
 	}
-	if((reg = find_register(key)) == NULL)
+	if((reg = regs_find(reg_name)) == NULL)
 	{
 		return 1;
 	}
-	if(os_lstat(file, &st) != 0)
+	if(!path_exists(file, NODEREF))
 	{
 		return 1;
 	}
@@ -133,74 +134,79 @@ append_to_register(int key, const char file[])
 		return 1;
 	}
 
-	reg->num_files = add_to_string_array(&reg->files, reg->num_files, 1, file);
+	reg->nfiles = add_to_string_array(&reg->files, reg->nfiles, 1, file);
 	return 0;
 }
 
 void
-clear_registers(void)
+regs_reset(void)
 {
 	const char *p = valid_registers;
 	while(*p != '\0')
 	{
-		clear_register(*p++);
+		regs_clear(*p++);
 	}
 }
 
 void
-clear_register(int key)
+regs_clear(int reg_name)
 {
-	registers_t *reg;
-
-	if((reg = find_register(key)) == NULL)
+	reg_t *const reg = regs_find(reg_name);
+	if(reg == NULL)
+	{
 		return;
+	}
 
-	free_string_array(reg->files, reg->num_files);
+	free_string_array(reg->files, reg->nfiles);
 	reg->files = NULL;
-	reg->num_files = 0;
+	reg->nfiles = 0;
 }
 
 void
-pack_register(int key)
+regs_pack(int reg_name)
 {
-	int x, y;
-	registers_t *reg;
-
-	if((reg = find_register(key)) == NULL)
+	int j, i;
+	reg_t *const reg = regs_find(reg_name);
+	if(reg == NULL)
+	{
 		return;
+	}
 
-	x = 0;
-	for(y = 0; y < reg->num_files; y++)
-		if(reg->files[y] != NULL)
-			reg->files[x++] = reg->files[y];
-	reg->num_files = x;
+	j = 0;
+	for(i = 0; i < reg->nfiles; ++i)
+	{
+		if(reg->files[i] != NULL)
+		{
+			reg->files[j++] = reg->files[i];
+		}
+	}
+	reg->nfiles = j;
 }
 
 char **
-list_registers_content(const char registers[])
+regs_list(const char registers[])
 {
 	char **list = NULL;
 	size_t len = 0;
 
 	while(*registers != '\0')
 	{
-		registers_t *reg;
-		char buf[56];
-		int y;
+		reg_t *reg = regs_find(*registers++);
+		char reg_str[16];
+		int i;
 
-		if((reg = find_register(*registers++)) == NULL)
-			continue;
-
-		if(reg->num_files <= 0)
-			continue;
-
-		snprintf(buf, sizeof(buf), "\"%c", reg->name);
-		len = add_to_string_array(&list, len, 1, buf);
-
-		y = reg->num_files;
-		while(y-- > 0)
+		if(reg == NULL || reg->nfiles <= 0)
 		{
-			len = add_to_string_array(&list, len, 1, reg->files[y]);
+			continue;
+		}
+
+		snprintf(reg_str, sizeof(reg_str), "\"%c", reg->name);
+		len = add_to_string_array(&list, len, 1, reg_str);
+
+		i = reg->nfiles;
+		while(i-- > 0)
+		{
+			len = add_to_string_array(&list, len, 1, reg->files[i]);
 		}
 	}
 
@@ -209,71 +215,103 @@ list_registers_content(const char registers[])
 }
 
 void
-rename_in_registers(const char old[], const char new[])
+regs_rename_contents(const char old[], const char new[])
 {
-	int x;
-	for(x = 0; x < NUM_REGISTERS; x++)
+	int i;
+	for(i = 0; i < NUM_REGISTERS; ++i)
 	{
-		int y, n;
-		n = registers[x].num_files;
-		for(y = 0; y < n; y++)
+		int j;
+		const int n = registers[i].nfiles;
+		for(j = 0; j < n; ++j)
 		{
-			if(stroscmp(registers[x].files[y], old) != 0)
+			if(stroscmp(registers[i].files[j], old) != 0)
 				continue;
 
-			(void)replace_string(&registers[x].files[y], new);
-			break; /* registers don't contain duplicates */
+			(void)replace_string(&registers[i].files[j], new);
+			/* Registers don't contain duplicates, so exit this loop. */
+			break;
 		}
 	}
 }
 
 void
-clean_regs_with_trash(void)
+regs_remove_trashed_files(const char trash_dir[])
 {
-	int x;
-	for(x = 0; x < NUM_REGISTERS; x++)
+	int i;
+	for(i = 0; i < NUM_REGISTERS; ++i)
 	{
-		int y, n, needs_pack = 0;
-		n = registers[x].num_files;
-		for(y = 0; y < n; y++)
+		int j, needs_packing = 0;
+		const int n = registers[i].nfiles;
+		for(j = 0; j < n; ++j)
 		{
-			if(!is_under_trash(registers[x].files[y]))
+			if(!trash_contains(trash_dir, registers[i].files[j]))
 				continue;
-			if(!path_exists(registers[x].files[y], DEREF))
+			if(!path_exists(registers[i].files[j], DEREF))
 				continue;
 
-			free(registers[x].files[y]);
-			registers[x].files[y] = NULL;
-			needs_pack = 1;
+			update_string(&registers[i].files[j], NULL);
+			needs_packing = 1;
+		}
+		if(needs_packing)
+		{
+			regs_pack(registers[i].name);
 		}
-		if(needs_pack)
-			pack_register(registers[x].name);
 	}
 }
 
 void
-update_unnamed_reg(int key)
+regs_update_unnamed(int reg_name)
 {
-	registers_t *unnamed, *reg;
+	reg_t *unnamed, *reg;
 	int i;
 
-	if(key == UNNAMED_REG_NAME)
+	if(reg_name == UNNAMED_REG_NAME)
 		return;
 
-	if((reg = find_register(key)) == NULL)
+	if((reg = regs_find(reg_name)) == NULL)
 		return;
 
-	if((unnamed = find_register(UNNAMED_REG_NAME)) == NULL)
+	if((unnamed = regs_find(UNNAMED_REG_NAME)) == NULL)
 		return;
 
-	clear_register(UNNAMED_REG_NAME);
+	regs_clear(UNNAMED_REG_NAME);
 
-	unnamed->num_files = reg->num_files;
-	unnamed->files = (char **)realloc(unnamed->files,
-			unnamed->num_files*sizeof(char *));
-	for(i = 0; i < unnamed->num_files; i++)
+	unnamed->nfiles = reg->nfiles;
+	unnamed->files = reallocarray(unnamed->files, unnamed->nfiles,
+			sizeof(char *));
+	for(i = 0; i < unnamed->nfiles; ++i)
+	{
 		unnamed->files[i] = strdup(reg->files[i]);
+	}
+}
+
+void
+regs_suggest(regs_suggest_cb cb, int max_entries_per_reg)
+{
+	wchar_t reg_name[] = L"reg: X";
+	const char *registers = valid_registers;
+
+	while(*registers != '\0')
+	{
+		reg_t *const reg = regs_find(*registers++);
+		int i, max = max_entries_per_reg;
+
+		if(reg == NULL || reg->nfiles <= 0)
+		{
+			continue;
+		}
+
+		i = reg->nfiles - 1;
+
+		reg_name[5] = reg->name;
+		cb(reg_name, L"", replace_home_part(reg->files[i]));
+
+		while(--max > 0 && i-- > 0)
+		{
+			cb(L"", L"", replace_home_part(reg->files[i]));
+		}
+	}
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
