@@ -38,37 +38,38 @@
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() */
-#include <stdlib.h> /* EXIT_FAILURE EXIT_SUCCESS malloc() free() */
+#include <stdlib.h> /* EXIT_FAILURE EXIT_SUCCESS free() realloc() */
 #include <string.h> /* strcmp() strerror() strrchr() strcat() strstr() strlen()
                        strchr() strdup() strncmp() */
 
 #include "cfg/config.h"
 #include "cfg/info.h"
+#include "compat/fs_limits.h"
 #include "compat/os.h"
+#include "int/file_magic.h"
+#include "int/fuse.h"
+#include "int/vim.h"
 #include "menus/users_menu.h"
 #include "modes/dialogs/msg_dialog.h"
 #include "ui/statusbar.h"
 #include "ui/ui.h"
 #include "utils/env.h"
 #include "utils/fs.h"
-#include "utils/fs_limits.h"
 #include "utils/log.h"
 #include "utils/path.h"
 #include "utils/str.h"
+#include "utils/string_array.h"
 #include "utils/test_helpers.h"
 #include "utils/utils.h"
 #include "utils/utf8.h"
 #include "background.h"
-#include "file_magic.h"
 #include "filelist.h"
 #include "filetype.h"
-#include "fuse.h"
 #include "macros.h"
 #include "opt_handlers.h"
 #include "status.h"
 #include "types.h"
 #include "vifm.h"
-#include "vim.h"
 
 /* Kinds of symbolic link file treatment on file handling. */
 typedef enum
@@ -89,7 +90,7 @@ static int is_dir_entry(const char full_path[], int type);
 static void run_win_executable(char full_path[], int elevate);
 static int run_win_executable_as_evaluated(const char full_path[]);
 #endif
-static int selection_is_consistent(const FileView *const view);
+static int selection_is_consistent(FileView *view);
 static void execute_file(const char full_path[], int elevate);
 static void run_selection(FileView *view, int dont_execute);
 static void run_file(FileView *view, int dont_execute);
@@ -101,6 +102,7 @@ static void run_implicit_prog(FileView *view, const char prog_spec[],
 		int pause, int force_bg);
 static void view_current_file(const FileView *view);
 static void follow_link(FileView *view, int follow_dirs);
+static int cd_to_parent_dir(FileView *view);
 static void extract_last_path_component(const char path[], char buf[]);
 static void setup_shellout_env(void);
 static void cleanup_shellout_env(void);
@@ -116,8 +118,8 @@ static int try_run_with_filetype(FileView *view, const assoc_records_t assocs,
 static void output_to_statusbar(const char cmd[]);
 static void output_to_nowhere(const char cmd[]);
 static void run_in_split(const FileView *view, const char cmd[]);
-static void output_to_custom_flist(FileView *view, const char cmd[], int very);
 static void path_handler(const char line[], void *arg);
+static void line_handler(const char line[], void *arg);
 
 /* Name of environment variable used to communicate path to file used to
  * initiate FUSE mounting of directory we're in. */
@@ -134,17 +136,18 @@ follow_file(FileView *view)
 {
 	if(flist_custom_active(view))
 	{
+		/* Entry might be freed on navigation, so make sure name and origin will
+		 * remain available for the call. */
 		const dir_entry_t *const entry = &view->dir_entry[view->list_pos];
 		char *const name = strdup(entry->name);
 		char *const origin = strdup(entry->origin);
-		navigate_to_file(view, origin, name);
+		navigate_to_file(view, origin, name, 0);
 		free(origin);
 		free(name);
+		return;
 	}
-	else
-	{
-		handle_file(view, FHE_RUN, FHL_FOLLOW);
-	}
+
+	handle_file(view, FHE_RUN, FHL_FOLLOW);
 }
 
 static void
@@ -321,37 +324,31 @@ run_win_executable_as_evaluated(const char full_path[])
 /* Returns non-zero if selection doesn't mix files and directories, otherwise
  * zero is returned. */
 static int
-selection_is_consistent(const FileView *const view)
+selection_is_consistent(FileView *view)
 {
-	if(view->selected_files > 1)
-	{
-		int files = 0, dirs = 0;
-		int i;
-		for(i = 0; i < view->list_rows; i++)
-		{
-			char full[PATH_MAX];
-			const dir_entry_t *const curr = &view->dir_entry[i];
-			if(!curr->selected)
-			{
-				continue;
-			}
+	int files = 0, dirs = 0;
+	dir_entry_t *entry;
 
-			get_full_path_of(curr, sizeof(full), full);
-			if(is_dir_entry(full, curr->type))
-			{
-				dirs++;
-			}
-			else
-			{
-				files++;
-			}
-		}
-		if(dirs > 0 && files > 0)
+	if(view->selected_files < 2)
+	{
+		return 1;
+	}
+
+	entry = NULL;
+	while(iter_selected_entries(view, &entry))
+	{
+		char full[PATH_MAX];
+		get_full_path_of(entry, sizeof(full), full);
+		if(is_dir_entry(full, entry->type))
 		{
-			return 0;
+			++dirs;
+		}
+		else
+		{
+			++files;
 		}
 	}
-	return 1;
+	return (dirs == 0 || files == 0);
 }
 
 /* Executes file, specified by the full_path.  Changes type of slashes on
@@ -360,8 +357,8 @@ static void
 execute_file(const char full_path[], int elevate)
 {
 #ifndef _WIN32
-	char *const escaped = escape_filename(full_path, 0);
-	shellout(escaped, 1, 1);
+	char *const escaped = shell_like_escape(full_path, 0);
+	shellout(escaped, PAUSE_ALWAYS, 1);
 	free(escaped);
 #else
 	char *const dquoted_full_path = strdup(enclose_in_dquotes(full_path));
@@ -406,7 +403,7 @@ run_file(FileView *view, int dont_execute)
 		clean_selected_files(view);
 	}
 
-	typed_fname = get_typed_current_fpath(view);
+	typed_fname = get_typed_entry_fpath(get_current_entry(view));
 	multi_prog_cmd = ft_get_program(typed_fname);
 	free(typed_fname);
 
@@ -420,14 +417,14 @@ run_file(FileView *view, int dont_execute)
 		char *typed_fname;
 		const char *entry_prog_cmd;
 
-		if(!path_exists(entry->name, DEREF))
+		if(!path_exists_at(entry->origin, entry->name, DEREF))
 		{
 			show_error_msgf("Broken Link", "Destination of \"%s\" link doesn't exist",
 					entry->name);
 			return;
 		}
 
-		typed_fname = get_typed_entry_fname(entry);
+		typed_fname = get_typed_entry_fpath(entry);
 		entry_prog_cmd = ft_get_program(typed_fname);
 		free(typed_fname);
 
@@ -509,7 +506,7 @@ run_selection_separately(FileView *view, int dont_execute)
 		char *typed_fname;
 		const char *entry_prog_cmd;
 
-		typed_fname = get_typed_entry_fname(entry);
+		typed_fname = get_typed_entry_fpath(entry);
 		entry_prog_cmd = ft_get_program(typed_fname);
 		free(typed_fname);
 
@@ -608,7 +605,8 @@ run_explicit_prog(const char prog_spec[], int pause, int force_bg)
 	}
 	else
 	{
-		(void)shellout(cmd, pause ? 1 : -1, flags != MF_NO_TERM_MUX);
+		(void)shellout(cmd, pause ? PAUSE_ALWAYS : PAUSE_ON_ERROR,
+				flags != MF_NO_TERM_MUX);
 	}
 
 	free(cmd);
@@ -648,7 +646,7 @@ run_implicit_prog(FileView *view, const char prog_spec[], int pause,
 	}
 	else
 	{
-		(void)shellout(cmd, pause ? 1 : -1, 1);
+		(void)shellout(cmd, pause ? PAUSE_ALWAYS : PAUSE_ON_ERROR, 1);
 	}
 }
 
@@ -729,7 +727,7 @@ open_dir(FileView *view)
 
 	if(is_parent_dir(filename))
 	{
-		cd_updir(view);
+		cd_updir(view, 1);
 		return;
 	}
 
@@ -742,32 +740,59 @@ open_dir(FileView *view)
 }
 
 void
-cd_updir(FileView *view)
+cd_updir(FileView *view, int levels)
+{
+	/* Do not save intermediate directories in directory history. */
+	curr_stats.drop_new_dir_hist = 1;
+
+	while(levels-- > 0)
+	{
+		if(cd_to_parent_dir(view) != 0)
+		{
+			break;
+		}
+	}
+
+	curr_stats.drop_new_dir_hist = 0;
+	save_view_history(view, NULL, NULL, -1);
+}
+
+/* Goes one directory up from current location.  Returns zero unless it won't
+ * make sense to continue going up (like on error or reaching root). */
+static int
+cd_to_parent_dir(FileView *view)
 {
 	char dir_name[strlen(view->curr_dir) + 1];
+	int ret;
 
 	/* Return to original directory from custom view. */
 	if(flist_custom_active(view))
 	{
 		navigate_to(view, view->custom.orig_dir);
-		return;
+		return 0;
 	}
 
 	/* Do nothing in root. */
 	if(is_root_dir(view->curr_dir))
 	{
-		return;
+		return 1;
 	}
 
 	dir_name[0] = '\0';
-
 	extract_last_path_component(view->curr_dir, dir_name);
 
-	if(change_directory(view, "../") != 1)
+	ret = change_directory(view, "../");
+	if(ret == -1)
+	{
+		return 1;
+	}
+
+	if(ret == 0)
 	{
 		load_dir_list(view, 0);
 		flist_set_pos(view, find_file_pos_in_list(view, dir_name));
 	}
+	return 0;
 }
 
 /* Extracts last part of the path into buf.  Assumes that size of the buf is
@@ -780,20 +805,20 @@ extract_last_path_component(const char path[], char buf[])
 }
 
 int
-shellout(const char command[], int pause, int use_term_multiplexer)
+shellout(const char command[], ShellPause pause, int use_term_multiplexer)
 {
 	char *cmd;
 	int result;
 	int ec;
 
-	if(pause > 0 && command != NULL && ends_with(command, "&"))
+	if(pause == PAUSE_ALWAYS && command != NULL && ends_with(command, "&"))
 	{
-		pause = -1;
+		pause = PAUSE_ON_ERROR;
 	}
 
 	setup_shellout_env();
 
-	cmd = gen_shell_cmd(command, pause > 0, use_term_multiplexer);
+	cmd = gen_shell_cmd(command, pause == PAUSE_ALWAYS, use_term_multiplexer);
 
 	if(curr_stats.load_stage != 0)
 	{
@@ -806,7 +831,7 @@ shellout(const char command[], int pause, int use_term_multiplexer)
 
 	cleanup_shellout_env();
 
-	if(result != 0 && pause < 0)
+	if(result != 0 && pause == PAUSE_ON_ERROR)
 	{
 		LOG_ERROR_MSG("Subprocess (%s) exit code: %d (0x%x); status = 0x%x", cmd,
 				result, result, ec);
@@ -829,7 +854,7 @@ shellout(const char command[], int pause, int use_term_multiplexer)
 
 	if(curr_stats.load_stage != 0)
 	{
-		curs_set(FALSE);
+		curs_set(0);
 	}
 
 	return result;
@@ -866,7 +891,7 @@ setup_shellout_env(void)
 			return;
 	}
 
-	escaped_path = escape_filename(mount_file, 0);
+	escaped_path = shell_like_escape(mount_file, 0);
 	cmd = format_str(term_multiplexer_fmt, FUSE_FILE_ENVVAR, escaped_path);
 	(void)vifm_system(cmd);
 	free(cmd);
@@ -933,7 +958,6 @@ static char *
 gen_term_multiplexer_cmd(const char cmd[], int pause)
 {
 	char *title_arg;
-	char *escaped_sh;
 	char *raw_shell_cmd;
 	char *escaped_shell_cmd;
 	char *shell_cmd = NULL;
@@ -945,17 +969,15 @@ gen_term_multiplexer_cmd(const char cmd[], int pause)
 		return NULL;
 	}
 
-	escaped_sh = escape_filename(cfg.shell, 0);
-
 	title_arg = gen_term_multiplexer_title_arg(cmd);
 
 	raw_shell_cmd = format_str("%s%s", cmd, pause ? PAUSE_STR : "");
-	escaped_shell_cmd = escape_filename(raw_shell_cmd, 0);
+	escaped_shell_cmd = shell_like_escape(raw_shell_cmd, 0);
 
 	if(curr_stats.term_multiplexer == TM_TMUX)
 	{
-		char *const arg = format_str("%s -c %s", escaped_sh, escaped_shell_cmd);
-		char *const escaped_arg = escape_filename(arg, 0);
+		char *const arg = format_str("%s -c %s", cfg.shell, escaped_shell_cmd);
+		char *const escaped_arg = shell_like_escape(arg, 0);
 
 		shell_cmd = format_str("tmux new-window %s %s", title_arg, escaped_arg);
 
@@ -964,9 +986,9 @@ gen_term_multiplexer_cmd(const char cmd[], int pause)
 	}
 	else if(curr_stats.term_multiplexer == TM_SCREEN)
 	{
-		set_pwd_in_screen(curr_view->curr_dir);
+		set_pwd_in_screen(flist_get_dir(curr_view));
 
-		shell_cmd = format_str("screen %s %s -c %s", title_arg, escaped_sh,
+		shell_cmd = format_str("screen %s %s -c %s", title_arg, cfg.shell,
 				escaped_shell_cmd);
 	}
 	else
@@ -977,7 +999,6 @@ gen_term_multiplexer_cmd(const char cmd[], int pause)
 	free(escaped_shell_cmd);
 	free(raw_shell_cmd);
 	free(title_arg);
-	free(escaped_sh);
 
 	return shell_cmd;
 }
@@ -1017,7 +1038,7 @@ gen_term_multiplexer_title_arg(const char cmd[])
 	else
 	{
 		const char opt_c = (curr_stats.term_multiplexer == TM_SCREEN) ? 't' : 'n';
-		char *const escaped_title = escape_filename(title, 0);
+		char *const escaped_title = shell_like_escape(title, 0);
 		title_arg = format_str("-%c %s", opt_c, escaped_title);
 		free(escaped_title);
 	}
@@ -1083,7 +1104,7 @@ gen_term_multiplexer_run_cmd(void)
 static void
 set_pwd_in_screen(const char path[])
 {
-	char *const escaped_dir = escape_filename(path, 0);
+	char *const escaped_dir = shell_like_escape(path, 0);
 	char *const set_pwd = format_str("screen -X setenv PWD %s", escaped_dir);
 
 	(void)vifm_system(set_pwd);
@@ -1095,7 +1116,7 @@ set_pwd_in_screen(const char path[])
 int
 run_with_filetype(FileView *view, const char beginning[], int background)
 {
-	char *const typed_fname = get_typed_current_fpath(view);
+	char *const typed_fname = get_typed_entry_fpath(get_current_entry(view));
 	assoc_records_t ft = ft_get_all_programs(typed_fname);
 	assoc_records_t magic = get_magic_handlers(typed_fname);
 	free(typed_fname);
@@ -1261,36 +1282,33 @@ output_to_nowhere(const char cmd[])
 static void
 run_in_split(const FileView *view, const char cmd[])
 {
-	const char *const cmd_to_run = (cmd == NULL) ? cfg.shell : cmd;
-
-	char *const escaped_cmd = escape_filename(cmd_to_run, 0);
+	char *const escaped_cmd = (cmd == NULL)
+	                        ? strdup(cfg.shell)
+	                        : shell_like_escape(cmd, 0);
 
 	setup_shellout_env();
 
 	if(curr_stats.term_multiplexer == TM_TMUX)
 	{
-		char buf[1024];
-		snprintf(buf, sizeof(buf), "tmux split-window %s", escaped_cmd);
-		(void)vifm_system(buf);
+		char cmd[1024];
+		snprintf(cmd, sizeof(cmd), "tmux split-window %s", escaped_cmd);
+		(void)vifm_system(cmd);
 	}
 	else if(curr_stats.term_multiplexer == TM_SCREEN)
 	{
-		char buf[1024];
-		char *escaped_chdir;
+		char cmd[1024];
 
-		char *const escaped_dir = escape_filename(flist_get_dir(view), 0);
-		snprintf(buf, sizeof(buf), "chdir %s", escaped_dir);
+		/* "eval" executes each argument as a separate argument, but escaping rules
+		 * are not exactly like in shell, so last command is run separately. */
+		char *const escaped_dir = shell_like_escape(flist_get_dir(view), 0);
+		snprintf(cmd, sizeof(cmd), "screen -X eval chdir\\ %s 'focus bottom' "
+				"split 'focus bottom'", escaped_dir);
 		free(escaped_dir);
+		(void)vifm_system(cmd);
 
-		escaped_chdir = escape_filename(buf, 0);
-		snprintf(buf, sizeof(buf), "screen -X eval %s", escaped_chdir);
-		free(escaped_chdir);
-
-		(void)vifm_system(buf);
-
-		snprintf(buf, sizeof(buf), "screen-open-region-with-program %s",
+		snprintf(cmd, sizeof(cmd), "screen -X screen vifm-screen-split %s",
 				escaped_cmd);
-		(void)vifm_system(buf);
+		(void)vifm_system(cmd);
 	}
 	else
 	{
@@ -1302,9 +1320,7 @@ run_in_split(const FileView *view, const char cmd[])
 	free(escaped_cmd);
 }
 
-/* Runs the cmd and parses its output as list of paths to compose custom view.
- * Very custom view implies unsorted list. */
-static void
+int
 output_to_custom_flist(FileView *view, const char cmd[], int very)
 {
 	char *title;
@@ -1322,30 +1338,11 @@ output_to_custom_flist(FileView *view, const char cmd[], int very)
 	if(error)
 	{
 		show_error_msgf("Trouble running command", "Unable to run: %s", cmd);
-		return;
+		return 1;
 	}
 
-	if(very)
-	{
-		memcpy(&view->custom.sort[0], &view->sort[0], sizeof(view->custom.sort));
-		memset(&view->sort[0], SK_NONE, sizeof(view->sort));
-	}
-	view->custom.unsorted = very;
-
-	if(flist_custom_finish(view) != 0)
-	{
-		show_error_msg("Custom view", "Ignoring empty list of files");
-		return;
-	}
-
-	if(very)
-	{
-		/* As custom view isn't activated until flist_custom_finish() is called,
-		 * need to update option separately from view sort array. */
-		load_sort_option(view);
-	}
-
-	flist_set_pos(view, 0);
+	flist_end_custom(view, very);
+	return 0;
 }
 
 /* Implements process_cmd_output() callback that loads paths into custom
@@ -1354,17 +1351,38 @@ static void
 path_handler(const char line[], void *arg)
 {
 	FileView *view = arg;
-	int line_num;
-	/* Skip empty lines. */
-	char *const path = (skip_whitespace(line)[0] == '\0')
-	                 ? NULL
-	                 : parse_file_spec(line, &line_num);
-	if(path != NULL)
+	flist_add_custom_line(view, line);
+}
+
+int
+run_cmd_for_output(const char cmd[], char ***files, int *nfiles)
+{
+	int error;
+	strlist_t list = {};
+
+	setup_shellout_env();
+	error = (process_cmd_output("Loading list", cmd, 1, &line_handler,
+				&list) != 0);
+	cleanup_shellout_env();
+
+	if(error)
 	{
-		flist_custom_add(view, path);
-		free(path);
+		free_string_array(list.items, list.nitems);
+		return 1;
 	}
+
+	*files = list.items;
+	*nfiles = list.nitems;
+	return 0;
+}
+
+/* Implements process_cmd_output() callback that collects lines into a list. */
+static void
+line_handler(const char line[], void *arg)
+{
+	strlist_t *const list = arg;
+	list->nitems = add_to_string_array(&list->items, list->nitems, 1, line);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
