@@ -32,6 +32,15 @@ typedef enum
 }
 FileHandleExec;
 
+/* When and why should we pause on shellout(). */
+typedef enum
+{
+	PAUSE_ALWAYS,   /* Execute command and pause. */
+	PAUSE_NEVER,    /* Don't pause after the command. */
+	PAUSE_ON_ERROR, /* Pause only on non-zero exit code from the command. */
+}
+ShellPause;
+
 /* Handles opening of current file/selection of the view. */
 void open_file(FileView *view, FileHandleExec exec);
 
@@ -47,16 +56,13 @@ void run_using_prog(FileView *view, const char prog_spec[], int dont_execute,
 /* Handles opening of current file of the view as directory. */
 void open_dir(FileView *view);
 
-/* Moves the view to parent directory taking care of special cases like root of
- * FUSE mount. */
-void cd_updir(FileView *view);
+/* Moves the view to levels-th parent directory taking care of special cases
+ * like root of FUSE mount. */
+void cd_updir(FileView *view, int levels);
 
-/* Values of pause:
- *  > 0 - pause always
- *  = 0 - do not pause
- *  < 0 - pause on error
- * Returns zero on success, otherwise non-zero is returned. */
-int shellout(const char command[], int pause, int use_term_multiplexer);
+/* Executes command in a shell.  Returns zero on success, otherwise non-zero is
+ * returned. */
+int shellout(const char command[], ShellPause pause, int use_term_multiplexer);
 
 /* Returns zero on successful running. */
 int run_with_filetype(FileView *view, const char beginning[], int background);
@@ -67,7 +73,16 @@ int run_with_filetype(FileView *view, const char beginning[], int background);
  *  - < 0 -- handled, exit. */
 int run_ext_command(const char cmd[], MacroFlags flags, int bg, int *save_msg);
 
+/* Runs the cmd and parses its output as list of paths to compose custom view.
+ * Very custom view implies unsorted list.  Returns zero on success, otherwise
+ * non-zero is returned. */
+int output_to_custom_flist(FileView *view, const char cmd[], int very);
+
+/* Executes external command capturing its output as list of lines.  Sets *files
+ * and *nfiles.  Returns zero on success, otherwise non-zero is returned. */
+int run_cmd_for_output(const char cmd[], char ***files, int *nfiles);
+
 #endif /* VIFM__RUNNING_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
