@@ -26,14 +26,15 @@
 #include <string.h>
 
 #include "cfg/config.h"
+#include "compat/fs_limits.h"
+#include "ui/fileview.h"
 #include "ui/statusbar.h"
 #include "ui/ui.h"
-#include "utils/fs_limits.h"
 #include "utils/path.h"
+#include "utils/regexp.h"
 #include "utils/str.h"
 #include "utils/utils.h"
 #include "filelist.h"
-#include "fileview.h"
 
 static int find_and_goto_pattern(FileView *view, int wrap_start, int backward);
 static int find_and_goto_match(FileView *view, int start, int backward);
@@ -104,7 +105,7 @@ find_and_goto_match(FileView *view, int start, int backward)
 
 int
 find_pattern(FileView *view, const char pattern[], int backward, int move,
-		int *const found, int interactive)
+		int *const found, int print_errors)
 {
 	int cflags;
 	int nmatches = 0;
@@ -146,7 +147,7 @@ find_pattern(FileView *view, const char pattern[], int backward, int move,
 				continue;
 			}
 
-			entry->search_match = 1;
+			entry->search_match = nmatches + 1;
 			entry->match_left = matches[0].rm_so;
 			entry->match_right = matches[0].rm_eo;
 			if(cfg.hl_search)
@@ -160,12 +161,12 @@ find_pattern(FileView *view, const char pattern[], int backward, int move,
 	}
 	else
 	{
-		if(interactive)
+		if(print_errors)
 		{
 			status_bar_errorf("Regexp error: %s", get_regexp_error(err, &re));
 		}
 		regfree(&re);
-		return 1;
+		return -1;
 	}
 
 	other = (view == &lwin) ? &rwin : &lwin;
@@ -194,7 +195,7 @@ find_pattern(FileView *view, const char pattern[], int backward, int move,
 
 		if(!cfg.hl_search)
 		{
-			if(interactive)
+			if(print_errors)
 			{
 				print_result(view, was_found, backward);
 			}
@@ -205,7 +206,7 @@ find_pattern(FileView *view, const char pattern[], int backward, int move,
 	else
 	{
 		fview_cursor_redraw(view);
-		if(interactive)
+		if(print_errors)
 		{
 			print_search_fail_msg(view, backward);
 		}
@@ -237,11 +238,20 @@ print_search_msg(const FileView *view, int backward)
 	}
 	else
 	{
-		status_bar_messagef("%d matching file%s for: %s", view->matches,
+		status_bar_messagef("%d of %d matching file%s for: %s",
+				view->dir_entry[view->list_pos].search_match,
+				view->matches,
 				(view->matches == 1) ? "" : "s", cfg_get_last_search_pattern());
 	}
 }
 
+void
+print_search_next_msg(const FileView *view, int backward)
+{
+	status_bar_messagef("(%d of %d) %c%s", view->dir_entry[view->list_pos].search_match,
+			view->matches, backward ? '?' : '/', cfg_get_last_search_pattern());
+}
+
 void
 print_search_fail_msg(const FileView *view, int backward)
 {
@@ -296,4 +306,4 @@ reset_search_results(FileView *view)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
