@@ -25,16 +25,17 @@
 
 #include <curses.h>
 
+#include <errno.h> /* errno */
 #include <signal.h>
 #include <stdio.h> /* fprintf() */
 #include <string.h> /* strsignal() */
 
 #include "cfg/info.h"
+#include "int/fuse.h"
+#include "int/term_title.h"
 #include "ui/cancellation.h"
 #include "utils/log.h"
 #include "utils/macros.h"
-#include "fuse.h"
-#include "term_title.h"
 
 static void _gnuc_noreturn shutdown_nicely(int sig, const char descr[]);
 
@@ -43,7 +44,7 @@ static void _gnuc_noreturn shutdown_nicely(int sig, const char descr[]);
 #include <sys/types.h> /* pid_t */
 #include <sys/wait.h> /* WEXITSTATUS() WIFEXITED() waitpid() */
 
-#include <stdlib.h> /* exit() */
+#include <stdlib.h> /* EXIT_FAILURE _Exit() */
 
 #include "utils/macros.h"
 #include "background.h"
@@ -92,6 +93,9 @@ received_sigchld(void)
 static void
 handle_signal(int sig)
 {
+	/* Try to not change errno value in the main program. */
+	const int saved_errno = errno;
+
 	switch(sig)
 	{
 		case SIGINT:
@@ -112,9 +116,9 @@ handle_signal(int sig)
 		case SIGTERM:
 			shutdown_nicely(sig, strsignal(sig));
 			break;
-		default:
-			break;
 	}
+
+	errno = saved_errno;
 }
 
 #else
@@ -150,11 +154,16 @@ shutdown_nicely(int sig, const char descr[])
   LOG_FUNC_ENTER;
 
 	endwin();
-	set_term_title(NULL);
+	term_title_update(NULL);
 	fuse_unmount_all();
 	write_info_file();
 	fprintf(stdout, "Vifm killed by signal: %d (%s).\n", sig, descr);
-	exit(0);
+
+	/* Alternatively we could do this sequence:
+	 *     signal(sig, SIG_DFL);
+	 *     raise(sig);
+	 * but only on *nix systems. */
+	_Exit(EXIT_FAILURE);
 }
 
 void
@@ -165,10 +174,15 @@ setup_signals(void)
 #ifndef _WIN32
 	struct sigaction handle_signal_action;
 
-	handle_signal_action.sa_handler = handle_signal;
+	handle_signal_action.sa_handler = &handle_signal;
 	sigemptyset(&handle_signal_action.sa_mask);
 	handle_signal_action.sa_flags = SA_RESTART;
 
+	/* Assumption: we work under shell with job control support.  If it's not the
+	 * case, this code enables handling of terminal related signals the shell
+	 * wanted us to have disabled (e.g. the app will catch Ctrl-C send to another
+	 * process). */
+
 	sigaction(SIGCHLD, &handle_signal_action, NULL);
 	sigaction(SIGHUP, &handle_signal_action, NULL);
 	sigaction(SIGINT, &handle_signal_action, NULL);
@@ -190,4 +204,4 @@ setup_signals(void)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
