@@ -19,18 +19,23 @@
 
 #include "sort.h"
 
+#include <regex.h> /* regex_t regcomp() regexec() regfree() */
+
 #include <assert.h> /* assert() */
 #include <ctype.h>
 #include <stdlib.h> /* abs() qsort() */
 #include <string.h> /* strcmp() strrchr() */
 
 #include "cfg/config.h"
+#include "compat/fs_limits.h"
 #include "ui/ui.h"
-#include "utils/fs_limits.h"
+#include "utils/fs.h"
+#include "utils/fsdata.h"
 #include "utils/path.h"
+#include "utils/regexp.h"
 #include "utils/str.h"
+#include "utils/string_array.h"
 #include "utils/test_helpers.h"
-#include "utils/tree.h"
 #include "utils/utils.h"
 #include "filelist.h"
 #include "status.h"
@@ -43,8 +48,11 @@ static int custom_view;
 static int sort_descending;
 /* Key used to sort entries in current sorting round. */
 static SortingKey sort_type;
+/* Sorting key specific data. */
+static void *sort_data;
 
-static void sort_by_key(char key);
+static void sort_by_groups(void);
+static void sort_by_key(char key, void *data);
 static int sort_dir_list(const void *one, const void *two);
 TSTATIC int strnumcmp(const char s[], const char t[]);
 #if !defined(HAVE_STRVERSCMP_FUNC) || !HAVE_STRVERSCMP_FUNC
@@ -57,6 +65,12 @@ static int compare_entry_names(const dir_entry_t *a, const dir_entry_t *b,
 static int compare_full_file_names(const char s[], const char t[],
 		int ignore_case);
 static int compare_file_names(const char s[], const char t[], int ignore_case);
+static int compare_file_sizes(const dir_entry_t *f, int fdir,
+		const dir_entry_t *s, int sdir);
+static int compare_item_count(const dir_entry_t *f, int fdir,
+		const dir_entry_t *s, int sdir);
+static int compare_group(const char f[], const char s[], regex_t *regex);
+static int compare_targets(const dir_entry_t *f, const dir_entry_t *s);
 
 void
 sort_view(FileView *v)
@@ -82,30 +96,71 @@ sort_view(FileView *v)
 			continue;
 		}
 
-		sort_by_key(sorting_key);
+		if(sorting_key == SK_BY_GROUPS)
+		{
+			sort_by_groups();
+			continue;
+		}
+
+		sort_by_key(sorting_key, NULL);
 	}
 
 	if(!ui_view_sort_list_contains(v->sort, SK_BY_DIR))
 	{
-		sort_by_key(SK_BY_DIR);
+		sort_by_key(SK_BY_DIR, NULL);
 	}
 }
 
+/* Sorts view according to sorting groups option. */
+static void
+sort_by_groups(void)
+{
+	char **groups = NULL;
+	int ngroups = 0;
+	int i;
+
+	char *const copy = strdup(view->sort_groups);
+	char *group = copy, *state = NULL;
+	while((group = split_and_get(group, ',', &state)) != NULL)
+	{
+		ngroups = add_to_string_array(&groups, ngroups, 1, group);
+	}
+	free(copy);
+
+	for(i = ngroups - 1; i >= 1; --i)
+	{
+		regex_t regex;
+		(void)regcomp(&regex, groups[i], REG_EXTENDED | REG_ICASE);
+		sort_by_key(SK_BY_GROUPS, &regex);
+		regfree(&regex);
+	}
+	if(ngroups != 0)
+	{
+		sort_by_key(SK_BY_GROUPS, &view->primary_group);
+	}
+
+	free_string_array(groups, ngroups);
+}
+
 /* Sorts view by the key in a stable way. */
 static void
-sort_by_key(char key)
+sort_by_key(char key, void *data)
 {
 	int j;
 
 	sort_descending = (key < 0);
 	sort_type = (SortingKey)abs(key);
+	sort_data = data;
 
-	for(j = 0; j < view->list_rows; j++)
+	for(j = 0; j < view->list_rows; ++j)
 	{
 		view->dir_entry[j].list_num = j;
 	}
 
-	qsort(view->dir_entry, view->list_rows, sizeof(dir_entry_t), sort_dir_list);
+	if(view->list_rows != 0)
+	{
+		qsort(view->dir_entry, view->list_rows, sizeof(dir_entry_t), sort_dir_list);
+	}
 }
 
 /* Compares file names containing numbers correctly. */
@@ -172,25 +227,25 @@ skip_leading_zeros(const char str[])
 static int
 sort_dir_list(const void *one, const void *two)
 {
+	/* TODO: refactor this function sort_dir_list(). */
+
 	int retval;
 	char *pfirst, *psecond;
-	dir_entry_t *const first = (dir_entry_t *)one;
-	dir_entry_t *const second = (dir_entry_t *)two;
-	int first_is_dir;
-	int second_is_dir;
+	const dir_entry_t *const first = (dir_entry_t *)one;
+	const dir_entry_t *const second = (dir_entry_t *)two;
 
-	if(is_parent_dir(first->name))
+	const int first_is_dir = is_directory_entry(first);
+	const int second_is_dir = is_directory_entry(second);
+
+	if(first_is_dir && is_parent_dir(first->name))
 	{
 		return -1;
 	}
-	else if(is_parent_dir(second->name))
+	if(second_is_dir && is_parent_dir(second->name))
 	{
 		return 1;
 	}
 
-	first_is_dir = is_directory_entry(first);
-	second_is_dir = is_directory_entry(second);
-
 	retval = 0;
 	switch(sort_type)
 	{
@@ -218,12 +273,34 @@ sort_dir_list(const void *one, const void *two)
 			retval = strcmp(get_type_str(first->type), get_type_str(second->type));
 			break;
 
+		case SK_BY_FILEEXT:
 		case SK_BY_EXTENSION:
 			pfirst = strrchr(first->name,  '.');
 			psecond = strrchr(second->name, '.');
 
-			if(pfirst && psecond)
-				retval = compare_file_names(++pfirst, ++psecond, 0);
+			if(first_is_dir && second_is_dir && sort_type == SK_BY_FILEEXT)
+			{
+				retval = compare_file_names(first->name, second->name, 0);
+			}
+			else if(first_is_dir != second_is_dir && sort_type == SK_BY_FILEEXT)
+			{
+				retval = first_is_dir ? -1 : 1;
+			}
+			else if(pfirst && psecond)
+			{
+				if(pfirst == first->name && psecond != second->name)
+				{
+					retval = -1;
+				}
+				else if(pfirst != first->name && psecond == second->name)
+				{
+					retval = 1;
+				}
+				else
+				{
+					retval = compare_file_names(++pfirst, ++psecond, 0);
+				}
+			}
 			else if(pfirst || psecond)
 				retval = pfirst ? -1 : 1;
 			else
@@ -231,25 +308,19 @@ sort_dir_list(const void *one, const void *two)
 			break;
 
 		case SK_BY_SIZE:
-			{
-				if(first_is_dir)
-				{
-					char full_path[PATH_MAX];
-					get_full_path_of(first, sizeof(full_path), full_path);
-					tree_get_data(curr_stats.dirsize_cache, full_path, &first->size);
-				}
+			retval = compare_file_sizes(first, first_is_dir, second, second_is_dir);
+			break;
 
-				if(second_is_dir)
-				{
-					char full_path[PATH_MAX];
-					get_full_path_of(second, sizeof(full_path), full_path);
-					tree_get_data(curr_stats.dirsize_cache, full_path, &second->size);
-				}
+		case SK_BY_NITEMS:
+			retval = compare_item_count(first, first_is_dir, second, second_is_dir);
+			break;
 
-				retval = (first->size < second->size)
-				       ? -1
-				       : (first->size > second->size);
-			}
+		case SK_BY_GROUPS:
+			retval = compare_group(first->name, second->name, sort_data);
+			break;
+
+		case SK_BY_TARGET:
+			retval = compare_targets(first, second);
 			break;
 
 		case SK_BY_TIME_MODIFIED:
@@ -263,6 +334,7 @@ sort_dir_list(const void *one, const void *two)
 		case SK_BY_TIME_CHANGED:
 			retval = first->ctime - second->ctime;
 			break;
+
 #ifndef _WIN32
 		case SK_BY_MODE:
 			retval = first->mode - second->mode;
@@ -286,6 +358,10 @@ sort_dir_list(const void *one, const void *two)
 				retval = strcmp(first_perm, second_perm);
 			}
 			break;
+
+		case SK_BY_NLINKS:
+			retval = first->nlinks - second->nlinks;
+			break;
 #endif
 	}
 
@@ -301,6 +377,103 @@ sort_dir_list(const void *one, const void *two)
 	return retval;
 }
 
+/* Compares two file sizes.  Returns standard -1, 0, 1 for comparisons. */
+static int
+compare_file_sizes(const dir_entry_t *f, int fdir, const dir_entry_t *s,
+		int sdir)
+{
+	uint64_t fsize = f->size;
+	uint64_t ssize = s->size;
+
+	if(fdir)
+	{
+		uint64_t size;
+		dcache_get_of(f, &size, NULL);
+		if(size != DCACHE_UNKNOWN)
+		{
+			fsize = size;
+		}
+	}
+
+	if(sdir)
+	{
+		uint64_t size;
+		dcache_get_of(s, &size, NULL);
+		if(size != DCACHE_UNKNOWN)
+		{
+			ssize = size;
+		}
+	}
+
+	return (fsize < ssize) ? -1 : (fsize > ssize);
+}
+
+/* Compares number of items in two directories (taken as zero for files).
+ * Returns standard -1, 0, 1 for comparisons. */
+static int
+compare_item_count(const dir_entry_t *f, int fdir, const dir_entry_t *s,
+		int sdir)
+{
+	/* We don't want to call entry_get_nitems() for files as sorting huge lists
+	 * of files can call this function a lot of times, thus even small extra
+	 * performance overhead is not desirable. */
+	const uint64_t fsize = fdir ? entry_get_nitems(view, f) : 0U;
+	const uint64_t ssize = sdir ? entry_get_nitems(view, s) : 0U;
+	return (fsize > ssize) ? 1 : (fsize < ssize) ? -1 : 0;
+}
+
+/* Compares two file names according to grouping regular expression.  Returns
+ * standard -1, 0, 1 for comparisons. */
+static int
+compare_group(const char f[], const char s[], regex_t *regex)
+{
+	char fname[NAME_MAX], sname[NAME_MAX];
+	regmatch_t fmatch = get_group_match(regex, f);
+	regmatch_t smatch = get_group_match(regex, s);
+
+	copy_str(fname, MIN(sizeof(fname), fmatch.rm_eo - fmatch.rm_so + 1U),
+			f + fmatch.rm_so);
+	copy_str(sname, MIN(sizeof(sname), smatch.rm_eo - smatch.rm_so + 1U),
+			s + smatch.rm_so);
+
+	return strcmp(fname, sname);
+}
+
+/* Compares two file names according to symbolic link target.  Returns standard
+ * -1, 0, 1 for comparisons. */
+static int
+compare_targets(const dir_entry_t *f, const dir_entry_t *s)
+{
+	char full_path[PATH_MAX];
+	char nlink[PATH_MAX], plink[PATH_MAX];
+
+	if((f->type == FT_LINK) != (s->type == FT_LINK))
+	{
+		/* One of the entries is not a link. */
+		return (f->type == FT_LINK) ? 1 : -1;
+	}
+	if(f->type != FT_LINK)
+	{
+		/* Both entries are not symbolic links. */
+		return 0;
+	}
+
+	/* Both entries are symbolic links. */
+
+	get_full_path_of(f, sizeof(full_path), full_path);
+	if(get_link_target(full_path, nlink, sizeof(nlink)) != 0)
+	{
+		return 0;
+	}
+	get_full_path_of(s, sizeof(full_path), full_path);
+	if(get_link_target(full_path, plink, sizeof(plink)) != 0)
+	{
+		return 0;
+	}
+
+	return stroscmp(nlink, plink);
+}
+
 /* Compares names of two file entries.  Returns positive value if a is greater
  * than b, zero if they are equal, otherwise negative value is returned. */
 static int
@@ -379,8 +552,11 @@ get_secondary_key(SortingKey primary_key)
 		case SK_BY_GROUP_ID:
 		case SK_BY_MODE:
 		case SK_BY_PERMISSIONS:
+		case SK_BY_NLINKS:
 #endif
 		case SK_BY_TYPE:
+		case SK_BY_NITEMS:
+		case SK_BY_TARGET:
 		case SK_BY_TIME_MODIFIED:
 		case SK_BY_TIME_ACCESSED:
 		case SK_BY_TIME_CHANGED:
@@ -388,7 +564,9 @@ get_secondary_key(SortingKey primary_key)
 		case SK_BY_NAME:
 		case SK_BY_INAME:
 		case SK_BY_EXTENSION:
+		case SK_BY_FILEEXT:
 		case SK_BY_SIZE:
+		case SK_BY_GROUPS:
 		case SK_BY_DIR:
 			return SK_BY_SIZE;
 	}
@@ -397,4 +575,4 @@ get_secondary_key(SortingKey primary_key)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
