@@ -26,34 +26,48 @@
 #undef MIN
 #endif
 
+#include <pthread.h> /* PTHREAD_* pthread_* */
+
 #include <assert.h> /* assert() */
 #include <limits.h> /* INT_MIN */
 #include <stddef.h> /* NULL */
 #include <string.h>
+#include <time.h> /* time_t time() */
 
 #include "cfg/config.h"
+#include "compat/fs_limits.h"
+#include "ui/colors.h"
 #include "ui/ui.h"
 #include "utils/env.h"
-#include "utils/fs_limits.h"
+#include "utils/fsdata.h"
 #include "utils/log.h"
 #include "utils/macros.h"
 #include "utils/path.h"
 #include "utils/str.h"
-#include "utils/tree.h"
 #include "utils/utils.h"
-#include "colors.h"
-#include "commands_completion.h"
+#include "cmd_completion.h"
+#include "filelist.h"
 
 /* Environment variables by which application hosted by terminal multiplexer can
  * identify the host. */
 #define SCREEN_ENVVAR "STY"
 #define TMUX_ENVVAR "TMUX"
 
+/* dcache entry. */
+typedef struct
+{
+	uint64_t value;   /* Stored value. */
+	time_t timestamp; /* When the value was set. */
+}
+dcache_data_t;
+
 static void load_def_values(status_t *stats, config_t *config);
 static void determine_fuse_umount_cmd(status_t *stats);
 static void set_gtk_available(status_t *stats);
-static int reset_dircache(status_t *stats);
+static int reset_dircache(void);
 static void set_last_cmdline_command(const char cmd[]);
+static void dcache_get(const char path[], uint64_t *size, uint64_t *nitems,
+		time_t ts);
 
 status_t curr_stats;
 
@@ -61,6 +75,15 @@ static int pending_redraw;
 static int inside_screen;
 static int inside_tmux;
 
+/* Thread-safety guard for dcache_size variable. */
+static pthread_mutex_t dcache_size_mutex = PTHREAD_MUTEX_INITIALIZER;
+/* Thread-safety guard for dcache_nitems variable. */
+static pthread_mutex_t dcache_nitems_mutex = PTHREAD_MUTEX_INITIALIZER;
+/* Cache for directory sizes. */
+static fsdata_t *dcache_size;
+/* Cache for directory item count. */
+static fsdata_t *dcache_nitems;
+
 int
 init_status(config_t *config)
 {
@@ -91,9 +114,9 @@ load_def_values(status_t *stats, config_t *config)
 	stats->register_saved = 0;
 	stats->number_of_windows = 2;
 	stats->use_input_bar = 1;
+	stats->drop_new_dir_hist = 0;
 	stats->load_stage = 0;
 	stats->term_state = TS_NORMAL;
-	stats->dirsize_cache = NULL_TREE;
 	stats->ch_pos = 1;
 	stats->confirmed = 0;
 	stats->skip_shellout_redraw = 0;
@@ -102,6 +125,8 @@ load_def_values(status_t *stats, config_t *config)
 
 	stats->view = 0;
 	stats->graphics_preview = 0;
+	update_string(&stats->preview_cleanup, NULL);
+	stats->clear_preview = 0;
 
 	stats->msg_head = 0;
 	stats->msg_tail = 0;
@@ -110,7 +135,7 @@ load_def_values(status_t *stats, config_t *config)
 
 	stats->scroll_bind_off = 0;
 	stats->split = VSPLIT;
-	stats->splitter_pos = -1.0;
+	stats->splitter_pos = -1;
 
 	stats->sourcing_state = SOURCING_NONE;
 
@@ -129,14 +154,16 @@ load_def_values(status_t *stats, config_t *config)
 
 	stats->original_stdout = NULL;
 
-	stats->chosen_files_out = NULL;
-	stats->chosen_dir_out = NULL;
+	update_string(&stats->chosen_files_out, NULL);
+	update_string(&stats->chosen_dir_out , NULL);
 	(void)replace_string(&stats->output_delimiter, "\n");
 
-	stats->on_choose = NULL;
+	update_string(&stats->on_choose, NULL);
 
 	stats->preview_hint = NULL;
 
+	stats->global_local_settings = 0;
+
 #ifdef HAVE_LIBGTK
 	stats->gtk_available = 0;
 #endif
@@ -180,16 +207,20 @@ reset_status(const config_t *config)
 	curr_stats.initial_lines = config->lines;
 	curr_stats.initial_columns = config->columns;
 
-	return reset_dircache(&curr_stats);
+	return reset_dircache();
 }
 
 /* Returns non-zero on error. */
 static int
-reset_dircache(status_t *stats)
+reset_dircache(void)
 {
-	tree_free(stats->dirsize_cache);
-	stats->dirsize_cache = tree_create(0, 0);
-	return stats->dirsize_cache == NULL_TREE;
+	fsdata_free(dcache_size);
+	dcache_size = fsdata_create(0);
+
+	fsdata_free(dcache_nitems);
+	dcache_nitems = fsdata_create(0);
+
+	return (dcache_size == NULL || dcache_nitems == NULL);
 }
 
 void
@@ -304,5 +335,87 @@ stats_file_choose_action_set(void)
 	    || !is_null_or_empty(curr_stats.on_choose);
 }
 
+void
+dcache_get_at(const char path[], uint64_t *size, uint64_t *nitems)
+{
+	dcache_get(path, size, nitems, 0);
+}
+
+void
+dcache_get_of(const dir_entry_t *entry, uint64_t *size, uint64_t *nitems)
+{
+	char full_path[PATH_MAX];
+	get_full_path_of(entry, sizeof(full_path), full_path);
+	dcache_get(full_path, size, nitems, entry->mtime);
+}
+
+/* Retrieves information about the path if data is newer than ts (0 requests to
+ * skip the check).  size and/or nitems can be NULL.  On unknown values
+ * variables are set to DCACHE_UNKNOWN. */
+static void
+dcache_get(const char path[], uint64_t *size, uint64_t *nitems, time_t ts)
+{
+	/* Initialization to make condition false by default. */
+	dcache_data_t size_data = { .timestamp = ts };
+	dcache_data_t nitems_data;
+
+	pthread_mutex_lock(&dcache_size_mutex);
+	if(fsdata_get(dcache_size, path, &size_data, sizeof(size_data)) != 0 ||
+			(ts != 0 && ts > size_data.timestamp))
+	{
+		size_data.value = DCACHE_UNKNOWN;
+
+		if(ts != 0 && ts > size_data.timestamp)
+		{
+			fsdata_invalidate(dcache_size, path);
+		}
+	}
+	pthread_mutex_unlock(&dcache_size_mutex);
+
+	pthread_mutex_lock(&dcache_nitems_mutex);
+	if(fsdata_get(dcache_nitems, path, &nitems_data, sizeof(nitems_data)) != 0 ||
+			(ts != 0 && ts > nitems_data.timestamp))
+	{
+		nitems_data.value = DCACHE_UNKNOWN;
+	}
+	pthread_mutex_unlock(&dcache_nitems_mutex);
+
+	if(size != NULL)
+	{
+		*size = size_data.value;
+	}
+	if(nitems != NULL)
+	{
+		*nitems = nitems_data.value;
+	}
+}
+
+int
+dcache_set_at(const char path[], uint64_t size, uint64_t nitems)
+{
+	int ret = 0;
+	const time_t ts = time(NULL);
+
+	if(size != DCACHE_UNKNOWN)
+	{
+		const dcache_data_t data = { .value = size, .timestamp = ts };
+
+		pthread_mutex_lock(&dcache_size_mutex);
+		ret |= fsdata_set(dcache_size, path, &data, sizeof(data));
+		pthread_mutex_unlock(&dcache_size_mutex);
+	}
+
+	if(nitems != DCACHE_UNKNOWN)
+	{
+		const dcache_data_t data = { .value = nitems, .timestamp = ts };
+
+		pthread_mutex_lock(&dcache_nitems_mutex);
+		ret |= fsdata_set(dcache_nitems, path, &data, sizeof(data));
+		pthread_mutex_unlock(&dcache_nitems_mutex);
+	}
+
+	return ret;
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
