@@ -20,14 +20,17 @@
 #ifndef VIFM__STATUS_H__
 #define VIFM__STATUS_H__
 
+#include <stdint.h> /* uint64_t */
 #include <stdio.h> /* FILE */
 
-#include "utils/tree.h"
-#include "utils/fs_limits.h"
+#include "compat/fs_limits.h"
+#include "ui/color_scheme.h"
 
-#include "color_scheme.h"
+/* Special value foe dcache fields meaning that it wasn't set. */
+#define DCACHE_UNKNOWN ((uint64_t)-1)
 
 struct config_t;
+struct dir_entry_t;
 
 typedef enum
 {
@@ -98,17 +101,17 @@ typedef struct
 	int curr_register;
 	int register_saved;
 	int number_of_windows;
-	int skip_history;
+	int drop_new_dir_hist; /* Skip recording of new directory history. */
 	int load_stage; /* 0 - no TUI, 1 - part of TUI, 2 - TUI, 3 - all */
 
-	int view;             /* Whether preview mode is active. */
-	int graphics_preview; /* Whether current preview displays graphics. */
+	int view;              /* Whether preview mode is active. */
+	int graphics_preview;  /* Whether current preview displays graphics. */
+	char *preview_cleanup; /* Cleanup command for preview. */
+	int clear_preview;     /* Whether we're in process of clearing preview. */
 
 	/* Describes terminal state with regard to its dimensions. */
 	TermState term_state;
 
-	tree_t dirsize_cache; /* ga command results */
-
 	int last_search_backward;
 
 	int ch_pos; /* for :cd, :pushd and 'letter */
@@ -128,6 +131,9 @@ typedef struct
 
 	int scroll_bind_off;
 	SPLIT split;
+	/* Splitter position relative to viewport, negative values mean "centred".
+	 * Handling it as a special case prevents drifting from center on resizes due
+	 * to rounding. */
 	int splitter_pos;
 
 	SourcingState sourcing_state;
@@ -162,6 +168,8 @@ typedef struct
 
 	void *preview_hint; /* Hint on which view is used for preview. */
 
+	int global_local_settings; /* Set local settings globally. */
+
 #ifdef HAVE_LIBGTK
 	int gtk_available; /* for mimetype detection */
 #endif
@@ -218,7 +226,23 @@ void stats_set_on_choose(const char command[]);
  * so, otherwise zero is returned. */
 int stats_file_choose_action_set(void);
 
+/* Caching of information about directories. */
+
+/* Retrieves information about the path.  size and/or nitems can be NULL.  On
+ * unknown values variables are set to DCACHE_UNKNOWN. */
+void dcache_get_at(const char path[], uint64_t *size, uint64_t *nitems);
+
+/* Retrieves information about the entry.  size and/or nitems can be NULL.  On
+ * unknown values variables are set to DCACHE_UNKNOWN.  Values older than entry
+ * modification date are considered unknown. */
+void dcache_get_of(const struct dir_entry_t *entry, uint64_t *size,
+		uint64_t *nitems);
+
+/* Updates information about the path.  Returns zero on success, otherwise
+ * non-zero is returned. */
+int dcache_set_at(const char path[], uint64_t size, uint64_t nitems);
+
 #endif /* VIFM__STATUS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
