@@ -20,6 +20,7 @@ const char *tags[] = {
 	"vifm-%i",
 	"vifm-%m",
 	"vifm-%n",
+	"vifm-%pc",
 	"vifm-%ph",
 	"vifm-%pw",
 	"vifm-%px",
@@ -40,6 +41,9 @@ const char *tags[] = {
 	"vifm-'confirm'",
 	"vifm-'cpo'",
 	"vifm-'cpoptions'",
+	"vifm-'cvoptions'",
+	"vifm-'deleteprg'",
+	"vifm-'dirsize'",
 	"vifm-'dotdirs'",
 	"vifm-'fastrun'",
 	"vifm-'fcs'",
@@ -58,6 +62,7 @@ const char *tags[] = {
 	"vifm-'iec'",
 	"vifm-'ignorecase'",
 	"vifm-'incsearch'",
+	"vifm-'iooptions'",
 	"vifm-'is'",
 	"vifm-'laststatus'",
 	"vifm-'lines'",
@@ -86,14 +91,17 @@ const char *tags[] = {
 	"vifm-'smartcase'",
 	"vifm-'so'",
 	"vifm-'sort'",
+	"vifm-'sortgroups'",
 	"vifm-'sortnumbers'",
 	"vifm-'sortorder'",
 	"vifm-'statusline'",
 	"vifm-'stl'",
+	"vifm-'suggestoptions'",
 	"vifm-'syscalls'",
 	"vifm-'tabstop'",
 	"vifm-'timefmt'",
 	"vifm-'timeoutlen'",
+	"vifm-'title'",
 	"vifm-'tm'",
 	"vifm-'to'",
 	"vifm-'trash'",
@@ -108,6 +116,7 @@ const char *tags[] = {
 	"vifm-'vimhelp'",
 	"vifm-'vixcmd'",
 	"vifm-'wildmenu'",
+	"vifm-'wildstyle'",
 	"vifm-'wmnu'",
 	"vifm-'wordchars'",
 	"vifm-'wrap'",
@@ -126,6 +135,8 @@ const char *tags[] = {
 	"vifm---on-choose",
 	"vifm---remote",
 	"vifm---select",
+	"vifm---server-list",
+	"vifm---server-name",
 	"vifm---version",
 	"vifm--c",
 	"vifm--f",
@@ -139,7 +150,12 @@ const char *tags[] = {
 	"vifm-:!!",
 	"vifm-:alink",
 	"vifm-:apropos",
+	"vifm-:au",
+	"vifm-:autocmd",
 	"vifm-:bar",
+	"vifm-:bmark",
+	"vifm-:bmarks",
+	"vifm-:bmgo",
 	"vifm-:c",
 	"vifm-:ca",
 	"vifm-:cabbrev",
@@ -169,6 +185,7 @@ const char *tags[] = {
 	"vifm-:cunabbrev",
 	"vifm-:cunmap",
 	"vifm-:d",
+	"vifm-:delbmarks",
 	"vifm-:delc",
 	"vifm-:delcommand",
 	"vifm-:delete",
@@ -183,6 +200,8 @@ const char *tags[] = {
 	"vifm-:edit",
 	"vifm-:el",
 	"vifm-:else",
+	"vifm-:elsei",
+	"vifm-:elseif",
 	"vifm-:empty",
 	"vifm-:en",
 	"vifm-:endif",
@@ -248,7 +267,9 @@ const char *tags[] = {
 	"vifm-:on",
 	"vifm-:only",
 	"vifm-:popd",
+	"vifm-:pu",
 	"vifm-:pushd",
+	"vifm-:put",
 	"vifm-:pw",
 	"vifm-:pwd",
 	"vifm-:q",
@@ -271,7 +292,12 @@ const char *tags[] = {
 	"vifm-:s",
 	"vifm-:screen",
 	"vifm-:se",
+	"vifm-:select",
 	"vifm-:set",
+	"vifm-:setg",
+	"vifm-:setglobal",
+	"vifm-:setl",
+	"vifm-:setlocal",
 	"vifm-:sh",
 	"vifm-:shell",
 	"vifm-:so",
@@ -291,6 +317,7 @@ const char *tags[] = {
 	"vifm-:unlet",
 	"vifm-:unm",
 	"vifm-:unmap",
+	"vifm-:unselect",
 	"vifm-:ve",
 	"vifm-:version",
 	"vifm-:vie",
@@ -306,6 +333,8 @@ const char *tags[] = {
 	"vifm-:vu",
 	"vifm-:vunmap",
 	"vifm-:w",
+	"vifm-:winc",
+	"vifm-:wincmd",
 	"vifm-:windo",
 	"vifm-:winrun",
 	"vifm-:wq",
@@ -436,6 +465,7 @@ const char *tags[] = {
 	"vifm-c_Up",
 	"vifm-cancellation",
 	"vifm-cg",
+	"vifm-chooseopt()",
 	"vifm-cl",
 	"vifm-clientserver",
 	"vifm-co",
@@ -482,6 +512,8 @@ const char *tags[] = {
 	"vifm-expr2",
 	"vifm-expr3",
 	"vifm-expr4",
+	"vifm-expr5",
+	"vifm-expr6",
 	"vifm-expression-syntax",
 	"vifm-f",
 	"vifm-filetype()",
@@ -494,6 +526,7 @@ const char *tags[] = {
 	"vifm-gUgU",
 	"vifm-ga",
 	"vifm-general-keys",
+	"vifm-getpanetype()",
 	"vifm-gf",
 	"vifm-gg",
 	"vifm-gh",
@@ -513,13 +546,26 @@ const char *tags[] = {
 	"vifm-j",
 	"vifm-k",
 	"vifm-l",
+	"vifm-layoutis()",
 	"vifm-literal-string",
 	"vifm-local-options",
 	"vifm-ls-view",
 	"vifm-m",
 	"vifm-m_/",
 	"vifm-m_:",
+	"vifm-m_:exi",
+	"vifm-m_:exit",
+	"vifm-m_:noh",
+	"vifm-m_:nohlsearch",
+	"vifm-m_:q",
+	"vifm-m_:quit",
+	"vifm-m_:range",
+	"vifm-m_:w",
+	"vifm-m_:write",
+	"vifm-m_:x",
+	"vifm-m_:xit",
 	"vifm-m_?",
+	"vifm-m_B",
 	"vifm-m_CTRL-B",
 	"vifm-m_CTRL-C",
 	"vifm-m_CTRL-D",
@@ -540,6 +586,7 @@ const char *tags[] = {
 	"vifm-m_ZQ",
 	"vifm-m_ZZ",
 	"vifm-m_b",
+	"vifm-m_c",
 	"vifm-m_e",
 	"vifm-m_gf",
 	"vifm-m_gg",
@@ -548,6 +595,7 @@ const char *tags[] = {
 	"vifm-m_l",
 	"vifm-m_n",
 	"vifm-m_q",
+	"vifm-m_v",
 	"vifm-m_zH",
 	"vifm-m_zL",
 	"vifm-m_zb",
@@ -558,10 +606,13 @@ const char *tags[] = {
 	"vifm-macros",
 	"vifm-mappings",
 	"vifm-menus-and-dialogs",
+	"vifm-more",
 	"vifm-n",
 	"vifm-normal",
 	"vifm-options",
 	"vifm-p",
+	"vifm-pager",
+	"vifm-paneisat()",
 	"vifm-patterns",
 	"vifm-plugin",
 	"vifm-q/",
@@ -662,7 +713,10 @@ const char *tags[] = {
 	"vifm-zf",
 	"vifm-zm",
 	"vifm-zo",
+	"vifm-zr",
 	"vifm-zt",
 	"vifm-zz",
+	"vifm-{",
+	"vifm-}",
 	0,
 };
