@@ -26,13 +26,15 @@
 #include <termios.h> /* struct winsize */
 #endif
 #include <sys/time.h> /* gettimeofday() */
+#include <pthread.h> /* PTHREAD_* pthread_* */
 #include <unistd.h>
 
+#include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL size_t wchar_t */
 #include <stdint.h> /* uint64_t */
-#include <stdlib.h> /* abs() free() malloc() */
+#include <stdlib.h> /* abs() free() */
 #include <stdio.h> /* snprintf() vsnprintf() */
 #include <string.h> /* memset() strcat() strcmp() strcpy() strdup() strlen() */
 #include <time.h> /* time() */
@@ -41,31 +43,33 @@
 #include "../cfg/config.h"
 #include "../cfg/info.h"
 #include "../compat/curses.h"
+#include "../compat/fs_limits.h"
 #include "../engine/mode.h"
+#include "../int/term_title.h"
 #include "../modes/dialogs/msg_dialog.h"
 #include "../modes/modes.h"
 #include "../modes/view.h"
+#include "../modes/wk.h"
 #include "../utils/fs.h"
-#include "../utils/fs_limits.h"
 #include "../utils/log.h"
 #include "../utils/macros.h"
+#include "../utils/matchers.h"
 #include "../utils/path.h"
 #include "../utils/str.h"
 #include "../utils/utf8.h"
 #include "../utils/utils.h"
-#include "../color_manager.h"
-#include "../color_scheme.h"
-#include "../colors.h"
 #include "../event_loop.h"
 #include "../filelist.h"
-#include "../fileview.h"
 #include "../opt_handlers.h"
-#include "../quickview.h"
 #include "../status.h"
-#include "../term_title.h"
 #include "../vifm.h"
 #include "private/statusline.h"
 #include "cancellation.h"
+#include "color_manager.h"
+#include "color_scheme.h"
+#include "colors.h"
+#include "fileview.h"
+#include "quickview.h"
 #include "statusbar.h"
 #include "statusline.h"
 
@@ -77,6 +81,14 @@ static WINDOW *ltop_line2;
 static WINDOW *rtop_line1;
 static WINDOW *rtop_line2;
 
+/* Mutexes for views, located out of FileView so that they are never moved nor
+ * copied, which would yield undefined behaviour. */
+static pthread_mutex_t lwin_timestamps_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t rwin_timestamps_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+FileView lwin = { .timestamps_mutex = &lwin_timestamps_mutex };
+FileView rwin = { .timestamps_mutex = &rwin_timestamps_mutex };
+
 static void create_windows(void);
 static void update_geometry(void);
 static int get_working_area_height(void);
@@ -92,6 +104,7 @@ static int get_ruler_width(FileView *view);
 static char * expand_ruler_macros(FileView *view, const char format[]);
 static void switch_panes_content(void);
 static void update_origins(FileView *view, const char *old_main_origin);
+static void set_splitter(int pos);
 static char * path_identity(const char path[]);
 static char * format_view_title(const FileView *view, path_func pf);
 static void print_view_title(const FileView *view, int active_view,
@@ -134,7 +147,7 @@ setup_ncurses_interface(void)
 	nonl();
 	raw();
 
-	curs_set(FALSE);
+	curs_set(0);
 
 	getmaxyx(stdscr, screen_y, screen_x);
 	/* screen is too small to be useful*/
@@ -149,7 +162,7 @@ setup_ncurses_interface(void)
 	start_color();
 	use_default_colors();
 
-	load_def_scheme();
+	cs_load_defaults();
 
 	create_windows();
 
@@ -221,8 +234,6 @@ ui_update_term_state(void)
 int
 ui_char_pressed(wint_t c)
 {
-	static const wint_t CTRL_C = L'\x03';
-
 	wint_t pressed = L'\0';
 	const int cancellation_state = ui_cancellation_pause();
 
@@ -230,7 +241,7 @@ ui_char_pressed(wint_t c)
 	wtimeout(status_bar, 0);
 	if(compat_wget_wch(status_bar, &pressed) != ERR)
 	{
-		if(pressed != c && pressed != CTRL_C)
+		if(pressed != c && pressed != NC_C_c)
 		{
 			compat_unget_wch(pressed);
 		}
@@ -238,7 +249,7 @@ ui_char_pressed(wint_t c)
 
 	ui_cancellation_resume(cancellation_state);
 
-	if(c != CTRL_C && pressed == CTRL_C)
+	if(c != NC_C_c && pressed == NC_C_c)
 	{
 		ui_cancellation_request();
 	}
@@ -344,7 +355,7 @@ horizontal_layout(int screen_x, int screen_y)
 	if(splitter_pos < 2)
 		splitter_pos = 2;
 	if(splitter_pos > get_working_area_height() - 1)
-		splitter_pos = get_working_area_height();
+		splitter_pos = get_working_area_height() - 1;
 	if(curr_stats.splitter_pos >= 0)
 		curr_stats.splitter_pos = splitter_pos;
 
@@ -432,6 +443,13 @@ resize_all(void)
 	wresize(rborder, border_height, 1);
 	mvwin(rborder, 1, screen_x - 1);
 
+	/* These need a resize at least after terminal size was zero or they grow and
+	 * produce bad looking effect. */
+	wresize(ltop_line1, 1, 1);
+	wresize(ltop_line2, 1, 1);
+	wresize(rtop_line1, 1, 1);
+	wresize(rtop_line2, 1, 1);
+
 	if(curr_stats.number_of_windows == 1)
 	{
 		only_layout(&lwin, screen_x, screen_y);
@@ -454,13 +472,13 @@ resize_all(void)
 	correct_size(&rwin);
 
 	wresize(stat_win, 1, screen_x);
-	(void)ui_stat_reposition(1);
+	(void)ui_stat_reposition(1, 0);
 
 	wresize(job_bar, 1, screen_x);
 
 	update_statusbar_layout();
 
-	curs_set(FALSE);
+	curs_set(0);
 }
 
 /* Calculates height available for main area that contains file lists.  Returns
@@ -536,8 +554,15 @@ update_screen(UpdateType update_kind)
 	curr_stats.need_update = UT_NONE;
 
 	update_views(update_kind == UT_FULL);
+	/* Redraw message dialog over updated panes.  It's not very nice to do it
+	 * here, but for sure better then blocking pane updates by checking for
+	 * message mode. */
+	if(vle_mode_is(MSG_MODE))
+	{
+		redraw_msg_dialog(0);
+	}
 
-	update_stat_window(curr_view);
+	update_stat_window(curr_view, 0);
 
 	if(!is_status_bar_multiline())
 	{
@@ -604,6 +629,8 @@ clear_border(WINDOW *border)
 	{
 		mvwaddstr(border, i, 0, cfg.border_filler);
 	}
+
+	wnoutrefresh(border);
 }
 
 /* Updates (redraws or reloads) views. */
@@ -652,7 +679,7 @@ change_window(void)
 {
 	swap_view_roles();
 
-	load_local_options(curr_view);
+	load_view_options(curr_view);
 
 	if(curr_stats.number_of_windows != 1)
 	{
@@ -816,38 +843,52 @@ clear_num_window(void)
 	}
 }
 
-/* msg can't be NULL
- * period - how often status bar should be updated
- * if period equals 0 reset inner counter
- */
 void
-show_progress(const char *msg, int period)
+show_progress(const char msg[], int period)
 {
 	static char marks[] = {'|', '/', '-', '\\'};
-	static int count = 0;
-	static int pause = 1;
+	static int mark = 0;
+	static int count = 1;
+	static int total = 0;
 
+	/* Do nothing if UI is not functional. */
 	if(curr_stats.load_stage < 1)
 	{
 		return;
 	}
 
+	/* Reset state. */
 	if(period == 0)
 	{
-		pause = 1;
+		count = 1;
+		total = 0;
 		return;
 	}
 
-	pause++;
+	/* Advance state. */
+	++count;
+	++total;
 
-	if(pause%period != 0)
+	/* Skip intermediate updates to do not hammer UI with refreshes. */
+	if(period != 1 && count%period != 1)
+	{
 		return;
+	}
+	count = 1;
 
-	pause = 1;
+	/* Assume that period equal to one means that message already contains counter
+	 * (maybe along with total number). */
+	if(period == 1)
+	{
+		ui_sb_quick_msgf("%s %c", msg, marks[mark]);
+	}
+	else
+	{
+		ui_sb_quick_msgf("%s %c %d", msg, marks[mark], total);
+	}
 
-	ui_sb_quick_msgf("%s %c", msg, marks[count]);
-
-	count = (count + 1) % sizeof(marks);
+	/* Pick next mark character. */
+	mark = (mark + 1) % sizeof(marks);
 }
 
 void
@@ -1000,12 +1041,14 @@ update_term_size(void)
 #ifndef _WIN32
 	struct winsize ws = { .ws_col = -1, .ws_row = -1 };
 
-	if(ioctl(0, TIOCGWINSZ, &ws) == -1)
+	if(ioctl(STDIN_FILENO, TIOCGWINSZ, &ws) == -1)
 	{
 		LOG_SERROR_MSG(errno, "Failed to query terminal size.");
 		vifm_finish("Terminal error.");
 	}
-	if(ws.ws_row <= 0 || ws.ws_col <= 0)
+	/* Allow 0 as GNU Hurd returns this, is_term_resized() returns false then, but
+	 * everything works. */
+	if(ws.ws_row == (typeof(ws.ws_row))-1 || ws.ws_col == (typeof(ws.ws_col))-1)
 	{
 		LOG_INFO_MSG("ws.ws_row = %d; ws.ws_col = %d", ws.ws_row, ws.ws_col);
 		vifm_finish("Terminal is unable to run vifm.");
@@ -1018,6 +1061,17 @@ update_term_size(void)
 #endif
 }
 
+int
+ui_term_is_alive(void)
+{
+#ifndef _WIN32
+	struct winsize ws;
+	return (ioctl(0, TIOCGWINSZ, &ws) != -1);
+#else
+	return 1;
+#endif
+}
+
 /* Re-layouts windows located on status bar (status bar itself, input and the
  * ruler). */
 static void
@@ -1085,7 +1139,7 @@ refresh_view_win(FileView *view)
 
 	wrefresh(view->win);
 	/* Use getmaxy(...) instead of multiline_status_bar to handle command line
-	 * mode, which doesn't use this module to show multilined messages. */
+	 * mode, which doesn't use this module to show multiline messages. */
 	if(cfg.display_statusline && getmaxy(status_bar) > 1)
 	{
 		touchwin(stat_win);
@@ -1101,17 +1155,13 @@ move_window(FileView *view, int horizontally, int first)
 	split_view(split_type);
 	if(view != desired_view)
 	{
-		switch_windows();
+		/* Switch two panes saving current windows as the active one (left/top or
+		 * right/bottom). */
+		switch_panes_content();
+		go_to_other_pane();
 	}
 }
 
-void
-switch_windows(void)
-{
-	switch_panes_content();
-	go_to_other_pane();
-}
-
 void
 switch_panes(void)
 {
@@ -1119,6 +1169,37 @@ switch_panes(void)
 	try_activate_view_mode();
 }
 
+void
+ui_view_pick(FileView *view, FileView **old_curr, FileView **old_other)
+{
+	*old_curr = curr_view;
+	*old_other = other_view;
+
+	curr_view = view;
+	other_view = (view == *old_curr) ? *old_other : *old_curr;
+	if(curr_view != *old_curr)
+	{
+		load_view_options(curr_view);
+	}
+}
+
+void
+ui_view_unpick(FileView *view, FileView *old_curr, FileView *old_other)
+{
+	if(curr_view != view)
+	{
+		/* Do nothing if view roles were switched from outside. */
+		return;
+	}
+
+	curr_view = old_curr;
+	other_view = old_other;
+	if(curr_view != view)
+	{
+		load_view_options(curr_view);
+	}
+}
+
 /* Switches panes content. */
 static void
 switch_panes_content(void)
@@ -1214,17 +1295,108 @@ only(void)
 	}
 }
 
+void
+move_splitter(int by, int fact)
+{
+	/* Determine exact splitter position if it's centered at the moment. */
+	if(curr_stats.splitter_pos < 0)
+	{
+		if(curr_stats.split == VSPLIT)
+		{
+			curr_stats.splitter_pos = getmaxx(stdscr)/2 - 1 + getmaxx(stdscr)%2;
+		}
+		else
+		{
+			curr_stats.splitter_pos = getmaxy(stdscr)/2 - 1;
+		}
+	}
+
+	set_splitter(curr_stats.splitter_pos + fact*by);
+}
+
+void
+ui_view_resize(FileView *view, int to)
+{
+	int pos;
+
+	if(curr_stats.split == HSPLIT)
+	{
+		const int height = get_working_area_height();
+		pos = (view == &lwin) ? (1 + to) : (height - to);
+	}
+	else
+	{
+		const int width = getmaxx(stdscr) - 1;
+		pos = (view == &lwin) ? to : (width - to);
+	}
+
+	set_splitter(pos);
+}
+
+/* Sets splitter position making sure it isn't negative value which has special
+ * meaning. */
+static void
+set_splitter(int pos)
+{
+	curr_stats.splitter_pos = (pos < 0) ? 0 : pos;
+	update_screen(UT_REDRAW);
+}
+
 void
 format_entry_name(const dir_entry_t *entry, size_t buf_len, char buf[])
 {
-	const FileType type = ui_view_entry_target_type(entry);
-
-	const char prefix[2] = { cfg.decorations[type][DECORATION_PREFIX] };
-	const char suffix[2] = { cfg.decorations[type][DECORATION_SUFFIX] };
-
+	const char *prefix, *suffix;
+	ui_get_decors(entry, &prefix, &suffix);
 	snprintf(buf, buf_len, "%s%s%s", prefix, entry->name, suffix);
 }
 
+void
+ui_get_decors(const dir_entry_t *entry, const char **prefix,
+		const char **suffix)
+{
+	if(entry->name_dec_num == -1)
+	{
+		/* Find a match and cache the result. */
+
+		((dir_entry_t *)entry)->name_dec_num = 0;
+		if(cfg.name_dec_count != 0)
+		{
+			char full_path[PATH_MAX];
+			int i;
+
+			get_full_path_of(entry, sizeof(full_path), full_path);
+
+			for(i = 0; i < cfg.name_dec_count; ++i)
+			{
+				const file_dec_t *const file_dec = &cfg.name_decs[i];
+				if(matchers_match(file_dec->matchers, full_path))
+				{
+					if(matchers_match(file_dec->matchers, full_path))
+					{
+						((dir_entry_t *)entry)->name_dec_num = i + 1;
+						break;
+					}
+				}
+			}
+		}
+	}
+
+	if(entry->name_dec_num == 0)
+	{
+		const FileType type = ui_view_entry_target_type(entry);
+		*prefix = cfg.type_decs[type][DECORATION_PREFIX];
+		*suffix = cfg.type_decs[type][DECORATION_SUFFIX];
+	}
+	else
+	{
+		assert(entry->name_dec_num - 1 >= 0 && "Wrong index.");
+		assert(entry->name_dec_num - 1 < cfg.name_dec_count && "Wrong index.");
+
+		*prefix = cfg.name_decs[entry->name_dec_num - 1].prefix;
+		*suffix = cfg.name_decs[entry->name_dec_num - 1].suffix;
+	}
+}
+
 void
 checked_wmove(WINDOW *win, int y, int x)
 {
@@ -1313,9 +1485,9 @@ ui_view_title_update(FileView *view)
 	title = format_view_title(view,
 			cfg.shorten_title_paths ? &replace_home_part : &path_identity);
 
-	if(view == selected)
+	if(view == selected && cfg.set_title)
 	{
-		set_term_title(title);
+		term_title_update(title);
 	}
 
 	print_view_title(view, view == selected, title);
@@ -1385,10 +1557,8 @@ fixup_titles_attributes(const FileView *view, int active_view)
 {
 	if(active_view)
 	{
-		col_attr_t col;
-
-		col = cfg.cs.color[TOP_LINE_COLOR];
-		mix_colors(&col, &cfg.cs.color[TOP_LINE_SEL_COLOR]);
+		col_attr_t col = cfg.cs.color[TOP_LINE_COLOR];
+		cs_mix_colors(&col, &cfg.cs.color[TOP_LINE_SEL_COLOR]);
 
 		wbkgdset(view->title, COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) |
 				(col.attr & A_REVERSE));
@@ -1428,14 +1598,14 @@ ui_view_sort_list_contains(const char sort[SK_COUNT], char key)
 }
 
 void
-ui_view_sort_list_ensure_well_formed(FileView *view)
+ui_view_sort_list_ensure_well_formed(FileView *view, char sort_keys[])
 {
 	int found_name_key = 0;
 	int i = -1;
 
 	while(++i < SK_COUNT)
 	{
-		const int sort_key = abs(view->sort[i]);
+		const int sort_key = abs(sort_keys[i]);
 		if(sort_key > SK_LAST)
 		{
 			break;
@@ -1447,17 +1617,26 @@ ui_view_sort_list_ensure_well_formed(FileView *view)
 	}
 
 	if(!found_name_key && i < SK_COUNT &&
-			(!flist_custom_active(view) || !view->custom.unsorted))
+			(!flist_custom_active(view) ||
+			 (sort_keys == view->sort && !view->custom.unsorted)))
 	{
-		view->sort[i++] = SK_DEFAULT;
+		sort_keys[i++] = SK_DEFAULT;
 	}
 
 	if(i < SK_COUNT)
 	{
-		memset(&view->sort[i], SK_NONE, SK_COUNT - i);
+		memset(&sort_keys[i], SK_NONE, SK_COUNT - i);
 	}
 }
 
+char *
+ui_view_sort_list_get(const FileView *view)
+{
+	return (flist_custom_active(view) && view->custom.unsorted)
+	     ? (char *)view->custom.sort
+	     : (char *)view->sort_g;
+}
+
 int
 ui_view_displays_numbers(const FileView *const view)
 {
@@ -1503,10 +1682,34 @@ ui_view_entry_target_type(const dir_entry_t *entry)
 int
 ui_view_available_width(const FileView *const view)
 {
-	const int correction = cfg.filelist_col_padding ? -2 : 0;
+	const int correction = cfg.extra_padding ? -2 : 0;
 	return ((int)view->window_width + 1) + correction;
 }
 
+int
+ui_qv_left(const FileView *view)
+{
+	return cfg.extra_padding ? 1 : 0;
+}
+
+int
+ui_qv_top(const FileView *view)
+{
+	return cfg.extra_padding ? 1 : 0;
+}
+
+int
+ui_qv_height(const FileView *view)
+{
+	return cfg.extra_padding ? view->window_rows - 1 : view->window_rows + 1;
+}
+
+int
+ui_qv_width(const FileView *view)
+{
+	return cfg.extra_padding ? view->window_width - 1 : view->window_width + 1;
+}
+
 const col_scheme_t *
 ui_view_get_cs(const FileView *view)
 {
@@ -1538,7 +1741,7 @@ ui_view_wipe(FileView *view)
 	(void)pair_content(PAIR_NUMBER(getbkgd(view->win)), &fg, &bg);
 	wattrset(view->win, COLOR_PAIR(colmgr_get_pair(bg, bg)));
 
-	memset(line_filler, '\x09', sizeof(line_filler));
+	memset(line_filler, '\t', sizeof(line_filler));
 	for(i = 0; i < height; ++i)
 	{
 		mvwaddstr(view->win, i, 0, line_filler);
@@ -1550,19 +1753,25 @@ ui_view_wipe(FileView *view)
 void
 ui_view_schedule_redraw(FileView *view)
 {
+	pthread_mutex_lock(view->timestamps_mutex);
 	view->postponed_redraw = get_updated_time(view->postponed_redraw);
+	pthread_mutex_unlock(view->timestamps_mutex);
 }
 
 void
 ui_view_schedule_reload(FileView *view)
 {
+	pthread_mutex_lock(view->timestamps_mutex);
 	view->postponed_reload = get_updated_time(view->postponed_reload);
+	pthread_mutex_unlock(view->timestamps_mutex);
 }
 
 void
 ui_view_schedule_full_reload(FileView *view)
 {
+	pthread_mutex_lock(view->timestamps_mutex);
 	view->postponed_full_reload = get_updated_time(view->postponed_full_reload);
+	pthread_mutex_unlock(view->timestamps_mutex);
 }
 
 /* Gets updated timestamp ensuring that it differs from the previous value.
@@ -1589,6 +1798,8 @@ ui_view_query_scheduled_event(FileView *view)
 {
 	UiUpdateEvent event;
 
+	pthread_mutex_lock(view->timestamps_mutex);
+
 	if(view->postponed_full_reload != view->last_reload)
 	{
 		event = UUE_FULL_RELOAD;
@@ -1610,8 +1821,10 @@ ui_view_query_scheduled_event(FileView *view)
 	view->last_reload = view->postponed_reload;
 	view->postponed_full_reload = view->postponed_reload;
 
+	pthread_mutex_unlock(view->timestamps_mutex);
+
 	return event;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
