@@ -20,30 +20,26 @@
 #ifndef VIFM__UI__UI_H__
 #define VIFM__UI__UI_H__
 
-#ifdef _WIN32
-#include <windows.h>
-#endif
-
 #include <sys/types.h>
 
 #include <curses.h>
+#include <regex.h> /* regex_t */
+#include <pthread.h> /* pthread_mutex_t */
+
 #include <stddef.h> /* size_t wchar_t */
-#include <stdint.h> /* uint64_t */
+#include <stdint.h> /* uint64_t uint32_t */
 #include <stdlib.h> /* mode_t */
 #include <time.h> /* time_t */
 #include <wchar.h> /* wint_t */
 
-#include "../utils/filemon.h"
+#include "../compat/fs_limits.h"
 #include "../utils/filter.h"
-#include "../utils/fs_limits.h"
+#include "../utils/fswatch.h"
 #include "../utils/trie.h"
-#include "../color_scheme.h"
-#include "../column_view.h"
 #include "../status.h"
 #include "../types.h"
-
-#define MIN_TERM_HEIGHT 10
-#define MIN_TERM_WIDTH 30
+#include "color_scheme.h"
+#include "column_view.h"
 
 #define SORT_WIN_WIDTH 32
 
@@ -54,6 +50,11 @@
  * line). */
 #define POS_WIN_MIN_WIDTH 13
 
+/* Menus don't look like menus as all if height is less than 5. */
+#define MIN_TERM_HEIGHT 5
+/* There is a lower limit on statusbar width. */
+#define MIN_TERM_WIDTH (INPUT_WIN_WIDTH + 1 + POS_WIN_MIN_WIDTH)
+
 /* Width of the ruler and input windows. */
 #define FIELDS_WIDTH() (INPUT_WIN_WIDTH + getmaxx(ruler_win))
 
@@ -62,25 +63,34 @@
  * updated accordingly. */
 typedef enum
 {
-	SK_BY_EXTENSION = 1,
-	SK_BY_NAME,
+	SK_BY_EXTENSION = 1,  /* Extension of files and directories. */
+	SK_BY_NAME,           /* Name (including extension). */
 #ifndef _WIN32
-	SK_BY_GROUP_ID,
-	SK_BY_GROUP_NAME,
-	SK_BY_MODE,
-	SK_BY_OWNER_ID,
-	SK_BY_OWNER_NAME,
+	SK_BY_GROUP_ID,       /* Group id. */
+	SK_BY_GROUP_NAME,     /* Group name. */
+	SK_BY_MODE,           /* File mode (file type + permissions) in octal. */
+	SK_BY_OWNER_ID,       /* Owner id. */
+	SK_BY_OWNER_NAME,     /* Owner name. */
 #endif
-	SK_BY_SIZE,
-	SK_BY_TIME_ACCESSED,
-	SK_BY_TIME_CHANGED,
-	SK_BY_TIME_MODIFIED,
-	SK_BY_INAME,
+	SK_BY_SIZE,           /* Size. */
+	SK_BY_TIME_ACCESSED,  /* Time accessed (e.g. read, executed). */
+	SK_BY_TIME_CHANGED,   /* Time changed (changes in metadata, e.g. mode). */
+	SK_BY_TIME_MODIFIED,  /* Time modified (when file contents is changed). */
+	SK_BY_INAME,          /* Name (including extension, ignores case). */
 #ifndef _WIN32
-	SK_BY_PERMISSIONS,
+	SK_BY_PERMISSIONS,    /* Permissions string. */
 #endif
-	SK_BY_DIR,
-	SK_BY_TYPE,
+	SK_BY_DIR,            /* Directory grouping (directory < file). */
+	SK_BY_TYPE,           /* File type (dir/reg/exe/link/char/block/sock/fifo). */
+	SK_BY_FILEEXT,        /* Extension of files only. */
+	SK_BY_NITEMS,         /* Number of items in a directory (zero for files). */
+	SK_BY_GROUPS,         /* Groups extracted via regexps from 'sortgroups'. */
+#ifndef _WIN32
+	SK_BY_NLINKS,         /* Number of hard links. */
+#endif
+	SK_BY_TARGET,         /* Symbolic link target (empty for other file types). */
+	/* New elements *must* be added here to keep values stored in existing
+	 * vifminfo files valid.  Don't forget to update SK_LAST below. */
 }
 SortingKey;
 
@@ -94,7 +104,7 @@ enum
 #endif
 
 	/* Value of the last sort option. */
-	SK_LAST = SK_BY_TYPE,
+	SK_LAST = SK_BY_TARGET,
 
 	/* Number of sort options. */
 	SK_COUNT = SK_LAST,
@@ -131,7 +141,7 @@ typedef struct
 }
 history_t;
 
-typedef struct
+typedef struct dir_entry_t
 {
 	char *name;
 	char *origin;     /* Location where this file comes from. */
@@ -141,25 +151,28 @@ typedef struct
 	gid_t gid;
 	mode_t mode;
 #else
-	DWORD attrs;
+	uint32_t attrs;
 #endif
 	time_t mtime;
 	time_t atime;
 	time_t ctime;
 	FileType type;
-
-	int selected;
-	int was_selected; /* Stores previous selection state in Visual mode. */
-
-	int search_match;      /* Whether the item matches last search. */
-	short int match_left;  /* Starting position of the match. */
-	short int match_right; /* Ending position of the match. */
+	int nlinks;       /* Number of hard links to the entry. */
 
 	int list_num;     /* Used by sorting comparer to perform stable sort. */
 
-	int marked;       /* Whether file should be processed. */
-
 	int hi_num;       /* File highlighting parameters cache (initially -1). */
+	int name_dec_num; /* File decoration parameters cache (initially -1).  The
+	                     value is shifted by one, 0 means type decoration. */
+
+	int search_match;      /* Non-zero if the item matches last search.  Equals to
+	                          search match number (top to bottom order). */
+	short int match_left;  /* Starting position of search match. */
+	short int match_right; /* Ending position of search match. */
+
+	unsigned int selected : 1;
+	unsigned int was_selected : 1; /* Previous selection state for Visual mode. */
+	unsigned int marked : 1;       /* Whether file should be processed. */
 }
 dir_entry_t;
 
@@ -174,10 +187,11 @@ typedef struct
 	/* Parameters related to custom filling. */
 	struct
 	{
-		/* File entries. */
-		dir_entry_t *entries;
-		/* Number of file entries. */
-		int entry_count;
+		/* This is temporary storage for custom list entries used during its
+		 * construction as well as storage for unfiltered custom list if local
+		 * filter is not empty. */
+		dir_entry_t *entries; /* File entries. */
+		int entry_count;      /* Number of file entries. */
 
 		/* Directory we were in before custom view activation. */
 		char *orig_dir;
@@ -190,19 +204,15 @@ typedef struct
 		char sort[SK_COUNT];
 
 		/* Names of files in custom view while it's being composed.  Used for
-		 * duplicate elimination. */
+		 * duplicate elimination during construction of custom list. */
 		trie_t paths_cache;
 	}
 	custom;
 
-#ifndef _WIN32
 	/* Monitor that checks for directory changes. */
-	filemon_t mon;
-#else
-	FILETIME dir_mtime;
-	HANDLE dir_watcher;
+	fswatch_t *watch;
 	char watched_dir[PATH_MAX];
-#endif
+
 	char last_dir[PATH_MAX];
 
 	/* Number of files that match current search pattern. */
@@ -220,12 +230,12 @@ typedef struct
 	int window_rows; /* number of rows shown in window */
 	unsigned int window_width;
 	int filtered;  /* number of files filtered out and not shown in list */
-	int selected_files;
+	int selected_files; /* Number of currently selected files. */
 	int local_cs; /* Whether directory-specific color scheme is in use. */
-	dir_entry_t *dir_entry;
-	char ** selected_filelist;
-	int nsaved_selection;
-	char ** saved_selection;
+	dir_entry_t *dir_entry; /* Must be handled via dynarray unit. */
+
+	int nsaved_selection;   /* Number of items in saved_selection. */
+	char **saved_selection; /* Names of selected files. */
 
 	/* Whether files are selected by user or via range on the command-line.  Say
 	 * some commands (e.g. ga/gA) implement "smart selection", but it shouldn't
@@ -272,7 +282,12 @@ typedef struct
 	}
 	local_filter;
 
-	char sort[SK_COUNT];
+	/* List of sorting keys. */
+	char sort[SK_COUNT], sort_g[SK_COUNT];
+	/* Sorting groups (comma-separated list of regular expressions). */
+	char *sort_groups, *sort_groups_g;
+	/* Primary group in compiled form. */
+	regex_t primary_group;
 
 	int history_num;
 	int history_pos;
@@ -281,27 +296,32 @@ typedef struct
 	col_scheme_t cs;
 
 	columns_t columns; /* handle for column_view unit */
-	char *view_columns; /* format string of columns */
+	char *view_columns, *view_columns_g; /* Format string of columns. */
 
 	/* ls-like view related fields */
-	int ls_view; /* non-zero if ls-like view is enabled */
+	int ls_view, ls_view_g; /* Non-zero if ls-like view is enabled. */
 	size_t max_filename_width; /* Maximum filename width (length in character
 	                            * positions on the screen) among all entries of
-	                            * the file list. */
+	                            * the file list.  Zero if not calculated. */
 	size_t column_count; /* number of columns in the view, used for list view */
 	size_t window_cells; /* max number of files that can be displayed */
 
-	NumberingType num_type; /* Whether and how line numbers are displayed. */
-	int num_width; /* Min number of characters reserved for number field. */
+	/* Whether and how line numbers are displayed. */
+	NumberingType num_type, num_type_g;
+	/* Min number of characters reserved for number field. */
+	int num_width, num_width_g;
 	int real_num_width; /* Real character count reserved for number field. */
 
 	/* Timestamps for controlling of scheduling update requests.  They are in
 	 * microseconds.  Real resolution is bigger than microsecond, but it's not
 	 * critical. */
 
-	uint64_t postponed_redraw;      /* Time of last redraw request. */
-	uint64_t postponed_reload;      /* Time of last redraw request. */
-	uint64_t postponed_full_reload; /* Time of last full redraw request. */
+	uint64_t postponed_redraw;         /* Time of last redraw request. */
+	uint64_t postponed_reload;         /* Time of last redraw request. */
+	uint64_t postponed_full_reload;    /* Time of last full redraw request. */
+	pthread_mutex_t *timestamps_mutex; /* Protects access to above variables.
+	                                      This is a pointer, because mutexes
+	                                      shouldn't be copied*/
 
 	uint64_t last_redraw; /* Time of last redraw. */
 	uint64_t last_reload; /* Time of last [full] reload. */
@@ -330,7 +350,7 @@ WINDOW *lborder;
 WINDOW *mborder;
 WINDOW *rborder;
 
-/* Updates the ruler with infomation from the view. */
+/* Updates the ruler with information from the view. */
 void ui_ruler_update(FileView *view);
 
 /* Sets text to be displayed on the ruler.  Real window update is postponed for
@@ -342,6 +362,10 @@ void ui_ruler_set(const char val[]);
  * term_state in status structure. */
 void ui_update_term_state(void);
 
+/* Performs a quick check whether terminal is functional.  Returns non-zero if
+ * so, otherwise zero is returned. */
+int ui_term_is_alive(void);
+
 /* Checks whether given character was pressed, ignores any other characters. */
 int ui_char_pressed(wint_t c);
 
@@ -353,7 +377,7 @@ float get_splitter_pos(int max);
  * argument). */
 void update_screen(UpdateType update_kind);
 
-/* Swaps curr_view and other_view pointers (activa and inactive panes).  Also
+/* Swaps curr_view and other_view pointers (active and inactive panes).  Also
  * updates things (including UI) that are bound to views. */
 void change_window(void);
 
@@ -370,7 +394,10 @@ void update_input_bar(const wchar_t *str);
 
 void clear_num_window(void);
 
-void show_progress(const char *msg, int period);
+/* Displays progress on the status bar, not updating it frequently.  msg can't
+ * be NULL.  period - how often status bar should be updated.  If period equals
+ * 0 inner counter is reset, do this on start of operation. */
+void show_progress(const char msg[], int period);
 
 void redraw_lists(void);
 
@@ -395,13 +422,16 @@ void refresh_view_win(FileView *view);
  * (horizontally/vertically, left-top/right-bottom). */
 void move_window(FileView *view, int horizontally, int first);
 
-/* Switches two panes saving current windows as the active one (left/top or
- * right/bottom). */
-void switch_windows(void);
-
 /* Swaps current and other views. */
 void switch_panes(void);
 
+/* Setups view to the the curr_view.  Saving previous state in supplied
+ * buffers.  Use ui_view_unpick() to revert the effect. */
+void ui_view_pick(FileView *view, FileView **old_curr, FileView **old_other);
+
+/* Restores what has been done by ui_view_pick(). */
+void ui_view_unpick(FileView *view, FileView *old_curr, FileView *old_other);
+
 /* Switches to other pane, ignoring state of the preview and entering view mode
  * in case the other pane has explore mode active. */
 void go_to_other_pane(void);
@@ -412,10 +442,22 @@ void split_view(SPLIT orientation);
 /* Switches view to one-window mode. */
 void only(void);
 
+/* Moves window splitter by specified amount of positions multiplied by the
+ * given factor. */
+void move_splitter(int by, int fact);
+
+/* Sets size of the view to specified value. */
+void ui_view_resize(FileView *view, int to);
+
 /* File name formatter which takes 'classify' option into account and applies
  * type dependent name decorations. */
 void format_entry_name(const dir_entry_t *entry, size_t buf_len, char buf[]);
 
+/* Retrieves decorations for file entry.  Sets *prefix and *suffix to strings
+ * stored in global configuration. */
+void ui_get_decors(const dir_entry_t *entry, const char **prefix,
+		const char **suffix);
+
 /* Moves cursor to position specified by coordinates checking result of the
  * movement. */
 void checked_wmove(WINDOW *win, int y, int x);
@@ -450,7 +492,11 @@ int ui_view_sort_list_contains(const char sort[SK_COUNT], char key);
 
 /* Ensures that list of sorting keys is sensible (i.e. contains either "name" or
  * "iname" for views, except for unsorted custom view). */
-void ui_view_sort_list_ensure_well_formed(FileView *view);
+void ui_view_sort_list_ensure_well_formed(FileView *view, char sort_keys[]);
+
+/* Picks sort array for the view taking custom view into account.  Returns
+ * pointer to the array. */
+char * ui_view_sort_list_get(const FileView *view);
 
 /* Checks whether file numbers should be displayed for the view.  Returns
  * non-zero if so, otherwise zero is returned. */
@@ -475,6 +521,20 @@ FileType ui_view_entry_target_type(const dir_entry_t *entry);
  * width. */
 int ui_view_available_width(const FileView *const view);
 
+/* Retrieves column number at which quickview content should be displayed.
+ * Returns the number. */
+int ui_qv_left(const FileView *view);
+
+/* Retrieves line number at which quickview content should be displayed.
+ * Returns the number. */
+int ui_qv_top(const FileView *view);
+
+/* Retrieves height of quickview area.  Returns the height. */
+int ui_qv_height(const FileView *view);
+
+/* Retrieves width of quickview area.  Returns the width. */
+int ui_qv_width(const FileView *view);
+
 /* Gets color scheme that corresponds to the view.  Returns pointer to the color
  * scheme. */
 const col_scheme_t * ui_view_get_cs(const FileView *view);
@@ -510,4 +570,4 @@ UiUpdateEvent ui_view_query_scheduled_event(FileView *view);
 #endif /* VIFM__UI__UI_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
