@@ -31,17 +31,17 @@
 
 #include <sys/stat.h> /* S_* statbuf */
 #include <sys/types.h> /* size_t mode_t */
-#include <unistd.h> /* getcwd() readlink() */
+#include <unistd.h> /* getcwd() pathconf() readlink() */
 
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL */
 #include <stdio.h> /* snprintf() remove() */
-#include <stdlib.h> /* free() realpath() */
+#include <stdlib.h> /* free() */
 #include <string.h> /* strcpy() strdup() strlen() strncmp() strncpy() */
 
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../io/iop.h"
-#include "fs_limits.h"
 #include "log.h"
 #include "path.h"
 #include "str.h"
@@ -51,6 +51,7 @@
 static int is_dir_fast(const char path[]);
 static int path_exists_internal(const char path[], const char filename[],
 		int deref);
+static int case_sensitive_paths(const char at[]);
 
 #ifndef _WIN32
 static int is_directory(const char path[], int dereference_links);
@@ -79,8 +80,8 @@ is_dir(const char path[])
 
 /* Checks if path is an existing directory faster than is_dir() does this at the
  * cost of less accurate results (fails on non-sufficient rights).
- * Automatically deferences symbolic links.  Returns non-zero if path points to
- * a directory, otherwise zero is returned. */
+ * Automatically dereferences symbolic links.  Returns non-zero if path points
+ * to a directory, otherwise zero is returned. */
 static int
 is_dir_fast(const char path[])
 {
@@ -114,9 +115,10 @@ is_dir_empty(const char path[])
 	DIR *dir;
 	struct dirent *d;
 
-	if((dir = os_opendir(path)) == NULL)
+	dir = os_opendir(path);
+	if(dir == NULL)
 	{
-		return 0;
+		return 1;
 	}
 
 	while((d = os_readdir(dir)) != NULL)
@@ -185,11 +187,11 @@ paths_are_same(const char s[], const char t[])
 	char s_real[PATH_MAX];
 	char t_real[PATH_MAX];
 
-	if(realpath(s, s_real) != s_real)
+	if(os_realpath(s, s_real) != s_real)
 	{
 		return 0;
 	}
-	if(realpath(t, t_real) != t_real)
+	if(os_realpath(t, t_real) != t_real)
 	{
 		return 0;
 	}
@@ -252,7 +254,7 @@ get_symlink_type(const char path[])
 	char *filename_copy;
 	char *p;
 
-	if(getcwd(cwd, sizeof(cwd)) == NULL)
+	if(get_cwd(cwd, sizeof(cwd)) == NULL)
 	{
 		/* getcwd() failed, just use "." rather than fail. */
 		strcpy(cwd, ".");
@@ -275,7 +277,7 @@ get_symlink_type(const char path[])
 	filename_copy = strdup(path);
 	chosp(filename_copy);
 
-	p = realpath(filename_copy, linkto);
+	p = os_realpath(filename_copy, linkto);
 	saved_errno = errno;
 
 	free(filename_copy);
@@ -407,6 +409,12 @@ make_path(const char dir_name[], mode_t mode)
 	return iop_mkdir(&args);
 }
 
+int
+create_path(const char dir_name[], mode_t mode)
+{
+	return is_dir(dir_name) ? 1 : make_path(dir_name, mode);
+}
+
 int
 symlinks_available(void)
 {
@@ -417,16 +425,6 @@ symlinks_available(void)
 #endif
 }
 
-int
-case_insensitive_paths(void)
-{
-#ifndef _WIN32
-	return 0;
-#else
-	return 1;
-#endif
-}
-
 int
 has_atomic_file_replace(void)
 {
@@ -646,12 +644,12 @@ is_in_subtree(const char path[], const char root[])
 	copy_str(path_copy, sizeof(path_copy), path);
 	remove_last_path_component(path_copy);
 
-	if(realpath(path_copy, path_real) != path_real)
+	if(os_realpath(path_copy, path_real) != path_real)
 	{
 		return 0;
 	}
 
-	if(realpath(root, root_real) != root_real)
+	if(os_realpath(root, root_real) != root_real)
 	{
 		return 0;
 	}
@@ -674,7 +672,7 @@ are_on_the_same_fs(const char s[], const char t[])
 int
 is_case_change(const char src[], const char dst[])
 {
-	if(!case_insensitive_paths())
+	if(case_sensitive_paths(src))
 	{
 		return 0;
 	}
@@ -682,6 +680,20 @@ is_case_change(const char src[], const char dst[])
 	return strcasecmp(src, dst) == 0 && strcmp(src, dst) != 0;
 }
 
+/* Whether paths are case sensitive at specified location.  Returns non-zero if
+ * so, otherwise zero is returned. */
+static int
+case_sensitive_paths(const char at[])
+{
+#if HAVE_DECL__PC_CASE_SENSITIVE
+	return pathconf(at, _PC_CASE_SENSITIVE) != 0;
+#elif !defined(_WIN32)
+	return 1;
+#else
+	return 0;
+#endif
+}
+
 int
 enum_dir_content(const char path[], dir_content_client_func client, void *param)
 {
@@ -710,7 +722,7 @@ enum_dir_content(const char path[], dir_content_client_func client, void *param)
 	HANDLE hfind;
 	WIN32_FIND_DATAW ffd;
 
-	snprintf(find_pat, sizeof(find_pat), "%s/""*", path);
+	snprintf(find_pat, sizeof(find_pat), "%s/*", path);
 	utf16_path = utf8_to_utf16(find_pat);
 	hfind = FindFirstFileW(utf16_path, &ffd);
 	free(utf16_path);
@@ -723,7 +735,10 @@ enum_dir_content(const char path[], dir_content_client_func client, void *param)
 	do
 	{
 		char *const utf8_name = utf8_from_utf16(ffd.cFileName);
-		client(utf8_name, &ffd, param);
+		if(client(utf8_name, &ffd, param) != 0)
+		{
+			break;
+		}
 		free(utf8_name);
 	}
 	while(FindNextFileW(hfind, &ffd));
@@ -733,8 +748,47 @@ enum_dir_content(const char path[], dir_content_client_func client, void *param)
 #endif
 }
 
+int
+count_dir_items(const char path[])
+{
+	DIR *dir;
+	struct dirent *d;
+	int count;
+
+	dir = os_opendir(path);
+	if(dir == NULL)
+	{
+		return -1;
+	}
+
+	count = 0;
+	while((d = os_readdir(dir)) != NULL)
+	{
+		if(!is_builtin_dir(d->d_name))
+		{
+			++count;
+		}
+	}
+	os_closedir(dir);
+
+	return count;
+}
+
 char *
-get_cwd(void)
+get_cwd(char buf[], size_t size)
+{
+	if(getcwd(buf, size) == NULL)
+	{
+		return NULL;
+	}
+#ifdef _WIN32
+	to_forward_slash(buf);
+#endif
+	return buf;
+}
+
+char *
+save_cwd(void)
 {
 	char cwd[PATH_MAX];
 	if(getcwd(cwd, sizeof(cwd)) != cwd)
@@ -756,8 +810,8 @@ restore_cwd(char saved_cwd[])
 
 #ifndef _WIN32
 
-/* Checks if path (dereferencer or not symbolic link) is an existing directory.
- * Automatically deferences symbolic links. */
+/* Checks if path (dereferenced for a symbolic link) is an existing directory.
+ * Automatically dereferences symbolic links. */
 static int
 is_directory(const char path[], int dereference_links)
 {
@@ -797,27 +851,6 @@ win_get_file_attrs(const char path[])
 	return attr;
 }
 
-char *
-realpath(const char path[], char buf[])
-{
-	const char *const resolved_path = win_resolve_mount_points(path);
-
-	if(!is_path_absolute(resolved_path))
-	{
-		/* Try to compose absolute path. */
-		char cwd[PATH_MAX];
-		if(getcwd(cwd, sizeof(cwd)) == cwd)
-		{
-			to_forward_slash(cwd);
-			snprintf(buf, PATH_MAX, "%s/%s", cwd, resolved_path);
-			return buf;
-		}
-	}
-
-	copy_str(buf, PATH_MAX, resolved_path);
-	return buf;
-}
-
 int
 S_ISLNK(mode_t mode)
 {
@@ -890,4 +923,4 @@ is_win_symlink(uint32_t attr, uint32_t tag)
 #endif
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
