@@ -49,12 +49,12 @@ SymLinkType;
 typedef int (*dir_content_client_func)(const char name[], const void *data,
 		void *param);
 
-/* Checks if path is an existing directory.  Automatically deferences symbolic
+/* Checks if path is an existing directory.  Automatically dereferences symbolic
  * links. */
 int is_dir(const char path[]);
 
-/* Checks whether directory is empty.  Returns non-zero if it isn't, in case of
- * error or when directory is empty zero is returned. */
+/* Checks whether directory is empty.  Returns non-zero if it is, in case of
+ * error or when directory is empty non-zero is returned. */
 int is_dir_empty(const char path[]);
 
 /* Checks if path could be a directory (e.g. it can be UNC root on Windows). */
@@ -90,17 +90,18 @@ int get_link_target_abs(const char link[], const char cwd[], char buf[],
 int get_link_target(const char *link, char *buf, size_t buf_len);
 
 /* Creates directory and all intermediate directories if needed using specified
- * access mode.  Returns zero on success, otherwise non-zero is returned. */
+ * access mode.  If target directory already exists, no error will be raised.
+ * Returns zero on success, otherwise non-zero is returned. */
 int make_path(const char dir_name[], mode_t mode);
 
+/* Same as make_path(), but fails if target path already exists and is a
+ * directory. */
+int create_path(const char dir_name[], mode_t mode);
+
 /* Whether symbolic links can be used.  Returns non-zero if so, otherwise zero
  * is returned. */
 int symlinks_available(void);
 
-/* Whether paths are case insensitive.  Returns non-zero if so, otherwise zero
- * is returned. */
-int case_insensitive_paths(void);
-
 /* Whether file rename-with-replace (deletion of file at destination) is
  * supported and atomic.  Returns non-zero if so, otherwise zero is returned. */
 int has_atomic_file_replace(void);
@@ -131,7 +132,7 @@ int is_regular_file(const char path[]);
  * error, otherwise zero is returned. */
 int rename_file(const char src[], const char dst[]);
 
-/* Removes directory content. */
+/* Removes directory content, but not the directory itself. */
 void remove_dir_content(const char path[]);
 
 struct dirent;
@@ -168,21 +169,24 @@ int is_case_change(const char src[], const char dst[]);
 int enum_dir_content(const char path[], dir_content_client_func client,
 		void *param);
 
+/* Counts number of files in the directory excluding . and .. entries.  Returns
+ * the count. */
+int count_dir_items(const char path[]);
+
+/* getcwd() wrapper that always uses forward slashes.  Returns buf on success or
+ * NULL on error. */
+char * get_cwd(char buf[], size_t size);
+
 /* Remembers current working directory.  If path can't be obtained, does
  * nothing.  Result should be passed to restore_cwd(), no checks are needed. */
-char * get_cwd(void);
+char * save_cwd(void);
 
-/* Restores previously remembered working directory via get_cwd().  If nothing
+/* Restores previously remembered working directory via save_cwd().  If nothing
  * was remembered, does nothing. */
 void restore_cwd(char saved_cwd[]);
 
 #ifdef _WIN32
 
-/* Resolves the path to the real path without any symbolic links.  buf should be
- * at least PATH_MAX in length.  Returns the buf on success, otherwise NULL is
- * returned. */
-char * realpath(const char path[], char buf[]);
-
 int S_ISLNK(mode_t mode);
 
 int readlink(const char *path, char *buf, size_t len);
@@ -200,4 +204,4 @@ int is_win_symlink(uint32_t attr, uint32_t tag);
 #endif /* VIFM__UTILS__FS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
