@@ -22,41 +22,64 @@
 
 #include <stddef.h> /* NULL */
 #include <stdlib.h> /* free() malloc() */
-#include <string.h> /* strlen() strrchr() strspn() */
+#include <string.h> /* strdup() strlen() strrchr() strspn() */
 
+#include "../int/file_magic.h"
 #include "globs.h"
 #include "path.h"
+#include "regexp.h"
 #include "str.h"
-#include "utils.h"
+
+/* Type of a matcher. */
+typedef enum
+{
+	MT_REGEX, /* Regular expression. */
+	MT_GLOBS, /* List of globs (translated to regular expression). */
+	MT_MIME,  /* List of mime types. */
+}
+MType;
 
 /* Wrapper for a regular expression, its state and compiled form. */
 struct matcher_t
 {
+	MType type;    /* Type of the matcher's pattern. */
 	char *expr;    /* User-entered pattern. */
-	char *raw;     /* Raw regular expression. */
-	int globs;     /* Whether original pattern is globs or regexp. */
+	char *undec;   /* User-entered pattern undecorated pattern. */
+	char *raw;     /* Raw stripped value (regular expression). */
 	int full_path; /* Matches full path instead of just file name. */
 	int cflags;    /* Regular expression compilation flags. */
+	int negated;   /* Whether match is inverted. */
 	regex_t regex; /* The expression in compiled form. */
 };
 
 static int is_full_path(const char expr[], int re, int glob, int *strip);
-static int compile_expr(matcher_t *m, int strip, int cs_by_def, char **error);
+static MType determine_type(const char expr[], int re, int glob,
+		int glob_by_def, int *strip);
+static int compile_expr(matcher_t *m, int strip, int cs_by_def,
+		const char on_empty_re[], char **error);
 static int parse_glob(matcher_t *m, int strip, char **error);
-static int parse_re(matcher_t *m, int strip, int cs_by_def, char **error);
+static int parse_re(matcher_t *m, int strip, int cs_by_def,
+		const char on_empty_re[], char **error);
 static void free_matcher_items(matcher_t *matcher);
-static int is_re_expr(const char expr[]);
+static int is_negated(const char **expr, int allow_empty);
+static int is_re_expr(const char expr[], int allow_empty);
 static int is_globs_expr(const char expr[]);
+static int is_mime_expr(const char expr[]);
 
 matcher_t *
-matcher_alloc(const char expr[], int cs_by_def, int glob_by_def, char **error)
+matcher_alloc(const char expr[], int cs_by_def, int glob_by_def,
+		const char on_empty_re[], char **error)
 {
-	const int re = is_re_expr(expr), glob = is_globs_expr(expr);
+	const char *orig_expr = expr;
+	const int negated = is_negated(&expr, 1);
+	const int re = is_re_expr(expr, 1), glob = is_globs_expr(expr);
 	int strip;
 	const int full_path = is_full_path(expr, re, glob, &strip);
+
 	matcher_t *matcher, m = {
+		.type = determine_type(expr, re, glob, glob_by_def, &strip),
 		.raw = strdup(expr + strip),
-		.globs = !(re || (!glob && !glob_by_def)),
+		.negated = negated,
 		.full_path = full_path,
 	};
 
@@ -68,13 +91,14 @@ matcher_alloc(const char expr[], int cs_by_def, int glob_by_def, char **error)
 		return NULL;
 	}
 
-	if(compile_expr(&m, strip, cs_by_def, error) != 0)
+	if(compile_expr(&m, strip, cs_by_def, on_empty_re, error) != 0)
 	{
 		free(m.raw);
+		free(m.undec);
 		return NULL;
 	}
 
-	m.expr = strdup(expr);
+	m.expr = strdup(orig_expr);
 	if(m.expr == NULL)
 	{
 		replace_string(error, "Failed to clone match expr.");
@@ -109,7 +133,7 @@ is_full_path(const char expr[], int re, int glob, int *strip)
 	if(re)
 	{
 		const char *const s = strrchr(expr, '/') - 1;
-		full_path = (expr[1] == '/' && s != expr + 1 && *s == '/');
+		full_path = (expr[1] == '/' && s > expr + 1 && *s == '/');
 		*strip = full_path ? 2 : 1;
 	}
 	else if(glob)
@@ -121,27 +145,50 @@ is_full_path(const char expr[], int re, int glob, int *strip)
 	return full_path;
 }
 
+/* Determines type of the matcher for given expression.  Returns the type. */
+static MType
+determine_type(const char expr[], int re, int glob, int glob_by_def, int *strip)
+{
+	if(re)
+	{
+		return MT_REGEX;
+	}
+	if(glob)
+	{
+		return MT_GLOBS;
+	}
+	if(is_mime_expr(expr))
+	{
+		*strip = 1;
+		return MT_MIME;
+	}
+	return glob_by_def ? MT_GLOBS : MT_REGEX;
+}
+
 /* Compiles m->raw into regular expression.  Also replaces global with
  * equivalent regexp.  Returns zero on success or non-zero on error with *error
  * containing description of it. */
 static int
-compile_expr(matcher_t *m, int strip, int cs_by_def, char **error)
+compile_expr(matcher_t *m, int strip, int cs_by_def, const char on_empty_re[],
+		char **error)
 {
 	int err;
 
-	if(m->globs)
+	switch(m->type)
 	{
-		if(parse_glob(m, strip, error) != 0)
-		{
-			return 1;
-		}
-	}
-	else
-	{
-		if(parse_re(m, strip, cs_by_def, error) != 0)
-		{
-			return 1;
-		}
+		case MT_MIME:
+		case MT_GLOBS:
+			if(parse_glob(m, strip, error) != 0)
+			{
+				return 1;
+			}
+			break;
+		case MT_REGEX:
+			if(parse_re(m, strip, cs_by_def, on_empty_re, error) != 0)
+			{
+				return 1;
+			}
+			break;
 	}
 
 	err = regcomp(&m->regex, m->raw, m->cflags);
@@ -168,10 +215,17 @@ parse_glob(matcher_t *m, int strip, char **error)
 		m->raw[strlen(m->raw) - strip] = '\0';
 	}
 
+	m->undec = strdup(m->raw);
+	if(m->undec == NULL)
+	{
+		replace_string(error, "Failed to allocate memory.");
+		return 1;
+	}
+
 	re = globs_to_regex(m->raw);
 	if(re == NULL)
 	{
-		replace_string(error, "Failed convert globs into regexp.");
+		replace_string(error, "Failed to convert globs into regexp.");
 		return 1;
 	}
 
@@ -185,21 +239,36 @@ parse_glob(matcher_t *m, int strip, char **error)
 /* Parses regexp flags.  Returns zero on success or non-zero on error with
  * *error containing description of it. */
 static int
-parse_re(matcher_t *m, int strip, int cs_by_def, char **error)
+parse_re(matcher_t *m, int strip, int cs_by_def, const char on_empty_re[],
+		char **error)
 {
 	if(strip != 0)
 	{
-		char *const flags = strrchr(m->raw, '/') + 1;
+		char *flags = strrchr(m->raw, '/');
+		flags = (flags == NULL) ? (m->raw + strlen(m->raw)) : (flags + 1);
+
 		if(parse_case_flag(flags, &cs_by_def) != 0)
 		{
 			replace_string(error, "Failed to parse flags.");
 			return 1;
 		}
 
-		/* Cut the flags off by replacing traling slash(es) with null character. */
+		/* Cut the flags off by replacing trailing slash(es) with null character. */
 		flags[-strip] = '\0';
 	}
 
+	if(m->raw[0] == '\0')
+	{
+		replace_string(&m->raw, on_empty_re);
+	}
+
+	m->undec = strdup(m->raw);
+	if(m->undec == NULL)
+	{
+		replace_string(error, "Failed to allocate memory.");
+		return 1;
+	}
+
 	m->cflags = REG_EXTENDED | (cs_by_def ? 0 : REG_ICASE);
 	return 0;
 }
@@ -215,14 +284,15 @@ matcher_clone(const matcher_t *matcher)
 		return NULL;
 	}
 
+	*clone = *matcher;
 	clone->expr = strdup(matcher->expr);
 	clone->raw = strdup(matcher->raw);
-	clone->globs = matcher->globs;
-	clone->cflags = matcher->cflags;
+	clone->undec = strdup(matcher->undec);
 
 	err = regcomp(&clone->regex, matcher->raw, matcher->cflags);
 
-	if(err != 0 || clone->expr == NULL || clone->raw == NULL)
+	if(err != 0 || clone->expr == NULL || clone->raw == NULL ||
+			clone->undec == NULL)
 	{
 		matcher_free(clone);
 		return NULL;
@@ -248,68 +318,73 @@ free_matcher_items(matcher_t *matcher)
 {
 	free(matcher->expr);
 	free(matcher->raw);
+	free(matcher->undec);
 	regfree(&matcher->regex);
 }
 
 int
-matcher_matches(matcher_t *matcher, const char path[])
+matcher_matches(const matcher_t *matcher, const char path[])
 {
-	if(!matcher->full_path)
+	if(matcher->type == MT_MIME)
+	{
+		path = get_mimetype(path);
+		if(path == NULL)
+		{
+			return matcher->negated;
+		}
+	}
+	else if(!matcher->full_path)
 	{
 		path = get_last_path_component(path);
 	}
 
-	return (regexec(&matcher->regex, path, 0, NULL, 0) == 0);
+	return (regexec(&matcher->regex, path, 0, NULL, 0) == 0)^matcher->negated;
 }
 
 const char *
-matcher_get_expr(const matcher_t *matcher)
+matcher_get_undec(const matcher_t *matcher)
 {
-	return matcher->expr;
+	return matcher->undec;
 }
 
 int
-matcher_includes(const matcher_t *like, const matcher_t *m)
+matcher_includes(const matcher_t *matcher, const matcher_t *like)
 {
-	if(m->globs != like->globs || m->cflags != like->cflags ||
-			m->full_path != like->full_path)
+	if(matcher->type != like->type || matcher->cflags != like->cflags ||
+			matcher->full_path != like->full_path)
 	{
 		return 0;
 	}
 
-	if(m->cflags & REG_ICASE)
-	{
-		if(strcasestr(like->raw, m->raw) != NULL)
-		{
-			return 1;
-		}
-	}
-	else
-	{
-		if(strstr(like->raw, m->raw) != NULL)
-		{
-			return 1;
-		}
-	}
-
-	return 0;
+	return (matcher->cflags & REG_ICASE)
+	     ? (strcasestr(matcher->raw, like->raw) != NULL)
+	     : (strstr(matcher->raw, like->raw) != NULL);
 }
 
-int
-matcher_is_expr(const char str[])
+/* Checks whether *expr specifies negated pattern.  Adjusts pointer if so.
+ * Returns non-zero if so, otherwise zero is returned. */
+static int
+is_negated(const char **expr, int allow_empty)
 {
-	return is_re_expr(str) || is_globs_expr(str);
+	if(**expr == '!' &&
+			(is_re_expr(*expr + 1, allow_empty) || is_globs_expr(*expr + 1) ||
+			 is_mime_expr(*expr + 1)))
+	{
+		++*expr;
+		return 1;
+	}
+	return 0;
 }
 
 /* Checks whether expr is a regular expression file name pattern.  Returns
  * non-zero if so, otherwise zero is returned. */
 static int
-is_re_expr(const char expr[])
+is_re_expr(const char expr[], int allow_empty)
 {
 	const char *e = strrchr(expr, '/');
 	return expr[0] == '/'                        /* Starts with slash. */
 	    && e != NULL && e != expr                /* Has second slash. */
-	    && e - expr > 1                          /* Not empty pattern. */
+	    && (allow_empty || e - expr > 1)         /* Not empty pattern. */
 	    && strspn(e + 1, "iI") == strlen(e + 1); /* Has only correct flags. */
 }
 
@@ -318,8 +393,22 @@ is_re_expr(const char expr[])
 static int
 is_globs_expr(const char expr[])
 {
-	return surrounded_with(expr, '{', '}');
+	return surrounded_with(expr, '{', '}') && expr[2] != '\0';
+}
+
+/* Checks whether expr is a mime type pattern.  Returns non-zero if so,
+ * otherwise zero is returned. */
+static int
+is_mime_expr(const char expr[])
+{
+	return surrounded_with(expr, '<', '>') && expr[2] != '\0';
+}
+
+int
+matcher_is_full_path(const matcher_t *matcher)
+{
+	return matcher->full_path;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
