@@ -22,8 +22,9 @@
 #ifndef _WIN32
 #include <pwd.h> /* getpwnam() */
 #endif
-#include <unistd.h> /* getcwd() */
+#include <unistd.h>
 
+#include <assert.h> /* assert() */
 #ifdef _WIN32
 #include <ctype.h>
 #endif
@@ -31,16 +32,15 @@
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h>  /* snprintf() */
 #include <stdlib.h> /* malloc() free() */
-#include <string.h> /* strcat() strcmp() strcasecmp() strdup() strncmp()
-                       strncasecmp() strncat() strchr() strcpy() strlen()
-                       strrchr() */
+#include <string.h> /* memset() strcat() strcmp() strcasecmp() strdup()
+                       strncmp() strncasecmp() strncat() strchr() strcpy()
+                       strlen() strrchr() */
 
 #include "../cfg/config.h"
-#include "../path_env.h"
+#include "../compat/fs_limits.h"
+#include "../int/path_env.h"
 #include "env.h"
 #include "fs.h"
-#include "fs_limits.h"
-#include "log.h"
 #include "str.h"
 #include "utils.h"
 
@@ -102,7 +102,7 @@ canonicalize_path(const char directory[], char buf[], size_t buf_size)
 	/* Destination string pointer. */
 	char *q = buf - 1;
 
-	buf[0] = '\0';
+	memset(buf, '\0', buf_size);
 
 #ifdef _WIN32
 	/* Handle first component of a UNC path. */
@@ -119,7 +119,7 @@ canonicalize_path(const char directory[], char buf[], size_t buf_size)
 	}
 #endif
 
-	while(*p != '\0' && (size_t)((q + 1) - buf) < buf_size - 1)
+	while(*p != '\0' && (size_t)((q + 1) - buf) < buf_size - 1U)
 	{
 		const int prev_dir_present = (q != buf - 1 && *q == '/');
 		if(skip_dotdir_if_any(&p, prev_dir_present))
@@ -128,7 +128,7 @@ canonicalize_path(const char directory[], char buf[], size_t buf_size)
 		}
 		else if(prev_dir_present &&
 				(strncmp(p, "../", 3) == 0 || strcmp(p, "..") == 0) &&
-				strcmp(buf, "../") != 0)
+				!ends_with(buf, "../") != 0)
 		{
 			/* Remove the last path component added. */
 #ifdef _WIN32
@@ -136,34 +136,43 @@ canonicalize_path(const char directory[], char buf[], size_t buf_size)
 			if(*(q - 1) != ':')
 #endif
 			{
-				p++;
-				q--;
+				++p;
+				--q;
 				while(q >= buf && *q != '/')
 				{
-					q--;
+					--q;
 				}
 			}
 		}
 		else if(*p == '/')
 		{
-			/* Don't add more than one slash between path components. */
-			if(!prev_dir_present)
+			/* Don't add more than one slash between path components.  And don't add
+			 * leading slash if initial path is relative. */
+			if(!prev_dir_present && !(q == buf - 1 && *directory != '/'))
 			{
 				*++q = '/';
 			}
 		}
 		else
 		{
-			/* Copy current path component till the end. */
+			/* Copy current path component till the end (slash or end of
+			 * line/buffer). */
 			*++q = *p;
 			while(p[1] != '\0' && p[1] != '/' &&
-					(size_t)((q + 1) - buf) < buf_size - 1)
+					(size_t)((q + 1) - buf) < buf_size - 1U)
 			{
 				*++q = *++p;
 			}
 		}
 
-		p++;
+		++p;
+	}
+
+	/* If initial path is relative and we ended up with an empty path, produce
+	 * "./". */
+	if(*directory != '/' && q < buf && (size_t)((q + 1) - buf) < buf_size - 2U)
+	{
+		*++q = '.';
 	}
 
 	if((*directory != '\0' && q < buf) || (q >= buf && *q != '/'))
@@ -256,7 +265,10 @@ make_rel_path(const char path[], const char base[])
 		strcat(buf, "../");
 	if(*p == '/')
 		p++;
-	canonicalize_path(p, buf + strlen(buf), sizeof(buf) - strlen(buf));
+	if(*p != '\0')
+	{
+		canonicalize_path(p, buf + strlen(buf), sizeof(buf) - strlen(buf));
+	}
 	chosp(buf);
 
 	if(buf[0] == '\0')
@@ -310,15 +322,8 @@ is_unc_root(const char *path)
 #endif
 }
 
-/*
- * Escape the filename for the purpose of inserting it into the shell.
- *
- * quote_percent means prepend percent sign with a percent sign
- *
- * Returns new string, caller should free it.
- */
 char *
-escape_filename(const char *string, int quote_percent)
+shell_like_escape(const char string[], int quote_percent)
 {
 	size_t len;
 	size_t i;
@@ -388,25 +393,33 @@ escape_filename(const char *string, int quote_percent)
 }
 
 char *
-replace_home_part(const char directory[])
+replace_home_part(const char path[])
+{
+	char *result = replace_home_part_strict(path);
+	if(!is_root_dir(result))
+	{
+		chosp(result);
+	}
+	return result;
+}
+
+char *
+replace_home_part_strict(const char path[])
 {
 	static char buf[PATH_MAX];
 	size_t len;
 
 	len = strlen(cfg.home_dir) - 1;
-	if(strnoscmp(directory, cfg.home_dir, len) == 0 &&
-			(directory[len] == '\0' || directory[len] == '/'))
+	if(strnoscmp(path, cfg.home_dir, len) == 0 &&
+			(path[len] == '\0' || path[len] == '/'))
 	{
-		strncat(strcpy(buf, "~"), directory + len, sizeof(buf) - strlen(buf) - 1);
+		strncat(strcpy(buf, "~"), path + len, sizeof(buf) - strlen(buf) - 1);
 	}
 	else
 	{
-		copy_str(buf, sizeof(buf), directory);
+		copy_str(buf, sizeof(buf), path);
 	}
 
-	if(!is_root_dir(buf))
-		chosp(buf);
-
 	return buf;
 }
 
@@ -521,7 +534,11 @@ remove_last_path_component(char path[])
 	}
 
 	slash = strrchr(path, '/');
-	if(slash != NULL)
+	if(slash == NULL)
+	{
+		path[0] = '\0';
+	}
+	else
 	{
 		const int offset = is_root_dir(path) ? 1 : 0;
 		slash[offset] = '\0';
@@ -555,22 +572,17 @@ ensure_path_well_formed(char *path)
 #endif
 }
 
-int
-to_canonic_path(const char path[], char buf[], size_t buf_len)
+void
+to_canonic_path(const char path[], const char base[], char buf[],
+		size_t buf_len)
 {
 	if(!is_path_absolute(path))
 	{
-		char cwd[PATH_MAX];
 		char full_path[PATH_MAX];
-
-		if(getcwd(cwd, sizeof(cwd)) == NULL)
-		{
-			/* getcwd() failed, we can't use relative path, so fail. */
-			LOG_SERROR_MSG(errno, "Can't get CWD");
-			return 1;
-		}
-
-		snprintf(full_path, sizeof(full_path), "%s/%s", cwd, path);
+		/* Assert is not level above to keep "." in curr_dir in tests, but this
+		 * should be possible to change. */
+		assert(is_path_absolute(base) && "Base path has to be absolute.");
+		snprintf(full_path, sizeof(full_path), "%s/%s", base, path);
 		canonicalize_path(full_path, buf, buf_len);
 	}
 	else
@@ -578,8 +590,10 @@ to_canonic_path(const char path[], char buf[], size_t buf_len)
 		canonicalize_path(path, buf, buf_len);
 	}
 
-	chosp(buf);
-	return 0;
+	if(!is_root_dir(buf))
+	{
+		chosp(buf);
+	}
 }
 
 int
@@ -736,7 +750,8 @@ generate_tmp_file_name(const char prefix[], char buf[], size_t buf_len)
 const char *
 get_tmpdir(void)
 {
-	return env_get_one_of_def("/tmp/", "TMPDIR", "TEMP", "TEMPDIR", "TMP", NULL);
+	return env_get_one_of_def("/tmp/", "TMPDIR", "TEMP", "TEMPDIR", "TMP",
+			(const char *)NULL);
 }
 
 #ifdef _WIN32
@@ -762,4 +777,4 @@ to_back_slash(char path[])
 #endif
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
