@@ -57,18 +57,25 @@ int is_root_dir(const char *path);
 
 int is_unc_root(const char *path);
 
-char * escape_filename(const char *string, int quote_percent);
+/* Escapes the string for the purpose of inserting it into the shell or
+ * command-line.  quote_percent means prepend percent sign with a percent sign.
+ * Returns new string, caller should free it. */
+char * shell_like_escape(const char string[], int quote_percent);
 
-/* Replaces leading path to home directory with a tilde.  Returns pointer to a
- * statically allocated buffer of size PATH_MAX. */
-char * replace_home_part(const char directory[]);
+/* Replaces leading path to home directory with a tilde, trims trailing slash.
+ * Returns pointer to a statically allocated buffer of size PATH_MAX. */
+char * replace_home_part(const char path[]);
 
-/* Expands tilde in the front of the path.  Returns newly allocated string
- * without tilde. */
+/* Same as replace_home_part(), but doesn't perform trailing slash trimming. */
+char * replace_home_part_strict(const char path[]);
+
+/* Expands tilde in the front of the path.  Does nothing for paths without
+ * tilde.  Returns newly allocated string without tilde. */
 char * expand_tilde(const char path[]);
 
-/* Expands tilde in the front of the path.  Can free the path.  Returns the path
- * or newly allocated string without tilde. */
+/* Expands tilde in the front of the path.  Can free the path.  Does nothing for
+ * paths without tilde.  Returns the path or newly allocated string without
+ * tilde. */
 char * replace_tilde(char path[]);
 
 /* Find beginning of the last component in the path ignoring trailing
@@ -86,8 +93,10 @@ int is_path_well_formed(const char *path);
 void ensure_path_well_formed(char *path);
 
 /* Ensures that path to a file is of canonic absolute form.  No trailing slash
- * in the buffer.  Returns zero on success, otherwise non-zero is returned. */
-int to_canonic_path(const char path[], char buf[], size_t buf_len);
+ * in the buffer except for the root path.  Canonic paths must be absolute, so
+ * if input path isn't base is prepended to it. */
+void to_canonic_path(const char path[], const char base[], char buf[],
+		size_t buf_len);
 
 /* Checks if path contains slash (also checks for backward slash on Windows). */
 int contains_slash(const char *path);
@@ -149,4 +158,4 @@ void to_back_slash(char path[]);
 #endif /* VIFM__UTILS__PATH_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
