@@ -29,9 +29,11 @@
 #include <string.h> /* strdup() strncmp() strlen() strcmp() strchr() strrchr()
                        strncpy() */
 #include <wchar.h> /* wint_t vswprintf() */
-#include <wctype.h> /* iswupper() towlower() towupper() */
+#include <wctype.h> /* iswprint() iswupper() towlower() towupper() */
 
+#include "../compat/reallocarray.h"
 #include "macros.h"
+#include "test_helpers.h"
 #include "utf8.h"
 #include "utils.h"
 
@@ -39,6 +41,7 @@ static int transform_ascii_str(const char str[], int (*f)(int), char buf[],
 		size_t buf_len);
 static int transform_wide_str(const char str[], wint_t (*f)(wint_t), char buf[],
 		size_t buf_len);
+TSTATIC void squash_double_commas(char str[]);
 
 void
 chomp(char str[])
@@ -63,7 +66,7 @@ to_wide(const char s[])
 	len = mbstowcs(NULL, s, 0);
 	if(len != (size_t)-1)
 	{
-		result = malloc((len + 1)*sizeof(wchar_t));
+		result = reallocarray(NULL, len + 1, sizeof(wchar_t));
 		if(result != NULL)
 		{
 			(void)mbstowcs(result, s, len + 1);
@@ -75,6 +78,40 @@ to_wide(const char s[])
 #endif
 }
 
+wchar_t *
+to_wide_force(const char s[])
+{
+	wchar_t *w = to_wide(s);
+	wchar_t *p;
+
+	if(w != NULL)
+	{
+		return w;
+	}
+
+	w = reallocarray(NULL, strlen(s) + 1U, sizeof(*w));
+	if(w == NULL)
+	{
+		return NULL;
+	}
+
+	/* There must be broken multi-byte sequence, do our best to convert string to
+	 * something meaningful rather than just failing. */
+
+	p = w;
+	while(*s != '\0')
+	{
+		const wchar_t wc = get_first_wchar(s++);
+		if(iswprint(wc))
+		{
+			*p++ = wc;
+		}
+	}
+	*p = L'\0';
+
+	return w;
+}
+
 size_t
 wide_len(const char s[])
 {
@@ -89,7 +126,7 @@ wchar_t *
 vifm_wcsdup(const wchar_t ws[])
 {
 	const size_t len = wcslen(ws) + 1;
-	wchar_t * const result = malloc(len*sizeof(wchar_t));
+	wchar_t * const result = reallocarray(NULL, len, sizeof(wchar_t));
 	if(result == NULL)
 	{
 		return NULL;
@@ -162,7 +199,7 @@ to_multibyte(const wchar_t *s)
 	char *result;
 
 	len = wcstombs(NULL, s, 0) + 1;
-	if((result = malloc(len*sizeof(char))) == NULL)
+	if((result = malloc(len)) == NULL)
 		return NULL;
 
 	wcstombs(result, s, len);
@@ -175,7 +212,7 @@ to_multibyte(const wchar_t *s)
 int
 str_to_lower(const char str[], char buf[], size_t buf_len)
 {
-	if(get_utf8_overhead(str) == 0U)
+	if(utf8_stro(str) == 0U)
 	{
 		return transform_ascii_str(str, &tolower, buf, buf_len);
 	}
@@ -188,7 +225,7 @@ str_to_lower(const char str[], char buf[], size_t buf_len)
 int
 str_to_upper(const char str[], char buf[], size_t buf_len)
 {
-	if(get_utf8_overhead(str) == 0U)
+	if(utf8_stro(str) == 0U)
 	{
 		return transform_ascii_str(str, &toupper, buf, buf_len);
 	}
@@ -204,6 +241,8 @@ str_to_upper(const char str[], char buf[], size_t buf_len)
 static int
 transform_ascii_str(const char str[], int (*f)(int), char buf[], size_t buf_len)
 {
+	int too_small;
+
 	if(buf_len == 0U)
 	{
 		return 1;
@@ -214,8 +253,10 @@ transform_ascii_str(const char str[], int (*f)(int), char buf[], size_t buf_len)
 		*buf++ = f(*str++);
 		--buf_len;
 	}
+	/* Check comes before the assignment to allow buf == str. */
+	too_small = (*str != '\0');
 	*buf = '\0';
-	return *str != '\0';
+	return too_small;
 }
 
 /* Transforms characters of the string to while they fit in the buffer by
@@ -364,6 +405,18 @@ replace_string(char **str, const char with[])
 	return 0;
 }
 
+int
+update_string(char **str, const char to[])
+{
+	if(to == NULL)
+	{
+		free(*str);
+		*str = NULL;
+		return 0;
+	}
+	return replace_string(str, to);
+}
+
 char *
 strcatch(char str[], char c)
 {
@@ -381,8 +434,15 @@ strprepend(char **str, size_t *len, const char prefix[])
 		return 1;
 	}
 
-	memmove(new + prefix_len, new, *len + 1);
-	strncpy(new + *len, prefix, prefix_len);
+	if(*len == 0)
+	{
+		new[prefix_len] = '\0';
+	}
+	else
+	{
+		memmove(new + prefix_len, new, *len + 1);
+	}
+	strncpy(new, prefix, prefix_len);
 	*str = new;
 	*len += prefix_len;
 
@@ -464,7 +524,7 @@ left_ellipsis(char str[], size_t max_width)
 		return NULL;
 	}
 
-	width = get_screen_string_length(str);
+	width = utf8_strsw(str);
 	if(width <= max_width)
 	{
 		return str;
@@ -481,8 +541,8 @@ left_ellipsis(char str[], size_t max_width)
 	tail = str;
 	while(width > max_width - 3U)
 	{
-		width -= utf8_get_screen_width_of_char(tail);
-		tail += get_char_width(tail);
+		width -= utf8_chrsw(tail);
+		tail += utf8_chrw(tail);
 	}
 	strcpy(str, "...");
 	memmove(str + 3U, tail, len - (tail - str) + 1U);
@@ -501,7 +561,7 @@ right_ellipsis(char str[], size_t max_width)
 		return NULL;
 	}
 
-	width = get_screen_string_length(str);
+	width = utf8_strsw(str);
 	if(width <= max_width)
 	{
 		return str;
@@ -514,7 +574,7 @@ right_ellipsis(char str[], size_t max_width)
 		return str;
 	}
 
-	prefix = get_normal_utf8_string_widthn(str, max_width - 3U);
+	prefix = utf8_nstrsnlen(str, max_width - 3U);
 	strcpy(&str[prefix], "...");
 	return str;
 }
@@ -529,7 +589,7 @@ break_in_two(char str[], size_t max)
 	if(break_point == NULL)
 		return str;
 
-	len = get_screen_string_length(str) - 2;
+	len = utf8_strsw(str) - 2;
 	size = strlen(str);
 	size = MAX(size, max);
 	result = malloc(size*4 + 2);
@@ -538,8 +598,8 @@ break_in_two(char str[], size_t max)
 
 	if(len > max)
 	{
-		const int l = get_screen_string_length(result) - (len - max);
-		break_point = str + get_real_string_width(str, MAX(l, 0));
+		const int l = utf8_strsw(result) - (len - max);
+		break_point = str + utf8_strsnlen(str, MAX(l, 0));
 	}
 
 	snprintf(result, break_point - str + 1, "%s", str);
@@ -567,10 +627,10 @@ vifm_swprintf(wchar_t str[], size_t len, const wchar_t format[], ...)
 
 	va_start(ap, format);
 
-#if !defined(_WIN32) || defined(_WIN64)
-	result = vswprintf(str, len, format, ap);
-#else
+#ifdef BROKEN_SWPRINTF
 	result = vswprintf(str, format, ap);
+#else
+	result = vswprintf(str, len, format, ap);
 #endif
 
 	va_end(ap);
@@ -628,6 +688,27 @@ escape_chars(const char string[], const char chars[])
 	return ret;
 }
 
+void
+unescape(char s[], int regexp)
+{
+	char *p;
+
+	p = s;
+	while(s[0] != '\0')
+	{
+		if(s[0] == '\\' && (!regexp || s[1] == '/'))
+		{
+			++s;
+		}
+		*p++ = s[0];
+		if(s[0] != '\0')
+		{
+			++s;
+		}
+	}
+	*p = '\0';
+}
+
 int
 is_null_or_empty(const char string[])
 {
@@ -663,7 +744,7 @@ expand_tabulation(const char line[], size_t max, size_t tab_stops, char buf[])
 	size_t col = 0;
 	while(col < max && *line != '\0')
 	{
-		const size_t char_width = get_char_width(line);
+		const size_t char_width = utf8_chrw(line);
 		const size_t char_screen_width = wcwidth(get_first_wchar(line));
 		if(char_screen_width != (size_t)-1 && col + char_screen_width > max)
 		{
@@ -744,7 +825,7 @@ has_uppercase_letters(const char str[])
 size_t
 copy_str(char dst[], size_t dst_len, const char src[])
 {
-	/* XXX: shouldn't we return "strlen(src)" instead "0U"? */
+	/* XXX: shouldn't we return "strlen(src)" instead of "0U"? */
 	return (dst == src) ? 0U : copy_substr(dst, dst_len, src, '\0');
 }
 
@@ -827,7 +908,134 @@ split_and_get(char str[], char sep, char **state)
 	return (*str == '\0') ? NULL : str;
 }
 
-#if defined(_WIN32)
+char *
+split_and_get_dc(char str[], char **state)
+{
+	if(str[0] == '\0')
+	{
+		return NULL;
+	}
+
+	if(*state != NULL)
+	{
+		if(**state == '\0')
+		{
+			return NULL;
+		}
+
+		str = *state;
+	}
+
+	while(str != NULL)
+	{
+		char *ptr = strchr(str, ',');
+
+		if(ptr != NULL)
+		{
+			while(ptr != NULL && ptr[1] == ',')
+			{
+				ptr = strchr(ptr + 2, ',');
+			}
+			if(ptr != NULL)
+			{
+				*ptr = '\0';
+				++ptr;
+			}
+		}
+		if(ptr == NULL)
+		{
+			ptr = str + strlen(str);
+		}
+
+		while(isspace(*str) || *str == ',')
+		{
+			++str;
+		}
+
+		if(str[0] != '\0')
+		{
+			squash_double_commas(str);
+			*state = ptr;
+			break;
+		}
+
+		str = ptr;
+	}
+
+	if(str == NULL)
+	{
+		*state = NULL;
+	}
+	return str;
+}
+
+/* Squashes two consecutive commas into one in place. */
+TSTATIC void
+squash_double_commas(char str[])
+{
+	char *p = str;
+	while(*str != '\0')
+	{
+		if(str[0] == ',')
+		{
+			if(str[1] == ',')
+			{
+				*p++ = *str++;
+				++str;
+				continue;
+			}
+		}
+		*p++ = *str++;
+	}
+	*p = '\0';
+}
+
+int
+count_lines(const char text[], int max_width)
+{
+	const char *start, *end;
+	int nlines;
+
+	nlines = 0;
+
+	start = text;
+	end = text - 1;
+	while((end = strchr(end + 1, '\n')) != NULL)
+	{
+		if(max_width == INT_MAX)
+		{
+			++nlines;
+		}
+		else
+		{
+			nlines += DIV_ROUND_UP(end - start, max_width);
+			if(start == end)
+			{
+				++nlines;
+			}
+		}
+
+		start = end + 1;
+	}
+	if(*start == '\0' || max_width == INT_MAX)
+	{
+		++nlines;
+	}
+	else
+	{
+		const size_t screen_width = utf8_strsw(start);
+		nlines += DIV_ROUND_UP(screen_width, max_width);
+	}
+
+	if(nlines == 0)
+	{
+		nlines = 1;
+	}
+
+	return nlines;
+}
+
+#ifdef _WIN32
 
 char *
 strcasestr(const char haystack[], const char needle[])
@@ -846,4 +1054,4 @@ strcasestr(const char haystack[], const char needle[])
 #endif
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
