@@ -20,34 +20,22 @@
 #ifndef VIFM__UTILS__STR_H__
 #define VIFM__UTILS__STR_H__
 
+#include <inttypes.h> /* PRIu64 */
 #include <stddef.h> /* size_t wchar_t */
 
-#if defined(_WIN64)
-#define WPRINTF_MBSTR L"s"
-#define WPRINTF_WSTR L"ls"
-#define PRINTF_PID_T "%llu"
-#define PRINTF_SIZE_T "%llu"
-#define TIME_T "%lld"
-#elif defined(_WIN32)
-#define WPRINTF_MBSTR L"S"
+#include "macros.h"
+#include "test_helpers.h"
+
+#if defined(_WIN32) && !defined(_WIN64)
+#ifdef BROKEN_SWPRINTF
 #define WPRINTF_WSTR L"s"
-#define PRINTF_PID_T "%d"
-#define PRINTF_SIZE_T "%u"
-#define TIME_T "%ld"
 #else
-#define WPRINTF_MBSTR L"s"
 #define WPRINTF_WSTR L"ls"
-#define PRINTF_PID_T "%d"
-#if defined(__OpenBSD__)
-#define TIME_T "%lld"
-#else
-#define TIME_T "%ld"
 #endif
-#if defined(_LP64)
-#define PRINTF_SIZE_T "%lu"
+#define PRINTF_ULL PRIu64
 #else
-#define PRINTF_SIZE_T "%u"
-#endif
+#define WPRINTF_WSTR L"ls"
+#define PRINTF_ULL "llu"
 #endif
 
 /* Various string functions. */
@@ -65,6 +53,10 @@ void chomp(char str[]);
  * returned. */
 wchar_t * to_wide(const char s[]);
 
+/* Same as to_wide() except that it never fails due to conversion errors and
+ * returns something, thus returns NULL only on memory allocation failure. */
+wchar_t * to_wide_force(const char s[]);
+
 /* Calculates length of multibyte string after converting it to wide string.
  * Returns the length. */
 size_t wide_len(const char s[]);
@@ -150,6 +142,10 @@ char * after_first(const char str[], char c);
  * failed. */
 int replace_string(char **str, const char with[]);
 
+/* Same as replace_string(), but also allows "to" to be NULL, in which case
+ * *str is freed and set to NULL. */
+int update_string(char **str, const char to[]);
+
 /* Adds a character to the end of the string.  Returns the str argument. */
 char * strcatch(char str[], char c);
 
@@ -200,7 +196,7 @@ char * break_in_two(char str[], size_t max);
  * platforms transparently in other parts of the program. */
 int vifm_swprintf(wchar_t str[], size_t len, const wchar_t format[], ...);
 
-/* Extracts next non-empry part of string with separators.  Returns pointer to
+/* Extracts next non-empty part of string with separators.  Returns pointer to
  * the beginning of the next part or NULL at the end of a string. */
 const char * extract_part(const char str[], char separator, char part_buf[]);
 
@@ -211,13 +207,17 @@ char * skip_char(const char str[], char c);
  * it. */
 char * escape_chars(const char string[], const char chars[]);
 
+/* Unescapes string in place (removes extra slashes).  regexp flag narrows set
+ * of unescaped characters to "/". */
+void unescape(char s[], int regexp);
+
 /* Returns non-zero if the string is NULL or empty. */
 int is_null_or_empty(const char string[]);
 
 /* Formats string like printf, but instead of printing it, allocates memory and
  * and prints it there.  Returns newly allocated string, which should be freed
  * by the caller, or NULL if there is not enough memory. */
-char * format_str(const char format[], ...);
+char * format_str(const char format[], ...) _gnuc_printf(1, 2);
 
 /* Replaces all occurrences of horizontal tabulation character with appropriate
  * number of spaces.  The max parameter designates the maximum number of screen
@@ -278,6 +278,14 @@ void replace_char(char str[], char from, char to);
  *   } */
 char * split_and_get(char str[], char sep, char **state);
 
+/* Like split_and_get(), but works on comma-separated list with double commas
+ * signifying literal comma character. */
+char * split_and_get_dc(char str[], char **state);
+
+/* Counts lines in the text.  Considers wrapping around max_width column unless
+ * it's equal to INT_MAX. */
+int count_lines(const char text[], int max_width);
+
 #ifdef _WIN32
 
 /* Same as strstr(), but in case insensitive way. */
@@ -289,7 +297,11 @@ char * strcasestr(const char haystack[], const char needle[]);
 
 #endif
 
+TSTATIC_DEFS(
+	void squash_double_commas(char str[]);
+)
+
 #endif /* VIFM__UTILS__STR_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
