@@ -26,14 +26,18 @@
 #include <stdlib.h> /* free() malloc() realloc() */
 #include <string.h> /* strcspn() */
 
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
-#include "fs_limits.h"
+#include "../compat/reallocarray.h"
+#include "file_streams.h"
 
 static char * read_whole_file(const char filepath[], size_t *read);
 static char * read_seekable_stream(FILE *const fp, size_t *read);
 static size_t get_remaining_stream_size(FILE *const fp);
-static char ** text_to_lines(char text[], size_t text_len, int *nlines);
-static char ** break_into_lines(char text[], size_t text_len, int *nlines);
+static char ** text_to_lines(char text[], size_t text_len, int *nlines,
+		int null_sep);
+static char ** break_into_lines(char text[], size_t text_len, int *nlines,
+		int null_sep);
 
 int
 add_to_string_array(char ***array, int len, int count, ...)
@@ -41,7 +45,7 @@ add_to_string_array(char ***array, int len, int count, ...)
 	char **p;
 	va_list va;
 
-	p = realloc(*array, sizeof(char *)*(len + count));
+	p = reallocarray(*array, len + count, sizeof(char *));
 	if(p == NULL)
 		return len;
 	*array = p;
@@ -64,7 +68,7 @@ add_to_string_array(char ***array, int len, int count, ...)
 int
 put_into_string_array(char ***array, int len, char item[])
 {
-	char **const arr = realloc(*array, sizeof(char *)*(len + 1));
+	char **const arr = reallocarray(*array, len + 1, sizeof(char *));
 	if(arr != NULL)
 	{
 		*array = arr;
@@ -107,7 +111,7 @@ is_in_string_array_os(char *array[], size_t len, const char item[])
 char **
 copy_string_array(char **array, size_t len)
 {
-	char **result = malloc(sizeof(char *)*len);
+	char **result = reallocarray(NULL, len, sizeof(char *));
 	size_t i;
 	for(i = 0U; i < len; ++i)
 	{
@@ -122,7 +126,7 @@ string_array_pos(char *array[], size_t len, const char item[])
 	size_t i = len;
 	if(item != NULL)
 	{
-		for(i = 0; i < len; i++)
+		for(i = 0U; i < len; ++i)
 		{
 			if(strcmp(array[i], item) == 0)
 			{
@@ -170,18 +174,14 @@ free_strings(char *array[], size_t len)
 	}
 }
 
-void
-free_wstring_array(wchar_t **array, size_t len)
-{
-	free_string_array((char **)array, len);
-}
-
 char **
 read_file_of_lines(const char filepath[], int *nlines)
 {
 	size_t text_len;
 	char *const text = read_whole_file(filepath, &text_len);
-	char **list = (text == NULL) ? NULL : text_to_lines(text, text_len, nlines);
+	char **list = (text == NULL)
+	            ? NULL
+	            : text_to_lines(text, text_len, nlines, 0);
 	if(list == NULL)
 	{
 		list = malloc(sizeof(*list));
@@ -215,15 +215,21 @@ read_file_lines(FILE *f, int *nlines)
 {
 	size_t text_len;
 	char *const text = read_seekable_stream(f, &text_len);
-	return text_to_lines(text, text_len, nlines);
+	return text_to_lines(text, text_len, nlines, 0);
 }
 
 char **
-read_stream_lines(FILE *f, int *nlines)
+read_stream_lines(FILE *f, int *nlines, int null_sep_heuristic)
 {
+	int null;
 	size_t text_len;
 	char *const text = read_nonseekable_stream(f, &text_len);
-	return (text == NULL) ? NULL : text_to_lines(text, text_len, nlines);
+	if(text == NULL)
+	{
+		return NULL;
+	}
+	null = (null_sep_heuristic && strlen(text) != text_len);
+	return text_to_lines(text, text_len, nlines, null);
 }
 
 char *
@@ -236,10 +242,12 @@ read_nonseekable_stream(FILE *fp, size_t *read)
 	{
 		char *last_allocated_block = content;
 		size_t len = 0U, piece_len;
+		skip_bom(fp);
 		while((piece_len = fread(content + len, 1, PIECE_LEN, fp)) != 0U)
 		{
 			const size_t new_size = len + piece_len + PIECE_LEN + 1U;
-			if((last_allocated_block = realloc(content, new_size)) == NULL)
+			last_allocated_block = realloc(content, new_size);
+			if(last_allocated_block == NULL)
 			{
 				break;
 			}
@@ -276,7 +284,10 @@ static char *
 read_seekable_stream(FILE *const fp, size_t *read)
 {
 	char *content;
-	const size_t len = get_remaining_stream_size(fp);
+	size_t len;
+
+	skip_bom(fp);
+	len = get_remaining_stream_size(fp);
 
 	*read = 0UL;
 
@@ -320,13 +331,13 @@ get_remaining_stream_size(FILE *const fp)
  * otherwise NULL is returned, *nlines is untouched.  For empty file non-NULL
  * will be returned, but *nlines will be zero. */
 static char **
-text_to_lines(char text[], size_t text_len, int *nlines)
+text_to_lines(char text[], size_t text_len, int *nlines, int null_sep)
 {
 	char **list = NULL;
 
 	if(text != NULL)
 	{
-		list = break_into_lines(text, text_len, nlines);
+		list = break_into_lines(text, text_len, nlines, null_sep);
 		free(text);
 	}
 	return list;
@@ -336,15 +347,16 @@ text_to_lines(char text[], size_t text_len, int *nlines)
  * on success, otherwise NULL is returned, *nlines is untouched.  For empty file
  * non-NULL will be returned, but *nlines will be zero. */
 static char **
-break_into_lines(char text[], size_t text_len, int *nlines)
+break_into_lines(char text[], size_t text_len, int *nlines, int null_sep)
 {
+	const char *const seps = null_sep ? "" : "\n\r";
 	const char *const end = text + text_len;
 	char **list = NULL;
 
 	*nlines = 0;
 	while(text < end)
 	{
-		const size_t line_len = strcspn(text, "\n\r");
+		const size_t line_len = strcspn(text, seps);
 
 		char *after_line = text + line_len;
 		if(after_line[0] == '\n')
@@ -376,17 +388,18 @@ break_into_lines(char text[], size_t text_len, int *nlines)
 int
 write_file_of_lines(const char filepath[], char *strs[], size_t nstrs)
 {
-	FILE *fp;
 	size_t i;
 
-	if((fp = os_fopen(filepath, "w")) == NULL)
+	FILE *const fp = os_fopen(filepath, "w");
+	if(fp == NULL)
 	{
 		return 1;
 	}
 
-	for(i = 0U; i < nstrs; i++)
+	for(i = 0U; i < nstrs; ++i)
 	{
-		fprintf(fp, "%s\n", strs[i]);
+		fputs(strs[i], fp);
+		putc('\n', fp);
 	}
 
 	fclose(fp);
@@ -394,4 +407,4 @@ write_file_of_lines(const char filepath[], char *strs[], size_t nstrs)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
