@@ -22,12 +22,21 @@
 #include <stddef.h> /* size_t wchar_t */
 #include <stdio.h> /* FILE */
 
+/* Convenience structure (not used here directly, but is related
+ * functionally). */
+typedef struct
+{
+	int nitems;   /* Number of items in the list. */
+	char **items; /* The list itself. */
+}
+strlist_t;
+
 /* Input pointers can be NULL.  Returns new length of the array. */
 int add_to_string_array(char ***array, int len, int count, ...);
 
 /* Puts pointer into string array without making a copy.  Reallocates *array.
- * Returns new size of the array, which can be equal to len on reallocation
- * failure. */
+ * item can be NULL.  Returns new size of the array, which can be equal to len
+ * on reallocation failure. */
 int put_into_string_array(char **array[], int len, char item[]);
 
 void remove_from_string_array(char **array, size_t len, int pos);
@@ -61,8 +70,6 @@ void free_string_array(char *array[], size_t len);
 /* Frees memory of all array items, but not from the array itself. */
 void free_strings(char *array[], size_t len);
 
-void free_wstring_array(wchar_t **array, size_t len);
-
 /* Reads file specified by filepath into an array of strings.  Returns non-NULL
  * on success, otherwise NULL is returned, *nlines is untouched and errno
  * contains error code.  For empty file non-NULL will be returned, but *nlines
@@ -74,8 +81,10 @@ char ** read_file_of_lines(const char filepath[], int *nlines);
 char ** read_file_lines(FILE *f, int *nlines);
 
 /* Reads content of the stream (not required to be seekable) f into array of
- * strings.  Returns NULL for an empty file stream. */
-char ** read_stream_lines(FILE *f, int *nlines);
+ * strings.  The flag enables heuristic according to which if stream contents
+ * includes null character, it's taken as a separator instead of regular newline
+ * characters.  Returns NULL for an empty file stream. */
+char ** read_stream_lines(FILE *f, int *nlines, int null_sep_heuristic);
 
 /* Reads content of the fp stream that doesn't support seek operation (e.g. it
  * points to a pipe) until end-of-file into null terminated string.  Returns
@@ -90,4 +99,4 @@ int write_file_of_lines(const char filepath[], char *strs[], size_t nstrs);
 #endif /* VIFM__UTILS__STRING_ARRAY_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
