@@ -27,46 +27,50 @@
 #include "utf8.h"
 #endif
 
-#include <regex.h>
-
 #include <sys/types.h> /* pid_t */
 #include <unistd.h>
 
 #include <ctype.h> /* isalnum() isalpha() */
+#include <errno.h> /* errno */
 #include <stddef.h> /* size_t */
 #include <stdio.h> /* snprintf() */
 #include <stdlib.h> /* free() malloc() */
-#include <string.h> /* strdup() strchr() strlen() strpbrk() */
+#include <string.h> /* strdup() strchr() strlen() strpbrk() strtol() */
 #include <wchar.h> /* wcwidth() */
 
-#include "../modes/dialogs/msg_dialog.h"
 #include "../cfg/config.h"
+#include "../compat/fs_limits.h"
 #include "../compat/os.h"
 #include "../engine/keys.h"
+#include "../int/fuse.h"
+#include "../modes/dialogs/msg_dialog.h"
 #include "../ui/cancellation.h"
 #include "../background.h"
-#include "../fuse.h"
 #include "../registers.h"
+#include "../status.h"
 #include "env.h"
 #include "file_streams.h"
 #include "fs.h"
-#include "fs_limits.h"
 #include "log.h"
 #include "macros.h"
 #include "path.h"
 #include "str.h"
+#include "string_array.h"
 
 #ifdef _WIN32
-
 static void unquote(char quoted[]);
-
 #endif
+static int is_line_spec(const char str[]);
 
 int
 vifm_system(char command[])
 {
 #ifdef _WIN32
-	system("cls");
+	/* The check is primarily for tests, otherwise screen is reset. */
+	if(curr_stats.load_stage != 0)
+	{
+		system("cls");
+	}
 #endif
 	LOG_INFO_MSG("Shell command: %s", command);
 	return run_in_shell_no_cls(command);
@@ -77,8 +81,10 @@ process_cmd_output(const char descr[], const char cmd[], int user_sh,
 		cmd_output_handler handler, void *arg)
 {
 	FILE *file, *err;
-	char *line;
 	pid_t pid;
+	int nlines;
+	char **lines;
+	int i;
 
 	LOG_INFO_MSG("Capturing output of the command: %s", cmd);
 
@@ -88,26 +94,22 @@ process_cmd_output(const char descr[], const char cmd[], int user_sh,
 		return 1;
 	}
 
-	show_progress("", 0);
-
 	ui_cancellation_reset();
 	ui_cancellation_enable();
 
 	wait_for_data_from(pid, file, 0);
-
-	line = NULL;
-	while((line = read_line(file, line)) != NULL)
-	{
-		show_progress(descr, 1000);
-		handler(line, arg);
-		wait_for_data_from(pid, file, 0);
-	}
+	lines = read_stream_lines(file, &nlines, 1);
 
 	ui_cancellation_disable();
-
 	fclose(file);
-	show_errors_from_file(err, descr);
 
+	for(i = 0; i < nlines; ++i)
+	{
+		handler(lines[i], arg);
+	}
+	free_string_array(lines, nlines);
+
+	show_errors_from_file(err, descr);
 	return 0;
 }
 
@@ -115,7 +117,7 @@ int
 vifm_chdir(const char path[])
 {
 	char curr_path[PATH_MAX];
-	if(getcwd(curr_path, sizeof(curr_path)) == curr_path)
+	if(get_cwd(curr_path, sizeof(curr_path)) == curr_path)
 	{
 		if(stroscmp(curr_path, path) == 0)
 		{
@@ -161,7 +163,7 @@ expand_envvars(const char str[], int escape_vals)
 				char *escaped_var_value = NULL;
 				if(escape_vals)
 				{
-					escaped_var_value = escape_filename(var_value, 1);
+					escaped_var_value = shell_like_escape(var_value, 1);
 					var_value = escaped_var_value;
 				}
 
@@ -235,62 +237,6 @@ friendly_size_notation(uint64_t num, int str_size, char *str)
 	return u > 0;
 }
 
-int
-get_regexp_cflags(const char pattern[])
-{
-	int result = REG_EXTENDED;
-	if(regexp_should_ignore_case(pattern))
-	{
-		result |= REG_ICASE;
-	}
-	return result;
-}
-
-int
-regexp_should_ignore_case(const char pattern[])
-{
-	int ignore_case = cfg.ignore_case;
-	if(cfg.ignore_case && cfg.smart_case)
-	{
-		if(has_uppercase_letters(pattern))
-		{
-			ignore_case = 0;
-		}
-	}
-	return ignore_case;
-}
-
-const char *
-get_regexp_error(int err, regex_t *re)
-{
-	static char buf[360];
-
-	regerror(err, re, buf, sizeof(buf));
-	return buf;
-}
-
-int
-parse_case_flag(const char flags[], int *case_sensitive)
-{
-	/* TODO: maybe generalize code with substitute_cmd(). */
-
-	while(*flags != '\0')
-	{
-		switch(*flags)
-		{
-			case 'i': *case_sensitive = 0; break;
-			case 'I': *case_sensitive = 1; break;
-
-			default:
-				return 1;
-		}
-
-		++flags;
-	}
-
-	return 0;
-}
-
 const char *
 enclose_in_dquotes(const char str[])
 {
@@ -588,15 +534,11 @@ def_count(int count)
 	return (count == NO_COUNT_GIVEN) ? 1 : count;
 }
 
-/* Extracts path and line number from the spec (default line number is 1).
- * Returns path in as newly allocated string and sets *line_num to line number,
- * otherwise NULL is returned. */
-TSTATIC char *
+char *
 parse_file_spec(const char spec[], int *line_num)
 {
 	char *path_buf;
 	const char *colon;
-	int colon_lookup_offset = 0;
 	const size_t bufs_len = 2 + strlen(spec) + 1 + 1;
 
 	path_buf = malloc(bufs_len);
@@ -605,7 +547,7 @@ parse_file_spec(const char spec[], int *line_num)
 		return NULL;
 	}
 
-	if(is_path_absolute(spec))
+	if(is_path_absolute(spec) || spec[0] == '~')
 	{
 		path_buf[0] = '\0';
 	}
@@ -615,13 +557,30 @@ parse_file_spec(const char spec[], int *line_num)
 	}
 
 #ifdef _WIN32
-	if(is_path_absolute(spec))
+	colon = strchr(spec + (is_path_absolute(spec) ? 2 : 0), ':');
+	if(colon != NULL && !is_line_spec(colon + 1))
 	{
-		colon_lookup_offset = 2;
+		colon = NULL;
+	}
+#else
+	colon = strchr(spec, ':');
+	while(colon != NULL)
+	{
+		if(is_line_spec(colon + 1))
+		{
+			char path[bufs_len];
+			strcpy(path, path_buf);
+			strncat(path, spec, colon - spec);
+			if(path_exists(path, NODEREF))
+			{
+				break;
+			}
+		}
+
+		colon = strchr(colon + 1, ':');
 	}
 #endif
 
-	colon = strchr(spec + colon_lookup_offset, ':');
 	if(colon != NULL)
 	{
 		strncat(path_buf, spec, colon - spec);
@@ -631,6 +590,11 @@ parse_file_spec(const char spec[], int *line_num)
 	{
 		strcat(path_buf, spec);
 		*line_num = 1;
+
+		while(!path_exists(path_buf, NODEREF) && strchr(path_buf, ':') != NULL)
+		{
+			break_atr(path_buf, ':');
+		}
 	}
 
 	chomp(path_buf);
@@ -639,7 +603,18 @@ parse_file_spec(const char spec[], int *line_num)
 	to_forward_slash(path_buf);
 #endif
 
-	return path_buf;
+	return replace_tilde(path_buf);
+}
+
+/* Checks whether str points to a valid line number.  Returns non-zero if so,
+ * otherwise zero is returned. */
+static int
+is_line_spec(const char str[])
+{
+	char *endptr;
+	errno = 0;
+	(void)strtol(str, &endptr, 10);
+	return (endptr != str && errno == 0 && *endptr == ':');
 }
 
 int
@@ -651,4 +626,4 @@ is_graphics_viewer(const char viewer[])
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
