@@ -20,8 +20,7 @@
 #ifndef VIFM__UTILS__UTILS_H__
 #define VIFM__UTILS__UTILS_H__
 
-#include <regex.h>
-
+#include <sys/stat.h> /* stat */
 #include <sys/types.h> /* gid_t mode_t uid_t */
 
 #include <stddef.h> /* size_t wchar_t */
@@ -42,23 +41,6 @@ EnvType;
 /* Callback for process_cmd_output() function. */
 typedef void (*cmd_output_handler)(const char line[], void *arg);
 
-/* Regular expressions. */
-
-/* Gets flags for compiling a regular expression specified by the pattern taking
- * 'ignorecase' and 'smartcase' options into account.  Returns regex flags. */
-int get_regexp_cflags(const char pattern[]);
-
-/* Decides whether case should be ignored for the pattern.  Considers
- * 'ignorecase' and 'smartcase' options.  Returns non-zero when case should be
- * ignored, otherwise zero is returned. */
-int regexp_should_ignore_case(const char pattern[]);
-
-const char * get_regexp_error(int err, regex_t *re);
-
-/* *case_sensitive should be initialized with default value outside the call.
- * Returns zero on success, otherwise non-zero is returned. */
-int parse_case_flag(const char flags[], int *case_sensitive);
-
 /* Shell and program running. */
 
 /* Executes an external command.  Clears the screen up on Windows before running
@@ -72,8 +54,10 @@ void pause_shell(void);
 void recover_after_shellout(void);
 
 /* Invokes handler for each line read from stdout of the command specified via
- * cmd.  Error stream is displayed separately.  Supports cancellation.  Returns
- * zero on success, otherwise non-zero is returned. */
+ * cmd.  Error stream is displayed separately.  Implements heuristic according
+ * to which if command output includes null character, it's taken as a separator
+ * instead of regular newline characters.  Supports cancellation.  Returns zero
+ * on success, otherwise non-zero is returned. */
 int process_cmd_output(const char descr[], const char cmd[], int user_sh,
 		cmd_output_handler handler, void *arg);
 
@@ -89,7 +73,7 @@ typedef int (*mptraverser)(struct mntent *entry, void *arg);
  * Returns non-zero if so, otherwise zero is returned. */
 int is_on_slow_fs(const char full_path[]);
 
-/* Checks whether accessing the to loation from the from location might cause
+/* Checks whether accessing the to location from the from location might cause
  * slowdown.  Returns non-zero if so, otherwise zero is returned. */
 int refers_to_slower_fs(const char from[], const char to[]);
 
@@ -215,10 +199,6 @@ int format_help_cmd(char cmd[], size_t cmd_size);
  * terminal. */
 void display_help(const char cmd[]);
 
-/* Updates dir_mtime field of the view.  Returns zero on success, otherwise
- * non-zero is returned. */
-int update_dir_mtime(FileView *view);
-
 /* Suspends process until external signal comes. */
 void wait_for_signal(void);
 
@@ -242,7 +222,11 @@ void get_gid_string(const dir_entry_t *entry, int as_num, size_t buf_len,
 /* Reopens real terminal and binds it to stdout.  Returns NULL on error (message
  * is printed to stderr), otherwise file previously used as stdout is
  * returned. */
-FILE * reopen_terminal(void);
+FILE * reopen_term_stdout(void);
+
+/* Reopens real terminal and binds it to stdin.  Returns non-zero on error
+ * (message is printed to stderr) and zero otherwise. */
+int reopen_term_stdin(void);
 
 /* Executes the command via shell and opens its output for reading.  Returns
  * NULL on error, otherwise stream valid for reading is returned. */
@@ -252,6 +236,11 @@ FILE * read_cmd_output(const char cmd[]);
  * pointer to a statically allocated buffer. */
 const char * get_installed_data_dir(void);
 
+/* Clones timestamps from file specified by from to file at path.  st is a hint
+ * to omit extra file system requests if possible, can be NULL on Windows. */
+void clone_timestamps(const char path[], const char from[],
+		const struct stat *st);
+
 #ifdef _WIN32
 #include "utils_win.h"
 #else
@@ -261,4 +250,4 @@ const char * get_installed_data_dir(void);
 #endif /* VIFM__UTILS__UTILS_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
