@@ -20,20 +20,22 @@
 #include "utils_nix.h"
 #include "utils_int.h"
 
-#if defined (HAVE_LINUX_BINFMTS_H) && defined (HAVE_SYS_USER_H)
+#ifdef HAVE_MAX_ARG_STRLEN
+/* MAX_ARG_STRLEN is the only reason we need these headers. */
 #include <linux/binfmts.h>
 #include <sys/user.h>
 #endif
 
 #include <sys/select.h> /* select() FD_SET FD_ZERO */
-#include <sys/stat.h> /* O_RDONLY O_WRONLY S_* */
-#include <sys/time.h> /* timeval */
+#include <sys/stat.h> /* O_* S_* */
+#include <sys/time.h> /* timeval futimens() utimes() */
 #include <sys/types.h> /* gid_t mode_t pid_t uid_t */
 #include <sys/wait.h> /* waitpid */
 #include <fcntl.h> /* open() close() */
 #include <grp.h> /* getgrnam() getgrgid_r() */
 #include <pwd.h> /* getpwnam() getpwuid_r() */
-#include <unistd.h> /* X_OK dup() dup2() getpid() pause() */
+#include <unistd.h> /* X_OK dup() dup2() getpid() isatty() pause() sysconf()
+                       ttyname() */
 
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
@@ -43,21 +45,23 @@
                        sigprocmask() */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* FILE stderr fdopen() fprintf() snprintf() */
-#include <stdlib.h> /* atoi() free() malloc() */
-#include <string.h> /* strchr() strdup() strlen() strncmp() */
+#include <stdlib.h> /* atoi() free() */
+#include <string.h> /* strchr() strdup() strerror() strlen() strncmp() */
 
 #include "../cfg/config.h"
+#include "../compat/fs_limits.h"
+#include "../compat/mntent.h" /* mntent setmntent() getmntent() endmntent() */
 #include "../compat/os.h"
+#include "../compat/reallocarray.h"
 #include "../ui/cancellation.h"
 #include "../running.h"
 #include "../status.h"
 #include "env.h"
 #include "filemon.h"
 #include "fs.h"
-#include "fs_limits.h"
+#include "fswatch.h"
 #include "log.h"
 #include "macros.h"
-#include "mntent.h" /* mntent setmntent() getmntent() endmntent() */
 #include "path.h"
 #include "str.h"
 #include "utils.h"
@@ -88,6 +92,7 @@ static int clone_mnt_entry(struct mntent *lhs, const struct mntent *rhs);
 static void free_mnt_entry(struct mntent *entry);
 static int starts_with_list_item(const char str[], const char list[]);
 static int find_path_prefix_index(const char path[], const char list[]);
+static int open_tty(void);
 
 void
 pause_shell(void)
@@ -98,17 +103,18 @@ pause_shell(void)
 int
 run_in_shell_no_cls(char command[])
 {
-	typedef void (*sig_handler)(int);
-
 	int pid;
 	int result;
 	extern char **environ;
-	sig_handler sigtstp_handler;
+	struct sigaction new, old;
 
 	if(command == NULL)
 		return 1;
 
-	sigtstp_handler = signal(SIGTSTP, SIG_DFL);
+	new.sa_handler = SIG_DFL;
+	sigemptyset(&new.sa_mask);
+	new.sa_flags = SA_RESTART;
+	sigaction(SIGTSTP, &new, &old);
 
 	/* We need to block SIGCHLD signal.  One can't just set it to SIG_DFL, because
 	 * it will possibly cause missing of SIGCHLD from a background process
@@ -118,7 +124,7 @@ run_in_shell_no_cls(char command[])
 	pid = fork();
 	if(pid == -1)
 	{
-		signal(SIGTSTP, sigtstp_handler);
+		sigaction(SIGTSTP, &old, NULL);
 		(void)set_sigchld(0);
 		return -1;
 	}
@@ -128,13 +134,14 @@ run_in_shell_no_cls(char command[])
 		signal(SIGINT, SIG_DFL);
 		(void)set_sigchld(0);
 
-		execve(cfg.shell, make_execv_array(cfg.shell, command), environ);
-		exit(127);
+		execve(get_execv_path(cfg.shell), make_execv_array(cfg.shell, command),
+				environ);
+		_Exit(127);
 	}
 
 	result = get_proc_exit_status(pid);
 
-	signal(SIGTSTP, sigtstp_handler);
+	sigaction(SIGTSTP, &old, NULL);
 	(void)set_sigchld(0);
 
 	return result;
@@ -186,7 +193,8 @@ process_cancel_request(pid_t pid)
 	{
 		if(kill(pid, SIGINT) != 0)
 		{
-			LOG_SERROR_MSG(errno, "Failed to send SIGINT to " PRINTF_PID_T, pid);
+			LOG_SERROR_MSG(errno, "Failed to send SIGINT to %" PRINTF_ULL,
+					(unsigned long long)pid);
 		}
 	}
 }
@@ -221,7 +229,7 @@ run_from_fork(int pipe[2], int err_only, char cmd[])
 	/* Redirect stderr and maybe stdout to write end of the pipe. */
 	if(dup2(pipe[1], STDERR_FILENO) == -1)
 	{
-		exit(1);
+		_Exit(EXIT_FAILURE);
 	}
 	if(err_only)
 	{
@@ -231,7 +239,7 @@ run_from_fork(int pipe[2], int err_only, char cmd[])
 	{
 		if(dup2(pipe[1], STDOUT_FILENO) == -1)
 		{
-			exit(1);
+			_Exit(EXIT_FAILURE);
 		}
 	}
 
@@ -245,22 +253,33 @@ run_from_fork(int pipe[2], int err_only, char cmd[])
 	{
 		if(dup2(nullfd, STDIN_FILENO) == -1)
 		{
-			exit(1);
+			_Exit(EXIT_FAILURE);
 		}
 		if(err_only && dup2(nullfd, STDOUT_FILENO) == -1)
 		{
-			exit(1);
+			_Exit(EXIT_FAILURE);
 		}
 	}
 
-	execvp(cfg.shell, make_execv_array(cfg.shell, cmd));
-	exit(1);
+	execvp(get_execv_path(cfg.shell), make_execv_array(cfg.shell, cmd));
+	_Exit(127);
+}
+
+char *
+get_execv_path(char shell[])
+{
+	static char name[NAME_MAX];
+	(void)extract_cmd_name(shell, 0, sizeof(name), name);
+	return name;
 }
 
 char **
 make_execv_array(char shell[], char cmd[])
 {
-#ifdef MAX_ARG_STRLEN
+#ifdef HAVE_MAX_ARG_STRLEN
+#ifndef PAGE_SIZE
+#define PAGE_SIZE sysconf(_SC_PAGESIZE)
+#endif
 	/* Don't use maximum length, leave some room or commands fail to run. */
 	const size_t safe_arg_len = MIN(MAX_ARG_STRLEN, MAX_ARG_STRLEN - 4096U);
 	const size_t npieces = DIV_ROUND_UP(strlen(cmd), safe_arg_len);
@@ -272,18 +291,28 @@ make_execv_array(char shell[], char cmd[])
 	const size_t npieces = 1U;
 #endif
 
-	char **args = malloc(sizeof(*args)*(3 + npieces + 1));
+	char name[NAME_MAX];
+
+	char **args = reallocarray(NULL, 4 + npieces + 1, sizeof(*args));
 	char *eval_cmd;
 	size_t len;
 	size_t i;
 
+	char *const sh_arg = extract_cmd_name(shell, 0, sizeof(name), name);
+	const int with_sh_arg = (*sh_arg != '\0');
+
 	/* Don't use eval hack unless necessary. */
 	if(npieces == 1)
 	{
-		args[0] = shell;
-		args[1] = "-c";
-		args[2] = cmd;
-		args[3] = NULL;
+		i = 0U;
+		args[i++] = shell;
+		if(with_sh_arg)
+		{
+			args[i++] = sh_arg;
+		}
+		args[i++] = "-c";
+		args[i++] = cmd;
+		args[i++] = NULL;
 		return args;
 	}
 
@@ -304,17 +333,21 @@ make_execv_array(char shell[], char cmd[])
 		c = cmd[safe_arg_len];
 		cmd[safe_arg_len] = '\0';
 
-		args[3 + i] = strdup(cmd);
+		args[(with_sh_arg ? 4 : 3) + i] = strdup(cmd);
 
 		cmd[safe_arg_len] = c;
 		cmd += safe_arg_len;
 	}
 	(void)strappend(&eval_cmd, &len, "\"");
 
-	args[0] = shell;
-	args[1] = "-c";
-	args[2] = eval_cmd;
-	args[3 + npieces] = NULL;
+	args[i++] = shell;
+	if(with_sh_arg)
+	{
+		args[i++] = sh_arg;
+	}
+	args[i++] = "-c";
+	args[i++] = eval_cmd;
+	args[i++ + npieces] = NULL;
 
 	return args;
 }
@@ -372,8 +405,7 @@ int
 is_on_slow_fs(const char full_path[])
 {
 	char fs_name[PATH_MAX];
-	get_mount_point_traverser_state state =
-	{
+	get_mount_point_traverser_state state = {
 		.type = MI_FS_TYPE,
 		.path = full_path,
 		.buf_len = sizeof(fs_name),
@@ -387,6 +419,14 @@ is_on_slow_fs(const char full_path[])
 		return 0;
 	}
 
+	/* If slowfs equals "*" then all file systems are considered slow.  On cygwin
+	 * obtaining list of mounts from /etc/mtab, which is linked to /proc/mounts,
+	 * is very slow in presence of network drives. */
+	if(strcmp(cfg.slow_fs_list, "*") == 0)
+	{
+		return 1;
+	}
+
 	if(traverse_mount_points(&get_mount_info_traverser, &state) == 0)
 	{
 		if(state.curr_len > 0)
@@ -404,8 +444,7 @@ is_on_slow_fs(const char full_path[])
 int
 get_mount_point(const char path[], size_t buf_len, char buf[])
 {
-	get_mount_point_traverser_state state =
-	{
+	get_mount_point_traverser_state state = {
 		.type = MI_MOUNT_POINT,
 		.path = path,
 		.buf_len = buf_len,
@@ -509,7 +548,7 @@ read_mnt_entries(unsigned int *nentries)
 
 	while((ent = getmntent(f)) != NULL)
 	{
-		void *p = realloc(entries, sizeof(*entries)*(*nentries + 1));
+		void *p = reallocarray(entries, *nentries + 1, sizeof(*entries));
 		if(p != NULL)
 		{
 			entries = p;
@@ -578,8 +617,8 @@ starts_with_list_item(const char str[], const char list[])
 	return prefix != NULL;
 }
 
-/* Finds such elemenent of comma separated list of paths (the list) that the
- * path is prefixed with it.  Returns the index or -1 on failure. */
+/* Finds such element of comma separated list of paths (the list) that the path
+ * is prefixed with it.  Returns the index or -1 on failure. */
 static int
 find_path_prefix_index(const char path[], const char list[])
 {
@@ -697,7 +736,7 @@ int
 format_help_cmd(char cmd[], size_t cmd_size)
 {
 	int bg;
-	char *const escaped = escape_filename(cfg.config_dir, 0);
+	char *const escaped = shell_like_escape(cfg.config_dir, 0);
 	snprintf(cmd, cmd_size, "%s %s/" VIFM_HELP, cfg_get_vicmd(&bg), escaped);
 	free(escaped);
 	return bg;
@@ -706,13 +745,7 @@ format_help_cmd(char cmd[], size_t cmd_size)
 void
 display_help(const char cmd[])
 {
-	(void)shellout(cmd, -1, 1);
-}
-
-int
-update_dir_mtime(FileView *view)
-{
-	return filemon_from_file(view->curr_dir, &view->mon);
+	(void)shellout(cmd, PAUSE_ON_ERROR, 1);
 }
 
 void
@@ -724,9 +757,16 @@ wait_for_signal(void)
 void
 stop_process(void)
 {
-	void (*saved_stp_sig_handler)(int) = signal(SIGTSTP, SIG_DFL);
+	struct sigaction new, old;
+
+	new.sa_handler = SIG_DFL;
+	sigemptyset(&new.sa_mask);
+	new.sa_flags = SA_RESTART;
+	sigaction(SIGTSTP, &new, &old);
+
 	kill(0, SIGTSTP);
-	signal(SIGTSTP, saved_stp_sig_handler);
+
+	sigaction(SIGTSTP, &old, NULL);
 }
 
 void
@@ -814,7 +854,7 @@ get_gid_string(const dir_entry_t *entry, int as_num, size_t buf_len, char buf[])
 }
 
 FILE *
-reopen_terminal(void)
+reopen_term_stdout(void)
 {
 	FILE *fp;
 	int outfd, ttyfd;
@@ -822,29 +862,30 @@ reopen_terminal(void)
 	outfd = dup(STDOUT_FILENO);
 	if(outfd == -1)
 	{
-		fprintf(stderr, "Failed to store original output stream.");
+		fprintf(stderr, "Failed to store original output stream.\n");
 		return NULL;
 	}
 
 	fp = fdopen(outfd, "w");
 	if(fp == NULL)
 	{
-		fprintf(stderr, "Failed to open original output stream.");
+		fprintf(stderr, "Failed to open original output stream.\n");
 		return NULL;
 	}
 
-	ttyfd = open("/dev/tty", O_WRONLY);
+	ttyfd = open_tty();
 	if(ttyfd == -1)
 	{
+		fprintf(stderr, "Failed to open terminal for output: %s\n",
+				strerror(errno));
 		fclose(fp);
-		fprintf(stderr, "Failed to open terminal for output.");
 		return NULL;
 	}
 	if(dup2(ttyfd, STDOUT_FILENO) == -1)
 	{
 		close(ttyfd);
 		fclose(fp);
-		fprintf(stderr, "Failed to setup terminal as standard output stream.");
+		fprintf(stderr, "Failed to setup terminal as standard output stream.\n");
 		return NULL;
 	}
 
@@ -852,6 +893,65 @@ reopen_terminal(void)
 	return fp;
 }
 
+int
+reopen_term_stdin(void)
+{
+	int ttyfd;
+
+	if(close(STDIN_FILENO))
+	{
+		fprintf(stderr, "Failed to close original input stream.\n");
+		return 1;
+	}
+
+	ttyfd = open_tty();
+	if(ttyfd != STDIN_FILENO)
+	{
+		if(ttyfd != -1)
+		{
+			close(ttyfd);
+		}
+
+		fprintf(stderr, "Failed to open terminal for input.\n");
+		return 1;
+	}
+
+	return 0;
+}
+
+/* Opens controlling terminal doing its best at guessing its name.  This might
+ * not work if all the standard streams are redirected, in which case /dev/tty
+ * is used, which can't be passed around between processes, so some applications
+ * won't work in this (quite rare) case.  Returns opened file descriptor or -1
+ * on error. */
+static int
+open_tty(void)
+{
+	int fd = -1;
+
+	/* XXX: we might be better off duplicating descriptor if possible. */
+
+	if(isatty(STDIN_FILENO))
+	{
+		fd = open(ttyname(STDIN_FILENO), O_RDWR);
+	}
+	if(fd == -1 && isatty(STDOUT_FILENO))
+	{
+		fd = open(ttyname(STDOUT_FILENO), O_RDWR);
+	}
+	if(fd == -1 && isatty(STDERR_FILENO))
+	{
+		fd = open(ttyname(STDERR_FILENO), O_RDWR);
+	}
+
+	if(fd == -1)
+	{
+		fd = open("/dev/tty", O_RDWR);
+	}
+
+	return fd;
+}
+
 FILE *
 read_cmd_output(const char cmd[])
 {
@@ -893,5 +993,39 @@ get_installed_data_dir(void)
 	return PACKAGE_DATA_DIR;
 }
 
+void
+clone_timestamps(const char path[], const char from[], const struct stat *st)
+{
+	struct timeval tv[2];
+
+#if defined(HAVE_STRUCT_STAT_ST_MTIM) && defined(HAVE_FUTIMENS)
+	const int fd = open(path, O_WRONLY);
+	if(fd != -1)
+	{
+		struct timespec ts[2] = { st->st_atim, st->st_mtim };
+		if(futimens(fd, ts) == 0)
+		{
+			close(fd);
+			return;
+		}
+		close(fd);
+		/* Not sure how useful this is, but we can try both functions in a row. */
+	}
+#endif
+
+#ifdef HAVE_STRUCT_STAT_ST_MTIM
+	tv[0].tv_sec = st->st_atim.tv_sec;
+	tv[0].tv_usec = st->st_atim.tv_nsec/1000;
+	tv[1].tv_sec = st->st_mtim.tv_sec;
+	tv[1].tv_usec = st->st_mtim.tv_nsec/1000;
+#else
+	tv[0].tv_sec = st->st_atime;
+	tv[0].tv_usec = 0;
+	tv[1].tv_sec = st->st_mtime;
+	tv[1].tv_usec = 0;
+#endif
+	utimes(path, tv);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
