@@ -20,9 +20,8 @@
 #include "utils.h"
 #include "utils_int.h"
 
-#include <ntdef.h>
 #include <windows.h>
-#include <winioctl.h>
+#include <ntdef.h>
 
 #include <curses.h>
 
@@ -33,21 +32,21 @@
 #include <stddef.h> /* NULL size_t */
 #include <stdint.h> /* uint32_t */
 #include <stdlib.h> /* EXIT_SUCCESS free() */
-#include <string.h> /* strcat() strchr() strcpy() strlen() */
+#include <string.h> /* strcat() strchr() strcpy() strdup() strlen() */
 #include <stdio.h> /* FILE SEEK_SET fread() fclose() snprintf() */
 
 #include "../cfg/config.h"
+#include "../compat/fs_limits.h"
+#include "../compat/mntent.h"
 #include "../compat/os.h"
 #include "../compat/wcwidth.h"
 #include "../ui/ui.h"
-#include "../commands_completion.h"
+#include "../cmd_completion.h"
 #include "../status.h"
 #include "env.h"
 #include "fs.h"
-#include "fs_limits.h"
 #include "log.h"
 #include "macros.h"
-#include "mntent.h"
 #include "path.h"
 #include "str.h"
 #include "utf8.h"
@@ -63,7 +62,6 @@ static int should_wait_for_program(const char cmd[]);
 static DWORD handle_process(const char cmd[], HANDLE proc, int *got_exit_code);
 static int get_subsystem(const char filename[]);
 static int get_stream_subsystem(FILE *fp);
-static int win_get_dir_mtime(const char dir_path[], FILETIME *ft);
 static FILE * use_info_prog_internal(const char cmd[], int out_pipe[2]);
 
 void
@@ -82,35 +80,23 @@ pause_shell(void)
 int
 run_in_shell_no_cls(char command[])
 {
-	char buf[strlen(cfg.shell) + 5 + strlen(command)*4 + 1 + 1];
+	int ret;
+	char *const sh_cmd = win_make_sh_cmd(command);
 
+	/* XXX: why do we use different functions for different cases? */
 	if(curr_stats.shell_type == ST_CMD)
 	{
-		/* See "cmd /?" for an "explanation" why extra double quotes are
-		 * omitted. */
-		snprintf(buf, sizeof(buf), "%s /C %s", cfg.shell, command);
-		return os_system(buf);
+		ret = os_system(sh_cmd);
 	}
 	else
 	{
-		char *p;
 		int returned_exit_code;
-
-		strcpy(buf, cfg.shell);
-		strcat(buf, " -c '");
-
-		p = buf + strlen(buf);
-		while(*command != '\0')
-		{
-			if(*command == '\\')
-				*p++ = '\\';
-			*p++ = *command++;
-		}
-		*p = '\0';
-
-		strcat(buf, "'");
-		return win_exec_cmd(buf, &returned_exit_code);
+		ret = win_exec_cmd(sh_cmd, &returned_exit_code);
 	}
+
+	free(sh_cmd);
+
+	return ret;
 }
 
 void
@@ -194,6 +180,42 @@ win_exec_cmd(char cmd[], int *const returned_exit_code)
 	return code;
 }
 
+char *
+win_make_sh_cmd(const char cmd[])
+{
+	char buf[strlen(cfg.shell) + 5 + strlen(cmd)*4 + 1 + 1];
+
+	if(curr_stats.shell_type == ST_CMD)
+	{
+		/* Documentation in `cmd /?` seems to LIE, can't make both spaces and
+		 * special characters work at the same time. */
+		const char *const fmt = (cmd[0] == '"') ? "%s /C \"%s\"" : "%s /C %s";
+		snprintf(buf, sizeof(buf), fmt, cfg.shell, cmd);
+	}
+	else
+	{
+		char *p;
+
+		strcpy(buf, cfg.shell);
+		strcat(buf, " -c '");
+
+		p = buf + strlen(buf);
+		while(*cmd != '\0')
+		{
+			if(*cmd == '\\')
+			{
+				*p++ = '\\';
+			}
+			*p++ = *cmd++;
+		}
+		*p = '\0';
+
+		strcat(buf, "'");
+	}
+
+	return strdup(buf);
+}
+
 /* Handles process execution.  Returns system error code when sets
  * *got_exit_code to 0 and exit code of the process otherwise. */
 static DWORD
@@ -347,7 +369,7 @@ is_vista_and_above(void)
 }
 
 const char *
-attr_str(DWORD attr)
+attr_str(uint32_t attr)
 {
 	static char buf[5 + 1];
 	buf[0] = '\0';
@@ -366,7 +388,7 @@ attr_str(DWORD attr)
 }
 
 const char *
-attr_str_long(DWORD attr)
+attr_str_long(uint32_t attr)
 {
 	static char buf[10 + 1];
 	snprintf(buf, sizeof(buf), "ahirscdepz");
@@ -405,157 +427,6 @@ escape_for_cd(const char str[])
 	return buf;
 }
 
-const char *
-win_resolve_mount_points(const char path[])
-{
-	static char resolved_path[PATH_MAX];
-
-	DWORD attr;
-	wchar_t *utf16_path;
-	HANDLE hfind;
-	WIN32_FIND_DATAW ffd;
-	HANDLE hfile;
-	char rdb[2048];
-	char *t;
-	int offset;
-	REPARSE_DATA_BUFFER *rdbp;
-
-	utf16_path = utf8_to_utf16(path);
-	attr = GetFileAttributesW(utf16_path);
-	free(utf16_path);
-
-	if(attr == INVALID_FILE_ATTRIBUTES)
-	{
-		return path;
-	}
-
-	if(!(attr & FILE_ATTRIBUTE_REPARSE_POINT))
-	{
-		return path;
-	}
-
-	copy_str(resolved_path, sizeof(resolved_path), path);
-	chosp(resolved_path);
-
-	utf16_path = utf8_to_utf16(resolved_path);
-	hfind = FindFirstFileW(utf16_path, &ffd);
-
-	if(hfind == INVALID_HANDLE_VALUE)
-	{
-		free(utf16_path);
-		return path;
-	}
-
-	FindClose(hfind);
-
-	if(ffd.dwReserved0 != IO_REPARSE_TAG_MOUNT_POINT)
-	{
-		free(utf16_path);
-		return path;
-	}
-
-	hfile = CreateFileW(utf16_path, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
-			OPEN_EXISTING, FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT,
-			NULL);
-
-	free(utf16_path);
-
-	if(hfile == INVALID_HANDLE_VALUE)
-	{
-		return path;
-	}
-
-	if(!DeviceIoControl(hfile, FSCTL_GET_REPARSE_POINT, NULL, 0, rdb, sizeof(rdb),
-			&attr, NULL))
-	{
-		CloseHandle(hfile);
-		return path;
-	}
-	CloseHandle(hfile);
-
-	rdbp = (REPARSE_DATA_BUFFER *)rdb;
-	t = to_multibyte(rdbp->MountPointReparseBuffer.PathBuffer);
-
-	offset = starts_with_lit(t, "\\??\\") ? 4 : 0;
-	strcpy(resolved_path, t + offset);
-
-	free(t);
-
-	return resolved_path;
-}
-
-int
-win_check_dir_changed(FileView *view)
-{
-	FILETIME ft;
-	int r;
-
-	if(stroscmp(view->watched_dir, view->curr_dir) != 0)
-	{
-		wchar_t *utf16_cwd;
-
-		FindCloseChangeNotification(view->dir_watcher);
-		strcpy(view->watched_dir, view->curr_dir);
-
-		utf16_cwd = utf8_to_utf16(view->curr_dir);
-
-		view->dir_watcher = FindFirstChangeNotificationW(utf16_cwd, 1,
-				FILE_NOTIFY_CHANGE_FILE_NAME | FILE_NOTIFY_CHANGE_DIR_NAME |
-				FILE_NOTIFY_CHANGE_ATTRIBUTES | FILE_NOTIFY_CHANGE_SIZE |
-				FILE_NOTIFY_CHANGE_LAST_WRITE | FILE_NOTIFY_CHANGE_SECURITY);
-
-		free(utf16_cwd);
-
-		if(view->dir_watcher == NULL || view->dir_watcher == INVALID_HANDLE_VALUE)
-		{
-			log_msg("ha%s", "d");
-		}
-	}
-
-	if(WaitForSingleObject(view->dir_watcher, 0) == WAIT_OBJECT_0)
-	{
-		FindNextChangeNotification(view->dir_watcher);
-		return 1;
-	}
-
-	if(win_get_dir_mtime(view->curr_dir, &ft) != 0)
-	{
-		return -1;
-	}
-
-	r = CompareFileTime(&view->dir_mtime, &ft);
-	view->dir_mtime = ft;
-
-	return r != 0;
-}
-
-/* Gets last directory modification time.  Returns non-zero on error, otherwise
- * zero is returned. */
-static int
-win_get_dir_mtime(const char dir_path[], FILETIME *ft)
-{
-	char selfref_path[PATH_MAX];
-	wchar_t *utf16_path;
-	HANDLE hfile;
-	int r;
-
-	snprintf(selfref_path, sizeof(selfref_path), "%s/.", dir_path);
-
-	utf16_path = utf8_to_utf16(selfref_path);
-	hfile = CreateFileW(utf16_path, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
-			OPEN_EXISTING, FILE_FLAG_BACKUP_SEMANTICS, NULL);
-	free(utf16_path);
-
-	if(hfile == INVALID_HANDLE_VALUE)
-	{
-		return 1;
-	}
-
-	r = !GetFileTime(hfile, NULL, NULL, ft);
-	CloseHandle(hfile);
-	return r;
-}
-
 int
 get_mount_point(const char path[], size_t buf_len, char buf[])
 {
@@ -678,23 +549,6 @@ display_help(const char cmd[])
 	update_screen(UT_REDRAW);
 }
 
-int
-update_dir_mtime(FileView *view)
-{
-	if(win_get_dir_mtime(view->curr_dir, &view->dir_mtime) != 0)
-	{
-		return -1;
-	}
-
-	/* Skip all directory change events accumulated so far. */
-	while(win_check_dir_changed(view) > 0)
-	{
-		/* Do nothing. */
-	}
-
-	return 0;
-}
-
 void
 wait_for_signal(void)
 {
@@ -730,7 +584,7 @@ get_gid_string(const dir_entry_t *entry, int as_num, size_t buf_len, char buf[])
 }
 
 FILE *
-reopen_terminal(void)
+reopen_term_stdout(void)
 {
 	int outfd;
 	FILE *fp;
@@ -758,8 +612,8 @@ reopen_terminal(void)
 	sec_attr.lpSecurityDescriptor = NULL;
 
 	handle_out = CreateFileW(L"CONOUT$", GENERIC_READ | GENERIC_WRITE,
-			FILE_SHARE_READ | FILE_SHARE_WRITE, &sec_attr, 0, 0, 0);
-	if(handle_out == NULL)
+			FILE_SHARE_READ | FILE_SHARE_WRITE, &sec_attr, OPEN_EXISTING, 0, 0);
+	if(handle_out == INVALID_HANDLE_VALUE)
 	{
 		fclose(fp);
 		fprintf(stderr, "Failed to open CONOUT$.");
@@ -776,6 +630,35 @@ reopen_terminal(void)
 	return fp;
 }
 
+int
+reopen_term_stdin(void)
+{
+	HANDLE handle_in;
+	SECURITY_ATTRIBUTES sec_attr;
+
+	/* Share this file handle with child processes so that could use standard
+	 * output. */
+	sec_attr.nLength = sizeof(SECURITY_ATTRIBUTES);
+	sec_attr.bInheritHandle = TRUE;
+	sec_attr.lpSecurityDescriptor = NULL;
+
+	handle_in = CreateFileW(L"CONIN$", GENERIC_READ | GENERIC_WRITE,
+			FILE_SHARE_READ | FILE_SHARE_WRITE, &sec_attr, 0, 0, 0);
+	if(handle_in == INVALID_HANDLE_VALUE)
+	{
+		fprintf(stderr, "Failed to open CONIN$.");
+		return 1;
+	}
+
+	if(SetStdHandle(STD_INPUT_HANDLE, handle_in) == FALSE)
+	{
+		fprintf(stderr, "Failed to set CONIN$ as standard input.");
+		return 1;
+	}
+
+	return 0;
+}
+
 FILE *
 read_cmd_output(const char cmd[])
 {
@@ -836,5 +719,83 @@ use_info_prog_internal(const char cmd[], int out_pipe[2])
 	return (retcode == 0) ? NULL : _fdopen(out_pipe[0], "r");
 }
 
+FILE *
+win_tmpfile()
+{
+	char dir[PATH_MAX];
+	char file[PATH_MAX];
+	HANDLE h;
+	int fd;
+	FILE *f;
+
+	if(GetTempPathA(sizeof(dir), dir) == 0)
+	{
+		return NULL;
+	}
+
+	if(GetTempFileNameA(dir, "dir-view", 0U, file) == 0)
+	{
+		return NULL;
+	}
+
+	h = CreateFileA(file, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING,
+			FILE_FLAG_DELETE_ON_CLOSE, NULL);
+	if(h == INVALID_HANDLE_VALUE)
+	{
+		return NULL;
+	}
+
+	fd = _open_osfhandle((intptr_t)h, _O_RDWR);
+	if(fd == -1)
+	{
+		CloseHandle(h);
+		return NULL;
+	}
+
+	f = fdopen(fd, "r+");
+	if(f == NULL)
+	{
+		close(fd);
+	}
+
+	return f;
+}
+
+void
+clone_timestamps(const char path[], const char from[], const struct stat *st)
+{
+	wchar_t *const utf16_path = utf8_to_utf16(path);
+	wchar_t *const utf16_from = utf8_to_utf16(from);
+
+	HANDLE hfrom = CreateFileW(utf16_from, GENERIC_READ,
+			FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
+			FILE_ATTRIBUTE_NORMAL, NULL);
+	HANDLE hto = CreateFileW(utf16_path, GENERIC_WRITE,
+			FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
+			FILE_ATTRIBUTE_NORMAL, NULL);
+
+	if(hto != INVALID_HANDLE_VALUE && hfrom != INVALID_HANDLE_VALUE)
+	{
+		FILETIME creationTime, lastAccessTime, lastWriteTime;
+
+		if(GetFileTime(hfrom, &creationTime, &lastAccessTime,
+					&lastWriteTime) != FALSE)
+		{
+			SetFileTime(hto, &creationTime, &lastAccessTime, &lastWriteTime);
+		}
+	}
+
+	if(hto != INVALID_HANDLE_VALUE)
+	{
+		CloseHandle(hto);
+	}
+	if(hfrom != INVALID_HANDLE_VALUE)
+	{
+		CloseHandle(hfrom);
+	}
+	free(utf16_from);
+	free(utf16_path);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
