@@ -26,63 +26,68 @@
 
 #include <curses.h>
 
-#include <unistd.h> /* getcwd() */
+#include <unistd.h>
 
 #include <errno.h> /* errno */
-#include <locale.h> /* setlocale */
+#include <locale.h> /* setlocale() LC_ALL */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* fprintf() fputs() puts() snprintf() */
-#include <stdlib.h> /* EXIT_FAILURE EXIT_SUCCESS exit() malloc() system() */
+#include <stdlib.h> /* EXIT_FAILURE EXIT_SUCCESS exit() srand() system() */
 #include <string.h>
+#include <time.h> /* time() */
 
 #include "cfg/config.h"
 #include "cfg/info.h"
+#include "engine/autocmds.h"
+#include "compat/fs_limits.h"
 #include "engine/cmds.h"
 #include "engine/keys.h"
 #include "engine/mode.h"
 #include "engine/options.h"
 #include "engine/variables.h"
+#include "int/fuse.h"
+#include "int/path_env.h"
+#include "int/term_title.h"
+#include "int/vim.h"
 #include "modes/dialogs/msg_dialog.h"
+#include "modes/cmdline.h"
 #include "modes/modes.h"
 #include "modes/view.h"
 #include "ui/cancellation.h"
+#include "ui/color_manager.h"
+#include "ui/color_scheme.h"
+#include "ui/quickview.h"
 #include "ui/statusbar.h"
 #include "ui/ui.h"
 #include "utils/fs.h"
-#include "utils/fs_limits.h"
 #include "utils/log.h"
 #include "utils/macros.h"
 #include "utils/path.h"
 #include "utils/str.h"
+#include "utils/string_array.h"
 #include "utils/utils.h"
 #include "args.h"
 #include "background.h"
-#include "bookmarks.h"
+#include "bmarks.h"
 #include "bracket_notation.h"
 #include "builtin_functions.h"
-#include "color_manager.h"
-#include "color_scheme.h"
-#include "commands.h"
-#include "commands_completion.h"
+#include "cmd_completion.h"
+#include "cmd_core.h"
 #include "dir_stack.h"
 #include "event_loop.h"
 #include "filelist.h"
 #include "fileops.h"
 #include "filetype.h"
-#include "fuse.h"
 #include "ipc.h"
+#include "marks.h"
 #include "ops.h"
 #include "opt_handlers.h"
-#include "path_env.h"
-#include "quickview.h"
 #include "registers.h"
 #include "running.h"
 #include "signals.h"
 #include "status.h"
-#include "term_title.h"
 #include "trash.h"
 #include "undo.h"
-#include "vim.h"
 
 static int pair_in_use(short int pair);
 static void move_pair(short int from, short int to);
@@ -94,8 +99,6 @@ static void check_path_for_file(FileView *view, const char path[], int handle);
 static int need_to_switch_active_pane(const char lwin_path[],
 		const char rwin_path[]);
 static void load_scheme(void);
-static void convert_configs(void);
-static int run_converter(int vifm_like_mode);
 static void exec_startup_commands(const args_t *args);
 static void _gnuc_noreturn vifm_leave(int exit_code, int cquit);
 
@@ -110,34 +113,43 @@ main(int argc, char *argv[])
 	static const int quit = 0;
 
 	char dir[PATH_MAX];
-	int old_config;
+	char **files = NULL;
+	int nfiles = 0;
 
-	if(getcwd(dir, sizeof(dir)) == NULL)
+	if(get_cwd(dir, sizeof(dir)) == NULL)
 	{
 		perror("getcwd");
 		return -1;
 	}
-#ifdef _WIN32
-	to_forward_slash(dir);
-#endif
 
 	(void)vifm_chdir(dir);
 	args_parse(&vifm_args, argc, argv, dir);
 	args_process(&vifm_args, 1);
 
+	if(strcmp(vifm_args.lwin_path, "-") == 0 ||
+			strcmp(vifm_args.rwin_path, "-") == 0)
+	{
+		files = read_stream_lines(stdin, &nfiles, 1);
+		if(reopen_term_stdin() != 0)
+		{
+			return EXIT_FAILURE;
+		}
+	}
+
 	(void)setlocale(LC_ALL, "");
+	srand(time(NULL));
 
 	cfg_init();
 
 	if(vifm_args.logging)
 	{
-		init_logger(1);
+		init_logger(1, vifm_args.startup_log_path);
 	}
 
 	init_filelists();
-	init_registers();
+	regs_init();
 	cfg_discover_paths();
-	reinit_logger();
+	reinit_logger(cfg.log_file);
 
 	/* Commands module also initializes bracket notation and variables. */
 	init_commands();
@@ -159,18 +171,19 @@ main(int argc, char *argv[])
 
 	init_option_handlers();
 
-	old_config = cfg_has_old_format();
-	if(!old_config && !vifm_args.no_configs)
+	if(!vifm_args.no_configs)
 	{
+		/* vifminfo must be processed this early so that it can restore last visited
+		 * directory. */
 		read_info_file(0);
 	}
 
-	ipc_init(&parse_received_arguments);
+	ipc_init(vifm_args.server_name, &parse_received_arguments);
 	args_process(&vifm_args, 0);
 
 	init_background();
 
-	init_fileops();
+	init_fileops(&enter_prompt_mode, &prompt_msg_custom);
 
 	set_view_path(&lwin, vifm_args.lwin_path);
 	set_view_path(&rwin, vifm_args.rwin_path);
@@ -190,7 +203,7 @@ main(int argc, char *argv[])
 	}
 
 	/* Prepare terminal for further operations. */
-	curr_stats.original_stdout = reopen_terminal();
+	curr_stats.original_stdout = reopen_term_stdout();
 	if(curr_stats.original_stdout == NULL)
 	{
 		return -1;
@@ -216,38 +229,30 @@ main(int argc, char *argv[])
 	init_modes();
 	init_undo_list(&undo_perform_func, NULL, &ui_cancellation_requested,
 			&cfg.undo_levels);
-	load_local_options(curr_view);
+	load_view_options(curr_view);
 
 	curr_stats.load_stage = 1;
 
-	if(!old_config && !vifm_args.no_configs)
+	if(!vifm_args.no_configs)
 	{
 		load_scheme();
 		cfg_load();
+
+		if(strcmp(vifm_args.lwin_path, "-") == 0)
+		{
+			flist_set(&lwin, "-", dir, files, nfiles);
+		}
+		else if(strcmp(vifm_args.rwin_path, "-") == 0)
+		{
+			flist_set(&rwin, "-", dir, files, nfiles);
+		}
 	}
 	/* Load colors in any case to load color pairs. */
-	load_color_scheme_colors();
+	cs_load_pairs();
 
-	write_color_scheme_file();
+	cs_write();
 	setup_signals();
 
-	if(old_config && !vifm_args.no_configs)
-	{
-		convert_configs();
-
-		curr_stats.load_stage = 0;
-		read_info_file(0);
-		curr_stats.load_stage = 1;
-
-		set_view_path(&lwin, vifm_args.lwin_path);
-		set_view_path(&rwin, vifm_args.rwin_path);
-
-		load_initial_directory(&lwin, dir);
-		load_initial_directory(&rwin, dir);
-
-		cfg_load();
-	}
-
 	/* Ensure trash directories exist, it might not have been called during
 	 * configuration file sourcing if there is no `set trashdir=...` command. */
 	(void)set_trash_dir(cfg.trash_dir);
@@ -257,10 +262,6 @@ main(int argc, char *argv[])
 
 	curr_stats.load_stage = 2;
 
-	exec_startup_commands(&vifm_args);
-	update_screen(UT_FULL);
-	modes_update();
-
 	/* Update histories of the views to ensure that their current directories,
 	 * which might have been set using command-line parameters, are stored in the
 	 * history.  This is not done automatically as history manipulation should be
@@ -269,6 +270,15 @@ main(int argc, char *argv[])
 	save_view_history(&lwin, NULL, NULL, -1);
 	save_view_history(&rwin, NULL, NULL, -1);
 
+	/* Trigger auto-commands for initial directories. */
+	vle_aucmd_execute("DirEnter", lwin.curr_dir, &lwin);
+	vle_aucmd_execute("DirEnter", rwin.curr_dir, &rwin);
+
+	exec_startup_commands(&vifm_args);
+
+	update_screen(UT_FULL);
+	modes_update();
+
 	curr_stats.load_stage = 3;
 
 	event_loop(&quit);
@@ -337,6 +347,7 @@ parse_received_arguments(char *argv[])
 	}
 
 	(void)vifm_chdir(argv[0]);
+	opterr = 0;
 	args_parse(&args, argc, argv, argv[0]);
 	args_process(&args, 0);
 
@@ -405,7 +416,7 @@ remote_cd(FileView *view, const char *path, int handle)
 static void
 check_path_for_file(FileView *view, const char path[], int handle)
 {
-	if(path[0] == '\0' || is_dir(path))
+	if(path[0] == '\0' || is_dir(path) || strcmp(path, "-") == 0)
 	{
 		return;
 	}
@@ -432,103 +443,19 @@ need_to_switch_active_pane(const char lwin_path[], const char rwin_path[])
 	    && curr_view != &lwin;
 }
 
-/* Loads color scheme.  Converts old format to the new one if needed.
- * Terminates application with error message on error. */
+/* Loads color scheme.  Converts old format to the new one if needed. */
 static void
 load_scheme(void)
 {
-	if(cfg_has_old_color_schemes())
+	if(cs_have_no_extensions())
 	{
-		const int err = run_converter(2);
-		if(err != 0)
-		{
-			endwin();
-			fputs("Problems with running vifmrc-converter\n", stderr);
-			exit(err);
-		}
-	}
-	else if(cs_have_no_extensions())
-	{
-		const int err = run_converter(3);
-		if(err != 0)
-		{
-			endwin();
-			fputs("Problems with running vifmrc-converter\n", stderr);
-			exit(err);
-		}
-	}
-	if(color_scheme_exists(curr_stats.color_scheme))
-	{
-		load_primary_color_scheme(curr_stats.color_scheme);
-	}
-}
-
-/* Converts old versions of configuration files to new ones.  Terminates
- * application with error message on error or when user chooses to do not update
- * anything. */
-static void
-convert_configs(void)
-{
-	int vifm_like_mode;
-	int err;
-
-	if(!prompt_msg("Configuration update", "Your vifmrc will be "
-			"upgraded to a new format.  Your current configuration will be copied "
-			"before performing any changes, but if you don't want to take the risk "
-			"and would like to make one more copy say No to exit vifm.  Continue?"))
-	{
-#ifdef _WIN32
-		system("cls");
-#endif
-		endwin();
-		exit(0);
+		cs_rename_all();
 	}
 
-	vifm_like_mode = !prompt_msg("Configuration update", "This version of "
-			"vifm is able to save changes in the configuration files automatically "
-			"when quitting, as it was possible in older versions.  It is from now "
-			"on recommended though, to save permanent changes manually in the "
-			"configuration file as it is done in vi/vim.  Do you want vifm to "
-			"behave like vi/vim?");
-
-	err = run_converter(vifm_like_mode);
-	if(err != 0)
+	if(cs_exists(curr_stats.color_scheme))
 	{
-		endwin();
-		fputs("Problems with running vifmrc-converter\n", stderr);
-		exit(err);
+		cs_load_primary(curr_stats.color_scheme);
 	}
-
-	show_error_msg("Configuration update", "Your vifmrc has been upgraded to "
-			"new format, you can find its old version in " CONF_DIR "/vifmrc.bak.  "
-			"vifm will not write anything to vifmrc, and all variables that are "
-			"saved between runs of vifm are stored in " CONF_DIR "/vifminfo now "
-			"(you can edit it by hand, but do it carefully).  You can control what "
-			"vifm stores in vifminfo with 'vifminfo' option.");
-}
-
-/* Runs vifmrc-converter in mode specified by the vifm_like_mode argument.
- * Returns zero on success, non-zero otherwise. */
-static int
-run_converter(int vifm_like_mode)
-{
-#ifndef _WIN32
-	char cmd[PATH_MAX];
-	snprintf(cmd, sizeof(cmd), "vifmrc-converter %d", vifm_like_mode);
-	return shellout(cmd, -1, 0);
-#else
-	char path[PATH_MAX];
-	char cmd[2*PATH_MAX];
-	int returned_exit_code;
-
-	if(get_exe_dir(path, sizeof(path)) != 0)
-	{
-		return -1;
-	}
-
-	snprintf(cmd, sizeof(cmd), "%s/vifmrc-converter %d", path, vifm_like_mode);
-	return win_exec_cmd(cmd, &returned_exit_code);
-#endif
 }
 
 void
@@ -539,11 +466,14 @@ vifm_restart(void)
 	curr_stats.restart_in_progress = 1;
 
 	/* All user mappings in all modes. */
-	clear_user_keys();
+	vle_keys_user_clear();
 
 	/* User defined commands. */
 	execute_cmd("comclear");
 
+	/* Autocommands. */
+	vle_aucmd_remove(NULL, NULL);
+
 	/* Directory histories. */
 	ui_view_clear_history(&lwin);
 	ui_view_clear_history(&rwin);
@@ -564,7 +494,7 @@ vifm_restart(void)
 	/* Options of other pane. */
 	tmp_view = curr_view;
 	curr_view = other_view;
-	load_local_options(other_view);
+	load_view_options(other_view);
 	reset_options_to_default();
 	curr_view = tmp_view;
 
@@ -578,10 +508,11 @@ vifm_restart(void)
 	clean_stack();
 
 	/* Registers. */
-	clear_registers();
+	regs_reset();
 
-	/* Clear all bookmarks. */
-	clear_all_bookmarks();
+	/* Clear all marks and bookmarks. */
+	clear_all_marks();
+	bmarks_clear();
 
 	/* Reset variables. */
 	clear_variables();
@@ -596,24 +527,28 @@ vifm_restart(void)
 
 	/* Color schemes. */
 	if(stroscmp(curr_stats.color_scheme, DEF_CS_NAME) != 0 &&
-			color_scheme_exists(curr_stats.color_scheme))
+			cs_exists(curr_stats.color_scheme))
 	{
-		if(load_primary_color_scheme(curr_stats.color_scheme) != 0)
+		if(cs_load_primary(curr_stats.color_scheme) != 0)
 		{
-			load_def_scheme();
+			cs_load_defaults();
 		}
 	}
 	else
 	{
-		load_def_scheme();
+		cs_load_defaults();
 	}
-	load_color_scheme_colors();
+	cs_load_pairs();
 
 	cfg_load();
 	exec_startup_commands(&vifm_args);
 
 	curr_stats.restart_in_progress = 0;
 
+	/* Trigger auto-commands for initial directories. */
+	vle_aucmd_execute("DirEnter", lwin.curr_dir, &lwin);
+	vle_aucmd_execute("DirEnter", rwin.curr_dir, &rwin);
+
 	update_screen(UT_REDRAW);
 }
 
@@ -697,7 +632,7 @@ vifm_leave(int exit_code, int cquit)
 		exit_code = EXIT_FAILURE;
 	}
 
-	set_term_title(NULL);
+	term_title_update(NULL);
 	exit(exit_code);
 }
 
@@ -705,11 +640,21 @@ void _gnuc_noreturn
 vifm_finish(const char message[])
 {
 	endwin();
-	write_info_file();
+
+	/* Update vifminfo only if we were able to startup, otherwise we can end up
+	 * writing from some intermediate half-initialized state.  One particular
+	 * case: after vifminfo read, but before configuration is processed, as a
+	 * result we write very little information to vifminfo file according to
+	 * default value of 'vifminfo' option. */
+	if(curr_stats.load_stage == 3)
+	{
+		write_info_file();
+	}
+
 	fprintf(stderr, "%s\n", message);
 	LOG_ERROR_MSG("Finishing: %s", message);
 	exit(EXIT_FAILURE);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
