@@ -4,8 +4,8 @@
 
 #include <limits.h>
 
-#include "../../src/colors.h"
-#include "../../src/color_manager.h"
+#include "../../src/ui/colors.h"
+#include "../../src/ui/color_manager.h"
 
 #include "test.h"
 
@@ -62,4 +62,4 @@ TEST(negative_fg_and_or_bg)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
