@@ -4,17 +4,18 @@
 #include <stdio.h> /* snprintf() */
 #include <string.h>
 
-#include "../../src/column_view.h"
+#include "../../src/ui/column_view.h"
 #include "test.h"
 
 static void column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset);
+		size_t offset, AlignType align);
 static void column1_func(int id, const void *data, size_t buf_len, char *buf);
 
 static const size_t MAX_WIDTH = 80;
 
 static size_t print_offset;
 static char print_buffer[800 + 1];
+static AlignType last_align;
 
 SETUP()
 {
@@ -30,8 +31,12 @@ TEARDOWN()
 
 static void
 column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset)
+		size_t offset, AlignType align)
 {
+	if(column_id == COL1_ID)
+	{
+		last_align = align;
+	}
 	print_offset = offset;
 	strncpy(print_buffer + offset, buf, strlen(buf));
 }
@@ -113,7 +118,6 @@ TEST(truncation_on_right_align)
 	columns_add_column(cols, column_info);
 
 	memset(print_buffer, '\0', MAX_WIDTH);
-	print_buffer[1] = '\0';
 	columns_format_line(cols, NULL, 1);
 
 	columns_free(cols);
@@ -121,5 +125,49 @@ TEST(truncation_on_right_align)
 	assert_string_equal(expected, print_buffer);
 }
 
+TEST(dyn_align)
+{
+	static column_info_t column_info = {
+		.column_id = COL1_ID, .full_width = 0UL, .text_width = 0UL,
+		.align = AT_DYN,      .sizing = ST_AUTO, .cropping = CT_TRUNCATE,
+	};
+
+	columns_t cols = columns_create();
+	col1_next = column1_func2;
+	columns_add_column(cols, column_info);
+
+	memset(print_buffer, '\0', MAX_WIDTH);
+	columns_format_line(cols, NULL, 8);
+	assert_string_equal("abcdefg ", print_buffer);
+	assert_int_equal(AT_LEFT, last_align);
+
+	memset(print_buffer, '\0', MAX_WIDTH);
+	columns_format_line(cols, NULL, 7);
+	assert_string_equal("abcdefg", print_buffer);
+	assert_int_equal(AT_LEFT, last_align);
+
+	memset(print_buffer, '\0', MAX_WIDTH);
+	columns_format_line(cols, NULL, 6);
+	assert_string_equal("bcdefg", print_buffer);
+	assert_int_equal(AT_RIGHT, last_align);
+
+	memset(print_buffer, '\0', MAX_WIDTH);
+	columns_format_line(cols, NULL, 4);
+	assert_string_equal("defg", print_buffer);
+	assert_int_equal(AT_RIGHT, last_align);
+
+	memset(print_buffer, '\0', MAX_WIDTH);
+	columns_format_line(cols, NULL, 2);
+	assert_string_equal("fg", print_buffer);
+	assert_int_equal(AT_RIGHT, last_align);
+
+	memset(print_buffer, '\0', MAX_WIDTH);
+	columns_format_line(cols, NULL, 1);
+	assert_string_equal("g", print_buffer);
+	assert_int_equal(AT_RIGHT, last_align);
+
+	columns_free(cols);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
