@@ -2,12 +2,12 @@
 
 #include <stddef.h> /* NULL size_t */
 
-#include "../../src/column_view.h"
+#include "../../src/ui/column_view.h"
 
 #include "test.h"
 
 static void column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset);
+		size_t offset, AlignType align);
 static void columns_func(int id, const void *data, size_t buf_len, char *buf);
 
 static const size_t MAX_WIDTH = 80;
@@ -51,7 +51,7 @@ TEARDOWN()
 
 static void
 column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset)
+		size_t offset, AlignType align)
 {
 	++print_counter;
 }
@@ -67,6 +67,7 @@ columns_func(int id, const void *data, size_t buf_len, char buf[])
 	{
 		++column2_counter;
 	}
+	buf[0] = '\0';
 }
 
 TEST(no_columns_one_print_callback_after_creation)
@@ -112,4 +113,4 @@ TEST(number_of_calls_to_print_function)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
