@@ -2,12 +2,12 @@
 
 #include <stddef.h> /* NULL size_t */
 
-#include "../../src/column_view.h"
+#include "../../src/ui/column_view.h"
 
 #include "test.h"
 
 static void print_not_less_than_zero(const void *data, int column_id,
-		const char buf[], size_t offset);
+		const char buf[], size_t offset, AlignType align);
 static void column12_func(int id, const void *data, size_t buf_len, char buf[]);
 
 static const size_t MAX_WIDTH = 80;
@@ -43,7 +43,7 @@ TEARDOWN()
 
 static void
 print_not_less_than_zero(const void *data, int column_id, const char buf[],
-		size_t offset)
+		size_t offset, AlignType align)
 {
 	assert_true(offset <= MAX_WIDTH);
 }
@@ -51,6 +51,7 @@ print_not_less_than_zero(const void *data, int column_id, const char buf[],
 static void
 column12_func(int id, const void *data, size_t buf_len, char buf[])
 {
+	buf[0] = '\0';
 }
 
 TEST(cant_add_columns_with_same_id)
@@ -81,4 +82,4 @@ TEST(add_duplicate_columns_ok)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
