@@ -2,7 +2,7 @@
 
 #include <stddef.h> /* NULL size_t */
 
-#include "../../src/column_view.h"
+#include "../../src/ui/column_view.h"
 
 #include "test.h"
 
@@ -30,7 +30,7 @@ column_line_print(const void *data, int column_id, const char buf[],
 		size_t offset, AlignType align, const char full_column[])
 {
 	assert_non_null(print_next);
-	print_next(data, column_id, buf, offset);
+	print_next(data, column_id, buf, offset, align);
 }
 
 static void
@@ -48,4 +48,4 @@ column2_func(int id, const void *data, size_t buf_len, char *buf)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
