@@ -6,12 +6,12 @@
 
 #include "../../src/utils/utf8.h"
 #include "../../src/utils/utils.h"
-#include "../../src/column_view.h"
+#include "../../src/ui/column_view.h"
 
 #include "test.h"
 
 static void column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset);
+		size_t offset, AlignType align);
 static void column1_func(int id, const void *data, size_t buf_len, char buf[]);
 static void column2_func(int id, const void *data, size_t buf_len, char buf[]);
 static int locale_works(void);
@@ -48,10 +48,10 @@ TEARDOWN()
 
 static void
 column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset)
+		size_t offset, AlignType align)
 {
-	strncpy(print_buffer + get_normal_utf8_string_widthn(print_buffer, offset),
-			buf, strlen(buf));
+	strncpy(print_buffer +
+			utf8_nstrsnlen(print_buffer, offset), buf, strlen(buf));
 }
 
 static void
@@ -217,4 +217,4 @@ locale_works(void)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
