@@ -4,11 +4,11 @@
 #include <stdio.h> /* snprintf() */
 #include <string.h>
 
-#include "../../src/column_view.h"
+#include "../../src/ui/column_view.h"
 #include "test.h"
 
 static void column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset);
+		size_t offset, AlignType align);
 static void column1_func(int id, const void *data, size_t buf_len, char buf[]);
 static void column2_func(int id, const void *data, size_t buf_len, char buf[]);
 
@@ -31,7 +31,7 @@ TEARDOWN()
 
 static void
 column_line_print(const void *data, int column_id, const char buf[],
-		size_t offset)
+		size_t offset, AlignType align)
 {
 	strncpy(print_buffer + offset, buf, strlen(buf));
 }
@@ -178,4 +178,4 @@ TEST(filling)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
