@@ -21,11 +21,18 @@ SETUP()
 
 	cfg.use_system_calls = 1;
 	cfg.slow_fs_list = strdup("");
+	cfg.delete_prg = strdup("");
+
+	lwin.list_rows = 0;
+	lwin.dir_entry = NULL;
+	rwin.list_rows = 0;
+	rwin.dir_entry = NULL;
 }
 
 TEARDOWN()
 {
 	free(cfg.slow_fs_list);
+	free(cfg.delete_prg);
 
 	reset_undo_list();
 }
@@ -49,4 +56,4 @@ op_avail(OPS op)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
