@@ -24,7 +24,7 @@ TEST(empty_files_are_ok)
 {
 	ioeta_estim_t *const estim = ioeta_alloc(NULL);
 
-	ioeta_calculate(estim, "test-data/existing-files", 0);
+	ioeta_calculate(estim, TEST_DATA_PATH "/existing-files", 0);
 
 	assert_int_equal(3, estim->total_items);
 	assert_int_equal(0, estim->current_item);
@@ -38,7 +38,7 @@ TEST(non_empty_files_are_ok)
 {
 	ioeta_estim_t *const estim = ioeta_alloc(NULL);
 
-	ioeta_calculate(estim, "test-data/various-sizes", 0);
+	ioeta_calculate(estim, TEST_DATA_PATH "/various-sizes", 0);
 
 	assert_int_equal(7, estim->total_items);
 	assert_int_equal(0, estim->current_item);
@@ -52,7 +52,7 @@ TEST(shallow_estimation_does_not_recur)
 {
 	ioeta_estim_t *const estim = ioeta_alloc(NULL);
 
-	ioeta_calculate(estim, "test-data/various-sizes", 1);
+	ioeta_calculate(estim, TEST_DATA_PATH "/various-sizes", 1);
 
 	assert_int_equal(1, estim->total_items);
 	assert_int_equal(0, estim->current_item);
@@ -69,15 +69,14 @@ TEST(symlink_calculated_as_zero_bytes)
 	ioeta_estim_t *const estim = ioeta_alloc(NULL);
 
 	{
-		io_args_t args =
-		{
-			.arg1.path = "test-data/existing-files",
-			.arg2.target = "link",
+		io_args_t args = {
+			.arg1.path = TEST_DATA_PATH "/existing-files",
+			.arg2.target = SANDBOX_PATH "/link",
 		};
 		assert_int_equal(0, iop_ln(&args));
 	}
 
-	ioeta_calculate(estim, "link", 0);
+	ioeta_calculate(estim, SANDBOX_PATH "/link", 0);
 
 	assert_int_equal(1, estim->total_items);
 	assert_int_equal(0, estim->current_item);
@@ -85,9 +84,8 @@ TEST(symlink_calculated_as_zero_bytes)
 	assert_int_equal(0, estim->current_byte);
 
 	{
-		io_args_t args =
-		{
-			.arg1.path = "link",
+		io_args_t args = {
+			.arg1.path = SANDBOX_PATH "/link",
 		};
 		assert_int_equal(0, iop_rmfile(&args));
 	}
@@ -98,4 +96,4 @@ TEST(symlink_calculated_as_zero_bytes)
 #endif
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
