@@ -18,8 +18,8 @@ TEST(file_is_copied)
 {
 	{
 		io_args_t args = {
-			.arg1.src = "../read/binary-data",
-			.arg2.dst = "binary-data",
+			.arg1.src = TEST_DATA_PATH "/read/binary-data",
+			.arg2.dst = SANDBOX_PATH "/binary-data",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -29,7 +29,7 @@ TEST(file_is_copied)
 
 	{
 		io_args_t args = {
-			.arg1.path = "binary-data",
+			.arg1.path = SANDBOX_PATH "/binary-data",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -40,12 +40,12 @@ TEST(file_is_copied)
 
 TEST(empty_directory_is_copied)
 {
-	create_empty_dir("empty-dir");
+	create_empty_dir(SANDBOX_PATH "/empty-dir");
 
 	{
 		io_args_t args = {
-			.arg1.src = "empty-dir",
-			.arg2.dst = "empty-dir-copy",
+			.arg1.src = SANDBOX_PATH "/empty-dir",
+			.arg2.dst = SANDBOX_PATH "/empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -55,7 +55,7 @@ TEST(empty_directory_is_copied)
 
 	{
 		io_args_t args = {
-			.arg1.path = "empty-dir",
+			.arg1.path = SANDBOX_PATH "/empty-dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -65,7 +65,7 @@ TEST(empty_directory_is_copied)
 
 	{
 		io_args_t args = {
-			.arg1.path = "empty-dir-copy",
+			.arg1.path = SANDBOX_PATH "/empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -76,12 +76,12 @@ TEST(empty_directory_is_copied)
 
 TEST(non_empty_directory_is_copied)
 {
-	create_non_empty_dir("non-empty-dir", "a-file");
+	create_non_empty_dir(SANDBOX_PATH "/non-empty-dir", "a-file");
 
 	{
 		io_args_t args = {
-			.arg1.src = "non-empty-dir",
-			.arg2.dst = "non-empty-dir-copy",
+			.arg1.src = SANDBOX_PATH "/non-empty-dir",
+			.arg2.dst = SANDBOX_PATH "/non-empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -89,11 +89,11 @@ TEST(non_empty_directory_is_copied)
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_success(access("non-empty-dir-copy/a-file", F_OK));
+	assert_success(access(SANDBOX_PATH "/non-empty-dir-copy/a-file", F_OK));
 
 	{
 		io_args_t args = {
-			.arg1.path = "non-empty-dir",
+			.arg1.path = SANDBOX_PATH "/non-empty-dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -103,7 +103,7 @@ TEST(non_empty_directory_is_copied)
 
 	{
 		io_args_t args = {
-			.arg1.path = "non-empty-dir-copy",
+			.arg1.path = SANDBOX_PATH "/non-empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -114,12 +114,12 @@ TEST(non_empty_directory_is_copied)
 
 TEST(empty_nested_directory_is_copied)
 {
-	create_empty_nested_dir("non-empty-dir", "empty-nested-dir");
+	create_empty_nested_dir(SANDBOX_PATH "/non-empty-dir", "empty-nested-dir");
 
 	{
 		io_args_t args = {
-			.arg1.src = "non-empty-dir",
-			.arg2.dst = "non-empty-dir-copy",
+			.arg1.src = SANDBOX_PATH "/non-empty-dir",
+			.arg2.dst = SANDBOX_PATH "/non-empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -127,11 +127,12 @@ TEST(empty_nested_directory_is_copied)
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_success(access("non-empty-dir-copy/empty-nested-dir", F_OK));
+	assert_success(access(SANDBOX_PATH "/non-empty-dir-copy/empty-nested-dir",
+				F_OK));
 
 	{
 		io_args_t args = {
-			.arg1.path = "non-empty-dir",
+			.arg1.path = SANDBOX_PATH "/non-empty-dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -141,7 +142,7 @@ TEST(empty_nested_directory_is_copied)
 
 	{
 		io_args_t args = {
-			.arg1.path = "non-empty-dir-copy",
+			.arg1.path = SANDBOX_PATH "/non-empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -152,12 +153,13 @@ TEST(empty_nested_directory_is_copied)
 
 TEST(non_empty_nested_directory_is_copied)
 {
-	create_non_empty_nested_dir("non-empty-dir", "nested-dir", "a-file");
+	create_non_empty_nested_dir(SANDBOX_PATH "/non-empty-dir", "nested-dir",
+			"a-file");
 
 	{
 		io_args_t args = {
-			.arg1.src = "non-empty-dir",
-			.arg2.dst = "non-empty-dir-copy",
+			.arg1.src = SANDBOX_PATH "/non-empty-dir",
+			.arg2.dst = SANDBOX_PATH "/non-empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -165,11 +167,12 @@ TEST(non_empty_nested_directory_is_copied)
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_success(access("non-empty-dir-copy/nested-dir/a-file", F_OK));
+	assert_success(access(SANDBOX_PATH "/non-empty-dir-copy/nested-dir/a-file",
+				F_OK));
 
 	{
 		io_args_t args = {
-			.arg1.path = "non-empty-dir",
+			.arg1.path = SANDBOX_PATH "/non-empty-dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -179,7 +182,7 @@ TEST(non_empty_nested_directory_is_copied)
 
 	{
 		io_args_t args = {
-			.arg1.path = "non-empty-dir-copy",
+			.arg1.path = SANDBOX_PATH "/non-empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -190,12 +193,12 @@ TEST(non_empty_nested_directory_is_copied)
 
 TEST(fails_to_overwrite_file_by_default)
 {
-	create_empty_file("a-file");
+	create_empty_file(SANDBOX_PATH "/a-file");
 
 	{
 		io_args_t args = {
-			.arg1.src = "../read/two-lines",
-			.arg2.dst = "a-file",
+			.arg1.src = TEST_DATA_PATH "/read/two-lines",
+			.arg2.dst = SANDBOX_PATH "/a-file",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -207,7 +210,7 @@ TEST(fails_to_overwrite_file_by_default)
 
 	{
 		io_args_t args = {
-			.arg1.path = "a-file",
+			.arg1.path = SANDBOX_PATH "/a-file",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -218,12 +221,12 @@ TEST(fails_to_overwrite_file_by_default)
 
 TEST(fails_to_overwrite_dir_by_default)
 {
-	create_empty_dir("empty-dir");
+	create_empty_dir(SANDBOX_PATH "/empty-dir");
 
 	{
 		io_args_t args = {
-			.arg1.src = "../read",
-			.arg2.dst = "empty-dir",
+			.arg1.src = TEST_DATA_PATH "/read",
+			.arg2.dst = SANDBOX_PATH "/empty-dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -235,7 +238,7 @@ TEST(fails_to_overwrite_dir_by_default)
 
 	{
 		io_args_t args = {
-			.arg1.path = "empty-dir",
+			.arg1.path = SANDBOX_PATH "/empty-dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -246,12 +249,12 @@ TEST(fails_to_overwrite_dir_by_default)
 
 TEST(overwrites_file_when_asked)
 {
-	create_empty_file("a-file");
+	create_empty_file(SANDBOX_PATH "/a-file");
 
 	{
 		io_args_t args = {
-			.arg1.src = "../read/two-lines",
-			.arg2.dst = "a-file",
+			.arg1.src = TEST_DATA_PATH "/read/two-lines",
+			.arg2.dst = SANDBOX_PATH "/a-file",
 			.arg3.crs = IO_CRS_REPLACE_FILES,
 		};
 		ioe_errlst_init(&args.result.errors);
@@ -262,7 +265,7 @@ TEST(overwrites_file_when_asked)
 
 	{
 		io_args_t args = {
-			.arg1.path = "a-file",
+			.arg1.path = SANDBOX_PATH "/a-file",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -273,12 +276,12 @@ TEST(overwrites_file_when_asked)
 
 TEST(overwrites_dir_when_asked)
 {
-	create_empty_dir("dir");
+	create_empty_dir(SANDBOX_PATH "/dir");
 
 	{
 		io_args_t args = {
-			.arg1.src = "../read",
-			.arg2.dst = "dir",
+			.arg1.src = TEST_DATA_PATH "/read",
+			.arg2.dst = SANDBOX_PATH "/dir",
 			.arg3.crs = IO_CRS_REPLACE_ALL,
 		};
 		ioe_errlst_init(&args.result.errors);
@@ -289,7 +292,7 @@ TEST(overwrites_dir_when_asked)
 
 	{
 		io_args_t args = {
-			.arg1.path = "dir",
+			.arg1.path = SANDBOX_PATH "/dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -299,9 +302,11 @@ TEST(overwrites_dir_when_asked)
 		ioe_errlst_free(&args.result.errors);
 	}
 
+	assert_success(chmod(SANDBOX_PATH "/dir", 0700));
+
 	{
 		io_args_t args = {
-			.arg1.path = "dir",
+			.arg1.path = SANDBOX_PATH "/dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -312,22 +317,16 @@ TEST(overwrites_dir_when_asked)
 
 TEST(directories_can_be_merged)
 {
-	create_empty_dir("first");
+	create_empty_dir(SANDBOX_PATH "/first");
+	create_empty_file(SANDBOX_PATH "/first/first-file");
 
-	assert_success(chdir("first"));
-	create_empty_file("first-file");
-	assert_success(chdir(".."));
-
-	create_empty_dir("second");
-
-	assert_success(chdir("second"));
-	create_empty_file("second-file");
-	assert_success(chdir(".."));
+	create_empty_dir(SANDBOX_PATH "/second");
+	create_empty_file(SANDBOX_PATH "/second/second-file");
 
 	{
 		io_args_t args = {
-			.arg1.src = "first",
-			.arg2.dst = "second",
+			.arg1.src = SANDBOX_PATH "/first",
+			.arg2.dst = SANDBOX_PATH "/second",
 			.arg3.crs = IO_CRS_REPLACE_FILES,
 		};
 		ioe_errlst_init(&args.result.errors);
@@ -336,12 +335,12 @@ TEST(directories_can_be_merged)
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_success(access("second/second-file", F_OK));
-	assert_success(access("second/first-file", F_OK));
+	assert_success(access(SANDBOX_PATH "/second/second-file", F_OK));
+	assert_success(access(SANDBOX_PATH "/second/first-file", F_OK));
 
 	{
 		io_args_t args = {
-			.arg1.path = "first",
+			.arg1.path = SANDBOX_PATH "/first",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -351,7 +350,7 @@ TEST(directories_can_be_merged)
 
 	{
 		io_args_t args = {
-			.arg1.path = "second",
+			.arg1.path = SANDBOX_PATH "/second",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -362,12 +361,12 @@ TEST(directories_can_be_merged)
 
 TEST(fails_to_copy_directory_inside_itself)
 {
-	create_empty_dir("empty-dir");
+	create_empty_dir(SANDBOX_PATH "/empty-dir");
 
 	{
 		io_args_t args = {
-			.arg1.src = "empty-dir",
-			.arg2.dst = "empty-dir/empty-dir-copy",
+			.arg1.src = SANDBOX_PATH "/empty-dir",
+			.arg2.dst = SANDBOX_PATH "/empty-dir/empty-dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -379,7 +378,7 @@ TEST(fails_to_copy_directory_inside_itself)
 
 	{
 		io_args_t args = {
-			.arg1.path = "empty-dir",
+			.arg1.path = SANDBOX_PATH "/empty-dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -395,7 +394,7 @@ TEST(dir_permissions_are_preserved)
 
 	{
 		io_args_t args = {
-			.arg1.path = "dir",
+			.arg1.path = SANDBOX_PATH "/dir",
 			.arg3.mode = 0711,
 		};
 		ioe_errlst_init(&args.result.errors);
@@ -406,8 +405,8 @@ TEST(dir_permissions_are_preserved)
 
 	{
 		io_args_t args = {
-			.arg1.src = "dir",
-			.arg2.dst = "dir-copy",
+			.arg1.src = SANDBOX_PATH "/dir",
+			.arg2.dst = SANDBOX_PATH "/dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -415,13 +414,13 @@ TEST(dir_permissions_are_preserved)
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_success(os_stat("dir", &src));
-	assert_success(os_stat("dir-copy", &dst));
+	assert_success(os_stat(SANDBOX_PATH "/dir", &src));
+	assert_success(os_stat(SANDBOX_PATH "/dir-copy", &dst));
 	assert_int_equal(src.st_mode & 0777, dst.st_mode & 0777);
 
 	{
 		io_args_t args = {
-			.arg1.path = "dir",
+			.arg1.path = SANDBOX_PATH "/dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -431,7 +430,7 @@ TEST(dir_permissions_are_preserved)
 
 	{
 		io_args_t args = {
-			.arg1.path = "dir-copy",
+			.arg1.path = SANDBOX_PATH "/dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -447,7 +446,7 @@ TEST(permissions_are_set_in_correct_order)
 
 	{
 		io_args_t args = {
-			.arg1.path = "dir/nested-dir",
+			.arg1.path = SANDBOX_PATH "/dir/nested-dir",
 			.arg2.process_parents = 1,
 			.arg3.mode = 0600,
 		};
@@ -457,12 +456,12 @@ TEST(permissions_are_set_in_correct_order)
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_success(chmod("dir", 0500));
+	assert_success(chmod(SANDBOX_PATH "/dir", 0500));
 
 	{
 		io_args_t args = {
-			.arg1.src = "dir",
-			.arg2.dst = "dir-copy",
+			.arg1.src = SANDBOX_PATH "/dir",
+			.arg2.dst = SANDBOX_PATH "/dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -470,16 +469,16 @@ TEST(permissions_are_set_in_correct_order)
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_success(os_stat("dir", &src));
-	assert_success(os_stat("dir-copy", &dst));
+	assert_success(os_stat(SANDBOX_PATH "/dir", &src));
+	assert_success(os_stat(SANDBOX_PATH "/dir-copy", &dst));
 	assert_int_equal(src.st_mode & 0777, dst.st_mode & 0777);
 
-	assert_success(chmod("dir", 0700));
-	assert_success(chmod("dir-copy", 0700));
+	assert_success(chmod(SANDBOX_PATH "/dir", 0700));
+	assert_success(chmod(SANDBOX_PATH "/dir-copy", 0700));
 
 	{
 		io_args_t args = {
-			.arg1.path = "dir",
+			.arg1.path = SANDBOX_PATH "/dir",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -489,7 +488,7 @@ TEST(permissions_are_set_in_correct_order)
 
 	{
 		io_args_t args = {
-			.arg1.path = "dir-copy",
+			.arg1.path = SANDBOX_PATH "/dir-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -503,8 +502,8 @@ TEST(symlink_to_file_is_symlink_after_copy, IF(not_windows))
 {
 	{
 		io_args_t args = {
-			.arg1.path = "../read/two-lines",
-			.arg2.target = "sym-link",
+			.arg1.path = TEST_DATA_PATH "/read/two-lines",
+			.arg2.target = SANDBOX_PATH "/sym-link",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -512,12 +511,12 @@ TEST(symlink_to_file_is_symlink_after_copy, IF(not_windows))
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_true(is_symlink("sym-link"));
+	assert_true(is_symlink(SANDBOX_PATH "/sym-link"));
 
 	{
 		io_args_t args = {
-			.arg1.src = "sym-link",
-			.arg2.dst = "sym-link-copy",
+			.arg1.src = SANDBOX_PATH "/sym-link",
+			.arg2.dst = SANDBOX_PATH "/sym-link-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -525,12 +524,12 @@ TEST(symlink_to_file_is_symlink_after_copy, IF(not_windows))
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_true(is_symlink("sym-link"));
-	assert_true(is_symlink("sym-link-copy"));
+	assert_true(is_symlink(SANDBOX_PATH "/sym-link"));
+	assert_true(is_symlink(SANDBOX_PATH "/sym-link-copy"));
 
 	{
 		io_args_t args = {
-			.arg1.path = "sym-link",
+			.arg1.path = SANDBOX_PATH "/sym-link",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -540,7 +539,7 @@ TEST(symlink_to_file_is_symlink_after_copy, IF(not_windows))
 
 	{
 		io_args_t args = {
-			.arg1.path = "sym-link-copy",
+			.arg1.path = SANDBOX_PATH "/sym-link-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -554,8 +553,8 @@ TEST(symlink_to_dir_is_symlink_after_copy, IF(not_windows))
 {
 	{
 		io_args_t args = {
-			.arg1.path = "../read",
-			.arg2.target = "sym-link",
+			.arg1.path = TEST_DATA_PATH "/read",
+			.arg2.target = SANDBOX_PATH "/sym-link",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -563,12 +562,12 @@ TEST(symlink_to_dir_is_symlink_after_copy, IF(not_windows))
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_true(is_symlink("sym-link"));
+	assert_true(is_symlink(SANDBOX_PATH "/sym-link"));
 
 	{
 		io_args_t args = {
-			.arg1.src = "sym-link",
-			.arg2.dst = "sym-link-copy",
+			.arg1.src = SANDBOX_PATH "/sym-link",
+			.arg2.dst = SANDBOX_PATH "/sym-link-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -576,12 +575,12 @@ TEST(symlink_to_dir_is_symlink_after_copy, IF(not_windows))
 		assert_int_equal(0, args.result.errors.error_count);
 	}
 
-	assert_true(is_symlink("sym-link"));
-	assert_true(is_symlink("sym-link-copy"));
+	assert_true(is_symlink(SANDBOX_PATH "/sym-link"));
+	assert_true(is_symlink(SANDBOX_PATH "/sym-link-copy"));
 
 	{
 		io_args_t args = {
-			.arg1.path = "sym-link",
+			.arg1.path = SANDBOX_PATH "/sym-link",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -591,7 +590,7 @@ TEST(symlink_to_dir_is_symlink_after_copy, IF(not_windows))
 
 	{
 		io_args_t args = {
-			.arg1.path = "sym-link-copy",
+			.arg1.path = SANDBOX_PATH "/sym-link-copy",
 		};
 		ioe_errlst_init(&args.result.errors);
 
@@ -607,4 +606,4 @@ not_windows(void)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
