@@ -4,17 +4,20 @@
 #include <string.h> /* strdup() */
 #include <wchar.h> /* wcsdup() wcslen() */
 
+#include "../../src/cfg/config.h"
 #include "../../src/engine/cmds.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/macros.h"
-#include "../../src/commands.h"
+#include "../../src/utils/str.h"
+#include "../../src/cmd_core.h"
 #include "../../src/status.h"
 
 static int builtin_cmd(const cmd_info_t* cmd_info);
 
 static const cmd_add_t commands[] = {
-	{ .name = "builtin", .abbr = NULL, .handler = builtin_cmd, .id = -1,    .range = 0,    .cust_sep = 0,
-		.emark = 1,        .qmark = 0,   .expand = 0,            .regexp = 0, .min_args = 0, .max_args = 0, .bg = 0, },
+	{ .name = "builtin",       .abbr = NULL,  .id = -1,      .descr = "descr",
+	  .flags = HAS_EMARK,
+	  .handler = &builtin_cmd, .min_args = 0, .max_args = 0, },
 };
 
 static int called;
@@ -28,10 +31,16 @@ SETUP()
 	init_commands();
 
 	add_builtin_commands(commands, ARRAY_LEN(commands));
+
+	update_string(&cfg.shell, "");
+	assert_success(init_status(&cfg));
 }
 
 TEARDOWN()
 {
+	assert_success(reset_status(&cfg));
+	update_string(&cfg.shell, NULL);
+
 	reset_cmds();
 }
 
@@ -42,13 +51,13 @@ builtin_cmd(const cmd_info_t* cmd_info)
 	return 0;
 }
 
-TEST(repeat_of_no_command_prints_message)
+TEST(repeat_of_no_command_prints_a_message)
 {
 	called = 0;
 	(void)exec_commands("builtin", &lwin, CIT_COMMAND);
 	assert_int_equal(1, called);
 
-	assert_string_equal(NULL, curr_stats.last_cmdline_command);
+	update_string(&curr_stats.last_cmdline_command, NULL);
 
 	called = 0;
 	assert_int_equal(1, exec_commands("!!", &lwin, CIT_COMMAND));
@@ -61,29 +70,21 @@ TEST(double_emark_repeats_last_command)
 	(void)exec_commands("builtin", &lwin, CIT_COMMAND);
 	assert_int_equal(1, called);
 
-	free(curr_stats.last_cmdline_command);
-	curr_stats.last_cmdline_command = strdup("builtin");
+	update_string(&curr_stats.last_cmdline_command, "builtin");
 
 	called = 0;
 	assert_int_equal(0, exec_commands("!!", &lwin, CIT_COMMAND));
 	assert_int_equal(1, called);
-
-	free(curr_stats.last_cmdline_command);
-	curr_stats.last_cmdline_command = NULL;
 }
 
 TEST(single_emark_without_args_fails)
 {
-	free(curr_stats.last_cmdline_command);
-	curr_stats.last_cmdline_command = strdup("builtin");
+	update_string(&curr_stats.last_cmdline_command, "builtin");
 
 	called = 0;
 	assert_false(exec_commands("!", &lwin, CIT_COMMAND) == 0);
 	assert_int_equal(0, called);
-
-	free(curr_stats.last_cmdline_command);
-	curr_stats.last_cmdline_command = NULL;
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
