@@ -7,6 +7,7 @@
 
 #include "../../src/cfg/config.h"
 #include "../../src/ui/ui.h"
+#include "../../src/utils/dynarray.h"
 #include "../../src/utils/str.h"
 #include "../../src/filelist.h"
 #include "../../src/macros.h"
@@ -25,7 +26,8 @@ setup_lwin(void)
 
 	lwin.list_rows = 4;
 	lwin.list_pos = 2;
-	lwin.dir_entry = calloc(lwin.list_rows, sizeof(*lwin.dir_entry));
+	lwin.dir_entry = dynarray_cextend(NULL,
+			lwin.list_rows*sizeof(*lwin.dir_entry));
 	lwin.dir_entry[0].name = strdup("lfi le0");
 	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[1].name = strdup("lfile1");
@@ -47,7 +49,8 @@ setup_rwin(void)
 
 	rwin.list_rows = 7;
 	rwin.list_pos = 5;
-	rwin.dir_entry = calloc(rwin.list_rows, sizeof(*rwin.dir_entry));
+	rwin.dir_entry = dynarray_cextend(NULL,
+			rwin.list_rows*sizeof(*rwin.dir_entry));
 	rwin.dir_entry[0].name = strdup("rfile0");
 	rwin.dir_entry[0].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[1].name = strdup("rfile1");
@@ -85,11 +88,11 @@ TEARDOWN()
 
 	for(i = 0; i < lwin.list_rows; i++)
 		free(lwin.dir_entry[i].name);
-	free(lwin.dir_entry);
+	dynarray_free(lwin.dir_entry);
 
 	for(i = 0; i < rwin.list_rows; i++)
 		free(rwin.dir_entry[i].name);
-	free(rwin.dir_entry);
+	dynarray_free(rwin.dir_entry);
 }
 
 TEST(b_both_have_selection)
@@ -218,9 +221,9 @@ TEST(r_well_formed)
 {
 	const char *p;
 
-	chdir("test-data/existing-files");
+	assert_success(chdir(TEST_DATA_PATH "/existing-files"));
 
-	init_registers();
+	regs_init();
 
 	p = valid_registers;
 	while(*p != '\0')
@@ -230,9 +233,9 @@ TEST(r_well_formed)
 		const char key = *p++;
 		snprintf(line, sizeof(line), "%%r%c", key);
 
-		append_to_register(key, "a");
-		append_to_register(key, "b");
-		append_to_register(key, "c");
+		regs_append(key, "a");
+		regs_append(key, "b");
+		regs_append(key, "c");
 
 		expanded = expand_macros(line, NULL, NULL, 0);
 		if(key == '_')
@@ -246,23 +249,21 @@ TEST(r_well_formed)
 		free(expanded);
 	}
 
-	clear_registers();
-
-	chdir("../..");
+	regs_reset();
 }
 
 TEST(r_ill_formed)
 {
-	char key;
+	unsigned char key;
 	char expected[] = "a b cx";
 
-	chdir("test-data/existing-files");
+	assert_success(chdir(TEST_DATA_PATH "/existing-files"));
 
-	init_registers();
+	regs_init();
 
-	append_to_register(DEFAULT_REG_NAME, "a");
-	append_to_register(DEFAULT_REG_NAME, "b");
-	append_to_register(DEFAULT_REG_NAME, "c");
+	regs_append(DEFAULT_REG_NAME, "a");
+	regs_append(DEFAULT_REG_NAME, "b");
+	regs_append(DEFAULT_REG_NAME, "c");
 
 	key = '\0';
 	do
@@ -280,9 +281,7 @@ TEST(r_ill_formed)
 	}
 	while(key != '\0');
 
-	clear_registers();
-
-	chdir("../..");
+	regs_reset();
 }
 
 TEST(with_quotes)
@@ -324,19 +323,18 @@ TEST(with_quotes)
 	assert_string_equal(" a m M s ", expanded);
 	free(expanded);
 
-	chdir("test-data/existing-files");
-	init_registers();
+	assert_success(chdir(TEST_DATA_PATH "/existing-files"));
+	regs_init();
 
-	append_to_register(DEFAULT_REG_NAME, "a");
-	append_to_register(DEFAULT_REG_NAME, "b");
-	append_to_register(DEFAULT_REG_NAME, "c");
+	regs_append(DEFAULT_REG_NAME, "a");
+	regs_append(DEFAULT_REG_NAME, "b");
+	regs_append(DEFAULT_REG_NAME, "c");
 
 	expanded = expand_macros("/%\"r ", "", NULL, 1);
 	assert_string_equal("/\"a\" \"b\" \"c\" ", expanded);
 	free(expanded);
 
-	clear_registers();
-	chdir("../..");
+	regs_reset();
 }
 
 TEST(single_percent_sign)
@@ -366,5 +364,92 @@ TEST(empty_line_ok)
 	free(expanded);
 }
 
+TEST(singly_expanded_is_not_escaped)
+{
+	char *expanded;
+	lwin.list_pos = 0;
+	expanded = ma_expand_single("%c");
+	assert_string_equal("lfi le0", expanded);
+	free(expanded);
+}
+
+TEST(singly_expanded_is_not_quoted)
+{
+	char *expanded;
+	lwin.list_pos = 0;
+	expanded = ma_expand_single("%\"c");
+	assert_string_equal("lfi le0", expanded);
+	free(expanded);
+}
+
+TEST(singly_expanded_single_macros)
+{
+	char *expanded;
+	lwin.list_pos = 0;
+	expanded = ma_expand_single("%d");
+	assert_string_equal("/lwin", expanded);
+	free(expanded);
+}
+
+TEST(singly_expanded_multiple_macros_single_file)
+{
+	char *expanded;
+	lwin.dir_entry[2].selected = 0;
+	lwin.selected_files = 1;
+	expanded = ma_expand_single("%f");
+	assert_string_equal("lfi le0", expanded);
+	free(expanded);
+}
+
+TEST(singly_not_expanded_multiple_macros_multiple_files)
+{
+	char *expanded;
+	lwin.list_pos = 0;
+	expanded = ma_expand_single("%f");
+	assert_string_equal("", expanded);
+	free(expanded);
+}
+
+TEST(singly_no_crash_on_wrong_register_name)
+{
+	assert_success(chdir(TEST_DATA_PATH "/spaces-in-names"));
+	regs_init();
+
+	assert_success(regs_append('r', "spaces in the middle"));
+	free(ma_expand_single("%r "));
+
+	regs_reset();
+}
+
+TEST(singly_expanded_single_file_register)
+{
+	char *expanded;
+
+	assert_success(chdir(TEST_DATA_PATH "/spaces-in-names"));
+	regs_init();
+
+	assert_success(regs_append('r', "spaces in the middle"));
+	expanded = ma_expand_single("%rr");
+	assert_string_equal("spaces in the middle", expanded);
+	free(expanded);
+
+	regs_reset();
+}
+
+TEST(singly_not_expanded_multiple_files_register)
+{
+	char *expanded;
+
+	regs_init();
+
+	regs_append('r', "a");
+	regs_append('r', "b");
+	expanded = ma_expand_single("%rr");
+	assert_string_equal("", expanded);
+	free(expanded);
+
+	regs_reset();
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
