@@ -5,6 +5,8 @@
 
 #include "../../src/cfg/config.h"
 #include "../../src/ui/statusline.h"
+#include "../../src/utils/dynarray.h"
+#include "../../src/utils/str.h"
 
 /* Checks that expanded string isn't equal to format string. */
 #define ASSERT_EXPANDED(format) \
@@ -28,11 +30,12 @@
 
 SETUP()
 {
-	cfg.time_format = strdup("");
+	update_string(&cfg.time_format, "+");
 
 	lwin.list_rows = 1;
 	lwin.list_pos = 0;
-	lwin.dir_entry = calloc(lwin.list_rows, sizeof(*lwin.dir_entry));
+	lwin.dir_entry = dynarray_cextend(NULL,
+			lwin.list_rows*sizeof(*lwin.dir_entry));
 	lwin.dir_entry[0].name = strdup("file");
 	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
 
@@ -48,9 +51,9 @@ TEARDOWN()
 	{
 		free(lwin.dir_entry[i].name);
 	}
-	free(lwin.dir_entry);
+	dynarray_free(lwin.dir_entry);
 
-	free(cfg.time_format);
+	update_string(&cfg.time_format, NULL);
 }
 
 TEST(empty_format)
@@ -114,6 +117,11 @@ TEST(L_macro_expanded)
 	ASSERT_EXPANDED("%L");
 }
 
+TEST(z_macro_expanded)
+{
+	ASSERT_EXPANDED("%z");
+}
+
 TEST(dash_macro_expanded)
 {
 	ASSERT_EXPANDED("%-t");
@@ -141,7 +149,7 @@ TEST(percent_macro_expanded)
 
 TEST(wrong_macros_ignored)
 {
-	static const char STATUS_CHARS[] = "tAugsEd-lLS%[]";
+	static const char STATUS_CHARS[] = "tAugsEd-lLS%[]z";
 	int i;
 
 	for(i = 1; i <= 255; ++i)
@@ -156,7 +164,7 @@ TEST(wrong_macros_ignored)
 
 TEST(wrong_macros_with_width_field_ignored)
 {
-	static const char STATUS_CHARS[] = "tAugsEd-lLS%[]";
+	static const char STATUS_CHARS[] = "tAugsEd-lLS%[]z";
 	int i;
 
 	for(i = 1; i <= 255; ++i)
@@ -196,6 +204,7 @@ TEST(nested_optional_non_empty)
 TEST(ignore_mismatched_opening_bracket)
 {
 	ASSERT_EXPANDED_TO("%[", "%[");
+	ASSERT_EXPANDED_TO("%[abcdef", "%[abcdef");
 }
 
 TEST(ignore_mismatched_closing_bracket)
@@ -204,4 +213,4 @@ TEST(ignore_mismatched_closing_bracket)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
