@@ -1,10 +1,13 @@
 #include <stic.h>
 
+#include <unistd.h> /* chdir() */
+
 #include <stdlib.h>
 #include <string.h>
 
 #include "../../src/cfg/config.h"
 #include "../../src/ui/ui.h"
+#include "../../src/utils/dynarray.h"
 #include "../../src/filelist.h"
 #include "../../src/macros.h"
 #include "../../src/registers.h"
@@ -22,7 +25,8 @@ setup_lwin(void)
 
 	lwin.list_rows = 5;
 	lwin.list_pos = 2;
-	lwin.dir_entry = calloc(lwin.list_rows, sizeof(*lwin.dir_entry));
+	lwin.dir_entry = dynarray_cextend(NULL,
+			lwin.list_rows*sizeof(*lwin.dir_entry));
 	lwin.dir_entry[0].name = strdup("lfile0");
 	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[1].name = strdup("lfile1");
@@ -46,7 +50,8 @@ setup_rwin(void)
 
 	rwin.list_rows = 7;
 	rwin.list_pos = 5;
-	rwin.dir_entry = calloc(rwin.list_rows, sizeof(*rwin.dir_entry));
+	rwin.dir_entry = dynarray_cextend(NULL,
+			rwin.list_rows*sizeof(*rwin.dir_entry));
 	rwin.dir_entry[0].name = strdup("rfile0");
 	rwin.dir_entry[0].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[1].name = strdup("rfile1");
@@ -71,14 +76,16 @@ setup_rwin(void)
 static void
 setup_registers(void)
 {
-	init_registers();
-	append_to_register('z', "test-data/existing-files/a");
-	append_to_register('z', "test-data/existing-files/b");
-	append_to_register('z', "test-data/existing-files/c");
+	regs_init();
+	regs_append('z', "existing-files/a");
+	regs_append('z', "existing-files/b");
+	regs_append('z', "existing-files/c");
 }
 
 SETUP()
 {
+	assert_success(chdir(TEST_DATA_PATH));
+
 	setup_lwin();
 	setup_rwin();
 
@@ -96,13 +103,13 @@ TEARDOWN()
 
 	for(i = 0; i < lwin.list_rows; i++)
 		free(lwin.dir_entry[i].name);
-	free(lwin.dir_entry);
+	dynarray_free(lwin.dir_entry);
 
 	for(i = 0; i < rwin.list_rows; i++)
 		free(rwin.dir_entry[i].name);
-	free(rwin.dir_entry);
+	dynarray_free(rwin.dir_entry);
 
-	clear_registers();
+	regs_reset();
 }
 
 TEST(colon_p)
@@ -143,7 +150,7 @@ TEST(colon_p)
 	free(expanded);
 
 	expanded = expand_macros(" cp %rz:p ", "", &flags, 1);
-	assert_string_equal(" cp " SL "lwin" SL "test-data" SL "existing-files" SL "a " SL "lwin" SL "test-data" SL "existing-files" SL "b " SL "lwin" SL "test-data" SL "existing-files" SL "c ", expanded);
+	assert_string_equal(" cp " SL "lwin" SL "existing-files" SL "a " SL "lwin" SL "existing-files" SL "b " SL "lwin" SL "existing-files" SL "c ", expanded);
 	free(expanded);
 }
 
@@ -186,7 +193,7 @@ TEST(colon_p_in_root)
 	free(expanded);
 
 	expanded = expand_macros(" cp %rz:p ", "", &flags, 1);
-	assert_string_equal(" cp " SL "test-data" SL "existing-files" SL "a " SL "test-data" SL "existing-files" SL "b " SL "test-data" SL "existing-files" SL "c ", expanded);
+	assert_string_equal(" cp " SL "existing-files" SL "a " SL "existing-files" SL "b " SL "existing-files" SL "c ", expanded);
 	free(expanded);
 }
 
@@ -241,7 +248,7 @@ TEST(colon_tilde)
 	free(expanded);
 
 	expanded = expand_macros(" cp %rz:~ ", "", &flags, 1);
-	assert_string_equal(" cp test-data" SL "existing-files" SL "a test-data" SL "existing-files" SL "b test-data" SL "existing-files" SL "c ", expanded);
+	assert_string_equal(" cp existing-files" SL "a existing-files" SL "b existing-files" SL "c ", expanded);
 	free(expanded);
 }
 
@@ -301,7 +308,7 @@ TEST(colon_dot)
 	free(expanded);
 
 	expanded = expand_macros(" cp %rz:. ", "", &flags, 1);
-	assert_string_equal(" cp test-data" SL "existing-files" SL "a test-data" SL "existing-files" SL "b test-data" SL "existing-files" SL "c ", expanded);
+	assert_string_equal(" cp existing-files" SL "a existing-files" SL "b existing-files" SL "c ", expanded);
 	free(expanded);
 }
 
@@ -352,7 +359,7 @@ TEST(colon_h)
 	free(expanded);
 
 	expanded = expand_macros(" cp %rz:h ", "", &flags, 1);
-	assert_string_equal(" cp test-data" SL "existing-files test-data" SL "existing-files test-data" SL "existing-files ", expanded);
+	assert_string_equal(" cp existing-files existing-files existing-files ", expanded);
 	free(expanded);
 }
 
@@ -656,4 +663,4 @@ TEST(with_quotes)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
