@@ -4,9 +4,11 @@
 #include <string.h>
 
 #include "../../src/cfg/config.h"
+#include "../../src/int/vim.h"
 #include "../../src/ui/ui.h"
+#include "../../src/utils/dynarray.h"
+#include "../../src/utils/str.h"
 #include "../../src/filelist.h"
-#include "../../src/vim.h"
 
 static void teardown_view(FileView *view);
 
@@ -17,7 +19,8 @@ setup_lwin(void)
 
 	lwin.list_rows = 4;
 	lwin.list_pos = 2;
-	lwin.dir_entry = calloc(lwin.list_rows, sizeof(*lwin.dir_entry));
+	lwin.dir_entry = dynarray_cextend(NULL,
+			lwin.list_rows*sizeof(*lwin.dir_entry));
 	lwin.dir_entry[0].name = strdup("lfile0");
 	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
 	lwin.dir_entry[1].name = strdup("lfile1");
@@ -39,7 +42,8 @@ setup_rwin(void)
 
 	rwin.list_rows = 6;
 	rwin.list_pos = 5;
-	rwin.dir_entry = calloc(rwin.list_rows, sizeof(*rwin.dir_entry));
+	rwin.dir_entry = dynarray_cextend(NULL,
+			rwin.list_rows*sizeof(*rwin.dir_entry));
 	rwin.dir_entry[0].name = strdup("rfile0");
 	rwin.dir_entry[0].origin = &rwin.curr_dir[0];
 	rwin.dir_entry[1].name = strdup("rfile1");
@@ -67,8 +71,8 @@ SETUP()
 	curr_view = &lwin;
 	other_view = &rwin;
 
-	cfg.vi_command = strdup("vim -p");
-	cfg.vi_x_command = strdup("");
+	update_string(&cfg.vi_command, "vim -p");
+	update_string(&cfg.vi_x_command, "");
 }
 
 TEARDOWN()
@@ -76,8 +80,8 @@ TEARDOWN()
 	teardown_view(&lwin);
 	teardown_view(&rwin);
 
-	free(cfg.vi_command);
-	free(cfg.vi_x_command);
+	update_string(&cfg.vi_command, NULL);
+	update_string(&cfg.vi_x_command, NULL);
 }
 
 static void
@@ -88,7 +92,7 @@ teardown_view(FileView *view)
 	{
 		free_dir_entry(view, &view->dir_entry[i]);
 	}
-	free(view->dir_entry);
+	dynarray_free(view->dir_entry);
 	view->list_rows = 0;
 	view->selected_files = 0;
 }
@@ -99,9 +103,13 @@ TEST(selection)
 	int bg;
 
 	cmd = format_edit_selection_cmd(&bg);
+#ifdef _WIN32
+	assert_string_equal("vim -p \"lfile0\" \"lfile2\"", cmd);
+#else
 	assert_string_equal("vim -p lfile0 lfile2", cmd);
+#endif
 	free(cmd);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
