@@ -1,6 +1,6 @@
 #include <stic.h>
 
-#include "../../src/fuse.h"
+#include "../../src/int/fuse.h"
 
 TEST(no_macro_string_unchanged)
 {
@@ -124,4 +124,4 @@ TEST(no_bar_symbol)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
