@@ -5,7 +5,7 @@
 
 #include "../../src/engine/cmds.h"
 #include "../../src/engine/completion.h"
-#include "../../src/commands.h"
+#include "../../src/cmd_core.h"
 
 #include "asserts.h"
 
@@ -18,6 +18,7 @@ TEST(vim_like_completion)
 	ASSERT_NEXT_MATCH("echo");
 	ASSERT_NEXT_MATCH("edit");
 	ASSERT_NEXT_MATCH("else");
+	ASSERT_NEXT_MATCH("elseif");
 	ASSERT_NEXT_MATCH("empty");
 	ASSERT_NEXT_MATCH("endif");
 	ASSERT_NEXT_MATCH("execute");
@@ -38,4 +39,4 @@ TEST(vim_like_completion)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
