@@ -4,8 +4,8 @@
 #include <string.h>
 
 #include "../../src/cfg/config.h"
+#include "../../src/ui/escape.h"
 #include "../../src/utils/utils.h"
-#include "../../src/escape.h"
 
 static int locale_works(void);
 
@@ -110,4 +110,4 @@ locale_works(void)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
