@@ -8,32 +8,36 @@
 
 static void dummy_handler(OPT_OP op, optval_t val);
 
-static const char cpoptions_charset[] = "abc";
-static const char * cpoptions_vals = cpoptions_charset;
-
-static const char * sort_enum[] = {
-	"ext",
-	"name",
-	"gid",
-	"gname",
-	"mode",
-	"uid",
-	"uname",
-	"size",
-	"atime",
-	"ctime",
-	"mtime",
+static const char *cpoptions_vals[][2] = {
+	{ "abc", "descr" },
+	{ "a", "descr" },
+	{ "b", "descr" },
+	{ "c", "descr" },
 };
 
-static const char * vifminfo_set[] = {
-	"options",
-	"filetypes",
-	"commands",
-	"bookmarks",
-	"tui",
-	"dhistory",
-	"state",
-	"cs",
+static const char *sort_enum[][2] = {
+	{ "ext",   "descr" },
+	{ "name",  "descr" },
+	{ "gid",   "descr" },
+	{ "gname", "descr" },
+	{ "mode",  "descr" },
+	{ "uid",   "descr" },
+	{ "uname", "descr" },
+	{ "size",  "descr" },
+	{ "atime", "descr" },
+	{ "ctime", "descr" },
+	{ "mtime", "descr" },
+};
+
+static const char *vifminfo_set[][2] = {
+	{ "options",   "descr" },
+	{ "filetypes", "descr" },
+	{ "commands",  "descr" },
+	{ "bookmarks", "descr" },
+	{ "tui",       "descr" },
+	{ "dhistory",  "descr" },
+	{ "state",     "descr" },
+	{ "cs",        "descr" },
 };
 
 SETUP()
@@ -44,28 +48,42 @@ SETUP()
 	init_options(&option_changed);
 
 	val.str_val = "bc";
-	add_option("cpoptions", "cpo", OPT_CHARSET, ARRAY_LEN(cpoptions_charset) - 1,
-			&cpoptions_vals, dummy_handler, val);
+	add_option("cpoptions", "cpo", "descr", OPT_CHARSET, OPT_GLOBAL,
+			ARRAY_LEN(cpoptions_vals), cpoptions_vals, dummy_handler, val);
 
 	val.bool_val = 0;
-	add_option("fastrun", "fr", OPT_BOOL, 0, NULL, dummy_handler, val);
+	add_option("fastrun", "fr", "descr", OPT_BOOL, OPT_GLOBAL, 0, NULL,
+			dummy_handler, val);
 
 	val.str_val = "fusehome-default";
-	add_option("fusehome", "fh", OPT_STR, 0, NULL, dummy_handler, val);
+	add_option("fusehome", "fh", "descr", OPT_STR, OPT_GLOBAL, 0, NULL,
+			dummy_handler, val);
 
 	val.str_val = "%r/.vifm-Trash,$HOME/.vifm/Trash";
-	add_option("trashdir", "fh", OPT_STRLIST, 0, NULL, dummy_handler, val);
+	add_option("trashdir", "td", "descr", OPT_STRLIST, OPT_GLOBAL, 0, NULL,
+			dummy_handler, val);
 
 	val.enum_item = 1;
-	add_option("sort", "so", OPT_ENUM, ARRAY_LEN(sort_enum), sort_enum,
+	add_option("sort", "so", "descr", OPT_ENUM, OPT_GLOBAL, ARRAY_LEN(sort_enum),
+			sort_enum, &dummy_handler, val);
+
+	/* Parsing unit doesn't accept options with underscores, although options unit
+	 * doesn't reject them.  Should it? */
+	val.enum_item = 1;
+	add_option("so_rt", "", "descr", OPT_ENUM, OPT_GLOBAL, ARRAY_LEN(sort_enum),
+			sort_enum, &dummy_handler, val);
+
+	val.int_val = 2;
+	add_option("tabstop", "ts", "descr", OPT_INT, OPT_GLOBAL, 0, NULL,
 			&dummy_handler, val);
 
 	val.int_val = 8;
-	add_option("tabstop", "ts", OPT_INT, 0, NULL, &dummy_handler, val);
+	add_option("tabstop", "ts", "descr", OPT_INT, OPT_LOCAL, 0, NULL,
+			&dummy_handler, val);
 
 	val.set_items = 0x81;
-	add_option("vifminfo", "", OPT_SET, ARRAY_LEN(vifminfo_set), vifminfo_set,
-			&dummy_handler, val);
+	add_option("vifminfo", "", "descr", OPT_SET, OPT_GLOBAL,
+			ARRAY_LEN(vifminfo_set), vifminfo_set, &dummy_handler, val);
 }
 
 TEARDOWN()
@@ -103,6 +121,21 @@ TEST(correct_full_option_name_ok)
 	ASSERT_INT_OK("&tabstop", 8);
 }
 
+TEST(correct_local_option_name_ok)
+{
+	ASSERT_INT_OK("&l:tabstop", 8);
+}
+
+TEST(correct_global_option_name_ok)
+{
+	ASSERT_INT_OK("&g:tabstop", 2);
+}
+
+TEST(wrong_option_syntax_fail)
+{
+	ASSERT_FAIL("&x:tabstop", PE_INVALID_EXPRESSION);
+}
+
 TEST(correct_short_option_name_ok)
 {
 	ASSERT_INT_OK("&ts", 8);
@@ -153,6 +186,11 @@ TEST(charset_option_ok)
 	ASSERT_OK("&cpoptions", "bc");
 }
 
+TEST(wrong_char_is_option_name_fail)
+{
+	ASSERT_FAIL("&so_rt", PE_INVALID_EXPRESSION);
+}
+
 TEST(very_long_option_name_fail)
 {
 	ASSERT_FAIL("&ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
@@ -161,4 +199,4 @@ TEST(very_long_option_name_fail)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
