@@ -17,11 +17,6 @@ TEST(simple_ok)
 	ASSERT_OK("\"test\"", "test");
 }
 
-TEST(not_closed_error)
-{
-	ASSERT_FAIL("\"test", PE_MISSING_QUOTE);
-}
-
 TEST(concatenation)
 {
 	ASSERT_OK("\"NV\".\"AR\"", "NVAR");
@@ -51,4 +46,4 @@ TEST(dot_ok)
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
