@@ -50,5 +50,23 @@ TEST(spaces_and_fail_position_correct)
 	assert_string_equal("a b", get_last_position());
 }
 
+TEST(nothing_but_comment)
+{
+	ASSERT_FAIL("\"", PE_INVALID_EXPRESSION);
+	ASSERT_FAIL("\" this is a comment", PE_INVALID_EXPRESSION);
+	ASSERT_FAIL("    \"this is a comment", PE_INVALID_EXPRESSION);
+	ASSERT_FAIL("    \"", PE_INVALID_EXPRESSION);
+}
+
+TEST(expression_and_comment)
+{
+	ASSERT_OK("1\"", "1");
+	ASSERT_OK("'str'\" this is a comment", "str");
+	ASSERT_OK(" 1 && 0 \"this is a comment", "0");
+
+	ASSERT_FAIL(" +   \"", PE_INVALID_EXPRESSION);
+	ASSERT_FAIL(" 4 || \"", PE_INVALID_EXPRESSION);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
